Package: arsenal
Title: An Arsenal of 'R' Functions for Large-Scale Statistical
        Summaries
Version: 3.7.1
Date: 2026-07-01
Authors@R: c(
    person("Ethan", "Heinzen", email = "heinzen.ethan@mayo.edu", role = c("aut", "cre")),
    person("Jason", "Sinnwell", role="aut"),
    person("Elizabeth", "Atkinson", role="aut"),
    person("Tina", "Gunderson", role="aut"),
    person("Gregory", "Dougherty", role="aut"),
    person("Patrick", "Votruba", role="ctb"),
    person("Ryan", "Lennon", role="ctb"),
    person("Andrew", "Hanson", role="ctb"),
    person("Krista", "Goergen", role="ctb"),
    person("Emily", "Lundt", role="ctb"),
    person("Brendan", "Broderick", role="ctb"),
    person("Maddie", "McCullough", role="art")
  )
Description: An Arsenal of 'R' functions for large-scale statistical summaries,
  which are streamlined to work within the latest reporting tools in 'R' and
  'RStudio' and which use formulas and versatile summary statistics for summary
  tables and models. The primary functions include tableby(), a Table-1-like
  summary of multiple variable types 'by' the levels of one or more categorical
  variables; paired(), a Table-1-like summary of multiple variable types paired across
  two time points; modelsum(), which performs simple model fits on one or more endpoints
  for many variables (univariate or adjusted for covariates);
  freqlist(), a powerful frequency table across many categorical variables;
  comparedf(), a function for comparing data.frames; and
  write2(), a function to output tables to a document.
Suggests: broom (>= 0.7.1), magrittr, rmarkdown, testthat, xtable,
        pander, survival (>= 2.43-1), coin, pROC, MASS, splines, rpart,
        yaml, stddiff, geepack
Depends: R (>= 3.4.0), stats (>= 3.4.0)
Imports: knitr (>= 1.29), utils (>= 3.4.0), glue
URL: https://github.com/mayoverse/arsenal,
        https://cran.r-project.org/package=arsenal,
        https://mayoverse.github.io/arsenal/
BugReports: https://github.com/mayoverse/arsenal/issues
VignetteBuilder: knitr
License: GPL (>= 2)
LazyData: true
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-02 02:15:16 UTC; m144326
Author: Ethan Heinzen [aut, cre],
  Jason Sinnwell [aut],
  Elizabeth Atkinson [aut],
  Tina Gunderson [aut],
  Gregory Dougherty [aut],
  Patrick Votruba [ctb],
  Ryan Lennon [ctb],
  Andrew Hanson [ctb],
  Krista Goergen [ctb],
  Emily Lundt [ctb],
  Brendan Broderick [ctb],
  Maddie McCullough [art]
Maintainer: Ethan Heinzen <heinzen.ethan@mayo.edu>
Repository: CRAN
Date/Publication: 2026-07-02 22:30:19 UTC

# Generated by roxygen2: do not edit by hand

S3method("[",arsenal_table)
S3method("[",keep_labels)
S3method("[",keep_tableby_attrs)
S3method("[",selectall)
S3method("[",tableby)
S3method("[",tbstat)
S3method("[",tbstat_countpct)
S3method("[",tbstat_multirow)
S3method("[<-",keep_labels)
S3method("[[",tbsta_multirowt)
S3method("[[",tbstat)
S3method("[[",tbstat_countpct)
S3method("labels<-",arsenal_table)
S3method("labels<-",data.frame)
S3method("labels<-",default)
S3method("labels<-",keep_labels)
S3method(Ops,tableby)
S3method(as.data.frame,freqlist)
S3method(as.data.frame,modelsum)
S3method(as.data.frame,summary.freqlist)
S3method(as.data.frame,summary.modelsum)
S3method(as.data.frame,summary.tableby)
S3method(as.data.frame,tableby)
S3method(as.matrix,selectall)
S3method(c,yaml)
S3method(diffs,comparedf)
S3method(diffs,summary.comparedf)
S3method(format,tbstat)
S3method(freqlist,formula)
S3method(freqlist,table)
S3method(head,summary.freqlist)
S3method(head,tableby)
S3method(includeNA,default)
S3method(includeNA,factor)
S3method(is.na,selectall)
S3method(keep.labels,data.frame)
S3method(keep.labels,default)
S3method(labels,arsenal_table)
S3method(labels,data.frame)
S3method(labels,keep_labels)
S3method(loosen.labels,data.frame)
S3method(loosen.labels,default)
S3method(merge,arsenal_table)
S3method(merge,freqlist)
S3method(n.diff.obs,comparedf)
S3method(n.diff.obs,summary.comparedf)
S3method(n.diffs,comparedf)
S3method(n.diffs,summary.comparedf)
S3method(padjust,default)
S3method(padjust,summary.tableby)
S3method(padjust,tableby)
S3method(print,arsenal_table)
S3method(print,code.chunk)
S3method(print,comparedf)
S3method(print,comparedf.frame.summary)
S3method(print,comparedf.vars.summary)
S3method(print,summary.arsenal_table)
S3method(print,summary.comparedf)
S3method(print,tbstat)
S3method(print,verbatim)
S3method(print,yaml)
S3method(sort,freqlist)
S3method(sort,tableby)
S3method(summary,comparedf)
S3method(summary,freqlist)
S3method(summary,modelsum)
S3method(summary,tableby)
S3method(tail,summary.freqlist)
S3method(tail,tableby)
S3method(tests,tableby)
S3method(write2,arsenal_table)
S3method(write2,character)
S3method(write2,code.chunk)
S3method(write2,comparedf)
S3method(write2,default)
S3method(write2,knitr_kable)
S3method(write2,list)
S3method(write2,summary.arsenal_table)
S3method(write2,summary.comparedf)
S3method(write2,verbatim)
S3method(write2,xtable)
S3method(write2,yaml)
S3method(xtfrm,tableby)
export("%nin%")
export("labels<-")
export(Date.mdy)
export(N)
export(Nevents)
export(NeventsSurv)
export(Nmiss)
export(Nmiss2)
export(Nmisspct)
export(Nmisspct2)
export(Npct)
export(Nrisk)
export(NriskSurv)
export(Nrowpct)
export(Nsigntest)
export(allNA)
export(as.selectall)
export(as.tbstat)
export(as.tbstat_multirow)
export(binomCI)
export(clog)
export(code.chunk)
export(comparedf)
export(comparedf.control)
export(countN)
export(countcellpct)
export(countpct)
export(countrowpct)
export(diffs)
export(formulize)
export(freq.control)
export(freqlist)
export(gmean)
export(gmeanCI)
export(gmeansd)
export(gsd)
export(has_strata)
export(includeNA)
export(iqr)
export(is.Date)
export(is.freqlist)
export(is.modelsum)
export(is.selectall)
export(is.summary.freqlist)
export(is.summary.modelsum)
export(is.summary.tableby)
export(is.tableby)
export(is.yaml)
export(keep.labels)
export(loosen.labels)
export(mdy.Date)
export(meanCI)
export(meanpmsd)
export(meanpmse)
export(meansd)
export(meanse)
export(medSurv)
export(medSurvCI)
export(medSurvQuant)
export(medTime)
export(medianmad)
export(medianq1q3)
export(medianrange)
export(modelsum)
export(modelsum.control)
export(modpval.tableby)
export(muck_up_mockstudy)
export(n.diff.obs)
export(n.diffs)
export(na.modelsum)
export(na.paired)
export(na.tableby)
export(negbin)
export(ordinal)
export(padjust)
export(paired)
export(paired.control)
export(pct)
export(q1q3)
export(relrisk)
export(replace2)
export(rowbinomCI)
export(rowpct)
export(selectall)
export(set_attr)
export(set_labels)
export(smart.split)
export(survival)
export(tableby)
export(tableby.control)
export(tbfmt)
export(tests)
export(tol.NA)
export(tol.char.both)
export(tol.char.case)
export(tol.char.none)
export(tol.char.trim)
export(tol.date.absolute)
export(tol.factor.labels)
export(tol.factor.levels)
export(tol.factor.none)
export(tol.logical.none)
export(tol.num.absolute)
export(tol.num.pct)
export(tol.num.percent)
export(tol.other.none)
export(verbatim)
export(write2)
export(write2html)
export(write2pdf)
export(write2word)
export(yaml)
importFrom(utils,head)
importFrom(utils,tail)

# The `arsenal` Package <a href="https://mayoverse.github.io/arsenal/"><img src="man/figures/logo.png" alt="Arsenal logo" style="float:right;height:232.25px" align="right" height="232.25"></a>

<!-- badges: start -->
[![CRAN_Status_Badge](http://www.r-pkg.org/badges/version/arsenal)](https://CRAN.R-project.org/package=arsenal)
[![Total Downloads](http://cranlogs.r-pkg.org/badges/grand-total/arsenal)](https://CRAN.R-project.org/package=arsenal)
[![Downloads](http://cranlogs.r-pkg.org/badges/arsenal)](https://CRAN.R-project.org/package=arsenal)
[![R-CMD-check](https://github.com/mayoverse/arsenal/workflows/R-CMD-check/badge.svg)](https://github.com/mayoverse/arsenal/actions)
[![R-CMD-check](https://github.com/mayoverse/arsenal/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/mayoverse/arsenal/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

## Overview

The goal of `library(arsenal)` is to make statistical reporting easy. It includes many functions which the useR will find useful to have
in his/her "arsenal" of functions. There are, at this time, 6 main functions, documented below. Each of these functions is
motivated by a local SAS macro or procedure of similar functionality.

Note that `arsenal` v3.0.0 is not backwards compatible with previous versions (mainly because `compare()` got renamed to `comparedf()`).
See the `NEWS` file for more details.

`arsenal` now has a `pkgdown` site: https://mayoverse.github.io/arsenal/

## The `tableby()` Function

`tableby()` is a function to easily summarize a set of independent variables by one or more categorical variables.
Optionally, an appropriate test is performed to test the distribution of the independent variables across
the levels of the categorical variable. Options for this function are easily controlled using `tableby.control()`.

The `tableby()` output is easily knitted in an Rmarkdown document or displayed in the command line using the `summary()` function.
Other S3 methods are implemented for objects from `tableby()`, including `print()`, `[`, `as.data.frame()`, `sort()`, `merge()`, `padjust()`,
`head()`, and `tail()`.

## The `paired()` Function

`paired()` is a function to easily summarize a set of independent variables across two time points.
Optionally, an appropriate test is performed to test the distribution of the independent variables across
the time points. Options for this function are easily controlled using `paired.control()`.

The `paired()` output is easily knitted in an Rmarkdown document or displayed in the command line using the `summary()` function.
It has the same S3 methods as `tableby()`, since it's a special case of the `tableby()` object.

## The `modelsum()` Function

`modelsum()` is a function to fit and summarize models for each independent variable with one or more response variables,
with options to adjust for covariates for each model. Options for this function are easily controlled using `modelsum.control()`.

The `modelsum` output is easily knitted in an Rmarkdown document or displayed in the command line using the `summary()` function.
Other S3 methods are implemented for objects from `modelsum()`, including `print()`, `[`, `as.data.frame()`, and `merge()`.

## The `freqlist()` Function

`freqlist()` is a function to approximate the output from SAS's `PROC FREQ` procedure when using the `/list` option of the `TABLE` statement.
Options for this function are easily controlled using `freq.control()`.

The `freqlist()` output is easily knitted in an Rmarkdown document or displayed in the command line using the `summary()` function.
Other S3 methods are implemented for objects from `freqlist()`, including `print()`, `[`, `as.data.frame()`, `sort()`, and `merge()`.
Additionally, the `summary()` output can be used with `head()` or `tail()`.

## The `comparedf()` Function

`comparedf()` compares two data.frames and reporting any differences between them,
much like SAS's `PROC COMPARE` procedure.

The `comparedf()` output is easily knitted in an Rmarkdown document or displayed in the command line using the `summary()` function.
Other S3 methods are implemented for objects of class `"comparedf"`, including `print()`, `n.diffs()`, `n.diff.obs()`, and `diffs()`.

## The `write2*()` Family of Functions

`write2word()`, `write2pdf()`, and `write2html()` are functions to output a table into a document, much like SAS's `ODS` procedure.
  The S3 method behind them is `write2()`. There are methods implemented for `tableby()`, `modelsum()`, `freqlist()`, and `comparedf()`, and
  also methods for `knitr::kable()`, `xtable::xtable()`, and `pander::pander_return()`. Another option is to coerce an object using
  `verbatim()` to print out the results monospaced (as if they were in the terminal)--the default method does this automatically.
  To output multiple tables into a document, simply make a list of them and call the same function as before. A YAML
  header can be added using `yaml()`. Code chunks can be written using `code.chunk()`.
  
  For more information, see `vignette("write2")`.

## Other Notable Functions

* `keep.labels()` keeps the `'label'` attribute on an R object when subsetting. `loosen.labels()` allows the labels to drop again.

* `formulize()` is a shortcut to collapse variable names into a formula.

* `mdy.Date()` and `Date.mdy()` convert numeric dates for month, day, and year to Date object, and vice versa.

* `is.Date`: tests if an object is a date.

* `%nin%` tests for "not in", the negation of `%in%`.

* `allNA()` tests for all elements being NA, and `includeNA()` makes NAs explicit values.

## ---- echo=FALSE, message=FALSE, results='hide', warning=FALSE----------------
require(knitr)
require(broom)
require(MASS)
require(pROC)
require(rpart)
 
opts_chunk$set(comment = NA, echo=TRUE, prompt=TRUE, collapse=TRUE)


## ---- load-data---------------------------------------------------------------
require(arsenal)
data(mockstudy) # load data
dim(mockstudy)  # look at how many subjects and variables are in the dataset 
# help(mockstudy) # learn more about the dataset and variables
str(mockstudy) # quick look at the data

## ----simple1------------------------------------------------------------------
tab1 <- modelsum(bmi ~ sex + age, data=mockstudy)

## ----simple-text--------------------------------------------------------------
summary(tab1, text=TRUE)

## ----simple-markdown, results='asis'------------------------------------------
summary(tab1)

## -----------------------------------------------------------------------------
as.data.frame(tab1)

## ----adjust, results="asis"---------------------------------------------------
tab2 <- modelsum(alk.phos ~ arm + ps + hgb, adjust= ~age + sex, data=mockstudy)
summary(tab2)

## -----------------------------------------------------------------------------
fit <- lm(alk.phos ~ arm + age + sex, data=mockstudy)
summary(fit)
plot(fit)

## -----------------------------------------------------------------------------
require(MASS)
boxcox(fit)

## -----------------------------------------------------------------------------
fit2 <- lm(log(alk.phos) ~ arm + age + sex, data=mockstudy)
summary(fit2)
plot(fit2)

## -----------------------------------------------------------------------------
require(splines)
fit3 <- lm(log(alk.phos) ~ arm + ns(age, df=2) + sex, data=mockstudy)

# test whether there is a difference between models 
stats::anova(fit2,fit3)

# look at functional form of age
termplot(fit3, term=2, se=T, rug=T)

## -----------------------------------------------------------------------------
tmp <- tidy(fit3) # coefficients, p-values
class(tmp)
tmp

glance(fit3)

## ---- results="asis"----------------------------------------------------------
ms.logy <- modelsum(log(alk.phos) ~ arm + ps + hgb, data=mockstudy, adjust= ~age + sex, 
                    family=gaussian,  
                    gaussian.stats=c("estimate","CI.lower.estimate","CI.upper.estimate","p.value"))
summary(ms.logy)

## -----------------------------------------------------------------------------
boxplot(age ~ mdquality.s, data=mockstudy, ylab=attr(mockstudy$age,'label'), xlab='mdquality.s')

fit <- glm(mdquality.s ~ age + sex, data=mockstudy, family=binomial)
summary(fit)

# create Odd's ratio w/ confidence intervals
tmp <- data.frame(summary(fit)$coef)
tmp

tmp$OR <- round(exp(tmp[,1]),2)
tmp$lower.CI <- round(exp(tmp[,1] - 1.96* tmp[,2]),2)
tmp$upper.CI <- round(exp(tmp[,1] + 1.96* tmp[,2]),2)
names(tmp)[4] <- 'P-value'

kable(tmp[,c('OR','lower.CI','upper.CI','P-value')])

# Assess the predictive ability of the model

# code using the pROC package
require(pROC)
pred <- predict(fit, type='response')
tmp <- pROC::roc(mockstudy$mdquality.s[!is.na(mockstudy$mdquality.s)]~ pred, plot=TRUE, percent=TRUE)
tmp$auc


## -----------------------------------------------------------------------------
tidy(fit, exp=T, conf.int=T) # coefficients, p-values, conf.intervals

glance(fit) # model summary statistics

## ---- results="asis"----------------------------------------------------------
summary(modelsum(mdquality.s ~ age + bmi, data=mockstudy, adjust=~sex, family=binomial))

fitall <- modelsum(mdquality.s ~ age, data=mockstudy, family=binomial,
                   binomial.stats=c("Nmiss2","OR","p.value"))
summary(fitall)

## ----survival-----------------------------------------------------------------
require(survival)

# multivariable model with all 3 terms
fit  <- coxph(Surv(fu.time, fu.stat) ~ age + sex + arm, data=mockstudy)
summary(fit)

# check proportional hazards assumption
fit.z <- cox.zph(fit)
fit.z
plot(fit.z[1], resid=FALSE) # makes for a cleaner picture in this case
abline(h=coef(fit)[1], col='red')

# check functional form for age using pspline (penalized spline)
# results are returned for the linear and non-linear components
fit2 <- coxph(Surv(fu.time, fu.stat) ~ pspline(age) + sex + arm, data=mockstudy)
fit2

# plot smoothed age to visualize why significant
termplot(fit2, se=T, terms=1)
abline(h=0)

# The c-statistic comes out in the summary of the fit
summary(fit2)$concordance

# It can also be calculated using the survConcordance function
survConcordance(Surv(fu.time, fu.stat) ~ predict(fit2), data=mockstudy)

## -----------------------------------------------------------------------------
tidy(fit) # coefficients, p-values

glance(fit) # model summary statistics

## ----results="asis"-----------------------------------------------------------
##Note: You must use quotes when specifying family="survival" 
##      family=survival will not work
summary(modelsum(Surv(fu.time, fu.stat) ~ arm, 
                 adjust=~age + sex, data=mockstudy, family="survival"))

##Note: the pspline term is not working yet
#summary(modelsum(Surv(fu.time, fu.stat) ~ arm, 
#                adjust=~pspline(age) + sex, data=mockstudy, family='survival'))

## ----poisson------------------------------------------------------------------
require(rpart) ##just to get access to solder dataset
data(solder)
hist(solder$skips)

fit <- glm(skips ~ Opening + Solder + Mask , data=solder, family=poisson)
stats::anova(fit, test='Chi')
summary(fit)

## -----------------------------------------------------------------------------
1-pchisq(fit$deviance, fit$df.residual)

## -----------------------------------------------------------------------------
fit2 <- glm(skips ~ Opening + Solder + Mask, data=solder, family=quasipoisson)
summary(fit2)

## -----------------------------------------------------------------------------
tidy(fit) # coefficients, p-values

glance(fit) # model summary statistics

## ----results='asis'-----------------------------------------------------------
summary(modelsum(skips~Opening + Solder + Mask, data=solder, family="quasipoisson"))
summary(modelsum(skips~Opening + Solder + Mask, data=solder, family="poisson"))

## -----------------------------------------------------------------------------
# add .01 to the follow-up time (.01*1 day) in order to keep everyone in the analysis
fit <- glm(fu.stat ~ offset(log(fu.time+.01)) + age + sex + arm, data=mockstudy, family=poisson)
summary(fit)
1-pchisq(fit$deviance, fit$df.residual)

coef(coxph(Surv(fu.time,fu.stat) ~ age + sex + arm, data=mockstudy))
coef(fit)[-1]

# results from the Poisson model can then be described as risk ratios (similar to the hazard ratio)
exp(coef(fit)[-1])

# As before, we can model the dispersion which alters the standard error
fit2 <- glm(fu.stat ~ offset(log(fu.time+.01)) + age + sex + arm, 
            data=mockstudy, family=quasipoisson)
summary(fit2)

## -----------------------------------------------------------------------------
tidy(fit) ##coefficients, p-values

glance(fit) ##model summary statistics

## ----results="asis", eval=TRUE------------------------------------------------
summary(modelsum(fu.stat ~ age, adjust=~offset(log(fu.time+.01))+ sex + arm, 
                 data=mockstudy, family=poisson))
                 

## ---- results='asis'----------------------------------------------------------
mycontrols  <- modelsum.control(gaussian.stats=c("estimate","std.error","adj.r.squared","Nmiss"),
                                show.adjust=FALSE, show.intercept=FALSE)                            
tab2 <- modelsum(bmi ~ age, adjust=~sex, data=mockstudy, control=mycontrols)
summary(tab2)

## ---- results='asis'----------------------------------------------------------
tab3 <- modelsum(bmi ~  age, adjust=~sex, data=mockstudy,
                 gaussian.stats=c("estimate","std.error","adj.r.squared","Nmiss"), 
                 show.intercept=FALSE, show.adjust=FALSE)
summary(tab3)

## ----check-labels-------------------------------------------------------------
## Look at one variable's label
attr(mockstudy$age,'label')

## See all the variables with a label
unlist(lapply(mockstudy,'attr','label'))

## or
cbind(sapply(mockstudy,attr,'label'))

## ----add-label, results='asis'------------------------------------------------
attr(mockstudy$age,'label')  <- 'Age, yrs'

tab1 <- modelsum(bmi ~  age, adjust=~sex, data=mockstudy)
summary(tab1)

## ---- results = 'asis'--------------------------------------------------------
labels(mockstudy)  <- c(age = 'Age, yrs')

tab1 <- modelsum(bmi ~  age, adjust=~sex, data=mockstudy)
summary(tab1)

## ---- results='asis'----------------------------------------------------------
mylabels <- list(sexFemale = "Female", age ="Age, yrs")
summary(tab1, labelTranslations = mylabels)

## ---- eval=TRUE---------------------------------------------------------------
labels(tab1)
labels(tab1) <- c(sexFemale="Female", age="Baseline Age (yrs)")
labels(tab1)

## ---- results='asis'----------------------------------------------------------
summary(tab1)

## ---- results='asis'----------------------------------------------------------
summary(modelsum(age~mdquality.s+sex, data=mockstudy), show.intercept=FALSE)

## ---- results='asis'----------------------------------------------------------
summary(modelsum(mdquality.s ~ age + bmi, data=mockstudy, adjust=~sex, family=binomial),
        show.adjust=FALSE)  

## ---- results='asis'----------------------------------------------------------
# create a vector specifying the variable names
myvars <- names(mockstudy)

# select the 8th through the 12th
# paste them together, separated by the + sign
RHS <- paste(myvars[8:12], collapse="+")
RHS

# create a formula using the as.formula function
as.formula(paste('mdquality.s ~ ', RHS))

# use the formula in the modelsum function
summary(modelsum(as.formula(paste('mdquality.s ~', RHS)), family=binomial, data=mockstudy))

## ---- results='asis'----------------------------------------------------------
## The formulize function does the paste and as.formula steps
tmp <- formulize('mdquality.s',myvars[8:10])
tmp

## More complex formulas could also be written using formulize
tmp2 <- formulize('mdquality.s',c('ps','hgb','sqrt(bmi)'))

## use the formula in the modelsum function
summary(modelsum(tmp, data=mockstudy, family=binomial))

## -----------------------------------------------------------------------------
newdata <- subset(mockstudy, subset=age>50 & arm=='F: FOLFOX', select = c(age,sex, bmi:alk.phos))
dim(mockstudy)
table(mockstudy$arm)
dim(newdata)
names(newdata)

## ---- results='asis'----------------------------------------------------------
summary(modelsum(alk.phos ~ ., data=newdata))

## ---- results='asis', eval=TRUE-----------------------------------------------
summary(modelsum(log(alk.phos) ~ sex + ps + bmi, subset=age>50 & arm=="F: FOLFOX", data=mockstudy))
summary(modelsum(alk.phos ~ ps + bmi, adjust=~sex, subset = age>50 & bmi<24, data=mockstudy))
summary(modelsum(alk.phos ~ ps + bmi, adjust=~sex, subset=1:30, data=mockstudy))

## -----------------------------------------------------------------------------
## create a variable combining the levels of mdquality.s and sex
with(mockstudy, table(interaction(mdquality.s,sex)))

## ---- results='asis'----------------------------------------------------------
summary(modelsum(age ~ interaction(mdquality.s,sex), data=mockstudy))

## ---- results='asis'----------------------------------------------------------
summary(modelsum(arm=="F: FOLFOX" ~ I(age/10) + log(bmi) + mdquality.s,
                 data=mockstudy, family=binomial))

## ---- results='asis'----------------------------------------------------------
mytab <- modelsum(bmi ~ sex + alk.phos + age, data=mockstudy)
mytab2 <- mytab[c('age','sex','alk.phos')]
summary(mytab2)
summary(mytab[c('age','sex')])
summary(mytab[c(3,1)])

## ---- results="asis"----------------------------------------------------------
## demographics
tab1 <- modelsum(bmi ~ sex + age, data=mockstudy)
## lab data
tab2 <- modelsum(mdquality.s ~ hgb + alk.phos, data=mockstudy, family=binomial)
                
tab12 <- merge(tab1, tab2, all = TRUE)
class(tab12)
summary(tab12)

## ---- results='asis'----------------------------------------------------------
t1 <- modelsum(bmi ~ sex + age, data=mockstudy)
summary(t1, title='Demographics')

## -----------------------------------------------------------------------------
## look at how many missing values there are for each variable
apply(is.na(mockstudy),2,sum)

## ---- results='asis'----------------------------------------------------------
## Show how many subjects have each variable (non-missing)
summary(modelsum(bmi ~ ast + age, data=mockstudy,
                control=modelsum.control(gaussian.stats=c("N","estimate"))))

## Always list the number of missing values
summary(modelsum(bmi ~ ast + age, data=mockstudy,
                control=modelsum.control(gaussian.stats=c("Nmiss2","estimate"))))

## Only show the missing values if there are some (default)
summary(modelsum(bmi ~ ast + age, data=mockstudy, 
                control=modelsum.control(gaussian.stats=c("Nmiss","estimate"))))

## Don't show N at all
summary(modelsum(bmi ~ ast + age, data=mockstudy, 
                control=modelsum.control(gaussian.stats=c("estimate"))))

## ---- results='asis'----------------------------------------------------------
summary(modelsum(bmi ~ sex + age + fu.time, data=mockstudy), digits=4, digits.test=2)

## -----------------------------------------------------------------------------
mockstudy$agegp <- cut(mockstudy$age, breaks=c(18,50,60,70,90), right=FALSE)

## create weights based on agegp and sex distribution
tab1 <- with(mockstudy,table(agegp, sex))
tab1
tab2 <- with(mockstudy, table(agegp, sex, arm))
gpwts <- rep(tab1, length(unique(mockstudy$arm)))/tab2

## apply weights to subjects
index <- with(mockstudy, cbind(as.numeric(agegp), as.numeric(sex), as.numeric(as.factor(arm)))) 
mockstudy$wts <- gpwts[index]

## show weights by treatment arm group
tapply(mockstudy$wts,mockstudy$arm, summary)

## ----results='asis'-----------------------------------------------------------
mockstudy$newvarA <- as.numeric(mockstudy$arm=='A: IFL')
tab1 <- modelsum(newvarA ~ ast + bmi + hgb, data=mockstudy, subset=(arm !='G: IROX'), 
                 family=binomial)
summary(tab1, title='No Case Weights used')

suppressWarnings({
tab2 <- modelsum(newvarA ~ ast + bmi + hgb, data=mockstudy, subset=(arm !='G: IROX'), 
                 weights=wts, family=binomial)
summary(tab2, title='Case Weights used')
})

## -----------------------------------------------------------------------------
summary(tab2, text=T)
tmp <- as.data.frame(summary(tab2, text = TRUE))
tmp
# write.csv(tmp, '/my/path/here/mymodel.csv')

## ----eval = FALSE-------------------------------------------------------------
#  ## write to an HTML document
#  write2html(tab2, "~/ibm/trash.html")
#  
#  ## write to a Word document
#  write2word(tab2, "~/ibm/trash.doc", title="My table in Word")

## ----eval=FALSE---------------------------------------------------------------
#  # A standalone shiny app
#  library(shiny)
#  library(arsenal)
#  data(mockstudy)
#  
#  shinyApp(
#    ui = fluidPage(tableOutput("table")),
#    server = function(input, output) {
#      output$table <- renderTable({
#        as.data.frame(summary(modelsum(age ~ sex, data = mockstudy), text = "html"))
#      }, sanitize.text.function = function(x) x)
#    }
#  )

## ----eval=FALSE---------------------------------------------------------------
#  summary(modelsum(age ~ sex, data = mockstudy), title="(\\#tab:mytableby) Caption here")

## ----results='asis'-----------------------------------------------------------
summary(modelsum(list(age, hgb) ~ bmi + sex, adjust = ~ arm, data = mockstudy))

## ----results='asis'-----------------------------------------------------------
summary(modelsum(list(age, hgb) ~ bmi + sex, adjust = ~ arm, data = mockstudy), term.name = TRUE)

## ----results='asis'-----------------------------------------------------------
summary(modelsum(list(age, hgb) ~ bmi + sex, strata = arm, data = mockstudy))

## -----------------------------------------------------------------------------
adj.list <- list(
  Unadjusted = ~ 1, # can also specify NULL here
  "Adjusted for Arm" = ~ arm
)
multi.adjust <- modelsum(list(age, bmi) ~ fu.time + ast, adjust = adj.list, data = mockstudy)
summary(multi.adjust, adjustment.names = TRUE)
summary(multi.adjust, adjustment.names = TRUE, show.intercept = FALSE, show.adjust = FALSE)

## -----------------------------------------------------------------------------
args(modelsum.control)

## -----------------------------------------------------------------------------
args(arsenal:::summary.modelsum)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE, tidy.opts=list(width.cutoff=80), tidy=TRUE, comment=NA)
options(width=80, max.print=1000)

## ----message = FALSE----------------------------------------------------------
require(arsenal)

## ----loading.data-------------------------------------------------------------
# load the data
data(mockstudy)

# retain NAs when creating the table using the useNA argument
tab.ex <- table(mockstudy[, c("arm", "sex", "mdquality.s")], useNA="ifany")

## ----console.output-----------------------------------------------------------
example1 <- freqlist(tab.ex)

str(example1)

# view the data frame portion of freqlist output
head(as.data.frame(example1)) ## or use as.data.frame(example1)

## ---- results = 'asis'--------------------------------------------------------
summary(example1)

## ---- results = 'asis'--------------------------------------------------------
summary(example1, title="Basic freqlist output")

## -----------------------------------------------------------------------------
head(as.data.frame(summary(example1)))

## ----results='asis'-----------------------------------------------------------
summary(freqlist(~ arm + sex + mdquality.s, data = mockstudy))

## ----results='asis'-----------------------------------------------------------
summary(freqlist(~ arm + sex + includeNA(mdquality.s, "Missing"), data = mockstudy))

## ----results='asis'-----------------------------------------------------------
mockstudy$weights <- c(10000, rep(1, nrow(mockstudy) - 1))
summary(freqlist(weights ~ arm + sex + addNA(mdquality.s), data = mockstudy))

## ----results='asis'-----------------------------------------------------------
mockstudy$weights2 <- c(rep(1, nrow(mockstudy) - 1), 10000)
summary(freqlist(list(weights, weights2) ~ arm + sex + addNA(mdquality.s), data = mockstudy))

## ----labelTranslations, results = 'asis'--------------------------------------
example2 <- freqlist(tab.ex, labelTranslations = c(arm = "Treatment Arm", sex = "Gender", mdquality.s = "LASA QOL"),
                      digits.pct = 1, digits.count = 1)
summary(example2)

## ----sparse, results = 'asis'-------------------------------------------------
summary(freqlist(~ race + sex + arm, data = mockstudy, sparse = TRUE, digits.pct=1))

## ----na.options, results = 'asis'---------------------------------------------
summary(freqlist(tab.ex, na.options="include"))
summary(freqlist(tab.ex, na.options="showexclude"))
summary(freqlist(tab.ex, na.options="remove"))

## ----freq.counts, results='asis'----------------------------------------------
example3 <- freqlist(tab.ex, strata = c("arm","sex"))
summary(example3)

#using the single = TRUE argument will collapse results into a single table for printing
summary(example3, single = TRUE)


## -----------------------------------------------------------------------------
head(summary(sort(example1, decreasing = TRUE), dupLabels = TRUE))

## ----changelabs, results = 'asis'---------------------------------------------
labs <- c(arm = "Arm", sex = "Sex", mdquality.s = "QOL", freqPercent = "%")
labels(example1) <- labs
summary(example1)

## ---- results = 'asis'--------------------------------------------------------
summary(example1, labelTranslations = labs)

## ----results='asis'-----------------------------------------------------------
require(xtable)

# set up custom function for xtable text
italic <- function(x) paste0('<i>', x, '</i>')

xftbl <- xtable(as.data.frame(summary(example1)), 
  caption = "xtable formatted output of freqlist data frame", align="|r|r|r|r|c|c|c|r|")

# change the column names
names(xftbl)[1:3] <- c("Arm", "Gender", "LASA QOL")

print(xftbl, sanitize.colnames.function = italic, include.rownames = FALSE, type = "html", comment = FALSE)

## ----eval=FALSE---------------------------------------------------------------
#  summary(freqlist(~ sex + age, data = mockstudy), title="(\\#tab:mytableby) Caption here")

## -----------------------------------------------------------------------------
# base table default removes NAs
tab.d1 <- base::table(mockstudy[, c("arm", "sex", "mdquality.s")], useNA="ifany")
tab.d1

## -----------------------------------------------------------------------------
# without specifying addNA
tab.d2 <- xtabs(formula = ~ arm + sex + mdquality.s, data = mockstudy)
tab.d2

# now with addNA
tab.d3 <- xtabs(~ arm + sex + addNA(mdquality.s), data = mockstudy)
tab.d3


## -----------------------------------------------------------------------------
# providing variables separately (as vectors) drops column names
table(mockstudy$arm, mockstudy$sex, mockstudy$mdquality.s)

## -----------------------------------------------------------------------------
# add the column name labels back using dnn option in base::table
table(mockstudy$arm, mockstudy$sex, mockstudy$mdquality.s, dnn=c("Arm", "Sex", "QOL"))

## -----------------------------------------------------------------------------
table(Arm = mockstudy$arm, Sex = mockstudy$sex, QOL = mockstudy$mdquality.s)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE, tidy.opts=list(width.cutoff=80), tidy=TRUE, comment=NA)
options(width=80, max.print=1000)

## ----message = FALSE----------------------------------------------------------
require(arsenal)

## ----loading.data-------------------------------------------------------------
# load the data
data(mockstudy)

# retain NAs when creating the table using the useNA argument
tab.ex <- table(mockstudy[, c("arm", "sex", "mdquality.s")], useNA="ifany")

## ----console.output-----------------------------------------------------------
example1 <- freqlist(tab.ex)

str(example1)

# view the data frame portion of freqlist output
head(as.data.frame(example1)) ## or use as.data.frame(example1)

## ----results = 'asis'---------------------------------------------------------
summary(example1)

## ----results = 'asis'---------------------------------------------------------
summary(example1, title="Basic freqlist output")

## -----------------------------------------------------------------------------
head(as.data.frame(summary(example1)))

## ----results='asis'-----------------------------------------------------------
summary(freqlist(~ arm + sex + mdquality.s, data = mockstudy))

## ----results='asis'-----------------------------------------------------------
summary(freqlist(~ arm + sex + includeNA(mdquality.s, "Missing"), data = mockstudy))

## ----results='asis'-----------------------------------------------------------
mockstudy$weights <- c(10000, rep(1, nrow(mockstudy) - 1))
summary(freqlist(weights ~ arm + sex + addNA(mdquality.s), data = mockstudy))

## ----results='asis'-----------------------------------------------------------
mockstudy$weights2 <- c(rep(1, nrow(mockstudy) - 1), 10000)
summary(freqlist(list(weights, weights2) ~ arm + sex + addNA(mdquality.s), data = mockstudy))

## ----labelTranslations, results = 'asis'--------------------------------------
example2 <- freqlist(tab.ex, labelTranslations = c(arm = "Treatment Arm", sex = "Gender", mdquality.s = "LASA QOL"),
                      digits.pct = 1, digits.count = 1)
summary(example2)

## ----sparse, results = 'asis'-------------------------------------------------
summary(freqlist(~ race + sex + arm, data = mockstudy, sparse = TRUE, digits.pct=1))

## ----na.options, results = 'asis'---------------------------------------------
summary(freqlist(tab.ex, na.options="include"))
summary(freqlist(tab.ex, na.options="showexclude"))
summary(freqlist(tab.ex, na.options="remove"))

## ----freq.counts, results='asis'----------------------------------------------
example3 <- freqlist(tab.ex, strata = c("arm","sex"))
summary(example3)

#using the single = TRUE argument will collapse results into a single table for printing
summary(example3, single = TRUE)


## -----------------------------------------------------------------------------
head(summary(sort(example1, decreasing = TRUE), dupLabels = TRUE))

## ----changelabs, results = 'asis'---------------------------------------------
labs <- c(arm = "Arm", sex = "Sex", mdquality.s = "QOL", freqPercent = "%")
labels(example1) <- labs
summary(example1)

## ----results = 'asis'---------------------------------------------------------
summary(example1, labelTranslations = labs)

## ----results='asis'-----------------------------------------------------------
require(xtable)

# set up custom function for xtable text
italic <- function(x) paste0('<i>', x, '</i>')

xftbl <- xtable(as.data.frame(summary(example1)), 
  caption = "xtable formatted output of freqlist data frame", align="|r|r|r|r|c|c|c|r|")

# change the column names
names(xftbl)[1:3] <- c("Arm", "Gender", "LASA QOL")

print(xftbl, sanitize.colnames.function = italic, include.rownames = FALSE, type = "html", comment = FALSE)

## ----eval=FALSE---------------------------------------------------------------
# summary(freqlist(~ sex + age, data = mockstudy), title="(\\#tab:mytableby) Caption here")

## -----------------------------------------------------------------------------
# base table default removes NAs
tab.d1 <- base::table(mockstudy[, c("arm", "sex", "mdquality.s")], useNA="ifany")
tab.d1

## -----------------------------------------------------------------------------
# without specifying addNA
tab.d2 <- xtabs(formula = ~ arm + sex + mdquality.s, data = mockstudy)
tab.d2

# now with addNA
tab.d3 <- xtabs(~ arm + sex + addNA(mdquality.s), data = mockstudy)
tab.d3


## -----------------------------------------------------------------------------
# providing variables separately (as vectors) drops column names
table(mockstudy$arm, mockstudy$sex, mockstudy$mdquality.s)

## -----------------------------------------------------------------------------
# add the column name labels back using dnn option in base::table
table(mockstudy$arm, mockstudy$sex, mockstudy$mdquality.s, dnn=c("Arm", "Sex", "QOL"))

## -----------------------------------------------------------------------------
table(Arm = mockstudy$arm, Sex = mockstudy$sex, QOL = mockstudy$mdquality.s)

#' Helper functions for paired
#'
#' A set of helper functions for \code{\link{paired}}.
#'
#' @param missings A character string denoting which action to take. See "Details", below.
#' @return \code{na.paired} returns a function used to subset data.frames in \code{\link{paired}}.
#' @details
#'   All methods subset out any NA time points or IDs.
#'   \code{"in.both"} (the default) subsets the data.frame to individuals who appear at both time points.
#'   \code{"fill"} adds explicit missings for the people missing second time points.
#'   \code{"asis"} does nothing to add or remove missings.
#' @seealso \link{tableby.internal}
#' @name paired.internal
NULL
#> NULL


# 'fill' puts in the missing time points
# 'asis' doesn't do anything
# 'in.both' subsets to the people in both

#' @rdname paired.internal
#' @export
na.paired <- function(missings = c("in.both", "fill", "asis"))
{
  missings <- match.arg(missings)
  switch(
    missings,
    in.both = function(object, ...)
    {
      obj.id <- object[["(id)"]]
      omit <- is.na(object[[1]]) | is.na(obj.id)
      if("(strata)" %in% names(object)) omit <- omit | is.na(object[["(strata)"]])

      by.col <- object[[1]][!omit]
      if(is.factor(by.col)) {
        by.col <- droplevels(by.col)
        by.levels <- levels(by.col)
      } else by.levels <- sort(unique(by.col))
      if(length(by.levels) != 2) stop("Please specify exactly 2 time points")

      ids <- object[["(id)"]][!omit]
      omit <- omit | (obj.id %nin% intersect(ids[by.col == by.levels[1]], ids[by.col == by.levels[2]]))
      xx <- object[!omit, , drop = FALSE]

      if(any(omit)) {
        temp <- stats::setNames(seq_along(omit)[omit], attr(object, "row.names")[omit])
        attr(temp, "class") <- "omit"
        attr(xx, "na.action") <- temp
      }
      xx
    },
    fill = function(object, ...)
    {
      omit <- is.na(object[[1]]) | is.na(object[["(id)"]])
      if("(strata)" %in% names(object)) omit <- omit | is.na(object[["(strata)"]])

      xx <- object[!omit, , drop = FALSE]

      all.pairs <- expand.grid(times = unique(xx[[1]]), id = unique(xx[["(id)"]]),
                               stringsAsFactors = FALSE, KEEP.OUT.ATTRS = FALSE)
      xx <- merge(xx, all.pairs, by.x = c(colnames(xx)[1], "(id)"), by.y = c("times", "id"),
            all = TRUE, sort = FALSE)[names(xx)]
      if(any(omit)) {
        temp <- stats::setNames(seq_along(omit)[omit], attr(object, "row.names")[omit])
        attr(temp, "class") <- "omit"
        attr(xx, "na.action") <- temp
      }
      xx
    },
    asis = function(object, ...)
    {
      # take away na's in tp and id
      omit <- is.na(object[[1]]) | is.na(object[["(id)"]])
      if("(strata)" %in% names(object)) omit <- omit | is.na(object[["(strata)"]])

      xx <- object[!omit, , drop = FALSE]
      if(any(omit)) {
        temp <- stats::setNames(seq_along(omit)[omit], attr(object, "row.names")[omit])
        attr(temp, "class") <- "omit"
        attr(xx, "na.action") <- temp
      }
      xx
    }
  )
}

release_questions <- function() {
  c(
    "Have you updated the DESCRIPTION file? Make sure the version number is right.",
    "Have you checked for reverse dependencies?",
    "Have you updated README.md?",
    "Have you updated NEWS.md?",
    "Have you updated cran-comments.md?",
    "Have you updated all the documentation using devtools::check_man()?",
    "Have you gotten approval from all authors to push to CRAN?",
    "Did you make sure the DESCRIPTION matches what's in arsenal.R?"
  )
}

## Created: 12/13/2016
## Author: Ethan Heinzen

#' An Arsenal of 'R' Functions for Large-Scale Statistical Summaries
#'
#' An Arsenal of 'R' functions for large-scale statistical summaries,
#'   which are streamlined to work within the latest reporting tools in 'R' and 'RStudio' and
#'   which use formulas and versatile summary statistics for summary tables and models.
#'
#' The package download, NEWS, and README are available on CRAN: \url{https://cran.r-project.org/package=arsenal}
#'
#' @section Functions:
#'
#' Below are listed some of the most widely used functions available in \code{arsenal}:
#'
#' \code{\link{tableby}}: Summary statistics of a set of independent variables by a categorical variable.
#'
#' \code{\link{paired}}: Summary statistics of a set of independent variables paired across two timepoints.
#'
#' \code{\link{modelsum}}: Fit models over each of a set of independent variables with a response variable.
#'
#' \code{\link{freqlist}}: Approximate the output from SAS's \code{PROC FREQ} procedure when using
#'  the \code{/list} option of the \code{TABLE} statement.
#'
#' \code{\link{comparedf}}: Compare two data.frames and report any differences between them,
#'  much like SAS's \code{PROC COMPARE} procedure.
#'
#' \code{\link{write2word}}, \code{\link{write2html}}, \code{\link{write2pdf}}: Functions to output
#'   tables to a single Word, HTML, or PDF document.
#'
#' \code{\link{write2}}: Functions to output tables to a single document.
#'   (Also the S3 backbone behind the \code{write2*} functions.)
#'
#' \code{\link{keep.labels}}: Keep the \code{'label'} attribute on an R object when subsetting.
#'
#' \code{\link{formulize}}: A shortcut to generate one-, two-, or many-sided formulas.
#'
#' \code{\link{mdy.Date}} and \code{\link{Date.mdy}}: Convert numeric dates for month, day, and year to Date object, and vice versa.
#'
#' \code{\link{is.Date}}: Test if an object is a date.
#'
#' \code{\link{\%nin\%}}: Test for "not in".
#'
#' \code{\link{allNA}} and \code{\link{includeNA}}: some useful functions for dealing with NAs.
#'
#' @section Data:
#'
#' \code{\link{mockstudy}}: Mock study data for examples.
#'
#' @examples
#' library(arsenal)
#'
#' @importFrom utils head tail
#'
"_PACKAGE"


#### commands to build the package using devtools
# devtools::check_man()
# devtools::test()
# devtools::check()
# withr::with_libpaths(c("../testinstalls/", .libPaths()), devtools::install(build_vignettes = TRUE, dependencies = FALSE))
# devtools::build("../rpkg-arsenal/")
## < restart R >
## library(arsenal, lib.loc = "../testinstalls/")

#### to upload to CRAN
## Update DESCRIPTION, README.md, NEWS.md, and cran-comments.md
# devtools::revdep_check()
# devtools::release()

#' summary.freqlist
#'
#' Summarize the \code{freqlist} object.
#'
#' @param object an object of class \code{\link{freqlist}}
#' @param ... For \code{summary.freqlist}, these are passed to \code{\link{as.data.frame.freqlist}} (and hence to
#'   \code{\link{freq.control}}). For the print method, these are
#'   additional arguments passed to the \code{\link[knitr]{kable}} function.
#' @param x An object of class \code{summary.freqlist}.
#' @inheritParams summary.tableby
#' @return An object of class \code{"summary.freqlist"} (invisibly for the print method).
#' @seealso \code{\link{freqlist}}, \code{\link[base]{table}}, \code{\link[stats]{xtabs}}, \code{\link[knitr]{kable}},
#'   \code{\link{freq.control}}, \code{\link{freqlist.internal}}
#'
#' @examples
#' # load mockstudy data
#' data(mockstudy)
#' tab.ex <- table(mockstudy[c("arm", "sex", "mdquality.s")], useNA = "ifany")
#' noby <- freqlist(tab.ex, na.options = "include")
#' summary(noby)
#' withby <- freqlist(tab.ex, strata = c("arm","sex"), na.options = "showexclude")
#' summary(withby)
#' summary(withby, dupLabels = TRUE)
#' @author Tina Gunderson, with major revisions by Ethan Heinzen
#' @name summary.freqlist
NULL
#> NULL

#' @rdname summary.freqlist
#' @export
summary.freqlist <- function(object, ..., labelTranslations = NULL, title = NULL)
{
  dat <- as.data.frame(object, ..., labelTranslations = labelTranslations, list.ok = TRUE)
  structure(list(
    object = set_attr(dat, "control", NULL),
    control = attr(dat, "control"),
    title = title
  ), class = c("summary.freqlist", "summary.arsenal_table"))
}

as_data_frame_summary_freqlist <- function(tb, labs, cntrl)
{
  fmtdups <- function(x, i)
  {
    x[i] <- lapply(x[i], as.character)
    if(nrow(x) == 0) return(x)
    tab <- as.matrix(x[i])
    tab[is.na(tab)] <- "NA"
    num <- max(lengths(gregexpr(",", tab)))

    for(col in seq_len(ncol(tab)))
    {
      tmp <- apply(tab[, 1:col, drop = FALSE], 1, paste, collapse = strrep(",", num + 1))
      x[c(FALSE, tmp[-1] == tmp[-length(tmp)]), colnames(tab)[col]] <- ""
    }
    x
  }

  fmtdigits <- function(x, digits.count, digits.pct)
  {
    if(nrow(x) == 0) return(x)
    if("Freq" %in% names(x)) x$Freq <- formatC(x$Freq, digits = digits.count, format = "f")
    if("cumFreq" %in% names(x)) x$cumFreq <- formatC(x$cumFreq, digits = digits.count, format = "f")
    if("freqPercent" %in% names(x)) x$freqPercent <- formatC(x$freqPercent, digits = digits.pct, format = "f")
    if("cumPercent" %in% names(x)) x$cumPercent <- formatC(x$cumPercent, digits = digits.pct, format = "f")
    x
  }
  idx <- names(tb) %nin% c("Freq", "cumFreq", "freqPercent", "cumPercent")
  tb <- fmtdigits(tb, digits.count = cntrl$digits.count, digits.pct = cntrl$digits.pct)
  if(!cntrl$dupLabels) tb <- fmtdups(tb, idx)
  tb <- stats::setNames(tb, labs[names(tb)])
  set_attr(set_attr(tb, "labels", NULL), "align", c("r", "l")[1 + idx])
}

#' @rdname summary.freqlist
#' @export
as.data.frame.summary.freqlist <- function(x, ..., list.ok = FALSE)
{
  out <- Map(x$object, lapply(x$object, attr, "labels"), f = as_data_frame_summary_freqlist, MoreArgs = list(cntrl = x$control))

  if(!list.ok)
  {
    if(length(out) == 1) out <- out[[1]] else warning("as.data.frame.summary.freqlist is returning a list of data.frames")
  }
  out
}

# set.seed(100)
# nsubj <- 90 # need multiple of 3
# mdat <- data.frame(
#   Group = c(rep("High", nsubj/3), rep("Med", nsubj/3), rep("Low", nsubj/3)),
#   Sex = sample(c("Male", "Female"), nsubj, replace=TRUE),
#   Age = round(rnorm(nsubj, mean=40, sd=5)),
#   Phase = ordered(sample(c("I", "II", "III"), nsubj, replace=TRUE), levels=c("I", "II", "III")),
#   ht_in = round(rnorm(nsubj, mean=65, sd=5)),
#   time = round(runif(nsubj,0,7)),
#   status = rbinom(nsubj, 1, prob=0.4),
#   dt = as.Date(round(rnorm(90, mean=100, sd=2000)), origin="1950/01/01"),
#   missing = as.character(NA),
#   trt = factor(sample(c("A", "B"), nsubj, replace=TRUE)),
#   ethan = factor(c(NA, NA, NA, sample(c("Ethan", "Heinzen"), nsubj - 3, replace=TRUE))),
#   weights = c(20, 1.5, rep(1, nsubj - 2)),
#   stringsAsFactors = FALSE
# )
# mdat$Group.fac <- factor(mdat$Group)
# attr(mdat$ht_in, "label") <- "Height in Inches"
# attr(mdat$trt, "label") <- "Treatment Arm"
# attr(mdat$Age, "label") <- "Age in Years"
#
# class(mdat$Sex) <- c("dummyClassToTriggerErrors", class(mdat$Sex))
#
# saveRDS(mdat, "tests/testthat/mdat.rds", compress = TRUE)


## As of 2019-02-26 the random number generator behind sample() changed, so I've saved out a copy to test against instead.
mdat <- readRDS("mdat.rds")

#' Labels
#'
#' Assign and extract the \code{'label'} attribute on an R object. \code{set_labels} is
#'   the same as \code{labels(x) <- value} but returns \code{x} for use in a pipe chain.
#'   \code{set_attr} is the same as \code{attr(x, which) <- value} but returns \code{x}
#'   for use in a pipe chain.
#'
#' @param x,object An R object.
#' @param value A vector or list containing labels to assign. Labels are assigned based on
#'   names, if available; otherwise, they're assigned in order. Can pass \code{NULL}
#'   to remove all labels.
#' @param which See \code{\link{attr<-}}
#' @param ... Other arguments (not in use at this time).
#' @return The labels of \code{object}, or \code{object} with new labels.
#' @details
#'   The \code{\link{data.frame}} methods put labels on and extract labels from
#'   the \emph{columns} of \code{object}.
#' @seealso \code{\link{keep.labels}}
#' @author Ethan Heinzen
#' @name labels
NULL
#> NULL

#' @rdname labels
#' @export
labels.data.frame <- function(object, ...)
{
  lapply(object, attr, which = "label", exact = TRUE)
}

#' @rdname labels
#' @export
labels.keep_labels <- function(object, ...)
{
  attr(object, "label", exact = TRUE)
}

#' @rdname labels
#' @export
'labels<-' <- function(x, value)
{
  UseMethod("labels<-")
}

#' @rdname labels
#' @export
`labels<-.keep_labels` <- function(x, value)
{
  set_attr(x, "label", value)
}

#' @rdname labels
#' @export
`labels<-.default` <- function(x, value) # we can't overwrite labels.default(), but nothing is stopping us from doing this
{
  set_attr(x, "label", value)
}

#' @rdname labels
#' @export
`labels<-.data.frame` <- function(x, value)
{
  if(is.null(value))
  {
    value <- rep(list(NULL), times = ncol(x))
  }

  if(is.null(names(x)) || is.null(names(value)))
  {
    if(length(x) != length(value) && length(x) > 0) stop("'x' and 'value' have unequal lengths, and don't have names")
    idx <- seq_along(x) # just map one-to-one
  } else
  {
    value <- value[names(value) %in% names(x)]
    idx <- match(names(value), names(x))
  }

  for(i in seq_along(idx))
  {
    attr(x[[idx[i]]], "label") <- value[[i]]
  }
  x
}

#' @rdname labels
#' @export
set_labels <- function(x, value)
{
  labels(x) <- value
  x
}

#' @rdname labels
#' @export
set_attr <- function(x, which, value)
{
  attr(x, which) <- value
  x
}

#' Family functions for modelsum
#'
#' A set of family functions for \code{\link{modelsum}}.
#'
#' @param method See \code{MASS::\link[MASS]{polr}}.
#' @param link See \code{MASS::\link[MASS]{glm.nb}}.
#' @return A list, in particular with element \code{family}.
#' @seealso \code{\link[stats]{family}}, \code{\link[survival]{coxph}}, \code{\link[MASS]{polr}}
#' @name modelsum.family
NULL
#> NULL

#' @rdname modelsum.family
#' @export
survival <- function() list(family="survival")

#' @rdname modelsum.family
#' @export
ordinal <- function(method = c("logistic", "probit", "loglog", "cloglog", "cauchit"))
{
  list(family = "ordinal", method = if(is.function(method)) method else match.arg(method))
}

#' @rdname modelsum.family
#' @export
negbin <- function(link = c("log", "identity", "sqrt"))
{
  list(family = "negbin", method = if(is.function(link)) link else match.arg(link))
}

#' @rdname modelsum.family
#' @export
clog <- function()
{
  list(family = "clog")
}


#' @rdname modelsum.family
#' @export
relrisk <- function(link = "log")
{
  list(family = "relrisk", link = link)
}

#' Control settings for \code{paired} function
#'
#' Control test and summary settings for the \code{\link{paired}} function.
#'
#' @param diff logical, telling \code{paired} whether to calculate a column of differences between time points.
#' @param numeric.test name of test for numeric RHS variables in \code{paired}: paired.t, signed.rank, signtest.
#' @param cat.test name of test for categorical variables: mcnemar
#' @param ordered.test name of test for ordered variables: signed.rank, signtest
#' @param date.test name of test to perform for date variables: paired.t, signed.rank, signtest
#' @param mcnemar.correct,signed.rank.exact,signed.rank.correct Options for statistical tests. See \code{\link{wilcox.test}}
#'   and \code{\link{mcnemar.test}} for details.
#' @param ... Arguments passed to \code{\link{tableby.control}}
#' @details Note that (with the exception of \code{total}) all arguments to \code{\link{tableby.control}} are accepted in
#'   this function (in fact, this function passes everything through to \code{\link{tableby.control}}).
#'   However, there are different defaults for the statistical tests (shown here). For details on the other arguments,
#'   please see the help page for \code{\link{tableby.control}}.
#' @return A list with settings to be used within the \code{\link{paired}} function.
#' @seealso \code{\link{paired}}, \code{\link{tableby}}, \code{\link{tableby.control}}, \code{\link{summary.tableby}}
#' @author Ethan Heinzen
#' @export
paired.control <- function(
  diff=TRUE, numeric.test="paired.t", cat.test="mcnemar", ordered.test="signed.rank", date.test="paired.t",
  mcnemar.correct = TRUE, signed.rank.exact = NULL, signed.rank.correct = TRUE, ...) {

  out <- tableby.control(numeric.test=numeric.test, cat.test=cat.test, ordered.test=ordered.test, date.test=date.test, ...)

  ## new args
  out$diff <- diff
  out$mcnemar.correct <- mcnemar.correct
  out$signed.rank.correct <- signed.rank.correct
  out$signed.rank.exact <- signed.rank.exact

  ## never show total
  out$total <- FALSE
  out$cat.droplevels <- FALSE
  out
}

## Purpose: summary method for modelsum object
## Author: Greg Dougherty, Jason Sinnwell and Beth Atkinson
## Updated: 9/29/2015


#' Summarize a \code{modelsum} object.
#'
#' Format the information in \code{object} as a table using Pandoc coding or plain text, and cat it to stdout.
#'
#' @param object A \code{\link{modelsum}} object.
#' @param ... For \code{summary.modelsum}, other arguments passed to \code{\link{as.data.frame.modelsum}}.
#'   For \code{as.data.frame.summary.modelsum}, "width" and "min.split" are passed to \code{\link{smart.split}}.
#'   For \code{print}ing the summary object, these are passed to both \code{as.data.frame.summary.modelsum} and
#'   \code{\link[knitr]{kable}}.
#' @inheritParams summary.tableby
#' @param x An object of class \code{"summary.modelsum"}.
#' @param adjustment.names Logical, denoting whether the names of the adjustment models should be printed.
#' @seealso \code{\link{modelsum}}, \code{\link{as.data.frame.modelsum}}
#' @return An object of class \code{"summary.modelsum"}
#' @author Ethan Heinzen, based on code originally by Greg Dougherty
#' @name summary.modelsum
NULL
#> NULL

#' @rdname summary.modelsum
#' @export
summary.modelsum <- function(object, ..., labelTranslations = NULL, text = FALSE, title = NULL, term.name = "", adjustment.names = FALSE)
{
  dat <- as.data.frame(object, ..., labelTranslations = labelTranslations, list.ok = TRUE)
  structure(list(
    object = set_attr(dat, "control", NULL),
    control = attr(dat, "control"),
    hasStrata = has_strata(object),
    text = text,
    title = title,
    term.name = term.name,
    adjustment.names = adjustment.names
  ), class = c("summary.modelsum", "summary.arsenal_table"))
}

as_data_frame_summary_modelsum <- function(df, control, hasStrata, term.name, text, adjustment.names, width, min.split)
{
  df.orig <- df

  #### format the digits and nsmall things ####
  # integers, one-per-model
  use.digits0 <- c("Nmiss", "N", "Nmiss2", "Nevents", "df.residual", "df.null", "edf")

  # non-integers, one-per-model
  use.digits1 <- c("logLik", "AIC", "BIC", "null.deviance", "deviance",
                   "statistic.F", "dispersion", "statistic.sc", "concordance", "std.error.concordance",
                   "adj.r.squared", "r.squared", "theta", "SE.theta")

  # non-integers, many-per-model
  use.digits2 <- c("estimate", "CI.lower.estimate", "CI.upper.estimate", "std.error", "statistic", "standard.estimate")

  use.digits.ratio <- c("OR", "CI.lower.OR", "CI.upper.OR", "RR", "CI.lower.RR", "CI.upper.RR", "HR", "CI.lower.HR", "CI.upper.HR")
  use.digits.p <- c("p.value.sc", "p.value.log", "p.value.wald", "p.value.F", "p.value.lrt") #"p.value"

  cn <- colnames(df)

  df[cn %in% c(use.digits1, use.digits2)] <- lapply(df[cn %in% c(use.digits1, use.digits2)], formatC, digits = control$digits, format = "f")
  df[cn %in% use.digits.ratio] <- lapply(df[cn %in% use.digits.ratio], formatC, digits = control$digits.ratio, format = "f")
  df[cn %in% c("p.value", use.digits.p)] <- lapply(df[cn %in% c("p.value", use.digits.p)], formatC, digits = control$digits.p,
                                                   format = if(control$format.p) "f" else "g")

  if(control$format.p)
  {
    cutoff <- 10^(-control$digits.p)
    fmt <- paste0("< ", format(cutoff, digits = control$digits.p, format = "f"))

    for(tst in c("p.value", use.digits.p))
    {
      if(tst %in% cn) df[[tst]][df.orig[[tst]] < cutoff] <- fmt
    }
  }
  df[cn %in% c("p.value", use.digits.p)] <- lapply(df[cn %in% c("p.value", use.digits.p)], sub,
                                                   pattern = "^\\s*NA\\s*$", replacement = "")

  #### don't show the same statistics more than once ####
  dups <- if(hasStrata) unlist(by(df, df[[4]], function(x) duplicated(x$model), simplify = FALSE), use.names = FALSE) else duplicated(df$model)
  df[cn %in% c(use.digits0, use.digits1, use.digits.p)] <- lapply(df[cn %in% c(use.digits0, use.digits1, use.digits.p)],
                                                                  replace, list = dups, values = "")
  if(hasStrata)
  {
    df[[4]] <- as.character(df[[4]])
    df[[4]][duplicated(df[[4]])] <- ""
  }

  #### get rid of unnecessary columns ####
  df$y.term <- NULL
  df$y.label <- NULL
  df$strata.term <- NULL

  df$model <- NULL
  df$term <- NULL
  term.type <- df$term.type
  df$term.type <- NULL
  if(!adjustment.names)
  {
    df$adjustment <- NULL
  } else df$adjustment[dups] <- ""

  #### Format if necessary ####
  if(!is.null(width))
  {
    firstcol <- smart.split(df[[1L + hasStrata]], width = width, min.split = min.split)
    lens <- vapply(firstcol, length, NA_integer_)

    df <- do.call(cbind.data.frame, c(list(label = unlist(firstcol, use.names = FALSE)), lapply(df[-1L - hasStrata], insert_elt, times = lens)))
    if(hasStrata) df <- df[replace(seq_along(df), 1:2, 2:1)]
    row.names(df) <- NULL
    term.type <- insert_elt(term.type, times = lens, elt = NULL)
  }
  df$label <- trimws(df$label)

  if(!is.null(text))
  {
    if(identical(text, "html"))
    {
      df$label <- ifelse(term.type == "Intercept", df$label, paste0("<strong>", df$label, "</strong>"))
    } else if(identical(text, "latex"))
    {
      df$label <- ifelse(term.type == "Intercept", df$label, paste0("\\textbf{", df$label, "}"))
    } else if(!text)
    {
      df$label <- ifelse(term.type == "Intercept", df$label, paste0("**", ifelse(df$label == "", "&nbsp;", df$label), "**"))
    }
  }

  #### tweak column names according to specifications ####
  cn <- stats::setNames(colnames(df), colnames(df))
  if(length(control$stat.labels) > 0)
  {
    nm <- intersect(cn, names(control$stat.labels))
    if(length(nm)) cn[nm] <- unlist(control$stat.labels[nm])
  }
  cn["label"] <- term.name
  colnames(df) <- cn

  set_attr(df, "align", rep("l", ncol(df)))
}
#' @rdname summary.modelsum
#' @export
as.data.frame.summary.modelsum <- function(x, ..., text = x$text, term.name = x$term.name, adjustment.names = x$adjustment.names,
                                           width = NULL, min.split = NULL, list.ok = FALSE)
{
  if(is.null(term.name) || identical(term.name, TRUE))
  {
    term.name <- vapply(x$object, attr, NA_character_, "ylabel")
  }
  stopifnot(length(term.name) <= length(x$object))
  out <- Map(as_data_frame_summary_modelsum, x$object, x$hasStrata, term.name,
             MoreArgs = list(control = x$control, text = text, width = width, min.split = min.split, adjustment.names = adjustment.names))
  if(!list.ok)
  {
    if(length(out) == 1) out <- out[[1]] else warning("as.data.frame.summary.modelsum is returning a list of data.frames")
  }
  out
}

#' Control settings for \code{comparedf} function
#'
#' Control tolerance definitions for the \code{\link{comparedf}} function.
#'
#' @param tol.logical,tol.num,tol.char,tol.factor,tol.date,tol.other A function or one of the shortcut character strings or a list thereof,
#'   denoting the tolerance function to use for a given data type. See "details", below.
#' @param tol.num.val Numeric; maximum value of differences allowed in numerics (fed to the function given in \code{tol.num}).
#' @param int.as.num Logical; should integers be coerced to numeric before comparison? Default FALSE.
#' @param factor.as.char Logical; should factors be coerced to character before comparison? Default FALSE.
#' @param tol.date.val Numeric; maximum value of differences allowed in dates (fed to the function given in \code{tol.date}).
#' @param tol.vars Either \code{"none"} (the default), denoting that variable names are to be matched as-is,
#'   a named vector manually specifying variable names to compare (where the names correspond to columns of
#'   \code{x} and the values correspond to columns of \code{y}), or a
#'   character vector denoting equivalence classes for characters in the variable names. See "details", below.
#' @param max.print.vars Integer denoting maximum number of variables to report in the "variables not shared" and "variables not compared"
#'   output. \code{NA} will print all differences.
#' @param max.print.obs Integer denoting maximum number of not-shared observations to report. \code{NA} will print all differences.
#' @param max.print.diffs.per.var,max.print.diffs Integers denoting the maximum number of differences to report for each variable or overall.
#'  \code{NA} will print all differences for each variable or overall.
#' @param max.print.attrs Integers denoting the maximum number of non-identical attributes to report.\code{NA} will print all differences.
#' @param max.print.diff Deprecated.
#' @param ... Other arguments (not in use at this time).
#' @return A list containing the necessary parameters for the \code{\link{comparedf}} function.
#' @details
#' The following character strings are accepted:
#' \itemize{
#'   \item{\code{tol.logical = "none"}: compare logicals exactly as they are.}
#'   \item{\code{tol.num = "absolute"}: compare absolute differences in numerics.}
#'   \item{\code{tol.num = "percent"}, \code{tol.num = "pct"} compare percent differences in numerics.}
#'   \item{\code{tol.char = "none"}: compare character strings exactly as they are.}
#'   \item{\code{tol.char = "trim"}: left-justify and trim all trailing white space.}
#'   \item{\code{tol.char = "case"}: allow differences in upper/lower case.}
#'   \item{\code{tol.char = "both"}: combine \code{"trim"} and \code{"case"}.}
#'   \item{\code{tol.factor = "none"}: match both character labels and numeric levels.}
#'   \item{\code{tol.factor = "levels"}: match only the numeric levels.}
#'   \item{\code{tol.factor = "labels"}: match only the labels.}
#'   \item{\code{tol.date = "absolute"}: compare absolute differences in dates.}
#'   \item{\code{tol.other = "none"}: expect objects of other classes to be exactly identical.}
#' }
#'
#' A list with names mapped to \code{x} can be used to specify tolerances by variable. One unnamed element is supported
#' as the default.
#'
#' \code{tol.vars}: If not set to \code{"none"} (the default) or a named vector,
#'   the \code{tol.vars} argument is a character vector denoting equivalence classes
#'   for the characters in the variable names. A single character in this vector means to replace that character
#'   with \code{""}. All other strings in this vector are split by character and replaced by the first character in the string.
#'
#' E.g., a character vector \code{c("._", "aA", " ")} would denote that the dot and underscore are equivalent (to be translated to a dot),
#'   that "a" and "A" are equivalent (to be translated to "a"), and that spaces should be removed.
#'
#' The special character string \code{"case"} in this vector is the same as specifying \code{paste0(letters, LETTERS)}.
#' @examples
#' cntl <- comparedf.control(
#'   tol.num = "pct",     # calculate percent differences
#'   tol.vars = c("case", # ignore case
#'                "._",   # set all underscores to dots.
#'                "e")    # remove all letter e's
#' )
#'
#' cntl <- comparedf.control(tol.char = list(
#'   "none",      # the default
#'   x1 = "case", # be case-insensitive for the variable "x1"
#'   x2 = function(x, y) tol.NA(x, y, x != y | y == "NA") # a custom-defined tolerance
#' ))
#' @seealso \code{\link{comparedf}}, \code{\link{comparedf.tolerances}}, \code{\link{summary.comparedf}}
#' @author Ethan Heinzen
#' @export
comparedf.control <- function(
  tol.logical = "none",
  tol.num = c("absolute", "percent", "pct"),
  tol.num.val = sqrt(.Machine$double.eps),
  int.as.num = FALSE,
  tol.char = c("none", "trim", "case", "both"),
  tol.factor = c("none", "levels", "labels"),
  factor.as.char = FALSE,
  tol.date = "absolute",
  tol.date.val = 0,
  tol.other = "none",
  tol.vars = "none",
  max.print.vars = NA, max.print.obs = NA, max.print.diffs.per.var = 10,
  max.print.diffs = 50, max.print.attrs = NA, ..., max.print.diff = 10)
{
  make_tol <- function(x, prefix, defaults)
  {
    if(is.function(x)) return(list(x))
    if(is.character(x)) return(list(match.fun(paste0(prefix, match.arg(x, defaults, several.ok = FALSE)))))
    if(!is.list(x) || (length(x) > 1 && (is.null(names(x)) || sum(names(x) == "") > 1)))
      stop("Provided tolerances should be a function, single character string, or a named list of functions/character strings")

    out <- lapply(x, function(y) {
      if(!is.function(y)) match.fun(paste0(prefix, match.arg(y, defaults, several.ok = FALSE))) else y
    })
    names(out) <- names(x)
    out
  }

  #### Logical ####
  tol.logical <- make_tol(tol.logical, "tol.logical.", "none")

  #### Numerics ####
  if(!is.numeric(tol.num.val)) stop("'tol.num.val' needs to be numeric.")
  tol.num <- make_tol(tol.num, "tol.num.", c("absolute", "percent", "pct"))
  if(!is.logical(int.as.num) || length(int.as.num) != 1 || is.na(int.as.num)) stop("'int.as.num' should be TRUE or FALSE.")

  #### Characters and factors ####
  tol.char <- make_tol(tol.char, "tol.char.", c("none", "trim", "case", "both"))

  tol.factor <- make_tol(tol.factor, "tol.factor.", c("none", "levels", "labels"))
  if(!is.logical(factor.as.char) || length(factor.as.char) != 1 || is.na(factor.as.char)) stop("'factor.as.char' should be TRUE or FALSE.")

  #### Dates ####
  if(!is.numeric(tol.date.val)) stop("'tol.date.val' needs to be numeric.")
  tol.date <- make_tol(tol.date, "tol.date.", "absolute")

  #### Other ####
  tol.other <- make_tol(tol.other, "tol.other.", "none")

  #### Variable names ####
  if(!is.character(tol.vars)) stop("'tol.vars' must be a character string or vector.")
  if(is.null(names(tol.vars)))
  {
    if("none" %in% tol.vars || length(tol.vars) == 0) tol.vars <- "none"
    if("case" %in% tol.vars) tol.vars <- c(paste0(letters, LETTERS), tol.vars[tol.vars != "case"])
  }

  if(!missing(max.print.diff))
  {
    .Deprecated(msg = "Using 'max.print.diff = ' is deprecated. Use 'max.print.diffs.per.var = ' instead.")
    max.print.diffs.per.var <- max.print.diff
  }
  chk <- function(x)
  {
    tmp <- deparse(substitute(x))
    if(length(x) != 1 || (!is.na(x) && (!is.numeric(x) || x <= 0))) stop("'", tmp, "' needs to be NA or a numeric > 0")
  }

  chk(max.print.vars)
  chk(max.print.obs)
  chk(max.print.diffs.per.var)
  chk(max.print.diffs)
  chk(max.print.attrs)

  list(tol.logical = tol.logical, tol.num = tol.num, tol.num.val = tol.num.val, int.as.num = int.as.num, tol.char = tol.char,
       tol.factor = tol.factor, factor.as.char = factor.as.char, tol.date = tol.date, tol.date.val = tol.date.val,
       tol.other = tol.other, tol.vars = tol.vars, max.print.vars = max.print.vars, max.print.obs = max.print.obs,
       max.print.diffs.per.var = max.print.diffs.per.var, max.print.diffs = max.print.diffs, max.print.attrs = max.print.attrs)
}

paired.t <- function(x, y, ..., na.rm = TRUE) {
  if(is.Date(x) && is.Date(y))
  {
    x <- as.integer(x)
    y <- as.integer(y)
  }
  if(na.rm) {
    idx <- is.na(x) | is.na(y)
    x <- x[!idx]
    y <- y[!idx]
  }
  stats::t.test(x, y, paired = TRUE)
}

mcnemar <- function(x, y, mcnemar.correct = TRUE, ..., na.rm = TRUE)
{
  if(na.rm) {
    idx <- is.na(x) | is.na(y)
    x <- x[!idx]
    y <- y[!idx]
  }
  stats::mcnemar.test(x, y, correct = mcnemar.correct)
}

signed.rank <- function(x, y, signed.rank.exact = NULL, signed.rank.correct = TRUE, ..., na.rm = TRUE)
{
  if(is.ordered(x) && is.ordered(y))
  {
    x <- as.integer(x)
    y <- as.integer(y)
  }
  if(na.rm) {
    idx <- is.na(x) | is.na(y)
    x <- x[!idx]
    y <- y[!idx]
  }
  stats::wilcox.test(x, y, paired = TRUE, exact = signed.rank.exact, correct = signed.rank.correct)
}

signtest <- function(x, y, ..., na.rm = TRUE)
{
  if(na.rm) {
    idx <- is.na(x) | is.na(y)
    x <- x[!idx]
    y <- y[!idx]
  }
  stats::binom.test(c(sum(x > y), sum(x < y)))
}

## allow stat functions to be passed as single arguments that are strings of function names
## Store this as attribute in the modeldf column, along with the actual name of the variable,
## rather than anova(age) showing up in the result (though anova(age) will be the column name in modeldf
## but we pull these attributes off later.
inline_tableby_stat_test <- function(x, ..., stats = list(...), digits = NULL, digits.count = NULL, digits.pct = NULL,
                                     digits.p = NULL, format.p = NULL,
                                     numeric.simplify = NULL, cat.simplify = NULL, cat.droplevels = NULL,
                                     ordered.simplify = NULL, date.simplify = NULL)
{
  attr(x, "term") <- attr(x, "name") <- deparse(substitute(x))
  attr(x, "stats") <- if(length(stats) == 0) NULL else stats
  attr(x, "control.list") <- list(digits = digits, digits.count = digits.count, digits.pct = digits.pct,
                                  digits.p = digits.p, format.p = format.p,
                                  numeric.simplify = numeric.simplify, cat.simplify = cat.simplify, cat.droplevels = cat.droplevels,
                                  ordered.simplify = ordered.simplify, date.simplify = date.simplify)
  class(x) <- c("keep_tableby_attrs", class(x)[class(x) != "keep_tableby_attrs"])
  x
}

#' @export
`[.keep_tableby_attrs` <- function(x, ...)
{
  y <- NextMethod()
  attr(y, "name") <- attr(x, "name")
  attr(y, "stats") <- attr(x, "stats")
  attr(y, "term") <- attr(x, "term")
  attr(y, "control.list") <- attr(x, "control.list")
  class(y) <- class(y)[class(y) != "keep_tableby_attrs"] # purposely drop the class
  y
}

get_attr <- function(x, which, default)
{
  x <- attr(x, which, exact = TRUE)
  if(is.null(x)) default else x
}

#' @export
print.tbstat <- function(x, ...) {
  cat("A 'tbstat' object with format:\n'", attr(x, "fmt"), "'", sep = "")
  invisible(x)
}

#' @rdname tableby.stats.internal
#' @export
tbfmt <- function(x, digits = NULL, digits.count = NULL, digits.pct = NULL, ...) {
  x <- x[] # to remove classes
  if(is.numeric(x)) {
    att <- attributes(x)
    x <- vapply(seq_along(x), function(i) {
      d <- if(i %in% att$which.pct) digits.pct else if(i %in% att$which.count) digits.count else digits
      formatC(x[i], digits = d, format = "f")
    }, NA_character_)
    x <- trimws(x)
  } else if(is.list(x) && any(idx <- vapply(x, inherits, NA, "difftime")))
  {
    x[idx] <- lapply(x[idx], function(xx) paste(trimws(formatC(unclass(xx), digits = digits, format = "f")), units(xx)))
  }
  x
}

#' @export
format.tbstat <- function(x, digits = NULL, digits.count = NULL, digits.pct = NULL, ...)
{
  if(is.null(fmt <- attr(x, "fmt"))) {
    warning("Could not find 'fmt' attribute")
    fmt <- "{y}"
  }
  glue::glue(
    fmt,
    x = x,
    y = tbfmt(x, digits = digits, digits.pct = digits.pct, digits.count = digits.count),
    digits = digits, digits.pct = digits.pct, digits.count = digits.count
  )
}

#' Internal \code{tableby} functions
#'
#' A collection of functions that may help users create custom functions that are formatted correctly.
#' @param x Usually a vector.
#' @param oldClass class(es) to add to the resulting object.
#' @param fmt A \code{\link[glue]{glue}} string, where the object is exposed as the variable \code{x},
#'   and a default-formatted version (using \code{tbfmt}) exposed as the variable \code{y}. \code{digits}, \code{digits.count},
#'   and \code{digits.pct} are also exposed.
#' @param which.pct Which statistics are percents? The default is 0, indicating that none are.
#' @param which.count Which statistics are counts? The default is 0, indicating that none are.
#' @param digits,digits.pct,digits.count Digits specifications
#' @param ... arguments to pass to \code{as.tbstat}.
#' @details
#'   The vignette has an example on how to use these.
#'
#'   \code{as.tbstat} defines a tableby statistic with its appropriate formatting.
#'
#'   \code{tbfmt} applies some default formatting.
#'
#'   \code{as.tbstat_multirow} marks an object (usually a list) for multiple-row printing.
#' @name tableby.stats.internal
NULL
#> NULL

#' @rdname tableby.stats.internal
#' @export
as.tbstat <- function(x, oldClass = NULL, fmt = "{y}", which.count = 0L, which.pct = 0L, ...)
{
  structure(x, class = c("tbstat", oldClass), fmt = fmt, which.count = which.count, which.pct = which.pct, ...)
}

#' @rdname tableby.stats.internal
#' @export
as.tbstat_multirow <- function(x)
{
  class(x) <- c("tbstat_multirow", class(x))
  x
}

extract_tbstat <- function(x, ...)
{
  x <- NextMethod("[")
  class(x) <- class(x)[class(x) %nin% c("tbstat", "tbstat_countpct", "tbstat_multirow")]
  x
}

extract2_tbstat <- function(x, ...)
{
  x <- NextMethod("[[")
  class(x) <- class(x)[class(x) %nin% c("tbstat", "tbstat_countpct", "tbstat_multirow")]
  x
}

#' @export
`[.tbstat` <- extract_tbstat
#' @export
`[.tbstat_countpct` <- extract_tbstat
#' @export
`[.tbstat_multirow` <- extract_tbstat
#' @export
`[[.tbstat` <- extract2_tbstat
#' @export
`[[.tbstat_countpct` <- extract2_tbstat
#' @export
`[[.tbsta_multirowt` <- extract2_tbstat


## merge two tableby objects
## both must have same "by" variable and levels
## if some RHS variables have same names, keep both, the one in y add ".y"

#' Helper functions for tableby
#'
#' A set of helper functions for \code{\link{tableby}}.
#'
#' @param ... Other arguments.
#' @param x A \code{tableby} object.
#' @param pdata A named data.frame where the first column is the by-variable names, the (optional) second is the strata value, the next is
#'   the x variable names, the next is p-values (or some test stat), and the (optional) next column is the method name.
#' @param e1,e2 \code{\link{tableby}} objects, or numbers to compare them to.
#' @param use.pname Logical, denoting whether the column name in \code{pdata} corresponding to the p-values should be used
#'   in the output of the object.
#' @param n A single integer. See \code{\link[utils]{head}} or \code{\link[utils:head]{tail}} for more details
#' @param lhs Logical, denoting whether to remove \code{NA}s from the first column of the data.frame (the "left-hand side")
#' @return \code{na.tableby} returns a subsetted version of \code{object} (with attributes). \code{Ops.tableby} returns
#'   a logical vector. \code{xtfrm.tableby} returns the p-values (which are ordered by \code{\link{order}} to \code{\link{sort}}).
#' @details
#' Logical comparisons are implemented for \code{Ops.tableby}.
#' @seealso \code{\link{arsenal_table}}, \code{\link{sort}}, \code{\link[utils]{head}}, \code{\link[utils:head]{tail}},
#'   \code{\link{tableby}}, \code{\link{summary.tableby}}, \code{\link{tableby.control}}
#' @name tableby.internal
NULL
#> NULL

#' @rdname tableby.internal
#' @export
is.tableby <- function(x) inherits(x, "tableby")

#' @rdname tableby.internal
#' @export
is.summary.tableby <- function(x) inherits(x, "summary.tableby")

## pdata is a named data.frame where the first column is the x variable names matched by name,
## p-values (or some test stat) are numbers and the name is matched
## method name is the third column (optional)
## to the x variable in the tableby object (x)

#' @rdname tableby.internal
#' @export
modpval.tableby <- function(x, pdata, use.pname=FALSE) {
  ## set control$test to TRUE
  if(any(pdata[[1]] %in% names(x$tables))) {
    x$control$test <- TRUE

    ## change test results
    for(k in seq_len(nrow(pdata))) {
      yname <- as.character(pdata[[1]][k])

      hasStrata <- x$tables[[yname]]$strata$hasStrata
      strat <- if(hasStrata) as.character(pdata[[2]][k]) else ""
      xname <- as.character(pdata[[2 + hasStrata]][k])
      p <- pdata[[3 + hasStrata]][k]
      method <- if(ncol(pdata) > 3 + hasStrata) pdata[[4 + hasStrata]][k] else "Modified by user"

      if(xname %in% names(x$tables[[yname]]$x) && strat %in% x$tables[[yname]]$strata$values)
      {
        idx <- which(x$tables[[yname]]$strata$values == strat)
        stopifnot(length(idx) == 1)

        x$tables[[yname]]$tables[[idx]][[xname]]$test$p.value <- p
        x$tables[[yname]]$tables[[idx]][[xname]]$test$method <- method
      }
    }
    if(use.pname & nchar(names(pdata)[3 + hasStrata]) > 0) {
      ## put different test column name in control
      x$control$test.pname <- names(pdata)[3 + hasStrata]
    }
  } else warning("Couldn't match any by-variables to the first column of 'x'.")
  return(x)
}

## Get the labels from the tableby object's elements in the order they appear in the fomula/Call
## including the y and x variables
# labels <- function(x) {
#   UseMethod("labels")
# }

## retrieve variable labels (y, x-vec) from tableby object


## define generic function for tests, so tests(tbObj) will work

#' @rdname tableby.internal
#' @export
tests <- function(x) UseMethod("tests")

## retrieve the names of the tests performed per variable
#' @rdname tableby.internal
#' @export
tests.tableby <- function(x) {
  if(x$control$test) {
    df <- as.data.frame(x, list.ok = TRUE)
    hasStrata <- has_strata(x)
    if(any(hasStrata) != all(hasStrata)) stop("Some tables in 'x' have strata, but others don't")
    testdf <- do.call(rbind_chr, Map(df, hasStrata, f = function(i, s) i[c("group.label", if(s) names(i)[4], "variable", "p.value", "test")]))

    testdf <- unique(testdf)
    row.names(testdf) <- NULL
    names(testdf)[c(1, any(hasStrata) + (2:4))] <- c("Group", "Variable",
                                                     if(!is.null(x$control$test.pname)) x$control$test.pname else "p.value", "Method")
  } else {
    cat("No tests run on tableby object\n")
    testdf <- NULL
  }
  testdf
}

## function to handle na.action for tableby formula, data.frame

#' @rdname tableby.internal
#' @export
na.tableby <- function(lhs = TRUE)
{
  if(is.data.frame(lhs)) stop("na.tableby now generates functions (and no longer accepts data.frames). ",
                              "Use 'na.tableby()' to generate the function that used to be 'na.tableby'.")
  if(lhs) return(na_lhs_strata)

  function(object, ...) {
    omit <- if("(strata)" %in% names(object)) is.na(object[["(strata)"]]) else rep(FALSE, nrow(object))

    xx <- object[!omit, , drop = FALSE]
    if(any(omit)) {
      temp <- stats::setNames(seq_along(omit)[omit], attr(object, "row.names")[omit])
      attr(temp, "class") <- "omit"
      attr(xx, "na.action") <- temp
    }
    xx
  }
}

#' @rdname tableby.internal
#' @export
xtfrm.tableby <- function(x)
{
  if(!x$control$test) stop("Can't extract p-values from a tableby object created with test=FALSE.")
  unlist(lapply(x$tables, function(lst) lapply(lst$tables, function(strat) lapply(strat, function(i) i$test$p.value))), use.names = FALSE)
}

#' @rdname tableby.internal
#' @export
sort.tableby <- function(x, ...)
{
  if(!x$control$test) stop("Can't sort a tableby object created with test=FALSE.")
  if(any(has_strata(x)) || length(x$tables) > 1) stop("Can't sort a tableby object with strata or multiple by variables")
  NextMethod()
}

#' @rdname tableby.internal
#' @export
Ops.tableby <- function(e1, e2)
{
  ok <- switch(.Generic, `<` = , `>` = , `<=` = , `>=` = , `==` = , `!=` = TRUE, FALSE)
  if(!ok) stop("'", .Generic, "' is not meaningful for tableby objects")

  if(inherits(e1, "tableby")) e1 <- xtfrm(e1)
  if(inherits(e2, "tableby")) e2 <- xtfrm(e2)
  get(.Generic, mode = "function")(e1, e2)
}

#' @rdname tableby.internal
#' @export
head.tableby <- function(x, n = 6L, ...)
{
  stopifnot(length(n) == 1L)
  xlen <- unique(vapply(x$tables, function(obj) length(obj$x), NA_integer_))
  if(length(xlen) != 1) stop("length isn't defined for tableby objects with differing x-variables per by-variable")
  n <- if(n < 0L) max(xlen + n, 0L) else min(n, xlen)
  x[seq_len(n)]
}

#' @rdname tableby.internal
#' @export
tail.tableby <- function(x, n = 6L, ...)
{
  stopifnot(length(n) == 1L)
  xlen <- unique(vapply(x$tables, function(obj) length(obj$x), NA_integer_))
  if(length(xlen) != 1) stop("length isn't defined for tableby objects with differing x-variables per by-variable")
  n <- if(n < 0L) max(xlen + n, 0L) else min(n, xlen)
  x[seq.int(to = xlen, length.out = n)]
}

as_data_frame_freqlist <- function(yList, single, sparse)
{
  filter_zero <- function(x) if(!sparse) droplevels(x[x$Freq != 0, , drop = FALSE]) else x
  labs <- vapply(yList$x, "[[", NA_character_, "label")
  if(single || !yList$strata$hasStrata)
  {
    list(set_attr(do.call(rbind_chr, lapply(yList$tables, filter_zero)), "labels", labs))
  } else
  {
    lapply(lapply(yList$tables, filter_zero), set_attr, "labels", labs)
  }
}

#' as.data.frame.freqlist
#'
#' Convert \code{\link{freqlist}} object to a data.frame.
#'
#' @param x An object of class \code{"freqlist"}.
#' @inheritParams summary.freqlist
#' @param ... Arguments to pass to \code{\link{freq.control}}
#' @return A data.frame corresponding to the \code{freqlist} object.
#' @export
as.data.frame.freqlist <- function(x, ..., labelTranslations = NULL, list.ok = FALSE)
{
  if(!is.null(labelTranslations)) labels(x) <- labelTranslations

  control <- c(list(...), x$control)
  control <- do.call("freq.control", control[!duplicated(names(control))])

  out <- lapply(x$tables, as_data_frame_freqlist, single = control$single, sparse = control$sparse)
  out <- unlist(out, recursive = FALSE, use.names = FALSE)
  if(!list.ok)
  {
    if(length(out) == 1) out <- out[[1]] else warning("as.data.frame.freqlist is returning a list of data.frames")
  }
  set_attr(out, "control", control)
}

#' Keep Labels
#'
#' Keep the \code{'label'} attribute on an R object when subsetting. \code{loosen.labels} allows the \code{'label'}
#'   attribute to be lost again.
#'
#' @param x An R object
#' @param i,value See \code{\link{[<-}}.
#' @param ... Other arguments (not in use at this time).
#' @return A copy of \code{x} with a "keep_labels" class appended on or removed. Note that for the \code{data.frame} method,
#'   only classes on the columns are changed; the \code{data.frame} won't have an extra class appended. This is different from previous
#'   versions of \code{arsenal}.
#' @author Ethan Heinzen
#' @seealso \code{\link{labels}}
#' @name keep.labels
NULL
#> NULL

#' @rdname keep.labels
#' @export
keep.labels <- function(x, ...)
{
  UseMethod("keep.labels")
}

#' @rdname keep.labels
#' @export
keep.labels.data.frame <- function(x, ...)
{
  x[] <- lapply(x, keep.labels)
  x
}

#' @rdname keep.labels
#' @export
keep.labels.default <- function(x, ...)
{
  class(x) <- c("keep_labels", class(x)[class(x) != "keep_labels"])
  x
}

#' @rdname keep.labels
#' @export
`[.keep_labels` <- function(x, ...)
{
  y <- NextMethod()
  keep.labels(set_attr(y, "label", attr(x, "label")))
}

#' @rdname keep.labels
#' @export
`[<-.keep_labels` <- function(x, i, value)
{
  x <- loosen.labels(x)
  out <- NextMethod()
  keep.labels(out)
}

#' @rdname keep.labels
#' @export
loosen.labels <- function(x, ...)
{
  UseMethod("loosen.labels")
}

#' @rdname keep.labels
#' @export
loosen.labels.data.frame <- function(x, ...)
{
  x[] <- lapply(x, loosen.labels)
  x
}

#' @rdname keep.labels
#' @export
loosen.labels.default <- function(x, ...)
{
  class(x) <- class(x)[class(x) != "keep_labels"]
  x
}

## ----echo=FALSE, message=FALSE, results='hide', warning=FALSE-----------------
require(knitr)
require(broom)
require(MASS)
require(pROC)
require(rpart)
 
opts_chunk$set(comment = NA, echo=TRUE, prompt=TRUE, collapse=TRUE)


## ----load-data----------------------------------------------------------------
require(arsenal)
data(mockstudy) # load data
dim(mockstudy)  # look at how many subjects and variables are in the dataset 
# help(mockstudy) # learn more about the dataset and variables
str(mockstudy) # quick look at the data

## ----simple1------------------------------------------------------------------
tab1 <- modelsum(bmi ~ sex + age, data=mockstudy)

## ----simple-text--------------------------------------------------------------
summary(tab1, text=TRUE)

## ----simple-markdown, results='asis'------------------------------------------
summary(tab1)

## -----------------------------------------------------------------------------
as.data.frame(tab1)

## ----adjust, results="asis"---------------------------------------------------
tab2 <- modelsum(alk.phos ~ arm + ps + hgb, adjust= ~age + sex, data=mockstudy)
summary(tab2)

## -----------------------------------------------------------------------------
fit <- lm(alk.phos ~ arm + age + sex, data=mockstudy)
summary(fit)
plot(fit)

## -----------------------------------------------------------------------------
require(MASS)
boxcox(fit)

## -----------------------------------------------------------------------------
fit2 <- lm(log(alk.phos) ~ arm + age + sex, data=mockstudy)
summary(fit2)
plot(fit2)

## -----------------------------------------------------------------------------
require(splines)
fit3 <- lm(log(alk.phos) ~ arm + ns(age, df=2) + sex, data=mockstudy)

# test whether there is a difference between models 
stats::anova(fit2,fit3)

# look at functional form of age
termplot(fit3, term=2, se=T, rug=T)

## -----------------------------------------------------------------------------
tmp <- tidy(fit3) # coefficients, p-values
class(tmp)
tmp

glance(fit3)

## ----results="asis"-----------------------------------------------------------
ms.logy <- modelsum(log(alk.phos) ~ arm + ps + hgb, data=mockstudy, adjust= ~age + sex, 
                    family=gaussian,  
                    gaussian.stats=c("estimate","CI.lower.estimate","CI.upper.estimate","p.value"))
summary(ms.logy)

## -----------------------------------------------------------------------------
boxplot(age ~ mdquality.s, data=mockstudy, ylab=attr(mockstudy$age,'label'), xlab='mdquality.s')

fit <- glm(mdquality.s ~ age + sex, data=mockstudy, family=binomial)
summary(fit)

# create Odd's ratio w/ confidence intervals
tmp <- data.frame(summary(fit)$coef)
tmp

tmp$OR <- round(exp(tmp[,1]),2)
tmp$lower.CI <- round(exp(tmp[,1] - 1.96* tmp[,2]),2)
tmp$upper.CI <- round(exp(tmp[,1] + 1.96* tmp[,2]),2)
names(tmp)[4] <- 'P-value'

kable(tmp[,c('OR','lower.CI','upper.CI','P-value')])

# Assess the predictive ability of the model

# code using the pROC package
require(pROC)
pred <- predict(fit, type='response')
tmp <- pROC::roc(mockstudy$mdquality.s[!is.na(mockstudy$mdquality.s)]~ pred, plot=TRUE, percent=TRUE)
tmp$auc


## -----------------------------------------------------------------------------
tidy(fit, exp=T, conf.int=T) # coefficients, p-values, conf.intervals

glance(fit) # model summary statistics

## ----results="asis"-----------------------------------------------------------
summary(modelsum(mdquality.s ~ age + bmi, data=mockstudy, adjust=~sex, family=binomial))

fitall <- modelsum(mdquality.s ~ age, data=mockstudy, family=binomial,
                   binomial.stats=c("Nmiss2","OR","p.value"))
summary(fitall)

## ----survival-----------------------------------------------------------------
require(survival)

# multivariable model with all 3 terms
fit  <- coxph(Surv(fu.time, fu.stat) ~ age + sex + arm, data=mockstudy)
summary(fit)

# check proportional hazards assumption
fit.z <- cox.zph(fit)
fit.z
plot(fit.z[1], resid=FALSE) # makes for a cleaner picture in this case
abline(h=coef(fit)[1], col='red')

# check functional form for age using pspline (penalized spline)
# results are returned for the linear and non-linear components
fit2 <- coxph(Surv(fu.time, fu.stat) ~ pspline(age) + sex + arm, data=mockstudy)
fit2

# plot smoothed age to visualize why significant
termplot(fit2, se=T, terms=1)
abline(h=0)

# The c-statistic comes out in the summary of the fit
summary(fit2)$concordance

# It can also be calculated using the survConcordance function
survConcordance(Surv(fu.time, fu.stat) ~ predict(fit2), data=mockstudy)

## -----------------------------------------------------------------------------
tidy(fit) # coefficients, p-values

glance(fit) # model summary statistics

## ----results="asis"-----------------------------------------------------------
##Note: You must use quotes when specifying family="survival" 
##      family=survival will not work
summary(modelsum(Surv(fu.time, fu.stat) ~ arm, 
                 adjust=~age + sex, data=mockstudy, family="survival"))

##Note: the pspline term is not working yet
#summary(modelsum(Surv(fu.time, fu.stat) ~ arm, 
#                adjust=~pspline(age) + sex, data=mockstudy, family='survival'))

## ----poisson------------------------------------------------------------------
require(rpart) ##just to get access to solder dataset
data(solder)
hist(solder$skips)

fit <- glm(skips ~ Opening + Solder + Mask , data=solder, family=poisson)
stats::anova(fit, test='Chi')
summary(fit)

## -----------------------------------------------------------------------------
1-pchisq(fit$deviance, fit$df.residual)

## -----------------------------------------------------------------------------
fit2 <- glm(skips ~ Opening + Solder + Mask, data=solder, family=quasipoisson)
summary(fit2)

## -----------------------------------------------------------------------------
tidy(fit) # coefficients, p-values

glance(fit) # model summary statistics

## ----results='asis'-----------------------------------------------------------
summary(modelsum(skips~Opening + Solder + Mask, data=solder, family="quasipoisson"))
summary(modelsum(skips~Opening + Solder + Mask, data=solder, family="poisson"))

## -----------------------------------------------------------------------------
# add .01 to the follow-up time (.01*1 day) in order to keep everyone in the analysis
fit <- glm(fu.stat ~ offset(log(fu.time+.01)) + age + sex + arm, data=mockstudy, family=poisson)
summary(fit)
1-pchisq(fit$deviance, fit$df.residual)

coef(coxph(Surv(fu.time,fu.stat) ~ age + sex + arm, data=mockstudy))
coef(fit)[-1]

# results from the Poisson model can then be described as risk ratios (similar to the hazard ratio)
exp(coef(fit)[-1])

# As before, we can model the dispersion which alters the standard error
fit2 <- glm(fu.stat ~ offset(log(fu.time+.01)) + age + sex + arm, 
            data=mockstudy, family=quasipoisson)
summary(fit2)

## -----------------------------------------------------------------------------
tidy(fit) ##coefficients, p-values

glance(fit) ##model summary statistics

## ----results="asis", eval=TRUE------------------------------------------------
summary(modelsum(fu.stat ~ age, adjust=~offset(log(fu.time+.01))+ sex + arm, 
                 data=mockstudy, family=poisson))
                 

## ----results='asis'-----------------------------------------------------------
mycontrols  <- modelsum.control(gaussian.stats=c("estimate","std.error","adj.r.squared","Nmiss"),
                                show.adjust=FALSE, show.intercept=FALSE)                            
tab2 <- modelsum(bmi ~ age, adjust=~sex, data=mockstudy, control=mycontrols)
summary(tab2)

## ----results='asis'-----------------------------------------------------------
tab3 <- modelsum(bmi ~  age, adjust=~sex, data=mockstudy,
                 gaussian.stats=c("estimate","std.error","adj.r.squared","Nmiss"), 
                 show.intercept=FALSE, show.adjust=FALSE)
summary(tab3)

## ----check-labels-------------------------------------------------------------
## Look at one variable's label
attr(mockstudy$age,'label')

## See all the variables with a label
unlist(lapply(mockstudy,'attr','label'))

## or
cbind(sapply(mockstudy,attr,'label'))

## ----add-label, results='asis'------------------------------------------------
attr(mockstudy$age,'label')  <- 'Age, yrs'

tab1 <- modelsum(bmi ~  age, adjust=~sex, data=mockstudy)
summary(tab1)

## ----results = 'asis'---------------------------------------------------------
labels(mockstudy)  <- c(age = 'Age, yrs')

tab1 <- modelsum(bmi ~  age, adjust=~sex, data=mockstudy)
summary(tab1)

## ----results='asis'-----------------------------------------------------------
mylabels <- list(sexFemale = "Female", age ="Age, yrs")
summary(tab1, labelTranslations = mylabels)

## ----eval=TRUE----------------------------------------------------------------
labels(tab1)
labels(tab1) <- c(sexFemale="Female", age="Baseline Age (yrs)")
labels(tab1)

## ----results='asis'-----------------------------------------------------------
summary(tab1)

## ----results='asis'-----------------------------------------------------------
summary(modelsum(age~mdquality.s+sex, data=mockstudy), show.intercept=FALSE)

## ----results='asis'-----------------------------------------------------------
summary(modelsum(mdquality.s ~ age + bmi, data=mockstudy, adjust=~sex, family=binomial),
        show.adjust=FALSE)  

## ----results='asis'-----------------------------------------------------------
# create a vector specifying the variable names
myvars <- names(mockstudy)

# select the 8th through the 12th
# paste them together, separated by the + sign
RHS <- paste(myvars[8:12], collapse="+")
RHS

# create a formula using the as.formula function
as.formula(paste('mdquality.s ~ ', RHS))

# use the formula in the modelsum function
summary(modelsum(as.formula(paste('mdquality.s ~', RHS)), family=binomial, data=mockstudy))

## ----results='asis'-----------------------------------------------------------
## The formulize function does the paste and as.formula steps
tmp <- formulize('mdquality.s',myvars[8:10])
tmp

## More complex formulas could also be written using formulize
tmp2 <- formulize('mdquality.s',c('ps','hgb','sqrt(bmi)'))

## use the formula in the modelsum function
summary(modelsum(tmp, data=mockstudy, family=binomial))

## -----------------------------------------------------------------------------
newdata <- subset(mockstudy, subset=age>50 & arm=='F: FOLFOX', select = c(age,sex, bmi:alk.phos))
dim(mockstudy)
table(mockstudy$arm)
dim(newdata)
names(newdata)

## ----results='asis'-----------------------------------------------------------
summary(modelsum(alk.phos ~ ., data=newdata))

## ----results='asis', eval=TRUE------------------------------------------------
summary(modelsum(log(alk.phos) ~ sex + ps + bmi, subset=age>50 & arm=="F: FOLFOX", data=mockstudy))
summary(modelsum(alk.phos ~ ps + bmi, adjust=~sex, subset = age>50 & bmi<24, data=mockstudy))
summary(modelsum(alk.phos ~ ps + bmi, adjust=~sex, subset=1:30, data=mockstudy))

## -----------------------------------------------------------------------------
## create a variable combining the levels of mdquality.s and sex
with(mockstudy, table(interaction(mdquality.s,sex)))

## ----results='asis'-----------------------------------------------------------
summary(modelsum(age ~ interaction(mdquality.s,sex), data=mockstudy))

## ----results='asis'-----------------------------------------------------------
summary(modelsum(arm=="F: FOLFOX" ~ I(age/10) + log(bmi) + mdquality.s,
                 data=mockstudy, family=binomial))

## ----results='asis'-----------------------------------------------------------
mytab <- modelsum(bmi ~ sex + alk.phos + age, data=mockstudy)
mytab2 <- mytab[c('age','sex','alk.phos')]
summary(mytab2)
summary(mytab[c('age','sex')])
summary(mytab[c(3,1)])

## ----results="asis"-----------------------------------------------------------
## demographics
tab1 <- modelsum(bmi ~ sex + age, data=mockstudy)
## lab data
tab2 <- modelsum(mdquality.s ~ hgb + alk.phos, data=mockstudy, family=binomial)
                
tab12 <- merge(tab1, tab2, all = TRUE)
class(tab12)
summary(tab12)

## ----results='asis'-----------------------------------------------------------
t1 <- modelsum(bmi ~ sex + age, data=mockstudy)
summary(t1, title='Demographics')

## -----------------------------------------------------------------------------
## look at how many missing values there are for each variable
apply(is.na(mockstudy),2,sum)

## ----results='asis'-----------------------------------------------------------
## Show how many subjects have each variable (non-missing)
summary(modelsum(bmi ~ ast + age, data=mockstudy,
                control=modelsum.control(gaussian.stats=c("N","estimate"))))

## Always list the number of missing values
summary(modelsum(bmi ~ ast + age, data=mockstudy,
                control=modelsum.control(gaussian.stats=c("Nmiss2","estimate"))))

## Only show the missing values if there are some (default)
summary(modelsum(bmi ~ ast + age, data=mockstudy, 
                control=modelsum.control(gaussian.stats=c("Nmiss","estimate"))))

## Don't show N at all
summary(modelsum(bmi ~ ast + age, data=mockstudy, 
                control=modelsum.control(gaussian.stats=c("estimate"))))

## ----results='asis'-----------------------------------------------------------
summary(modelsum(bmi ~ sex + age + fu.time, data=mockstudy), digits=4, digits.test=2)

## -----------------------------------------------------------------------------
mockstudy$agegp <- cut(mockstudy$age, breaks=c(18,50,60,70,90), right=FALSE)

## create weights based on agegp and sex distribution
tab1 <- with(mockstudy,table(agegp, sex))
tab1
tab2 <- with(mockstudy, table(agegp, sex, arm))
gpwts <- rep(tab1, length(unique(mockstudy$arm)))/tab2

## apply weights to subjects
index <- with(mockstudy, cbind(as.numeric(agegp), as.numeric(sex), as.numeric(as.factor(arm)))) 
mockstudy$wts <- gpwts[index]

## show weights by treatment arm group
tapply(mockstudy$wts,mockstudy$arm, summary)

## ----results='asis'-----------------------------------------------------------
mockstudy$newvarA <- as.numeric(mockstudy$arm=='A: IFL')
tab1 <- modelsum(newvarA ~ ast + bmi + hgb, data=mockstudy, subset=(arm !='G: IROX'), 
                 family=binomial)
summary(tab1, title='No Case Weights used')

suppressWarnings({
tab2 <- modelsum(newvarA ~ ast + bmi + hgb, data=mockstudy, subset=(arm !='G: IROX'), 
                 weights=wts, family=binomial)
summary(tab2, title='Case Weights used')
})

## -----------------------------------------------------------------------------
summary(tab2, text=T)
tmp <- as.data.frame(summary(tab2, text = TRUE))
tmp
# write.csv(tmp, '/my/path/here/mymodel.csv')

## ----eval = FALSE-------------------------------------------------------------
# ## write to an HTML document
# write2html(tab2, "~/ibm/trash.html")
# 
# ## write to a Word document
# write2word(tab2, "~/ibm/trash.doc", title="My table in Word")

## ----eval=FALSE---------------------------------------------------------------
# # A standalone shiny app
# library(shiny)
# library(arsenal)
# data(mockstudy)
# 
# shinyApp(
#   ui = fluidPage(tableOutput("table")),
#   server = function(input, output) {
#     output$table <- renderTable({
#       as.data.frame(summary(modelsum(age ~ sex, data = mockstudy), text = "html"))
#     }, sanitize.text.function = function(x) x)
#   }
# )

## ----eval=FALSE---------------------------------------------------------------
# summary(modelsum(age ~ sex, data = mockstudy), title="(\\#tab:mytableby) Caption here")

## ----results='asis'-----------------------------------------------------------
summary(modelsum(list(age, hgb) ~ bmi + sex, adjust = ~ arm, data = mockstudy))

## ----results='asis'-----------------------------------------------------------
summary(modelsum(list(age, hgb) ~ bmi + sex, adjust = ~ arm, data = mockstudy), term.name = TRUE)

## ----results='asis'-----------------------------------------------------------
summary(modelsum(list(age, hgb) ~ bmi + sex, strata = arm, data = mockstudy))

## -----------------------------------------------------------------------------
adj.list <- list(
  Unadjusted = ~ 1, # can also specify NULL here
  "Adjusted for Arm" = ~ arm
)
multi.adjust <- modelsum(list(age, bmi) ~ fu.time + ast, adjust = adj.list, data = mockstudy)
summary(multi.adjust, adjustment.names = TRUE)
summary(multi.adjust, adjustment.names = TRUE, show.intercept = FALSE, show.adjust = FALSE)

## -----------------------------------------------------------------------------
args(modelsum.control)

## -----------------------------------------------------------------------------
args(arsenal:::summary.modelsum)

# original function mockup done by Andy Hanson; re-envisioned by EPH starting 3/20/17

#' Compare two data.frames and report differences
#'
#' Compare two data.frames and report any differences between them,
#'  much like SAS's \code{PROC COMPARE} procedure.
#'
#' @param x,y A data.frame to compare
#' @param by,by.x,by.y Which variables are IDs to merge the two data.frames?
#'   If set to \code{"row.names"}, merging will occur over the row.names.
#'   If set to \code{NULL} (default), merging will occur row-by-row.
#' @param control A list of control parameters from \code{\link{comparedf.control}}.
#' @param ... Other arguments, passed to \code{\link{comparedf.control}} when appropriate.
#' @examples
#'
#' df1 <- data.frame(id = paste0("person", 1:3), a = c("a", "b", "c"),
#'                   b = c(1, 3, 4), c = c("f", "e", "d"),
#'                   row.names = paste0("rn", 1:3), stringsAsFactors = FALSE)
#' df2 <- data.frame(id = paste0("person", 3:1), a = c("c", "b", "a"),
#'                   b = c(1, 3, 4), d = paste0("rn", 1:3),
#'                   row.names = paste0("rn", c(1,3,2)), stringsAsFactors = FALSE)
#' summary(comparedf(df1, df2))
#' summary(comparedf(df1, df2, by = "id"))
#' summary(comparedf(df1, df2, by = "row.names"))
#' @author Ethan Heinzen, adapted from code from Andrew Hanson
#' @seealso \code{\link{summary.comparedf}}, \code{\link{comparedf.control}},
#'   \code{\link{diffs}}, \code{\link{n.diffs}}, \code{\link{n.diff.obs}}
#' @name comparedf
NULL
#> NULL

#' @rdname comparedf
#' @export
comparedf <- function(x, y, by = NULL, by.x = by, by.y = by, control = NULL, ...) {

  control <- c(list(...), control)
  control <- do.call("comparedf.control", control[!duplicated(names(control))])

  xname <- paste0(deparse(substitute(x)), collapse = "")
  yname <- paste0(deparse(substitute(y)), collapse = "")

  if(!is.data.frame(x) || !is.data.frame(y))
  {
    stop("Both 'x' and 'y' must be data.frames")
  }

  if(any(c("..row.x..", "..row.y..", "..row.names..") %in% c(colnames(x), colnames(y))))
  {
    stop("'..row.x..', '..row.y..', and '..row.names..' are reserved colnames in this function.")
  }

  if(is.null(by) && is.null(by.x) && is.null(by.y))
  {
    # user didn't supply any by-variables, so we'll merge by row
    by.x <- by.y <- "row.names"
    byrow <- TRUE
  } else if(is.null(by.x) || is.null(by.y))
  {
    stop("Either 'by' or both of 'by.x' and 'by.y' must be specified")
  } else byrow <- FALSE

  if(any(by.x %nin% c("row.names", colnames(x))) || any(by.y %nin% c("row.names", colnames(y))))
  {
    stop("One or more of 'by.x' doesn't match colnames(x) or 'by.y' doesn't match colnames(y).")
  }

  #### data frame summary ####

  frame.summary <- data.frame(
    version = c("x", "y"),
    arg = c(xname, yname),
    ncol = c(ncol(x), ncol(y)),
    nrow = c(nrow(x), nrow(y)), stringsAsFactors = FALSE
  )
  tmp.attrs <- list(attributes(x), attributes(y))

  if("row.names" %in% by.x)
  {
    x[["..row.names.."]] <- if(byrow) seq_len(nrow(x)) else row.names(x)
    by.x[by.x == "row.names"] <- "..row.names.."
  }
  if("row.names" %in% by.y)
  {
    y[["..row.names.."]] <- if(byrow) seq_len(nrow(y)) else row.names(y)
    by.y[by.y == "row.names"] <- "..row.names.."
  }
  frame.summary$by <- set_attr(list(by.x, by.y), "byrow", byrow)
  frame.summary$attrs <- tmp.attrs

  #### tweak the column names ####

  tcn <- tweakcolnames(by.x, by.y, colnames(x), colnames(y), control)
  rm(by.x, by.y) # just to make sure we don't try to use those anymore
  by <- tcn$by

  #### now merge the things together ####

  together <- merge(cbind(stats::setNames(x, tcn$cn.x), ..row.x.. = seq_len(nrow(x))),
                    cbind(stats::setNames(y, tcn$cn.y), ..row.y.. = seq_len(nrow(y))), by = by, all = TRUE)

  both <- together[!is.na(together[["..row.x.."]]) & !is.na(together[["..row.y.."]]), , drop = FALSE]

  #### get the unshared observations ####

  getunique <- function(whch, whch2, by. = by)
  {
    tmp <- together[is.na(together[[whch2]]), c(by., whch), drop = FALSE]
    colnames(tmp)[colnames(tmp) == whch] <- "observation"
    tmp
  }

  frame.summary$unique <- list(getunique("..row.x..", "..row.y.."), getunique("..row.y..", "..row.x.."))
  frame.summary$n.shared <- rep(nrow(both), times = 2)

  #### Make the main object in this function ####

  mkdf <- function(df, cn.new)
  {
    df2 <- data.frame(tmp = cn.new, var = colnames(df), pos = seq_along(df), stringsAsFactors = FALSE)
    df2$class <- unname(lapply(df, class))
    df2
  }

  vars.summary <- merge(mkdf(x, tcn$cn.x), mkdf(y, tcn$cn.y), by = "tmp", all = TRUE)
  ord <- order(is.na(vars.summary$var.x), is.na(vars.summary$var.y), vars.summary$pos.x, vars.summary$pos.y, na.last = TRUE)
  vars.summary <- vars.summary[ord, , drop = FALSE]
  row.names(vars.summary) <- NULL

  vars.summary$class.x <- lapply(vars.summary$class.x, cleanup.null.na)
  vars.summary$class.y <- lapply(vars.summary$class.y, cleanup.null.na)

  vars.summary$values <- lapply(seq_len(nrow(vars.summary)), compare_values, v = vars.summary, df = both, byvars = by, contr = control)
  vars.summary$attrs <- lapply(seq_len(nrow(vars.summary)), compare_attrs, v = vars.summary, x_ = x, y_ = y)
  vars.summary$tmp <- NULL

  structure(list(frame.summary = structure(frame.summary, class = c("comparedf.frame.summary", "data.frame")),
                 vars.summary = structure(vars.summary, class = c("comparedf.vars.summary", "data.frame")),
                 control = control, Call = match.call()), class = "comparedf")
}

#' @rdname comparedf
#' @export
print.comparedf <- function(x, ...)
{
  cat("Compare Object\n\n")
  cat("Function Call: \n")
  print(x$Call)
  cat("\n")
  cat("Shared: ", sum(idx_var_sum(x, "nonby.vars.shared")), " non-by variables and ", x$frame.summary$n.shared[1], " observations.\n", sep = "")
  cat("Not shared: ", sum(idx_var_sum(x, "vars.not.shared")), " variables and ",
      n.diff.obs(x), " observations.\n", sep = "")
  cat("\n")
  cat("Differences found in ", sum(idx_var_sum(x, "differences.found")), "/", sum(idx_var_sum(x, "vars.compared")), " variables compared.\n", sep = "")
  cat(sum(idx_var_sum(x, "non.identical.attributes")), " variables compared have non-identical attributes.\n", sep = "")
  invisible(x)
}

#' @export
print.comparedf.vars.summary <- function(x, ...)
{
  orig <- x
  f <- function(elt, txt1, txt2)
  {
    if(is.data.frame(elt)) paste0(nrow(elt), txt1) else if(is.null(elt)) txt2 else elt
  }

  x$values <- vapply(x$values, f, character(1), txt1 = " differences", txt2 = "Not compared")
  x$attrs <- vapply(x$attrs, f, character(1), txt1 = " attributes", txt2 = "0 attributes")
  NextMethod()
  invisible(orig)
}


#' @export
print.comparedf.frame.summary <- function(x, ...)
{
  orig <- x
  f <- function(elt, txt1, txt2)
  {
    if(is.data.frame(elt)) paste0(nrow(elt), txt1) else if(is.list(elt)) paste0(length(elt), txt1) else if(is.null(elt)) txt2 else elt
  }

  x$attrs <- vapply(x$attrs, f, character(1), txt1 = " attributes", txt2 = "0 attributes")
  x$unique <- vapply(x$unique, f, character(1), txt1 = " unique obs", txt2 = "")
  NextMethod()
  invisible(orig)
}

#' Fit models over each of a set of independent variables with a response variable
#'
#' Fit and summarize models for each independent (x) variable with a response variable (y), with options to adjust by variables for each model.
#'
#' @param formula an object of class \code{\link{formula}}; a symbolic description of the variables to be modeled.  See "Details" for more information.
#' @param adjust an object of class \code{\link{formula}} or a list of formulas, listing variables to adjust by in all models.
#'  Specify as a one-sided formula, like: \code{~Age+ Sex}. If a list, the names are used for the summary function. Unadjusted models
#'  can be specified as \code{~ 1} or as a list: \code{list(Unadjusted = NULL)}.
#' @param family similar mechanism to \code{\link[stats]{glm}}, where the model to be fit is driven by the family.
#'   Options include: binomial, gaussian, survival, poisson, negbin, clog, and ordinal. These can be passed as a string, as a function,
#'   or as a list resulting from a call to one of the functions. See \code{\link{modelsum.family}} for details on
#'   survival, ordinal, negbin, and clog families.
#' @param data an optional data.frame, list or environment (or object coercible by \code{\link[base]{as.data.frame}} to a data frame) containing the
#'   variables in the model. If not found in \code{data}, the variables are taken from \code{environment(formula)}, typically
#'   the environment from which \code{modelsum} is called.
#' @param subset an optional vector specifying a subset of observations (rows of \code{data}) to be used in the results.
#'   If \code{strata} is missing, this works as vector of logicals or an index; otherwise, it should be a logical vector.
#' @param weights an optional vector specifying the weights to apply to each data observation (rows of \code{data})
#' @param strata a vector of strata to separate model summaries by an additional group. Note that for families like "clog",
#'   the "usual" strata term to indicate subject groupings should be given in the \code{adjust} argument.
#' @param na.action a function which indicates what should happen when the data contain \code{NA}s.
#'   The default (\code{NULL}) is to use the defaults of \code{\link[stats]{lm}}, \code{\link[stats]{glm}}, or \code{\link[survival]{coxph}},
#'   depending on the \code{family} specifications.
#' @param id A vector to identify clusters. Only used for \code{\link{relrisk}} at this time.
#' @param control control parameters to handle optional settings within \code{modelsum}.  Arguments for \code{modelsum.control}
#'   can be passed to \code{modelsum} via the \code{...} argument, but if a control object and \code{...} arguments are both supplied,
#'   the latter are used. See \code{\link{modelsum.control}} for other details.
#' @param ... additional arguments to be passed to internal \code{modelsum} functions.
#' @return An object with class \code{c("modelsum", "arsenal_table")}
#' @author Jason Sinnwell, Patrick Votruba, Beth Atkinson, Gregory Dougherty, and Ethan Heinzen, adapted from SAS Macro of the same name
#' @seealso \code{\link{arsenal_table}}, \code{\link{modelsum.control}}, \code{\link{summary.modelsum}},
#'   \code{\link{modelsum.internal}}, \code{\link{formulize}}
#' @examples
#'
#' data(mockstudy)
#'
#' tab1 <- modelsum(bmi ~ sex + age, data = mockstudy)
#' summary(tab1, text = TRUE)
#'
#' tab2 <- modelsum(alk.phos ~ arm + ps + hgb, adjust = ~ age + sex,
#'                  family = "gaussian", data = mockstudy)
#' summary(tab2, text = TRUE)
#'
#' summary(tab2, show.intercept = FALSE, text = TRUE)
#'
#' tab2.df <- as.data.frame(tab2)
#'
#' tab2.df[1:5,]
#' @name modelsum
NULL
#> NULL

#' @rdname modelsum
#' @export

modelsum <- function(formula,  family="gaussian", data, adjust=NULL, na.action = NULL,
                     subset=NULL, weights=NULL, id, strata, control = NULL, ...) {
  Call <- match.call()

  ## Allow family parameter to passed with or without quotes
  ## Here, we force quotes to simplify in for loop below
  family.list <- if(is.function(family) || is.character(family)) match.fun(family)() else family

  if(family.list$family %nin% c("survival", "gaussian", "binomial", "poisson", "quasibinomial", "quasipoisson", "ordinal", "negbin", "clog", "relrisk"))
    stop("Family ", family.list$family, " not supported.\n")

  if(family.list$family != "survival" && any(grepl("Surv\\(", formula))) {
    warning("Found Surv in formula, assuming family='survival'\n")
    family.list <- survival()
  }
  ## pick up extra control arguments from command via ...
  control <- c(list(...), control)
  control <- do.call("modelsum.control", control[!duplicated(names(control))])

  ## Tell user if they passed an argument that was not expected, either here or in control
  expectArgs <- c("formula", "family", "adjust", "data", "na.action", "subset", "weights", "id", "strata", "control", names(control))
  match.idx <- match(names(Call)[-1], expectArgs)
  if(anyNA(match.idx)) warning("Unused arguments: ", paste(names(Call)[c(FALSE, is.na(match.idx))], collapse=", "), "\n")

  #### Set up "main effects" dataset ####
  indx.main <- match(c("formula", "data", "subset", "weights", "strata"), names(Call), 0L)
  if(indx.main[1] == 0) stop("A formula argument is required")
  if(length(formula) == 2) stop("'formula' should have a response variable!")

  main.call <- Call[c(1, indx.main)]
  main.call[[1]] <- quote(stats::model.frame)
  main.call$na.action <- quote(stats::na.pass) # for now, keep all rows, except for what is subset out
  if(!missing(data))
  {
    # instead of call("keep.labels", ...), which breaks when arsenal isn't loaded (Can't find "keep.labels")
    main.call$data <- as.call(list(keep.labels, main.call$data))
  }

  #### Set up "adjustment" dataset ####
  if(is.null(adjust))
  {
    adjust <- list(unadjusted = NULL)
    adjustdf <- NULL
    adjTerms <- NULL
    adjLabels <- NULL
  } else
  {
    adjust <- as_list_formula(adjust)
    if(any(lengths(adjust) != 2)) stop("'adjust' formula(s) shouldn't have a response variable!")
    if(is.null(names(adjust)))
    {
      names(adjust) <- paste0("adjusted", seq_along(adjust))
    } else if(anyDuplicated(names(adjust))) stop("Names of 'adjust' must be unique.")
    adj.call <- main.call
    adj.call$formula <- Reduce(join_formula, adjust)
    adj.call$weights <- NULL
    adj.call$strata <- NULL
    adjustdf <- eval(adj.call, parent.frame())

    Terms.a <- stats::terms(adjustdf)
    adjTerms <- make_ms_term_labels(adjustdf, Terms.a)
    adjLabels <- lapply(adjTerms, make_ms_labs)
  }

  is.numericish <- function(x) is.numeric(x) || is.Date(x)

  out.tables <- list()
  formula.list <- as_list_formula(formula)
  for(FORM in formula.list)
  {
    main.call$formula <- FORM
    maindf <- loosen.labels(eval(main.call, parent.frame()))
    if(nrow(maindf) == 0) stop("No (non-missing) observations")
    Terms <- stats::terms(maindf)

    #### Check for weights ####
    if(hasWeights <- "(weights)" %in% colnames(maindf)) maindf[["(weights)"]] <- NULL

    #### Check for strata ####
    if(hasStrata <- "(strata)" %in% colnames(maindf))
    {
      strata.col <- maindf[["(strata)"]]
      strataTerm <- paste(deparse(Call$strata), collapse = " ")
      if(is.null(strataLabel <- attr(strata.col, "label"))) strataLabel <- strataTerm
      if(is.factor(strata.col))
      {
        strata.col <- droplevels(strata.col)
        strata.levels <- levels(strata.col)
      } else strata.levels <- sort(unique(strata.col))

      maindf[["(strata)"]] <- NULL
    } else
    {
      strata.col <- rep("", nrow(maindf))
      strataTerm <- strataLabel <- strata.levels <- ""
    }

    #### Get info on y-variable ####
    yCol <- maindf[[1]]
    if(family.list$family == "gaussian" && length(unique(yCol)) <= 5) warning("Input family=gaussian, but dependent variable has 5 or fewer categories\n")

    yTerm <- colnames(maindf)[1]
    if(is.null(yLabel <- attr(yCol, "label"))) yLabel <- yTerm
    yList <- list(label = yLabel, term = yTerm)
    maindf[[1]] <- NULL
    Terms.x <- stats::delete.response(Terms)

    #### Now finish the x-variables ####
    effCols <- seq_len(ncol(attr(Terms, "factors")))
    xTerms <- make_ms_term_labels(maindf, Terms.x)

    strataList <- vector("list", length(strata.levels))
    if(hasStrata) names(strataList) <- paste0("(", strataTerm, ") == ", strata.levels)

    for(strat in strata.levels)
    {
      xList <- vector("list", length(effCols))
      names(xList) <- names(xTerms)
      idx <- if(!hasStrata) NULL else call("==", call("(", Call$strata), strat)

      for(eff in effCols)
      {
        xList[[eff]] <- vector("list", length(adjust))
        names(xList[[eff]]) <- names(adjust)

        for(adj.i in seq_along(adjust))
        {
          curr.formula <- stats::drop.terms(Terms, if(length(effCols) > 1) setdiff(effCols, eff) else NULL, keep.response = TRUE)
          adj.formula <- join_formula(curr.formula, adjust[[adj.i]])

          temp.call <- Call[c(1, match(c("data", "subset", "na.action", "weights", "id"), names(Call), 0L))]
          temp.call$formula <- adj.formula
          if(hasStrata)
          {
            temp.call$subset <- if(!is.null(temp.call$subset)) call("&", call("(", temp.call$subset), idx) else idx
          }

          currCols <- maindf[strata.col == strat, attr(Terms.x, "factors")[, eff] > 0, drop=FALSE]
          results <- modelsum_guts(family.list, temp.call, envir = parent.frame(), conf.level = control$conf.level,
                                   scope = stats::delete.response(curr.formula), anyna = anyNA(currCols))

          nmiss <- length(results$fit$na.action)
          xList[[eff]][[adj.i]] <- list(
            coeff=results$coeffTidy,
            glance = c(
              results$modelGlance,
              N = sum(strata.col == strat) - nmiss,
              Nmiss = nmiss,
              Nmiss2 = nmiss,
              endpoint=yTerm,
              endlabel=yLabel,
              x=xTerms[[eff]]$variable,
              contrasts=list(results$fit$contrasts)
            )
          )
        }
      }
      strataList[[if(!hasStrata) 1 else paste0("(", strataTerm, ") == ", strat)]] <- xList
    }

    out.tables[[yTerm]] <- list(y = yList, strata = list(term = strataTerm, values = strata.levels, label = strataLabel, hasStrata = hasStrata),
                                x = lapply(xTerms, make_ms_labs),
                                adjust = adjLabels,
                                tables = strataList, family = family.list$family, hasWeights = hasWeights)
  }

  structure(list(Call = Call, control = control, tables = out.tables), class = c("modelsum", "arsenal_table"))
}

#' Mock study data for examples
#'
#' Mock clinical study data for examples to test data manipulation and statistical functions.
#'   The function \code{muck_up_mockstudy()} is used in examples for \code{\link{comparedf}}.
#'
#' @format A data frame with 1499 observations on the following 15 variables:
#'   \describe{
#'     \item{\code{case}}{a numeric identifier-patient ID}
#'     \item{\code{age}}{age in years}
#'     \item{\code{arm}}{treatment arm divided into 3 groups, character string }
#'     \item{\code{sex}}{a factor with levels \code{Male} \code{Female}}
#'     \item{\code{race}}{self-reported race/ethnicity, character string}
#'     \item{\code{fu.time}}{survival or censoring time in years}
#'     \item{\code{fu.stat}}{censoring status; 1=censor, 2=death}
#'     \item{\code{ps}}{integer, ECOG performance score }
#'     \item{\code{hgb}}{numeric, hemoglobin count}
#'     \item{\code{bmi}}{numeric, body mass index, kg/m^2}
#'     \item{\code{alk.phos}}{numeric, alkaline phosphatase}
#'     \item{\code{ast}}{numeric, aspartate transaminase }
#'     \item{\code{mdquality.s}}{integer, LASA QOL 0=Clinically Deficient, 1=Not Clinically Deficient  }
#'     \item{\code{age.ord}}{an ordered factor split of age, with levels
#'       \code{10-19} < \code{20-29} < \code{30-39} < \code{40-49} <
#'         \code{50-59} < \code{60-69} < \code{70-79} < \code{80-89}}
#'   }
#' @examples
#' data(mockstudy)
#' str(mockstudy)
#' @name mockstudy
NULL
#> NULL

#' @rdname mockstudy
"mockstudy"

#' @rdname mockstudy
#' @export
muck_up_mockstudy <- function()
{
  mockstudy <- arsenal::mockstudy
  mockstudy2 <- mockstudy[c(1, 3, 6, 10:nrow(mockstudy), 2, 4), ]
  mockstudy2$sex <- factor(mockstudy2$sex, levels = c("Female", "Male"))
  mockstudy2$ps[3] <- NA
  mockstudy2$ast[1:3] <- 36
  mockstudy2$age <- NULL
  mockstudy2$fu_time <- mockstudy2$fu.time
  mockstudy2$fu.time <- NULL
  mockstudy2$`fu stat` <- mockstudy2$fu.stat
  mockstudy2$fu.stat <- NULL
  mockstudy2$Arm <- mockstudy2$arm
  mockstudy2$arm <- NULL
  mockstudy2$hgb[is.na(mockstudy2$hgb)] <- -9
  mockstudy2$race[mockstudy2$race == "Caucasian"] <- "caucasian"
  mockstudy2$race <- factor(mockstudy2$race)
  attr(mockstudy2$sex, "label") <- "Sex (M/F)"
  mockstudy2
}

#' Make a column for "select all" input
#'
#' @param ... Named arguments of the same length. These should be logical, numeric (0/1) or a factor with two levels.
#' @param i,j,drop Arguments to `[.matrix`
#' @param x An object of class "selectall"
#' @examples
#' d <- data.frame(grp = rep(c("A", "B"), each = 5))
#' d$s <- selectall(
#'   `Option 1` = c(rep(1, 4), rep(0, 6)),
#'   `Option 2` = c(0, 1, 0, 0, 0, 1, 1, 1, 0, 0),
#'   `Option 3` = 1,
#'   `Option 4` = 0
#' )
#' summary(tableby(grp ~ s, data = d), text = TRUE)
#' @seealso \code{\link{tableby}}, \code{\link{paired}}
#' @name selectall
NULL
#> NULL

#' @rdname selectall
#' @export
selectall <- function(...)
{
  Call <- match.call()
  args <- lapply(list(...), function(x) if(is.factor(x)) as.numeric(x)-1 else if(is.numeric(x)) x else if(is.logical(x)) +x else as.numeric(factor(x))-1)
  out <- do.call(cbind, args)

  if(any(out %nin% c(0, 1, NA))) stop("Some elements may not have two levels")
  as.selectall(out)
}

#' @rdname selectall
#' @export
as.selectall <- function(x)
{
  structure(x, class = c("selectall", class(x)[class(x) != "selectall"]))
}

#' @rdname selectall
#' @export
as.matrix.selectall <- function(x, ...)
{
  class(x) <- "matrix"
  x
}

#' @rdname selectall
#' @export
`[.selectall` <- function(x, i, j, drop = FALSE)
{
  as.selectall(as.matrix(x)[i, j, drop = FALSE])
}

#' @rdname selectall
#' @export
is.na.selectall <- function(x)
{
  rowSums(is.na(as.matrix(x))) > 0
}

#' @rdname selectall
#' @export
is.selectall <- function(x) inherits(x, "selectall")

get_ms_strata_part <- function(msList, sValue, xList, ...)
{
  Map(get_ms_part, msList, c(0, cumsum(lengths(msList)[-1])), xList, MoreArgs = list(sValue = sValue, ...))
}

get_ms_part <- function(msList, modelnum, xList, yList, aList, sList, sValue, fam, cntrl)
{
  get_labs <- function(x) stats::setNames(x$label, x$term)
  # even if the model doesn't have an intercept, that's okay
  labs <- c("(Intercept)" = "(Intercept)", get_labs(xList), adj <- unlist(unname(lapply(aList, get_labs))))
  statFields <- switch(
    fam,
    quasibinomial = cntrl$binomial.stats, binomial = cntrl$binomial.stats,
    quasipoisson = cntrl$poisson.stats, poisson = cntrl$poisson.stats,
    negbin = cntrl$negbin.stats, clog = cntrl$clog.stats, survival = cntrl$survival.stats,
    ordinal = cntrl$ordinal.stats, gaussian = cntrl$gaussian.stats, relrisk = cntrl$relrisk.stats
  )

  OUT <- NULL
  for(adj.i in seq_along(msList))
  {
    msLst <- msList[[adj.i]]
    trms <- msLst$coeff$term
    out <- data.frame(
      y.term = yList$term,
      y.label = yList$label,
      strata.term = if(!sList$hasStrata) "" else paste0("(", sList$term, ") == ", sValue),
      strata.value = if(!sList$hasStrata) "" else sValue,
      adjustment = names(msList)[adj.i],
      model = modelnum + adj.i,
      term = trms,
      label = ifelse(trms %in% names(labs), labs[trms], trms),
      term.type = ifelse(trms %in% names(adj), "Adjuster", ifelse(trms %in% xList$term, "Term", "Intercept")),
      stringsAsFactors = FALSE
    )
    if(!sList$hasStrata) out$strata.value <- NULL else names(out)[4] <- sList$label
    if(any(names(msLst$coeff) %in% statFields)) out <- cbind(out, msLst$coeff[intersect(statFields, names(msLst$coeff))])
    if(any(names(msLst$glance) %in% statFields)) out <- cbind(out, msLst$glance[intersect(statFields, names(msLst$glance))])
    OUT <- rbind_chr(OUT, out)
  }
  OUT
}

#' as.data.frame.modelsum
#'
#' Coerce a \code{\link{modelsum}} object to a \code{data.frame}.
#'
#' @param x A \code{\link{modelsum}} object.
#' @param ... Arguments to pass to \code{\link{modelsum.control}}.
#' @inheritParams summary.modelsum
#' @seealso \code{\link{modelsum}}, \code{\link{summary.modelsum}}
#' @return A \code{data.frame}.
#' @author Ethan Heinzen, based on code originally by Greg Dougherty
#' @export
as.data.frame.modelsum <- function(x, ..., labelTranslations = NULL, list.ok = FALSE)
{
  if(!is.null(labelTranslations)) labels(x) <- labelTranslations

  control <- c(list(...), x$control)
  control <- do.call("modelsum.control", control[!duplicated(names(control))])

  out <- lapply(x$tables, as_data_frame_modelsum, control = control)

  if(!list.ok)
  {
    if(length(out) == 1) out <- out[[1]] else warning("as.data.frame.modelsum is returning a list of data.frames")
  }

  set_attr(out, "control", control)
}

as_data_frame_modelsum <- function(lhsList, control)
{
  stopifnot(length(lhsList$tables) == length(lhsList$strata$values))
  tabs <- Map(get_ms_strata_part, msList = lhsList$tables, sValue = lhsList$strata$values,
              MoreArgs = list(yList = lhsList$y, sList = lhsList$strata, xList = lhsList$x, aList = lhsList$adjust,
                              fam = lhsList$family, cntrl = control))
  out <- do.call(rbind_chr, unlist(tabs, recursive = FALSE, use.names = FALSE))

  out <- out[out$term.type %in% c("Term", if(control$show.intercept) "Intercept", if(control$show.adjust) "Adjuster"), , drop = FALSE]
  row.names(out) <- NULL

  # Get rid of Nmiss if none missing
  if("Nmiss" %in% names(out) && all(out$Nmiss == 0)) out$Nmiss <- NULL
  set_attr(out, "ylabel", lhsList$y$label)
}

#' The summary method for a \code{tableby} object
#'
#' The summary method for a \code{\link{tableby}} object, which is a pretty rendering of a \code{\link{tableby}}
#' object into a publication-quality results table in R Markdown, and can render well in text-only.
#'
#' @param object An object of class \code{"tableby"}, made by the \code{\link{tableby}} function.
#' @param x An object of class \code{"summary.tableby"}.
#' @param ... For \code{summary.tableby}, other arguments passed to \code{\link{as.data.frame.tableby}}.
#'   For \code{print}ing the summary object, these are passed to both \code{as.data.frame.summary.tableby} and
#'   \code{\link[knitr]{kable}}.
#' @param title	Title/caption for the table, defaulting to \code{NULL} (no title). Passed to \code{\link[knitr]{kable}}.
#'   Can be length > 1 if the more than one table is being printed.
#' @param labelTranslations  A named list (or vector) where the name is the label in the
#'        output to be replaced in the pretty rendering by the character string
#'        value for the named element of the list, e.g., \code{list(age = "Age(Years)", meansd = "Mean(SD)")}.
#' @param text An argument denoting how to print the summary to the screen.
#'		Default is \code{FALSE} (show markdown output). \code{TRUE} and \code{NULL} output a text-only version, with
#'		the latter avoiding all formatting.
#'		\code{"html"} uses the HTML tag \code{<strong>} instead of the markdown formatting, and \code{"latex"} uses
#'		the LaTeX command \code{\\textbf}.
#' @param pfootnote Logical, denoting whether to put footnotes describing the tests used to generate the p-values. Alternatively,
#'   "html" to surround the outputted footnotes with \code{<li>}.
#' @param term.name A character vector denoting the column name for the "terms" column. It should be the same length
#'   as the number of tables or less (it will get recycled if needed). The special value \code{TRUE} will
#'   use the y-variable's label for each table.
#' @param list.ok If the object has multiple by-variables, is it okay to return a list of data.frames instead of a single data.frame?
#'   If \code{FALSE} but there are multiple by-variables, a warning is issued.
#' @inheritParams arsenal_table
#' @return An object of class \code{summary.tableby}
#' @seealso \code{\link{tableby.control}}, \code{\link{tableby}}
#' @author Ethan Heinzen, based on code by Gregory Dougherty, Jason Sinnwell, Beth Atkinson,
#'   adapted from SAS Macros written by Paul Novotny and Ryan Lennon
#' @examples
#'
#' set.seed(100)
#' ## make 3+ categories for response
#' nsubj <- 90
#' mdat <- data.frame(Response=sample(c(1,2,3),nsubj, replace=TRUE),
#'                    Sex=sample(c("Male", "Female"), nsubj,replace=TRUE),
#'                    Age=round(rnorm(nsubj,mean=40, sd=5)),
#'                    HtIn=round(rnorm(nsubj,mean=65,sd=5)))
#'
#' ## allow default summaries on RHS variables
#' out <- tableby(Response ~ Sex + Age + HtIn, data=mdat)
#' summary(out, text=TRUE)
#' labels(out)
#' labels(out) <- c(Age="Age (years)", HtIn="Height (inches)")
#' summary(out, stats.labels=c(meansd="Mean-SD", q1q3 = "Q1-Q3"), text=TRUE)
#'
#' @name summary.tableby
NULL
#> NULL

#' @rdname summary.tableby
#' @export
summary.tableby <- function(object, ..., labelTranslations = NULL, text = FALSE, title = NULL, pfootnote = FALSE, term.name = "")
{
  dat <- as.data.frame(object, ..., labelTranslations = labelTranslations, list.ok = TRUE)
  structure(list(
    object = set_attr(dat, "control", NULL),
    control = attr(dat, "control"),
    totals = lapply(object$tables, function(x) x$y$stats),
    hasStrata = has_strata(object),
    text = text,
    title = title,
    pfootnote = pfootnote,
    term.name = term.name
  ), class = c("summary.tableby", "summary.arsenal_table"))
}

as_data_frame_summary_tableby <- function(df, totals, hasStrata, term.name, control, text, pfootnote, width, min.split)
{
  df.orig <- df
  idx <- names(df)[names(df) %nin% c("group.term", "group.label", "strata.term", "strata.label", "variable",
                                     "term", "label", "variable.type", "test", "p.value")]
  dgt <- attr(df, "control.list")

  f <- function(j, whch) if(is.null(dgt[[j]]) || is.null(dgt[[j]][[whch]])) control[[whch]] else dgt[[j]][[whch]]

  digits <- vapply(df$variable, function(i) f(i, "digits"), numeric(1))
  digits.count <- vapply(df$variable, function(i) f(i, "digits.count"), numeric(1))
  digits.pct <- vapply(df$variable, function(i) f(i, "digits.pct"), numeric(1))

  df[idx] <- lapply(df[idx], function(col) {
    out <- Map(
      function(xx, ...) if(is.character(xx)) xx else format(xx, ...),
      col,
      digits = digits, format = "f", # the format="f" is not used for tbstat objects
      digits.count = digits.count, digits.pct = digits.pct
    )
    unlist(out)
  })

  if(is.numeric(df$p.value))
  {
    my_format_p <- function(p, digits.p, format.p) {
      p2 <- formatC(p, digits = digits.p, format = "f")
      cutoff <- 10^(-digits.p)
      fmt <- paste0("< ", format(cutoff, digits = digits.p, format = "f"))
      p2[p < cutoff] <- fmt
      if(identical(format.p, TRUE)) return(p2)

      p3 <- formatC(p, digits = digits.p, format = "g")
      if(identical(format.p, FALSE)) return(p3)

      glue::glue(format.p, p = p, digits.p = digits.p)
    }
    digits.p <- vapply(df$variable, function(i) f(i, "digits.p"), numeric(1))
    format.p <- lapply(df$variable, function(i) f(i, "format.p"))

    df$p.value <- unlist(Map(
      my_format_p,
      df$p.value,
      digits.p = digits.p,
      format.p = format.p
    ))
  }
  if(!is.null(df$p.value)) df$p.value[grepl("^\\s*NA$", df$p.value) | is.na(df$p.value)] <- ""

  tests.used <- NULL
  if(control$test && (isTRUE(pfootnote) || identical(pfootnote, "html")))
  {
    tests.used <- unique(df$test[df$test != "No test"])
    sup <- if(!is.null(text) && identical(text, "html")) c("<sup>", "</sup>") else if(isTRUE(text)) c(" (", ")") else c("^", "^")

    df$p.value <- ifelse(df$p.value == "", "", paste0(df$p.value, sup[1], as.integer(factor(df[["test"]], levels = tests.used)), sup[2]))
    tests.used <- if(identical(pfootnote, "html")) c("<ol>", paste0("<li>", tests.used, "</li>"), "</ol>") else paste0(seq_along(tests.used), ". ", tests.used)
  }

  #### don't show the same statistics more than once ####
  dups <- if(hasStrata)
  {
    unlist(by(df, factor(df[[4]], levels = unique(df[[4]])), function(x) duplicated(x$variable), simplify = FALSE), use.names = FALSE)
  } else duplicated(df$variable)
  df$p.value[dups] <- ""
  if(hasStrata)
  {
    df[[4]] <- as.character(df[[4]])
    df[[4]][duplicated(df[[4]])] <- ""
  }

  #### get rid of unnecessary columns ####
  df$group.term <- NULL
  df$group.label <- NULL
  df$strata.term <- NULL

  df$variable <- NULL
  df$term <- NULL
  df$test <- NULL
  df$variable.type <- NULL
  if(!control$test) df$p.value <- NULL
  if(!control$total && !identical(control$stats.labels$overall, "Total")) df[[control$stats.labels$total]] <- NULL

  #### Format if necessary ####
  if(!is.null(width))
  {
    firstcol <- smart.split(df[[1L + hasStrata]], width = width, min.split = min.split)
    lens <- vapply(firstcol, length, NA_integer_)

    df <- do.call(cbind.data.frame, c(list(label = unlist(firstcol, use.names = FALSE)), lapply(df[-1L - hasStrata], insert_elt, times = lens)))
    if(hasStrata) df <- df[replace(seq_along(df), 1:2, 2:1)]
    row.names(df) <- NULL
    dups <- insert_elt(dups, times = lens, elt = NULL)
  }
  df$label <- trimws(df$label)

  if(!is.null(text))
  {
    df$label <- if(identical(text, "html"))
    {
      ifelse(dups, paste0("&nbsp;&nbsp;&nbsp;", df$label), paste0("<strong>", df$label, "</strong>"))
    } else if(identical(text, "latex"))
    {
      ifelse(dups, paste0("~~~", df$label), paste0("\\textbf{", df$label, "}"))
    } else if(text)
    {
      ifelse(dups, paste0("-  ", df$label), df$label)
    } else ifelse(dups, paste0("&nbsp;&nbsp;&nbsp;", df$label), paste0("**", ifelse(df$label == "", "&nbsp;", df$label), "**"))

    if(identical(text, "latex")) df[] <- lapply(df, gsub, pattern = "%", replacement = "\\%", fixed = TRUE)
  }

  #### tweak column names according to specifications ####
  cn <- stats::setNames(colnames(df), colnames(df))
  align <- c(if(hasStrata) "l", "l", rep("c", times = sum(cn != "p.value")-1), if("p.value" %in% cn) "r")
  nm <- intersect(cn, names(totals))
  if(length(nm) && (is.null(control$digits.n) || !is.na(control$digits.n)))
    cn[nm] <- paste0(cn[nm], " (N=", formatC(totals[nm], digits = control$digits.n, format = "f"), ")")
  cn["label"] <- term.name
  if("p.value" %in% cn && is.null(control$test.pname)) cn["p.value"] <- "p value" else if("p.value" %in% cn) cn["p.value"] <- control$test.pname
  colnames(df) <- cn

  if(control$total.pos == "before" && control$stats.labels$total %in% nm) {
    idx <- seq_along(cn)
    idx2 <- idx[names(cn) %in% nm]
    idx[idx2] <- c(utils::tail(idx2, 1), utils::head(idx2, -1))
    df <- df[idx]
  }

  set_attr(set_attr(df, "tests", tests.used), "align", align)
}

#' @rdname summary.tableby
#' @export
as.data.frame.summary.tableby <- function(x, ..., text = x$text, pfootnote = x$pfootnote, term.name = x$term.name,
                                          width = NULL, min.split = NULL, list.ok = FALSE)
{
  if(is.null(term.name) || identical(term.name, TRUE))
  {
    term.name <- vapply(x$object, attr, NA_character_, "ylabel")
  }
  stopifnot(length(term.name) <= length(x$object))
  out <- Map(as_data_frame_summary_tableby, x$object, x$totals, x$hasStrata, term.name,
             MoreArgs = list(control = x$control, text = text, pfootnote = pfootnote,
                             width = width, min.split = min.split))
  if(!list.ok)
  {
    if(length(out) == 1) out <- out[[1]] else warning("as.data.frame.summary.tableby is returning a list of data.frames")
  }
  out
}

## Purpose: internal functions (and methods) for tableby function
## Authors: Jason Sinnwell, Beth Atkinson
## Created: 9/4/2015

## Helper functions for modelsum:  merge, subset, and labels (work like names)

#' Helper functions for modelsum
#'
#' A set of helper functions for \code{\link{modelsum}}.
#'
#' @param object A \code{data.frame} resulting from evaluating a \code{modelsum} formula.
#' @param ... Other arguments, or a vector of indices for extracting.
#' @param x A \code{modelsum} object.
#' @return \code{na.modelsum} returns a subsetted version of \code{object} (with attributes).
#' @seealso \code{\link{arsenal_table}}
#' @name modelsum.internal
NULL
#> NULL

join_formula <- function(x, y)
{
  x <- stats::formula(x)
  if(is.null(y)) return(x)
  y <- stats::formula(y)
  stopifnot(length(y) == 2)
  if(length(x) == 2)
  {
    x[[2]] <- call("+", x[[2]], y[[2]])
  } else
  {
    stopifnot(length(x) == 3)
    x[[3]] <- call("+", x[[3]], y[[2]])
  }
  x
}

#' @rdname modelsum.internal
#' @export
is.modelsum <- function(x) inherits(x, "modelsum")

#' @rdname modelsum.internal
#' @export
is.summary.modelsum <- function(x) inherits(x, "summary.modelsum")

#' @rdname modelsum.internal
#' @export
na.modelsum <- na_lhs_strata

##standardized beta function (for gaussian stat)
lm.beta  <- function (MOD) {
    b <- stats::coef(MOD)[-1]
    sx <- rep(NA,length(b))
    b.idx <- 1
    for(k in 2:ncol(MOD$model)) {
      ## skip factors and char variables,
      ## psplines consider doing sx, but need a second for loop for the ncol of those
      if(any(class(MOD$model[,k]) %in% c("character","factor", "pspline"))) {
        b.idx <- b.idx + ifelse(is.null(ncol(MOD$model[,k])), length(unique(MOD$model[,k]))-1, ncol(MOD$model[,k]))
        ## skip as many elements of beta as there are N.levels-1 of categorical variables
      } else {
        sx[b.idx] <- stats::sd(as.double(MOD$model[,k]),na.rm=TRUE)
        b.idx <- b.idx + 1
      }
    }
    sy <- stats::sd(as.double(MOD$model[,1]),na.rm=TRUE)
    beta <- c(NA,round(b * sx/sy,3))
    return(beta)
}

make_ms_term_labels <- function(mf, trms)
{
  factors <- attr(trms, "factors")
  mm <- stats::model.matrix(trms, mf)
  assign <- attr(mm, "assign")
  mm <- mm[, assign > 0, drop = FALSE]
  assign <- assign[assign > 0]
  lvls <- colnames(factors)[assign]
  names(lvls) <- colnames(mm)

  out <- lapply(colnames(factors), function(nm2) {
    idx <- factors[, nm2] > 0
    nm <- names(mf)[idx]
    labelEff <- vapply(nm, function(nam) {
      if(is.null(lab <- attr(mf[[nam]], "label"))) lab <- nam
      lab
    }, NA_character_)
    list(variable = paste(nm, collapse = ":"), variable2 = nm2,
         varterm = nm, varterm2 = row.names(factors)[idx], varlabel = unname(labelEff),
         term = names(lvls)[lvls == nm2])
  })
  names(out) <- vapply(out, "[[", NA_character_, "variable")
  out
}



modelsum_guts <- function(fam, temp.call, envir, conf.level, scope, anyna)
{
  check_pkg("broom")

  try_lrt <- function(f, s, a)
  {
    if(a) return(NA_real_)
    out <- setdiff(stats::drop1(f, scope = s, test = "Chisq")[["Pr(>Chi)"]], NA_real_)
    if(length(out) == 1) out else NA_real_
  }

  ## y is ordered factor
  if (fam$family == "ordinal") {
    check_pkg("MASS")
    temp.call[[1]] <- quote(MASS::polr)
    temp.call$Hess <- TRUE
    temp.call$method <- fam$method
    fit <- eval(temp.call, envir)
    coeffORTidy <- broom::tidy(fit, exponentiate=TRUE, conf.int=TRUE, conf.level=conf.level)
    coeffORTidy[coeffORTidy$coef.type != "coefficient", names(coeffORTidy) %nin% c("term", "coef.type")] <- NA
    coeffTidy <- broom::tidy(fit, exponentiate=FALSE, conf.int=TRUE, conf.level=conf.level)
    coeffTidy$p.value <- 2*stats::pnorm(abs(coeffTidy$statistic), lower.tail = FALSE)
    coeffTidy <- cbind(coeffTidy, OR=coeffORTidy$estimate, CI.lower.OR=coeffORTidy$conf.low, CI.upper.OR=coeffORTidy$conf.high)
    # sort so that zeta comes first, but hold all else fixed
    coeffTidy <- coeffTidy[order(coeffTidy$coef.type == "coefficient", seq_len(nrow(coeffTidy))), ]
    modelGlance <- broom::glance(fit)
    modelGlance$p.value.lrt <- try_lrt(fit, scope, anyna)

  } else if (fam$family == "gaussian") {
    # ## issue warning if appears categorical

    temp.call[[1]] <- quote(stats::lm)
    temp.call$x <- TRUE
    fit <- eval(temp.call, envir)
    coeffTidy <- broom::tidy(fit, conf.int=TRUE, conf.level=conf.level)

    if("(weights)" %in% colnames(fit$model)) fit$model[["(weights)"]] <- NULL

    coeffTidy$standard.estimate <- lm.beta(fit)
    ## Continuous variable (numeric) ###############
    ## Note: Using tidy changes colname from 't value' to 'statistic'
    modelGlance <- broom::glance(fit)
    names(modelGlance)[names(modelGlance) == "statistic"] <- "statistic.F"
    names(modelGlance)[names(modelGlance) == "p.value"] <- "p.value.F"
    modelGlance$p.value.lrt <- try_lrt(fit, scope, FALSE)

  } else if (fam$family == "binomial" || fam$family == "quasibinomial") {
    ## These families are used in glm

    check_pkg("pROC")
    temp.call[[1]] <- quote(stats::glm)
    temp.call$x <- TRUE
    temp.call$family <- fam
    fit <- eval(temp.call, envir)
    #coeffbeta <- summary(fit)$coef
    ## find out that broom:::tidy.lm allows conf.int and exp
    coeffORTidy <- broom::tidy(fit, exponentiate=TRUE, conf.int=TRUE, conf.level=conf.level)
    coeffORTidy[coeffORTidy$term == "Intercept", -1] <- NA
    coeffTidy <- broom::tidy(fit, exponentiate=FALSE, conf.int=TRUE, conf.level=conf.level)

    waldTidy <- suppressMessages(stats::confint.default(fit, conf.level=conf.level))
    all_na <- apply(waldTidy, 1, allNA)
    waldTidy <- stats::setNames(as.data.frame(waldTidy[!all_na, , drop = FALSE]), c("conf.low", "conf.high"))

    coeffTidy <- cbind(coeffTidy, OR=coeffORTidy$estimate, CI.lower.OR=coeffORTidy$conf.low, CI.upper.OR=coeffORTidy$conf.high,
                       CI.lower.wald=waldTidy$conf.low, CI.upper.wald=waldTidy$conf.high,
                       CI.lower.OR.wald=exp(waldTidy$conf.low), CI.upper.OR.wald=exp(waldTidy$conf.high))
    modelGlance <- broom::glance(fit)
    modelGlance$concordance <- as.numeric(pROC::auc(fit$y ~ predict(fit, type='response'), direction = "<", levels = 0:1))
    modelGlance$p.value.lrt <- try_lrt(fit, scope, FALSE)
    modelGlance$Nevents <- table(fit$y)[2]

  } else if (fam$family == "quasipoisson" || fam$family == "poisson") {
    ## These families use glm

    temp.call[[1]] <- quote(stats::glm)
    temp.call$x <- TRUE
    temp.call$family <- fam
    fit <- eval(temp.call, envir)
    coeffRRTidy <- broom::tidy(fit, exponentiate=TRUE, conf.int=TRUE, conf.level=conf.level)
    coeffRRTidy[coeffRRTidy$term == "Intercept", -1] <- NA
    coeffTidy <- broom::tidy(fit, exponentiate=FALSE, conf.int=TRUE, conf.level=conf.level)
    coeffTidy <- cbind(coeffTidy, RR=coeffRRTidy$estimate, CI.lower.RR=coeffRRTidy$conf.low, CI.upper.RR=coeffRRTidy$conf.high)
    modelGlance <- broom::glance(fit)
    modelGlance$p.value.lrt <- try_lrt(fit, scope, FALSE)

  } else if (fam$family == "negbin") {
    ## Also uses glm
    check_pkg("MASS")
    temp.call[[1]] <- quote(MASS::glm.nb)
    temp.call$x <- TRUE
    temp.call$link <- fam$link
    fit <- eval(temp.call, envir)
    coeffRRTidy <- suppressWarnings(broom::tidy(fit, exponentiate=TRUE, conf.int=TRUE, conf.level=conf.level))
    coeffRRTidy[coeffRRTidy$term == "Intercept", -1] <- NA
    coeffTidy <- suppressWarnings(broom::tidy(fit, exponentiate=FALSE, conf.int=TRUE, conf.level=conf.level))
    coeffTidy <- cbind(coeffTidy, RR=coeffRRTidy$estimate, CI.lower.RR=coeffRRTidy$conf.low, CI.upper.RR=coeffRRTidy$conf.high)
    modelGlance <- suppressWarnings(broom::glance(fit))
    modelGlance$theta <- fit$theta
    modelGlance$SE.theta <- fit$SE.theta
    modelGlance$p.value.lrt <- try_lrt(fit, scope, anyna)

  } else if(fam$family == "clog") {

    check_pkg("survival")
    temp.call[[1]] <- quote(survival::clogit)
    fit <- eval(temp.call, envir)
    ## use tidy to get both CIs, merge
    coeffORTidy <- broom::tidy(fit, exponentiate=TRUE, conf.int=conf.level)
    coeffTidy <- broom::tidy(fit, exponentiate=FALSE, conf.int=conf.level)
    coeffTidy <- cbind(coeffTidy, OR=coeffORTidy$estimate, CI.lower.OR=coeffORTidy$conf.low, CI.upper.OR=coeffORTidy$conf.high)
    modelGlance <-  broom::glance(fit)
    names(modelGlance)[names(modelGlance) == "nevent"] <- "Nevents"
    modelGlance$p.value.lrt <- try_lrt(fit, scope, anyna)

  } else if (fam$family == "relrisk") {

    check_pkg("geepack")
    temp.call[[1]] <- quote(geepack::geeglm)
    temp.call$family <- stats::poisson(fam$link)
    temp.call$corstr <- "independence"
    fit <- eval(temp.call, envir)
    coeffRRTidy <- broom::tidy(fit, exponentiate=TRUE, conf.int=TRUE, conf.level=conf.level)
    coeffRRTidy[coeffRRTidy$term == "Intercept", -1] <- NA
    coeffTidy <- broom::tidy(fit, exponentiate=FALSE, conf.int=TRUE, conf.level=conf.level)
    coeffTidy <- cbind(coeffTidy, RR=coeffRRTidy$estimate, CI.lower.RR=coeffRRTidy$conf.low, CI.upper.RR=coeffRRTidy$conf.high)
    modelGlance <- broom::glance(fit)

  } else if(fam$family == "survival") {

    check_pkg("survival")
    temp.call[[1]] <- quote(survival::coxph)
    fit <- eval(temp.call, envir)
    ## use tidy to get both CIs, merge
    coeffHRTidy <- broom::tidy(fit, exponentiate=TRUE, conf.int=TRUE, conf.level=conf.level)
    coeffTidy <- broom::tidy(fit, exponentiate=FALSE, conf.int=TRUE, conf.level=conf.level)
    coeffTidy <- cbind(coeffTidy, HR=coeffHRTidy$estimate, CI.lower.HR=coeffHRTidy$conf.low, CI.upper.HR=coeffHRTidy$conf.high)
    modelGlance <-  broom::glance(fit)
    names(modelGlance)[names(modelGlance) == "nevent"] <- "Nevents"
    modelGlance$p.value.lrt <- try_lrt(fit, scope, anyna)
  }

  names(coeffTidy)[names(coeffTidy) == "conf.low"] <- "CI.lower.estimate"
  names(coeffTidy)[names(coeffTidy) == "conf.high"] <- "CI.upper.estimate"
  modelGlance[] <- lapply(modelGlance, unname)
  list(coeffTidy = coeffTidy, modelGlance = modelGlance, fit = fit)
}

#####################################################
## Testing and Summary stats methods for internal use in tableby
###########################################

#' tableby Summary Statistics Functions
#'
#' A collection of functions that will report summary statistics. To create a custom function,
#'   consider using a function with all three arguments and \code{...}. See the \code{\link{tableby}} vignette
#'   for an example.
#'
#' @param x Usually a vector.
#' @param na.rm Should NAs be removed?
#' @param weights A vector of weights.
#' @param levels A vector of levels that character \code{x}s should have.
#' @param times A vector of times to use for survival summaries.
#' @param by a vector of the by-values.
#' @param by.levels a vector of the levels of \code{by}.
#' @param conf.level Numeric, denoting what confidence level to use for confidence intervals.
#' @param ... Other arguments.
#' @param totallab What to call the total "column"
#' @return Usually a vector of the appropriate numbers.
#' @details Not all these functions are exported, in order to avoid conflicting NAMESPACES.
#'   Note also that the functions prefixed with \code{"arsenal_"} can be referred to by their short names
#'   (e.g., \code{"min"} for \code{"arsenal_min"}).
#' @seealso \code{\link{includeNA}}, \code{\link{tableby.control}}
#' @name tableby.stats
NULL
#> NULL

get_stat_function <- function(x) switch(x, sum = , min = , max = , range = , mean = , sd = , var = , median = paste0("arsenal_", x), x)

#' @rdname tableby.stats
arsenal_sum <- function(x, na.rm=TRUE, ...) {
  y <- if(na.rm && allNA(x)) {
    NA_real_
  } else {
    sum(x, na.rm=na.rm)
  }
  as.tbstat(y) # unclear what the sum of dates should be?
}

#' @rdname tableby.stats
arsenal_min <- function(x, na.rm=TRUE, ...) {
  y <- if(na.rm && allNA(x)) {
    NA_real_
  } else {
    min(x, na.rm=na.rm)
  }
  as.tbstat(y, oldClass = if(is.Date(x)) "Date" else NULL)
}

#' @rdname tableby.stats
arsenal_max <- function(x, na.rm=TRUE, ...) {
  y <- if(na.rm && allNA(x)) {
    NA_real_
  } else {
    max(x, na.rm=na.rm)
  }
  as.tbstat(y, oldClass = if(is.Date(x)) "Date" else NULL)
}

#' @rdname tableby.stats
arsenal_mean <- function(x, na.rm=TRUE, weights = NULL, ...) {
  y <- if(na.rm && allNA(x))
  {
    NA_real_
  } else {
    wtd.mean(x, weights=weights, na.rm=na.rm)
  }
  as.tbstat(y, oldClass = if(is.Date(x)) "Date" else NULL)
}

#' @rdname tableby.stats
arsenal_sd <- function(x, na.rm=TRUE, weights = NULL, ...) {
  y <- if(na.rm && allNA(x))
  {
    NA_real_
  } else {
    s <- sqrt(wtd.var(x, weights=weights, na.rm=na.rm))
    if(is.Date(x)) list(as.difftime(s, units = "days")) else s
  }
  as.tbstat(y)
}

#' @rdname tableby.stats
arsenal_var <- function(x, na.rm=TRUE, weights = NULL, ...) {
  y <- if(na.rm && allNA(x))
  {
    NA_real_
  } else {
    wtd.var(x, weights=weights, na.rm=na.rm)
    # if(is.Date(x)) as.difftime(s, units = "days") else s
  }
  as.tbstat(y)
}

#' @rdname tableby.stats
#' @export
meansd <- function(x, na.rm=TRUE, weights = NULL, ...) {
  if(na.rm && allNA(x))
  {
    as.tbstat(NA_real_)
  } else {
    m <- wtd.mean(x, weights=weights, na.rm=na.rm)
    s <- sqrt(wtd.var(x, weights=weights, na.rm=na.rm))
    y <- if(is.Date(x)) list(as.character(m), as.difftime(s, units = "days")) else c(m, s)
    as.tbstat(y, fmt = "{y[1]} ({y[2]})")
  }
}

#' @rdname tableby.stats
#' @export
meanse <- function(x, na.rm=TRUE, weights = NULL, ...) {
  if(na.rm && allNA(x))
  {
    as.tbstat(NA_real_)
  } else {
    if(!is.null(weights)) stop("'meanse' can only be used without weights")
    m <- mean(x, na.rm=na.rm)
    s <- stats::sd(x, na.rm=na.rm)/sqrt(sum(!is.na(x)))
    y <- if(is.Date(x)) list(as.character(m), as.difftime(s, units = "days")) else c(m, s)
    as.tbstat(y, fmt = "{y[1]} ({y[2]})")
  }
}

#' @rdname tableby.stats
#' @export
meanpmsd <- function(x, na.rm=TRUE, weights = NULL, ...) {
  if(na.rm && allNA(x))
  {
    as.tbstat(NA_real_)
  } else {
    y <- meansd(x = x, na.rm = na.rm, weights = weights, ...)
    attr(y, "fmt") <- "{y[1]} &pm; {y[2]}"
    y
  }
}

#' @rdname tableby.stats
#' @export
meanpmse <- function(x, na.rm=TRUE, weights = NULL, ...) {
  if(na.rm && allNA(x))
  {
    as.tbstat(NA_real_)
  } else {
    y <- meanse(x = x, na.rm = na.rm, weights = weights, ...)
    attr(y, "fmt") <- "{y[1]} &pm; {y[2]}"
    y
  }
}

#' @rdname tableby.stats
#' @export
meanCI <- function(x, na.rm=TRUE, weights = NULL, conf.level = 0.95, ...) {
  if(!is.null(weights) || (na.rm && allNA(x)))
  {
    as.tbstat(NA_real_)
  } else
  {
    if(na.rm) x <- x[!is.na(x)]
    s <- stats::sd(x, na.rm = na.rm)
    m <- mean(x, na.rm = na.rm)
    n <- length(x)
    a <- (1 - conf.level)/2
    y <- c(m, m + stats::qt(c(a, 1 - a), df = n - 1) * s / sqrt(n))
    as.tbstat(y, oldClass = if(is.Date(x)) "Date" else NULL, fmt = "{y[1]} ({y[2]}, {y[3]})")
  }
}

#' @rdname tableby.stats
#' @export
medianrange <- function(x, na.rm=TRUE, weights = NULL, ...) {
  if(na.rm && allNA(x)) {
    as.tbstat(NA_real_)
  } else {
    y <- wtd.quantile(x, probs=c(0.5, 0, 1), na.rm=na.rm, weights=weights)
    as.tbstat(y, oldClass = if(is.Date(x)) "Date" else NULL, fmt = "{y[1]} ({y[2]}, {y[3]})")
  }
}


#' @rdname tableby.stats
#' @export
medianmad <- function(x, na.rm=TRUE, weights = NULL, ...) {
  if(!is.null(weights) || (na.rm && allNA(x)))
  {
    as.tbstat(NA_real_)
  } else {
    m <- stats::median(x, na.rm=na.rm)
    s <- stats::mad(x, na.rm=na.rm, constant = 1)
    y <- if(is.Date(x)) list(as.character(m), as.difftime(s, units = "days")) else c(m, s)
    as.tbstat(y, fmt = "{y[1]} ({y[2]})")
  }
}


#' @rdname tableby.stats
arsenal_median <- function(x, na.rm=TRUE, weights = NULL, ...) {
  y <- if(na.rm && allNA(x)) {
    NA_real_
  } else if(is.Date(x)) {
    as.Date(wtd.quantile(as.integer(x), weights=weights, probs=0.5, na.rm=na.rm), origin="1970/01/01")
  } else {
    wtd.quantile(x, weights=weights, probs=0.5, na.rm=na.rm)
  }
  as.tbstat(y, oldClass = if(is.Date(x)) "Date" else NULL)
}

#' @rdname tableby.stats
arsenal_range <- function(x, na.rm=TRUE, ...) {
  if(na.rm && allNA(x)) {
    as.tbstat(NA_real_)
  } else {
    y <- range(x, na.rm=na.rm)
    as.tbstat(y, oldClass = if(is.Date(x)) "Date" else NULL, fmt = "{y[1]} - {y[2]}")
  }
}

#' @rdname tableby.stats
#' @export
gmean <- function(x, na.rm=TRUE, weights = NULL, ...) {
  y <- if((na.rm && allNA(x)) || any(x < 0, na.rm = TRUE) || is.Date(x))
  {
    NA_real_
  } else {
    exp(wtd.mean(log(x), weights=weights, na.rm=na.rm))
  }
  as.tbstat(y)
}

#' @rdname tableby.stats
#' @export
gsd <- function(x, na.rm=TRUE, weights = NULL, ...) {
  y <- if((na.rm && allNA(x)) || any(x <= 0, na.rm = TRUE) || is.Date(x))
  {
    NA_real_
  } else {
    n <- sum(!is.na(x))
    exp(sqrt(wtd.var(log(x), weights = weights, na.rm = na.rm) * (n-1)/n))
  }
  as.tbstat(y)
}

#' @rdname tableby.stats
#' @export
gmeansd <- function(x, na.rm=TRUE, weights = NULL, ...) {
  if((na.rm && allNA(x)) || any(x < 0, na.rm = TRUE) || is.Date(x))
  {
    as.tbstat(NA_real_)
  } else {
    m <- exp(wtd.mean(log(x), weights=weights, na.rm=na.rm))
    n <- sum(!is.na(x))
    s <- if(any(x == 0, na.rm = TRUE)) {
      NA_real_
    } else exp(sqrt(wtd.var(log(x), weights = weights, na.rm = na.rm) * (n-1)/n))
    as.tbstat(c(m, s), fmt = "{y[1]} ({y[2]})")
  }
}

#' @rdname tableby.stats
#' @export
gmeanCI <- function(x, na.rm=TRUE, weights = NULL, conf.level = 0.95, ...) {
  if(!is.null(weights) || (na.rm && allNA(x)) || any(x < 0, na.rm = TRUE) || is.Date(x))
  {
    as.tbstat(NA_real_)
  } else
  {
    if(na.rm) x <- x[!is.na(x)]
    n <- length(x)
    s <- sqrt(stats::var(log(x), na.rm = na.rm) * (n-1)/n)
    m <- mean(log(x), na.rm = na.rm)
    a <- (1 - conf.level)/2
    bad <- any(x == 0, na.rm = TRUE)
    ci <- if(bad) NA_real_ else m + stats::qt(c(a, 1 - a), df = n - 1) * s / sqrt(n)
    as.tbstat(exp(c(m, ci)), fmt = if(bad) "{y[1]} ({y[2]})" else "{y[1]} ({y[2]}, {y[3]})")
  }
}

#' @rdname tableby.stats
#' @export
Nsigntest <- function(x, na.rm = TRUE, weights = NULL, ...) {
  if(is.null(weights)) weights <- rep(1, NROW(x))
  as.tbstat(sum(weights*(x != 0), na.rm = na.rm), which.count = 1L)
}

## survival stats
#' @rdname tableby.stats
#' @export
Nevents <- function(x, na.rm = TRUE, weights = NULL, ...) {
  y <- if(na.rm && allNA(x))
  {
    NA_real_
  } else {
    check_pkg("survival")
    mat <- summary(survival::survfit(x ~ 1, weights = weights))$table
    as.numeric(mat["events"])
  }
  as.tbstat(y, which.count = 1L)
}

## Median survival
#' @rdname tableby.stats
#' @export
medSurv <- function(x, na.rm = TRUE, weights = NULL, ...) {
  y <- if(na.rm && allNA(x))
  {
    NA_real_
  } else {
    check_pkg("survival")
    mat <- summary(survival::survfit(x ~ 1, weights = weights))$table
    as.numeric(mat["median"]) # if we don't hit the median, or if all obs are censors, this is NA
  }
  as.tbstat(y)
}

## Median survival with confidence interval
#' @rdname tableby.stats
#' @param robust,conf.type Passed to \code{survival::\link[survival]{survfit}}.
#' @export
medSurvCI <- function (x, na.rm = TRUE, weights = NULL, robust = FALSE, conf.type = 'log', ...) {
  y <- if (na.rm && allNA(x)) {
    NA_real_
  }
  else {
    check_pkg("survival")
    mat <- summary(survival::survfit(x ~ 1, weights = weights, robust = robust, conf.type =conf.type))$table
    m <- as.numeric(mat["median"])
    ci <- c(as.numeric(mat["0.95LCL"]), as.numeric(mat["0.95UCL"]))
    c(m, ci)
  }
  as.tbstat(y, fmt = "{y[1]} ({y[2]}, {y[3]})")
}

## Median survival with 25th/75th quantiles
#' @rdname tableby.stats
#' @export
medSurvQuant <- function (x, na.rm = TRUE, weights = NULL, robust = FALSE, ...) {
  y <- if (na.rm && allNA(x)) {
    NA_real_
  }
  else {
    check_pkg("survival")
    mat <- stats::quantile(survival::survfit(x ~ 1, weights = weights, robust = robust), probs=c(0.5,0.25, 0.75))$quantile
    mat
  }
  as.tbstat(y, fmt = "{y[1]} ({y[2]}, {y[3]})")
}


#' @rdname tableby.stats
#' @export
NeventsSurv <- function(x, na.rm = TRUE, weights = NULL, times=1:5, ...) {
  if(na.rm && allNA(x))
  {
    y <- matrix(NA_real_, nrow = 1, ncol = length(times))
    a <- TRUE
  } else
  {
    check_pkg("survival")
    xsumm <- summary(survival::survfit(x ~ 1, weights = weights), times=times)
    y <- t(cbind(cumsum(xsumm$n.event), 100*xsumm$surv))
    a <- FALSE
  }
  out <- stats::setNames(as.list(as.data.frame(y)), paste0("time = ", times))
  as.tbstat_multirow(lapply(out, as.tbstat, fmt = if(a) '{y}' else "{y[1]} ({y[2]})", which.count = 1L, which.pct = 2L))
}

#' @rdname tableby.stats
#' @export
NriskSurv <- function(x, na.rm = TRUE, weights = NULL, times=1:5, ...) {
  if(na.rm && allNA(x))
  {
    y <- matrix(NA_real_, nrow = 1, ncol = length(times))
    a <- TRUE
  } else {
    check_pkg("survival")
    xsumm <- summary(survival::survfit(x ~ 1, weights = weights), times=times)
    y <- t(cbind(xsumm$n.risk, 100*xsumm$surv))
    a <- FALSE
  }
  out <- stats::setNames(as.list(as.data.frame(y)), paste0("time = ", times))
  as.tbstat_multirow(lapply(out, as.tbstat, fmt = if(a) '{y}' else "{y[1]} ({y[2]})", which.count = 1L, which.pct = 2L))
}

#' @rdname tableby.stats
#' @export
Nrisk <- function(x, na.rm = TRUE, weights = NULL, times=1:5, ...) {
  y <- if(na.rm && allNA(x))
  {
    rep(NA_real_, times = length(times))
  } else
  {
    check_pkg("survival")
    summary(survival::survfit(x ~ 1, weights = weights), times=times)$n.risk
  }
  out <- stats::setNames(as.list(y), paste0("time = ", times))
  as.tbstat_multirow(lapply(out, as.tbstat, which.count = 1L))
}

#' @rdname tableby.stats
#' @export
medTime <- function(x, na.rm = TRUE, weights = NULL, ...)
{
  y <- if(na.rm && allNA(x))
  {
    NA_real_
  } else
  {
    x[, 2] <- 1 - x[, 2] # censor events instead
    check_pkg("survival")
    mat <- summary(survival::survfit(x ~ 1, weights = weights))$table
    as.numeric(mat["median"]) # if we don't hit the median, or if all obs are events, this is NA
  }
  as.tbstat(y)
}

## quantiles
#' @rdname tableby.stats
#' @export
q1q3 <- function(x, na.rm=TRUE, weights = NULL, ...) {
  if(na.rm && allNA(x)) {
    as.tbstat(NA_real_)
  } else {
    y <- wtd.quantile(x, weights=weights, probs=c(0.25, .75), na.rm=na.rm)
    as.tbstat(y, oldClass = if(is.Date(x)) "Date" else NULL, fmt = "{y[1]}, {y[2]}")
  }
}

#' @rdname tableby.stats
#' @export
medianq1q3 <- function(x, na.rm=TRUE, weights = NULL, ...) {
  if(na.rm && allNA(x)) {
    as.tbstat(NA_real_)
  } else {
    y <- wtd.quantile(x, weights=weights, probs=c(0.5, 0.25, 0.75), na.rm=na.rm)
    as.tbstat(y, oldClass = if(is.Date(x)) "Date" else NULL, fmt = "{y[1]} ({y[2]}, {y[3]})")
  }
}

#' @rdname tableby.stats
#' @export
iqr <- function(x, na.rm=TRUE, weights = NULL, ...) {
  y <- if(na.rm && allNA(x)) {
    NA_real_
  } else {
    s <- diff(wtd.quantile(x, weights=weights, probs=c(0.25, 0.75), na.rm=na.rm))
    if(is.Date(x)) list(as.difftime(s, units = "days")) else s
  }

  as.tbstat(y)
}

## Count of missings: always show missings
#' @rdname tableby.stats
#' @export
Nmiss <- function(x, weights = NULL, ...) {
  if(is.null(weights)) weights <- rep(1, NROW(x))
  weights <- weights[is.na(x)]
  as.tbstat(sum(weights), which.count = 1L)
}

#' @rdname tableby.stats
#' @export
Nmisspct <- function(x, weights = NULL, ...) {
  if(is.null(weights)) weights <- rep(1, NROW(x))
  num <- sum(weights[is.na(x)])
  denom <- sum(weights)
  as.tbstat(c(num, 100*num/denom), fmt = "{y[1]} ({y[2]}%)", which.count = 1L, which.pct = 2L)
}

## Nmiss2 make similar, but in tableby, always keep nmiss,
## even if there are no missings
#' @rdname tableby.stats
#' @export
Nmiss2 <- Nmiss

#' @rdname tableby.stats
#' @export
Nmisspct2 <- Nmisspct

## count of complete samples
#' @rdname tableby.stats
#' @export
N <- function(x, weights = NULL, ...) {
  if(is.null(weights)) weights <- rep(1, NROW(x))
  weights <- weights[!is.na(x)]
  as.tbstat(sum(weights), which.count = 1L)
}


#' @rdname tableby.stats
#' @export
Npct <- function(x, weights = NULL, ...) {
  if(is.null(weights)) weights <- rep(1, NROW(x))
  num <- sum(weights[!is.na(x)])
  denom <- sum(weights)
  as.tbstat(c(num, 100*num/denom), fmt = "{y[1]} ({y[2]}%)", which.count = 1L, which.pct = 2L)
}

#' @rdname tableby.stats
#' @export
Nrowpct <- function(x, levels=NULL, by, by.levels=sort(unique(by)), weights = NULL, ..., totallab = "Total") {
  if(is.null(levels)) levels <- sort(unique(x))

  idx <- !is.na(x) & !is.na(by)
  x <- x[idx]
  by <- by[idx]
  weights <- weights[idx]

  tmp <- wtd.table(factor(by, levels = by.levels), weights = weights)
  wtbl <- c(tmp, stats::setNames(sum(tmp), totallab))
  lapply(wtbl, function(elt) as.tbstat(c(elt, 100*elt/sum(tmp)), fmt = "{y[1]} ({y[2]}%)", which.count = 1L, which.pct = 2L))
}

## count within group variable
#' @rdname tableby.stats
count <- function (x, levels=NULL, na.rm = TRUE, weights = NULL, ...)  {
  if(is.null(levels)) levels <- sort(unique(x))
  if(na.rm)
  {
    idx <- !is.na(x)
    x <- x[idx]
    weights <- weights[idx]
  }
  if(is.selectall(x))
  {
    if(is.null(weights)) weights <- rep(1, nrow(x))
    wtbl <- apply(as.matrix(x) == 1, 2, function(y) sum(weights[y]))
  } else wtbl <- wtd.table(factor(x, levels=levels), weights=weights)
  as.tbstat_multirow(lapply(as.list(wtbl), as.tbstat, which.count = 1L))
}

## count (pct) where pct is within group variable total
#' @rdname tableby.stats
#' @export
countpct <- function(x, levels=NULL, na.rm=TRUE, weights = NULL, ...) {
  if(is.null(levels)) levels <- sort(unique(x))
  if(na.rm)
  {
    idx <- !is.na(x)
    x <- x[idx]
    weights <- weights[idx]
  }
  if(is.selectall(x))
  {
    if(is.null(weights)) weights <- rep(1, nrow(x))
    wtbl <- apply(as.matrix(x) == 1, 2, function(y) sum(weights[y]))
    denom <- sum(weights)
  } else
  {
    wtbl <- wtd.table(factor(x, levels=levels), weights=weights)
    denom <- sum(wtbl)
  }
  a <- any(wtbl > 0)
  tmp <- Map(c, wtbl, if(a) 100*wtbl/denom else rep(list(NULL), times = length(wtbl)))
  as.tbstat_multirow(lapply(tmp, as.tbstat, fmt = if(a) "{y[1]} ({y[2]}%)" else "{y}", which.count = 1L, which.pct = 2L))
}

#' @rdname tableby.stats
#' @export
pct <- function(x, levels=NULL, na.rm=TRUE, weights = NULL, ...) {
  if(is.null(levels)) levels <- sort(unique(x))
  if(na.rm)
  {
    idx <- !is.na(x)
    x <- x[idx]
    weights <- weights[idx]
  }
  if(is.selectall(x))
  {
    if(is.null(weights)) weights <- rep(1, nrow(x))
    wtbl <- apply(as.matrix(x) == 1, 2, function(y) sum(weights[y]))
    denom <- sum(weights)
  } else
  {
    wtbl <- wtd.table(factor(x, levels=levels), weights=weights)
    denom <- sum(wtbl)
  }
  a <- any(wtbl > 0)
  tmp <- if(a) 100*wtbl/denom else rep_len(NA, length(wtbl))
  as.tbstat_multirow(lapply(tmp, as.tbstat, fmt = "{y}%", which.pct = 1L))
}

#' @rdname tableby.stats
#' @export
countN <- function(x, levels=NULL, na.rm=TRUE, weights = NULL, ...) {
  if(is.null(levels)) levels <- sort(unique(x))
  wtbl <- wtd.table(factor(x[!is.na(x)], levels=levels), weights=weights[!is.na(x)])
  n <- sum(wtbl)
  as.tbstat_multirow(lapply(Map(c, wtbl, rep(n, times = length(wtbl))), as.tbstat, which.count = 1:2, fmt = "{y[1]}/{y[2]}"))
}

transpose_list <- function(x, levels, by.levels)
  stats::setNames(lapply(by.levels, function(i) as.tbstat_multirow(stats::setNames(lapply(x, "[[", i), levels))), by.levels)


#' @rdname tableby.stats
#' @export
countrowpct <- function(x, levels=NULL, by, by.levels=sort(unique(by)), na.rm=TRUE, weights = NULL, ..., totallab = "Total") {
  if(is.null(levels)) levels <- sort(unique(x))
  if(na.rm)
  {
    idx <- !is.na(x) & !is.na(by)
    x <- x[idx]
    by <- by[idx]
    weights <- weights[idx]
  }

  wtbls <- lapply(levels, function(L) {
    tmp <- wtd.table(factor(by[x == L], levels = by.levels), weights = weights[x == L])
    wtbl <- c(tmp, stats::setNames(sum(tmp), totallab))
    lapply(wtbl, function(elt) as.tbstat(c(elt, 100*elt/sum(tmp)), fmt = "{y[1]} ({y[2]}%)", which.count = 1L, which.pct = 2L))
  })
  transpose_list(wtbls, levels, c(by.levels, totallab))
}

#' @rdname tableby.stats
#' @export
rowpct <- function(x, levels=NULL, by, by.levels=sort(unique(by)), na.rm=TRUE, weights = NULL, ..., totallab = "Total") {
  if(is.null(levels)) levels <- sort(unique(x))
  if(na.rm)
  {
    idx <- !is.na(x) & !is.na(by)
    x <- x[idx]
    by <- by[idx]
    weights <- weights[idx]
  }

  wtbls <- lapply(levels, function(L) {
    tmp <- wtd.table(factor(by[x == L], levels = by.levels), weights = weights[x == L])
    wtbl <- c(tmp, stats::setNames(sum(tmp), totallab))
    lapply(wtbl, function(elt) as.tbstat(100*elt/sum(tmp), fmt = "{y[1]}%", which.pct = 1L))
  })
  transpose_list(wtbls, levels, c(by.levels, totallab))
}

#' @rdname tableby.stats
#' @export
countcellpct <- function(x, levels=NULL, by, by.levels=sort(unique(by)), na.rm=TRUE, weights = NULL, ..., totallab = "Total") {
  if(is.null(levels)) levels <- sort(unique(x))
  if(na.rm)
  {
    idx <- !is.na(x) & !is.na(by)
    x <- x[idx]
    by <- by[idx]
    weights <- weights[idx]
  }

  tot <- sum(vapply(levels, function(L) {
    sum(wtd.table(factor(by[x == L], levels = by.levels), weights = weights[x == L]))
  }, numeric(1)))

  wtbls <- lapply(levels, function(L) {
    tmp <- wtd.table(factor(by[x == L], levels = by.levels), weights = weights[x == L])
    wtbl <- c(tmp, stats::setNames(sum(tmp), totallab))
    lapply(wtbl, function(elt) as.tbstat(c(elt, 100*elt/tot), fmt = "{y[1]} ({y[2]}%)", which.count = 1L, which.pct = 2L))
  })
  transpose_list(wtbls, levels, c(by.levels, totallab))
}

get_binom_est_ci <- function(x, tot, setNA, conf.level = 0.95) {
  if(setNA) return(NA_real_)

  b <- stats::binom.test(x, tot, conf.level = conf.level)
  unname(c(b$estimate, b$conf.int))
}

#' @rdname tableby.stats
#' @export
binomCI <- function(x, levels=NULL, na.rm=TRUE, weights = NULL, conf.level = 0.95, ...) {
  if(is.null(levels)) levels <- sort(unique(x))
  wtbl <- wtd.table(factor(x[!is.na(x)], levels=levels), weights=weights[!is.na(x)])

  ests <- lapply(wtbl, get_binom_est_ci, tot = sum(wtbl), setNA = !is.null(weights), conf.level = conf.level)
  as.tbstat_multirow(lapply(ests, as.tbstat, fmt = "{y[1]} ({y[2]}, {y[3]})"))
}

#' @rdname tableby.stats
#' @export
rowbinomCI <- function(x, levels=NULL, by, by.levels=sort(unique(by)), na.rm=TRUE, weights = NULL, conf.level = 0.95, ..., totallab = "Total") {
  if(is.null(levels)) levels <- sort(unique(x))
  wts <- !is.null(weights)
  if(na.rm)
  {
    idx <- !is.na(x) & !is.na(by)
    x <- x[idx]
    by <- by[idx]
    weights <- weights[idx]
  }

  wtbls <- lapply(levels, function(L) {
    tmp <- wtd.table(factor(by[x == L], levels = by.levels), weights = weights[x == L])
    wtbl <- c(tmp, stats::setNames(sum(tmp), totallab))
    wtbl <- lapply(wtbl, get_binom_est_ci, tot = sum(tmp), setNA = wts, conf.level = conf.level)
    lapply(wtbl, as.tbstat, fmt = "{y[1]} ({y[2]}, {y[3]})")
  })
  # as.tbstat_multirow(lapply(wtbl, f))
  transpose_list(wtbls, levels, c(by.levels, totallab))
}

######## internal functions that we use above ########

wtd.table <- function(x, weights = NULL, na.rm = TRUE)
{
  if(!length(weights)) return(table(x))
  tmp <- tapply(weights, x, sum, na.rm = na.rm)
  tmp[is.na(tmp)] <- 0 # (tapply(default = 0) would be enough in R >= 3.4, but we'll make this backwards-compatible)
  tmp
}

wtd.mean <- function(x, weights = NULL, na.rm = TRUE) {
  if(!length(weights)) return(mean(x, na.rm = na.rm))
  if(na.rm) {
    idx <- !is.na(x + weights)
    x <- x[idx]
    weights <- weights[idx]
  }
  sum(weights * x)/sum(weights)
}

wtd.quantile <- function(x, weights=NULL, probs=c(0,0.25,0.5,0.75,1), na.rm=TRUE) {

  if(!length(weights)) return(stats::quantile(as.numeric(x), probs = probs, na.rm = na.rm))
  if(any(probs < 0) || any(probs > 1)) stop("Probabilities must be between 0 and 1 inclusive")

  wts <- wtd.table(x, weights, na.rm = na.rm)
  x <- if(is.Date(x)) as.numeric(as.Date(names(wts))) else as.numeric(names(wts))
  n <- sum(wts)
  order <- 1 + (n - 1) * probs
  low <- pmax(floor(order), 1)
  high <- pmin(low + 1, n)
  order <- order%%1
  allq <- stats::approx(cumsum(wts), x, xout = c(low, high), method = "constant", f = 1, rule = 2)$y
  k <- length(probs)
  stats::setNames((1 - order) * allq[1:k] + order * allq[-(1:k)], probs)
}

wtd.var <- function(x, weights = NULL, na.rm=TRUE, method = c("unbiased", "ML")) {
  method <- match.arg(method)
  if(!length(weights)) return(stats::var(x, na.rm = na.rm))

  if(na.rm) {
    idx <- !is.na(x + weights)
    x <- x[idx]
    weights <- weights[idx]
  }
  if(length(x) < 2) return(NA_real_)
  as.numeric(stats::cov.wt(matrix(x, ncol = 1), weights, method = method)$cov)
}

## Author: Terry Therneau
## Contributed on 8/30/2013
## Updated 7/23/2014 by Jason Sinnwell

#' Convert numeric dates to Date object, and vice versa
#'
#' Convert numeric dates for month, day, and year to Date object, and vice versa.
#'
#' Test if an object is a date.
#'
#' @param month integer, month (1-12).
#' @param day integer, day of the month (1-31, depending on the month).
#' @param year integer, either 2- or 4-digit year. If two-digit number, will add 1900 onto it, depending on range.
#' @param yearcut cutoff for method to know if to convert to 4-digit year.
#' @param date A date value.
#' @param x An object.
#' @return \code{mdy.Date} returns a Date object, and Date.mdy returns a list with integer values for month, day, and year.
#'   \code{is.Date} returns a single logical value.
#' @details More work may need to be done with yearcut and 2-digit years. Best to give a full 4-digit year.
#' @seealso \code{\link{Date}}, \code{\link{DateTimeClasses}}
#' @examples
#' mdy.Date(9, 2, 2013)
#'
#' tmp <- mdy.Date(9, 2, 2013)
#' Date.mdy(tmp)
#'
#' is.Date(tmp)
#' @name mdy.Date
NULL
#> NULL

#' @rdname mdy.Date
#' @export
# mdy.Date(c(0,5),c(1, 1),c(2014, 2013))  # should return NA, "2013-05-01"
mdy.Date <- function(month, day, year, yearcut=120) {
    ## keep operations vectorized
    ## NA for day or month out of range
    day <- as.numeric(day)
    day <- ifelse(day < 1 | day > 31, NA, day) # stop ("invalid day")

    month <- as.numeric(month)
    month <- ifelse(month < 1 | month > 12 | month != floor(month), NA, month)

    year <- ifelse(year < yearcut, year + 1900, year)
    temp <- cbind(year, month, day)  # force them all to the same length
    ## allow NAs
    dtext <- rep(NA, nrow(temp))
    dtext[rowSums(is.na(temp)) < 1] <- paste(temp[rowSums(is.na(temp)) < 1, 1, drop=FALSE],
                            sprintf("%2d", temp[rowSums(is.na(temp)) < 1, 2, drop=FALSE]),
                            sprintf("%2d", temp[rowSums(is.na(temp)) < 1, 3, drop=FALSE]), sep='/')

   as.Date(dtext)
}

#' @rdname mdy.Date
#' @export
Date.mdy <- function(date) {
    temp <- unclass(as.POSIXlt(date))
    list(month=temp$mon+1, day=temp$mday, year=1900+temp$year)
}

#' @rdname mdy.Date
#' @export
is.Date <- function(x) inherits(x, "Date")

check_pkg <- function(p)
{
  if(!requireNamespace(p, quietly = TRUE)) stop("Package '", p, "' is required for this functionality, but is not installed.")
}

locate <- function(string, pattern) gregexpr(pattern, string)[[1L]]

smartsplit <- function(string, width, min.split)
{
  if(width < min.split) stop("Desired width < min.split?")
  if(nchar(string) <= width) return(string)

  pos <- locate(string, "[ \t\n_.;:,-]")
  splt <- if((length(pos) == 1 && pos == -1) || !any(idx <- (pos <= width & pos >= min.split))) width else max(pos[idx])

  c(substring(string, 1L, splt), smartsplit(substring(string, splt+1L), width = width, min.split = min.split))
}

#' Internal Functions
#'
#' @param string A character vector
#' @param width Either \code{Inf} or \code{NULL} to specify no splitting,
#'   or a positive integer giving the largest allowed string length.
#' @param min.split Either \code{-Inf} or \code{NULL} to specify no
#'   lower bound on the string length, or a positive integer giving the minimum string length.
#' @inheritParams base::replace
#' @return For \code{smart.split}, a list of the same length as \code{string}, with each element being
#'   the "intelligently" split string.
#'
#'   For \code{replace2}, a vector with the proper values replaced.
#' @seealso \code{\link[base]{replace}}
#' @name internal.functions
NULL
#> NULL

#' @rdname internal.functions
#' @export
smart.split <- function(string, width = Inf, min.split = -Inf)
{
  if(is.null(width)) width <- Inf
  if(is.null(min.split)) min.split <- -Inf
  lapply(string, smartsplit, width = width, min.split = min.split)
}


insert_elt <- function(col, times, elt = "")
{
  f <- if(is.null(elt)) rep else function(x, i) c(x, rep(elt, times = i - 1L))
  unlist(Map(f, col, times), use.names = FALSE)
}

#' @rdname internal.functions
#' @export
replace2 <- function(x, list, values)
{
  x[[list]] <- values
  x
}

as_list_formula <- function(formula)
{
  if(is.list(formula)) return(formula)
  if(length(formula) == 2 || is.name(formula[[2]]) || !identical(formula[[2]][[1]], as.name("list")))
    return(list(formula)) # one-sided or LHS is single arg
  if(length(formula[[2]]) == 1) return(list(replace2(formula, 2, NULL))) # for empty LHS-list()
  lapply(formula[[2]][-1], replace2, list = 2, x = formula)
}

# set all factors to characters
rbind_chr <- function(...)
{
  out <- rbind(..., make.row.names = FALSE)
  if(is.data.frame(out))
  {
    idx <- vapply(out, is.factor, NA)
    out[idx] <- lapply(out[idx], as.character)
  }
  out
}

#' Include a YAML header in \code{write2}
#'
#' @param x An object of class \code{"yaml"}.
#' @param ... For \code{yaml()}, arguments to be bundled into a list and passed to \code{\link[yaml]{as.yaml}}.
#'   For \code{print.yaml()}, extra arguments. For \code{c.yaml()}, "yaml" objects to be concatenated.
#' @param recursive Not in use at this time.
#' @return A text string of class \code{"yaml"}.
#' @author Ethan Heinzen, adapted from an idea by Brendan Broderick
#' @seealso \code{\link[yaml]{as.yaml}}, \code{\link{write2}}
#' @examples
#' x <- yaml(title = "My cool title", author = "Ethan P Heinzen")
#' x
#' y <- yaml("header-includes" = list("\\usepackage[labelformat=empty]{caption}"))
#' y
#' c(x, y)
#' @name yaml
NULL
#> NULL

#' @rdname yaml
#' @export
yaml <- function(...)
{
  check_pkg("yaml")
  x <- yaml::as.yaml(list(...))
  class(x) <- "yaml"
  x
}

#' @rdname yaml
#' @export
print.yaml <- function(x, ...)
{
  cat(c("---\n", x, "---\n"), sep = "")
  invisible(x)
}

#' @rdname yaml
#' @export
c.yaml <- function(..., recursive = FALSE)
{
  structure(paste0(c(unlist(lapply(list(...), unclass))), collapse = ""), class = "yaml")
}

#' @rdname yaml
#' @export
is.yaml <- function(x)
{
  inherits(x, "yaml")
}

internalTable <- function(data, na.options, keep_cols = c("cumFreq", "freqPercent", "cumPercent"), sort = FALSE, decreasing = FALSE) {
  if("Freq" %nin% names(data)) stop("You tried to create or sort a freqlist table with no 'Freq' column!")

  data <- if(!sort)
  {
    data[do.call(order, unname(data)), ]
  } else data[order(data$Freq, decreasing = decreasing), ]

  na.index <- rowSums(is.na(data))
  if (na.options == 'remove') {
    data  <- data[na.index == 0, ]
    cumFreq <- cumsum(data$Freq)
    freqPct <- if(sum(data$Freq) > 0) 100 * data$Freq / sum(data$Freq) else NA_real_
    cumPct <- cumsum(freqPct)
  } else if(na.options == 'include') {
    cumFreq <- cumsum(data$Freq)
    freqPct <- if(sum(data$Freq) > 0) 100 * data$Freq / sum(data$Freq) else NA_real_
    cumPct <- cumsum(freqPct)
  } else if(na.options == 'showexclude') {
    freq_tmp <- data$Freq
    freq_tmp[na.index != 0] <- NA
    cumFreq <- cumfun(freq_tmp)
    denom <- max(stats::na.omit(cumFreq), na.rm = TRUE)
    freqPct <- if(denom > 0) 100 * freq_tmp / denom else NA_real_
    cumPct <- cumfun(freqPct)
  }

  if("cumFreq" %in% keep_cols) data$cumFreq <- cumFreq
  if("freqPercent" %in% keep_cols) data$freqPercent <- freqPct
  if("cumPercent" %in% keep_cols) data$cumPercent <- cumPct
  row.names(data) <- NULL
  data
}

cumfun <- function(x) {
  # function to create a cumulative sum retaining NAs, but omitting in sum function
  x2 <- rep(NA, length(x))
  x.om <- stats::na.omit(x)
  if (length(x.om) == 0) {
    warning("For at least one level, all entries have NAs")
  } else {
    x2[!is.na(x)] <- cumsum(x.om)
  }
  x2
}

add_freqlist_xterms <- function(xTerms)
{
  xTerms$Freq <- list(variable="Freq", label="Freq", term="Freq")
  xTerms$cumFreq <- list(variable="cumFreq", label="Cumulative Freq", term="cumFreq")
  xTerms$freqPercent <- list(variable="freqPercent", label="Percent", term="freqPercent")
  xTerms$cumPercent <- list(variable="cumPercent", label="Cumulative Percent", term="cumPercent")
  xTerms
}

#' Helper functions for freqlist
#'
#' A set of helper functions for \code{\link{freqlist}}.
#'
#' @param x A \code{freqlist} object.
#' @inheritParams tableby.internal
#' @param decreasing Should the sort be increasing or decreasing?
#' @seealso \code{\link{merge.freqlist}}, \code{\link{arsenal_table}}, \code{\link{sort}},
#'   \code{\link{freqlist}}, \code{\link{summary.freqlist}}, \code{\link{freq.control}},
#' @details
#' Note that \code{sort()} has to recalculate cumulative statistics. Note also that the reordering of rows
#'   will also affect which labels are duplicates; you may also want to consider using
#'   \code{dupLabels=TRUE} in \code{\link{freq.control}()}.
#' @name freqlist.internal
NULL
#> NULL

#' @rdname freqlist.internal
#' @export
is.freqlist <- function(x) inherits(x, "freqlist")

#' @rdname freqlist.internal
#' @export
is.summary.freqlist <- function(x) inherits(x, "summary.freqlist")

#' @rdname freqlist.internal
#' @export
head.summary.freqlist <- function(x, n = 6L, ...)
{
  x$object <- lapply(x$object, function(d) set_attr(utils::head(d, n = n, ...), "labels", attr(d, "labels")))
  x
}

#' @rdname freqlist.internal
#' @export
tail.summary.freqlist <- function(x, n = 6L, ...)
{
  x$object <- lapply(x$object, function(d) set_attr(utils::tail(d, n = n, ...), "labels", attr(d, "labels")))
  x
}

#' @rdname freqlist.internal
#' @export
sort.freqlist <- function(x, decreasing = FALSE, ...)
{
  for(i in seq_along(x$tables))
  {
    na.opts <- x$tables[[i]]$na.options
    keepcols <- names(x$tables[[i]]$x) # in case they subsetted away some of the columns

    x$tables[[i]]$tables[] <- lapply(x$tables[[i]]$tables, internalTable, na.options = na.opts,
                                     sort = TRUE, decreasing = decreasing, keep_cols = keepcols)
  }
  x
}

## Purpose: control parameters for modelsum function
## Authors: P Votruba, Jason Sinnwell, Beth Atkinson
## Created: 9/3/2015

#' Control settings for \code{modelsum} function
#'
#' Control test and summary settings for \code{\link{modelsum}} function.
#'
#' @param digits Numeric, denoting the number of digits after the decimal point for beta coefficients and standard errors.
#' @param digits.ratio Numeric, denoting the number of digits after the decimal point for ratios, e.g. OR, RR, HR.
#' @param digits.p Numeric, denoting the number of digits for p-values. See "Details", below.
#' @param format.p Logical, denoting whether to format p-values. See "Details", below.
#' @param show.adjust Logical, denoting whether to show adjustment terms.
#' @param show.intercept Logical, denoting whether to show intercept terms.
#' @param conf.level Numeric, giving the confidence level.
#' @param ordinal.stats,binomial.stats,survival.stats,gaussian.stats,poisson.stats,negbin.stats,clog.stats,relrisk.stats
#'   Character vectors denoting which stats to show for the various model types.
#' @param stat.labels A named list of labels for all the stats used above.
#' @param ... Other arguments (not in use at this time).
#' @return A list with settings to be used within the \code{modelsum} function.
#' @details
#'   If \code{format.p} is \code{FALSE}, \code{digits.p} denotes the number of significant digits shown. The
#'   p-values will be in exponential notation if necessary. If \code{format.p} is \code{TRUE},
#'   \code{digits.p} will determine the number of digits after the decimal point to show. If the p-value
#'   is less than the resulting number of places, it will be formatted to show so.
#' @seealso \code{\link{modelsum}}, \code{\link{summary.modelsum}}, \code{\link{modelsum.internal}}
#' @export
modelsum.control <- function(
  digits = 3L, digits.ratio = 3L, digits.p = 3L, format.p = TRUE,
  show.adjust = TRUE, show.intercept = TRUE, conf.level = 0.95,
  ordinal.stats=c("OR","CI.lower.OR","CI.upper.OR", "p.value","Nmiss"),
  binomial.stats=c("OR","CI.lower.OR","CI.upper.OR","p.value", "concordance","Nmiss"),
  gaussian.stats=c("estimate","std.error","p.value","adj.r.squared","Nmiss"),
  poisson.stats=c("RR","CI.lower.RR", "CI.upper.RR","p.value","Nmiss"),
  negbin.stats=c("RR","CI.lower.RR", "CI.upper.RR","p.value","Nmiss"),
  relrisk.stats=c("RR","CI.lower.RR", "CI.upper.RR","p.value","Nmiss"),
  clog.stats=c("OR", "CI.lower.OR", "CI.upper.OR", "p.value", "concordance", "Nmiss"),
  survival.stats=c("HR","CI.lower.HR","CI.upper.HR","p.value","concordance","Nmiss"),
  stat.labels = list(), ...
) {

  if("nsmall" %in% names(list(...))) .Deprecated(msg = "Using 'nsmall = ' is deprecated. Use 'digits = ' instead.")
  if("nsmall.ratio" %in% names(list(...))) .Deprecated(msg = "Using 'nsmall.ratio = ' is deprecated. Use 'digits.ratio = ' instead.")
  if("digits.test" %in% names(list(...))) .Deprecated(msg = "Using 'digits.test = ' is deprecated. Use 'digits.p = ' instead.")

  # digits and digits.test are OK to be NULL. See ?format
  if(!is.null(digits) && digits < 0L)
  {
    warning("digits must be >= 0. Set to default.")
    digits <- 3L
  }
  if(!is.null(digits.ratio) && digits.ratio < 0L)
  {
    warning("digits.ratio must be >= 0. Set to default.")
    digits.ratio <- 3L
  }
  if(!is.null(digits.p) && digits.p < 0L)
  {
    warning("digits.p must be >= 0. Set to default.")
    digits.p <- 3L
  }

  if(conf.level <= 0 || conf.level >= 1) {
    warning("conf.level must be between (0,1). Setting to default.\n")
    conf.level <- 0.95
  }

  ##########################
  ## Ordinal stats:
  ##########################

  ordinal.stats.valid <- c(
    "Nmiss", "OR", "CI.lower.OR", "CI.upper.OR", "p.value", # default
    "estimate", "CI.OR", "CI.estimate", "CI.lower.estimate", "CI.upper.estimate", "N", "Nmiss2", "endpoint", "std.error", "statistic",
    "logLik", "AIC", "BIC", "edf", "deviance", "df.residual", "p.value.lrt"
  )

  if(any(ordinal.stats %nin% ordinal.stats.valid)) {
    stop("Invalid binomial stats: ",
         paste(ordinal.stats[ordinal.stats %nin% ordinal.stats.valid],collapse=","), "\n")
  }
  ## let CI.OR decode to CI.lower.OR and CI.upper.OR
  if(any(ordinal.stats == "CI.OR")) {
    ordinal.stats <- unique(c(ordinal.stats[ordinal.stats != "CI.OR"], "CI.lower.OR", "CI.upper.OR"))
  }
  if(any(ordinal.stats == "CI.estimate")) {
    ordinal.stats <- unique(c(ordinal.stats[ordinal.stats != "CI.estimate"], "CI.lower.estimate", "CI.upper.estimate"))
  }

  ##########################
  ## Binomial stats:
  ##########################
  ##Other coefficient columns:
  ##CI.estimate, N, Nmiss2, depvar (show name of dependent variable), estimate, se, zstat
  ##Other  model fits:  logLik,AIC,BIC
  binomial.stats.valid <- c(
    "Nmiss", "OR", "CI.lower.OR", "CI.upper.OR", "p.value", "concordance",  # default
    "estimate", "CI.OR", "CI.estimate", "CI.lower.estimate", "CI.upper.estimate",
    "CI.wald", "CI.lower.wald", "CI.upper.wald", "CI.OR.wald", "CI.lower.OR.wald", "CI.upper.OR.wald",
    "N", "Nmiss2", "endpoint", "std.error", "statistic", "Nevents",
    "logLik", "AIC", "BIC", "null.deviance", "deviance", "df.residual", "df.null", "p.value.lrt"
  )

  if(any(binomial.stats %nin% binomial.stats.valid)) {
    stop("Invalid binomial stats: ",
         paste(binomial.stats[binomial.stats %nin% binomial.stats.valid],collapse=","), "\n")
  }
  ## let CI.OR decode to CI.lower.OR and CI.upper.OR
  if(any(binomial.stats == "CI.OR")) {
    binomial.stats <- unique(c(binomial.stats[binomial.stats != "CI.OR"], "CI.lower.OR", "CI.upper.OR"))
  }
  if(any(binomial.stats == "CI.estimate")) {
    binomial.stats <- unique(c(binomial.stats[binomial.stats != "CI.estimate"], "CI.lower.estimate", "CI.upper.estimate"))
  }
  if(any(binomial.stats == "CI.wald")) {
    binomial.stats <- unique(c(binomial.stats[binomial.stats != "CI.wald"], "CI.lower.wald", "CI.upper.wald"))
  }
  if(any(binomial.stats == "CI.OR.wald")) {
    binomial.stats <- unique(c(binomial.stats[binomial.stats != "CI.OR.wald"], "CI.lower.OR.wald", "CI.upper.OR.wald"))
  }

  ##########################
  ## Gaussian stats:
  ##########################
  ##Other coefficient columns: CI.estimate, N, Nmiss2, t.stat, standard.estimate, endpoint
  ##Other model fits: r.squared, AIC, BIC,logLik
  gaussian.stats.valid <- c(
    "Nmiss", "estimate", "std.error", "p.value", "adj.r.squared",  #default
    "CI.estimate", "CI.lower.estimate", "CI.upper.estimate", "N", "Nmiss2", "statistic", "standard.estimate", "endpoint",
    "r.squared", "AIC", "BIC", "logLik", "statistic.F", "p.value.F", "p.value.lrt"
  )

  if(any(gaussian.stats %nin% gaussian.stats.valid)) {
    stop("Invalid gaussian stats: ",
         paste(gaussian.stats[gaussian.stats %nin% gaussian.stats.valid],collapse=","), "\n")
  }
  if(any(gaussian.stats == "CI.estimate")) {
    gaussian.stats <- unique(c(gaussian.stats[gaussian.stats != "CI.estimate"], "CI.lower.estimate", "CI.upper.estimate"))
  }


  ##########################
  ## Poisson stats:
  ##########################
  ##(quasi)/poisson.stats=c("Nmiss","RR","CI.RR", "p.value","concordance"),
  ##Other coeff columns: CI.estimate, CI.RR  (ci for relrisk),N,Nmiss2, std.error, estimate, z.stat, endpoint
  ##Other model fits: AIC,BIC,logLik, dispersion
  ##  dispersion = deviance/df.residual
  poisson.stats.valid <- c(
    "RR", "CI.lower.RR", "CI.upper.RR", "p.value", "Nmiss", # default
    "CI.RR", "CI.estimate", "CI.lower.estimate", "CI.upper.estimate", "CI.RR", "Nmiss2", "std.error", "estimate", "statistic", "endpoint",
    "AIC", "BIC", "logLik", "dispersion", "null.deviance", "deviance", "df.residual", "df.null", "p.value.lrt"
  )

  if(any(poisson.stats %nin% poisson.stats.valid)) {
    stop("Invalid poisson stats: ",
         paste(poisson.stats[poisson.stats %nin% poisson.stats.valid],collapse=","), "\n")
  }
  ## let CI.RR decode to CI.lower.RR and CI.upper.RR
  if(any(poisson.stats == "CI.RR")) {
    poisson.stats <- unique(c(poisson.stats[poisson.stats != "CI.RR"], "CI.lower.RR", "CI.upper.RR"))
  }
  if(any(poisson.stats == "CI.estimate")) {
    poisson.stats <- unique(c(poisson.stats[poisson.stats != "CI.estimate"], "CI.lower.estimate", "CI.upper.estimate"))
  }

  ##########################
  ## Negbin stats:
  ##########################

  negbin.stats.valid <- c(
    "RR", "CI.lower.RR", "CI.upper.RR", "p.value", "Nmiss", # default
    "CI.RR", "CI.estimate", "CI.lower.estimate", "CI.upper.estimate", "CI.RR", "Nmiss2", "std.error", "estimate", "statistic", "endpoint",
    "AIC", "BIC", "logLik", "dispersion", "null.deviance", "deviance", "df.residual", "df.null", "theta", "SE.theta", "p.value.lrt"
  )

  if(any(negbin.stats %nin% negbin.stats.valid)) {
    stop("Invalid poisson stats: ",
         paste(negbin.stats[negbin.stats %nin% negbin.stats.valid],collapse=","), "\n")
  }
  ## let CI.RR decode to CI.lower.RR and CI.upper.RR
  if(any(negbin.stats == "CI.RR")) {
    negbin.stats <- unique(c(negbin.stats[negbin.stats != "CI.RR"], "CI.lower.RR", "CI.upper.RR"))
  }
  if(any(negbin.stats == "CI.estimate")) {
    negbin.stats <- unique(c(negbin.stats[negbin.stats != "CI.estimate"], "CI.lower.estimate", "CI.upper.estimate"))
  }

  ##########################
  ## clog stats:
  ##########################
  clog.stats.valid <- c(
    "OR", "CI.lower.OR", "CI.upper.OR", "p.value", "concordance", "Nmiss", # default
    "CI.OR", "CI.estimate", "CI.lower.estimate", "CI.upper.estimate", "N", "Nmiss2", "estimate", "std.error", "endpoint", "Nevents", "statistic",
    "r.squared", "r.squared.max", "logLik", "AIC", "BIC", "statistic.log", "p.value.log", "statistic.sc", "p.value.sc",
    "statistic.wald", "p.value.wald", "N", "std.error.concordance", "p.value.lrt"
  )

  if(any(clog.stats %nin% clog.stats.valid)) {
    stop("Invalid clog stats: ",
         paste(clog.stats[clog.stats %nin% clog.stats.valid],collapse=","), "\n")
  }
  ## let CI.OR decode to CI.lower.OR and CI.upper.OR
  if(any(clog.stats == "CI.OR")) {
    clog.stats <- unique(c(clog.stats[clog.stats != "CI.OR"], "CI.lower.OR", "CI.upper.OR"))
  }
  if(any(clog.stats == "CI.estimate")) {
    clog.stats <- unique(c(clog.stats[clog.stats != "CI.estimate"], "CI.lower.estimate", "CI.upper.estimate"))
  }

  ##########################
  ## relrisk stats:
  ##########################
  ##(quasi)/poisson.stats=c("Nmiss","RR","CI.RR", "p.value","concordance"),
  ##Other coeff columns: CI.estimate, CI.RR  (ci for relrisk),N,Nmiss2, std.error, estimate, z.stat, endpoint
  ##Other model fits: AIC,BIC,logLik, dispersion
  ##  dispersion = deviance/df.residual
  relrisk.stats.valid <- c(
    "RR", "CI.lower.RR", "CI.upper.RR", "p.value", "Nmiss", # default
    "CI.RR", "CI.estimate", "CI.lower.estimate", "CI.upper.estimate", "CI.RR", "Nmiss2", "std.error", "estimate", "statistic", "endpoint",
    "AIC", "BIC", "logLik", "dispersion", "null.deviance", "deviance", "df.residual", "df.null"
  )

  if(any(relrisk.stats %nin% relrisk.stats.valid)) {
    stop("Invalid relrisk stats: ",
         paste(relrisk.stats[relrisk.stats %nin% relrisk.stats.valid],collapse=","), "\n")
  }
  ## let CI.RR decode to CI.lower.RR and CI.upper.RR
  if(any(relrisk.stats == "CI.RR")) {
    relrisk.stats <- unique(c(relrisk.stats[relrisk.stats != "CI.RR"], "CI.lower.RR", "CI.upper.RR"))
  }
  if(any(relrisk.stats == "CI.estimate")) {
    relrisk.stats <- unique(c(relrisk.stats[relrisk.stats != "CI.estimate"], "CI.lower.estimate", "CI.upper.estimate"))
  }

  ##########################
  ## Survival stats:
  ##########################
  ##surv.stats=c(Nmiss,HR,CI.HR,p.value,concorance)
  ##Other possible coefficient table columns: CI.estimate,N,Nmiss2,estimate,se,endpoint,Nevents,z.stat
  ##Other possible model fits: r.squared, logLik, AIC, BIC
  surv.stats.valid <- c(
    "HR", "CI.lower.HR", "CI.upper.HR", "p.value", "concordance", "Nmiss", # default
    "CI.HR", "CI.estimate", "CI.lower.estimate", "CI.upper.estimate", "N", "Nmiss2", "estimate", "std.error", "endpoint", "Nevents", "statistic",
    "r.squared", "r.squared.max", "logLik", "AIC", "BIC", "statistic.log", "p.value.log", "statistic.sc", "p.value.sc",
    "statistic.wald", "p.value.wald", "N", "std.error.concordance", "p.value.lrt"
  )

  if(any(survival.stats %nin% surv.stats.valid)) {
    stop("Invalid survival stats: ",
         paste(survival.stats[survival.stats %nin% surv.stats.valid], collapse=","), "\n")
  }

  ## let CI.HR decode to CI.lower.HR and CI.upper.HR
  if(any(survival.stats == "CI.HR")) {
    survival.stats <- unique(c(survival.stats[survival.stats != "CI.HR"], "CI.lower.HR", "CI.upper.HR"))
  }
  if(any(survival.stats == "CI.estimate")) {
    survival.stats <- unique(c(survival.stats[survival.stats != "CI.estimate"], "CI.lower.estimate", "CI.upper.estimate"))
  }
  list(digits=digits, digits.ratio=digits.ratio, digits.p = digits.p, format.p = format.p,
       show.adjust=show.adjust, show.intercept=show.intercept, conf.level=conf.level,
       ordinal.stats=ordinal.stats, binomial.stats=binomial.stats, gaussian.stats=gaussian.stats,
       poisson.stats=poisson.stats, negbin.stats = negbin.stats, clog.stats=clog.stats, relrisk.stats=relrisk.stats,
       survival.stats=survival.stats, stat.labels = stat.labels)
}

#' Control settings for \code{tableby} function
#'
#' Control test and summary settings for the \code{\link{tableby}} function.
#'
#' @param test logical, telling \code{tableby} whether to perform tests of x variables across levels of the group variable.
#' @param total logical, telling \code{tableby} whether to calculate a column of totals across group variable.
#' @param total.pos One of \code{"before"} or \code{"after"}, denoting where to put the total column relative to the by-variable columns.
#' @param test.pname character string denoting the p-value column name in \code{\link{summary.tableby}}.
#'   Modifiable also with \code{\link{modpval.tableby}}.
#' @param cat.simplify,ordered.simplify logical, tell \code{tableby} whether to remove the first level of the categorical/ordinal variable if binary.
#'   If \code{TRUE}, only the summary stats of the second level are reported (unless there's only one level, in which case it's reported).
#'   If \code{"label"}, the second level's label is additionally appended to the label.
#'   NOTE: this only simplifies to one line if there is only one statistic reported, such as \code{countpct}.
#'   In particular, if \code{Nmiss} is specified and there are missings, then the output is not simplified.
#' @param cat.droplevels Should levels be dropped for categorical variables? If set to true, p-values will not be displayed
#'   unless \code{test.always = TRUE} as well.
#' @param numeric.simplify,date.simplify logical, tell \code{tableby} whether to condense numeric/date output to a single line.
#'   NOTE: this only simplifies to one line if there is only one statistic reported, such as \code{meansd}.
#'   In particular, if \code{Nmiss} is specified and there are missings, then the output is not simplified.
#' @param numeric.test name of test for numeric RHS variables in \code{tableby}: anova, kwt (Kruskal-Wallis), medtest (median test).
#'   If no LHS variable exists, then a mean is required for a univariate test.
#' @param numeric.stats,cat.stats,ordered.stats,surv.stats,date.stats,selectall.stats summary statistics to include for the respective class of RHS variables
#'  within the levels of the group LHS variable.
#' @param cat.test name of test for categorical variables: chisq, fe (Fisher's Exact)
#' @param wilcox.correct,wilcox.exact See \code{\link[stats]{wilcox.test}}
#' @param chisq.correct logical, correction factor for chisq.test
#' @param simulate.p.value logical, simulate p-value for categorical tests (fe and chisq)
#' @param B number of simulations to perform for simulation-based p-value
#' @param ordered.test name of test for ordered variables: trend
#' @param surv.test name of test for survival variables: logrank
#' @param date.test name of test for date variables: kwt
#' @param selectall.test name of test for date variables: notest
#' @param stats.labels A named list of labels for all the statistics function names, where the function name is the named element in the list
#'   and the value that goes with it is a string containing the formal name that will be printed in all printed renderings of the output,
#'   e.g., \code{list(countpct="Count (Pct)")}. Any unnamed elements will be ignored. Passing \code{NULL} will disable labels.
#' @param digits Number of decimal places for numeric values.
#' @param digits.count Number of decimal places for count values.
#' @param digits.pct Number of decimal places for percents.
#' @param digits.p Number of decimal places for p-values.
#' @param format.p Logical, denoting whether to format p-values, or character, a \code{\link[glue]{glue}} specification for how to format. See "Details", below.
#' @param digits.n Number of decimal places for N's in the header. Set it to NA to suppress the N's.
#' @param conf.level Numeric, denoting what confidence level to use for confidence intervals.
#'   (See, e.g., \code{\link{binomCI}})
#' @param times A vector of times to use for survival summaries.
#' @param test.always Should the test be performed even if one or more by-group has 0 observations? Relevant
#'   for kwt and anova.
#' @param ... additional arguments.
#' @details
#' All tests can be turned off by setting \code{test} to FALSE.
#'   Otherwise, test are set to default settings in this list, or set explicitly in the formula of \code{tableby}.
#'
#' If \code{format.p} is \code{FALSE}, \code{digits.p} denotes the number of significant digits shown. The
#'   p-values will be in exponential notation if necessary. If \code{format.p} is \code{TRUE},
#'   \code{digits.p} will determine the number of digits after the decimal point to show. If the p-value
#'   is less than the resulting number of places, it will be formatted to show so. If \code{format.p} is a character string,
#'   it will be treated as a \code{\link[glue]{glue}} specification: the p-value is exposed as "p", and "digits.p" as "digits.p".
#'
#' Options for statistics are described more thoroughly in the vignette and are listed in \link{tableby.stats}
#'
#'
#' @return A list with settings to be used within the \code{tableby} function.
#'
#' @seealso \code{\link[stats]{anova}}, \code{\link[stats]{chisq.test}}, \code{\link{tableby}}, \code{\link{summary.tableby}},
#'   \code{\link{tableby.stats}}.
#' @author Jason Sinnwell, Beth Atkinson, Ethan Heinzen, Terry Therneau, adapted from SAS Macros written by Paul Novotny and Ryan Lennon
#' @examples
#' set.seed(100)
#' ## make 3+ categories for Response
#' mdat <- data.frame(Response=c(0,0,0,0,0,1,1,1,1,1),
#'                    Sex=sample(c("Male", "Female"), 10,replace=TRUE),
#'                    Age=round(rnorm(10,mean=40, sd=5)),
#'                    HtIn=round(rnorm(10,mean=65,sd=5)))
#'
#' ## allow default summaries in RHS variables, and pass control args to
#' ## main function, to be picked up with ... when calling tableby.control
#' outResp <- tableby(Response ~ Sex + Age + HtIn, data=mdat, total=FALSE, test=TRUE)
#' outCtl <- tableby(Response ~ Sex + Age + HtIn, data=mdat,
#'                   control=tableby.control(total=TRUE, cat.simplify=TRUE,
#'                   cat.stats=c("Nmiss","countpct"),digits=1))
#' summary(outResp, text=TRUE)
#' summary(outCtl, text=TRUE)
#' @export
tableby.control <- function(
  test=TRUE,total=TRUE, total.pos = c("after", "before"), test.pname=NULL,
  numeric.simplify=FALSE, cat.simplify=FALSE, cat.droplevels=FALSE, ordered.simplify=FALSE, date.simplify=FALSE,
  numeric.test="anova", cat.test="chisq", ordered.test="trend", surv.test="logrank", date.test="kwt", selectall.test="notest",
  test.always = FALSE,
  numeric.stats=c("Nmiss","meansd","range"), cat.stats=c("Nmiss","countpct"),
  ordered.stats=c("Nmiss", "countpct"), surv.stats=c("Nmiss", "Nevents","medSurv"), date.stats=c("Nmiss", "median","range"),
  selectall.stats=c("Nmiss", "countpct"),
  stats.labels = list(),
  digits = 3L, digits.count = 0L, digits.pct = 1L, digits.p = 3L, format.p = TRUE, digits.n = 0L, conf.level = 0.95,
  wilcox.correct = FALSE, wilcox.exact = NULL,
  chisq.correct=FALSE, simulate.p.value=FALSE, B=2000, times = 1:5, ...) {

  nm <- names(list(...))
  if("digits.test" %in% nm) .Deprecated(msg = "Using 'digits.test = ' is deprecated. Use 'digits.p = ' instead.")
  if("nsmall" %in% nm) .Deprecated(msg = "Using 'nsmall = ' is deprecated. Use 'digits = ' instead.")
  if("nsmall.pct" %in% nm) .Deprecated(msg = "Using 'nsmall.pct = ' is deprecated. Use 'digits.pct = ' instead.")

  ## validate digits
  # digits and digits.test are OK to be NULL. See ?format
  if(!is.null(digits) && digits < 0L)
  {
    warning("digits must be >= 0. Set to default.")
    digits <- 3L
  }
  if(!is.null(digits.count) && digits.count < 0L)
  {
    warning("digits.count must be >= 0. Set to default.")
    digits.count <- 0L
  }
  if(!is.null(digits.pct) && digits.pct < 0L)
  {
    warning("digits.pct must be >= 0. Set to default.")
    digits.pct <- 1L
  }
  if(!is.null(digits.p) && digits.p < 0L)
  {
    warning("digits.p must be >= 0. Set to default.")
    digits.p <- 3L
  }
  if(!is.null(digits.n) && !is.na(digits.n) && digits.p < 0L)
  {
    warning("digits.n must be >= 0 or NA or NULL. Set to default.")
    digits.n <- 0L
  }

  stats.labels <- if(is.null(stats.labels)) NULL else add_tbc_stats_labels(stats.labels)

  list(test=test, total=total, total.pos = match.arg(total.pos), test.pname=test.pname,
       numeric.simplify=numeric.simplify, cat.simplify=cat.simplify, cat.droplevels = cat.droplevels, ordered.simplify=ordered.simplify, date.simplify=date.simplify,
       numeric.test=numeric.test, cat.test=cat.test, ordered.test=ordered.test, surv.test=surv.test, date.test=date.test, selectall.test=selectall.test,
       test.always=test.always,
       numeric.stats=numeric.stats, cat.stats=cat.stats, ordered.stats=ordered.stats, surv.stats=surv.stats, date.stats=date.stats, selectall.stats=selectall.stats,
       stats.labels=stats.labels,
       digits=digits, digits.p=digits.p, digits.count = digits.count, digits.pct = digits.pct, format.p = format.p, digits.n = digits.n,
       conf.level=conf.level,
       wilcox.correct = wilcox.correct, wilcox.exact = wilcox.exact,
       chisq.correct=chisq.correct, simulate.p.value=simulate.p.value, B=B, times=times)
}

add_tbc_stats_labels <- function(x) {
  start <- list(
    Nmiss="N-Miss", Nmiss2="N-Miss", Nmisspct="N-Miss (%)", Nmisspct2="N-Miss (%)",
    meansd="Mean (SD)", meanse = "Mean (SE)", meanpmsd="Mean &pm; SD", meanpmse = "Mean &pm; SE", medianrange="Median (Range)",
    median="Median", medianq1q3="Median (Q1, Q3)", q1q3="Q1, Q3", iqr = "IQR",
    mean = "Mean", sd = "SD", var = "Var", max = "Max", min = "Min", meanCI = "Mean (CI)", sum = "Sum",
    gmean = "Geom Mean", gsd = "Geom SD", gmeansd = "Geom Mean (Geom SD)", gmeanCI = "Geom Mean (CI)",
    range="Range", Npct="N (%)", Nrowpct="N (%)", Nevents="Events", medSurv="Median Survival",
    medSurvQuant="Median (Q1, Q3) Survival", medSurvCI="Median (CI)",
    medTime = "Median Follow-Up", medianmad="Median (MAD)", Nsigntest = "N (sign test)",
    overall = "Overall", total = "Total", difference = "Difference"
  )
  nms <- setdiff(names(x), "")
  start[nms] <- x[nms]
  start
}

#' Defunct functions in \code{arsenal}
#'
#' Details about defunct functions in \code{arsenal}
#'
#' @param x,y See \code{\link{comparedf}}.
#' @param ... Other arguments.
#' @details
#' \code{comparison.control} was renamed to \code{\link{comparedf.control}} in version 3.0.0.
#'
#' \code{compare.data.frame} was renamed to \code{\link{comparedf}} in version 3.0.0.
#'
#' \code{length.tableby} was removed in version 2.0.0.
#'
#' \code{includeNA.character} and \code{includeNA.numeric} were removed in version 2.0.0 and
#'   replaced with a default method.
#'
#' \code{rangeTime} was removed in version 1.5.0.
#' @seealso \code{\link{arsenal-deprecated}}, \code{\link{comparedf}}
#' @name arsenal-defunct
NULL
#> NULL

#' The summary method for a \code{comparedf} object
#'
#' Print a more detailed output of the \code{\link{comparedf}} object.
#'
#' @param object An object of class \code{"comparedf"}, as made by the \code{\link{comparedf}} S3 method.
#' @param ... Other arguments passed to \code{\link{comparedf.control}}. In \code{print}, these are passed to \code{\link[knitr]{kable}}.
#' @param show.attrs Logical, denoting whether to show the actual attributes which are different. For (e.g.) factors with lots
#'   of levels, this can make the tables quite wide, so this feature is \code{FALSE} by default.
#' @param x An object returned by the \code{summary.comparedf} function.
#' @param format Passed to \code{\link[knitr]{kable}}: the format for the table. The default here is "pandoc".
#'   To use the default in \code{kable}, pass \code{NULL}.
#' @return An object of class \code{"summary.comparedf"} is returned.
#' @seealso \code{\link{comparedf}}, \code{\link{comparedf.control}}
#' @name summary.comparedf
NULL
#> NULL

#' @rdname summary.comparedf
#' @export
summary.comparedf <- function(object, ..., show.attrs = FALSE)
{
  control <- c(list(...), object$control)
  control <- do.call("comparedf.control", control[!duplicated(names(control))])

  #### start with summaries of the data.frames ####

  frame.summary <- as.data.frame(object$frame.summary[c("version", "arg", "ncol", "nrow")])

  #### after we've done all that, summaries of the overall comparison ####
  diffs.byvar <- diffs(object, by.var = TRUE)
  diffs.tab <- diffs(object)
  nobs.shared <- object$frame.summary$n.shared[1]
  nobs.uneq <- length(unique(diffs.tab$row.x))
  comparison.summary <- data.frame(
    statistic = c(
      "Number of by-variables",
      "Number of non-by variables in common", "Number of variables compared",
      "Number of variables in x but not y", "Number of variables in y but not x",
      "Number of variables compared with some values unequal", "Number of variables compared with all values equal",
      "Number of observations in common", "Number of observations in x but not y", "Number of observations in y but not x",
      "Number of observations with some compared variables unequal", "Number of observations with all compared variables equal",
      "Number of values unequal"
    ),
    value = c(
      (!attr(object$frame.summary$by, "byrow"))*sum(idx_var_sum(object, "by.variables")),
      sum(idx_var_sum(object, "nonby.vars.shared")), sum(idx_var_sum(object, "vars.compared")),
      sum(is.na(object$vars.summary$var.y)), sum(is.na(object$vars.summary$var.x)),
      sum(diffs.byvar$n > 0), sum(diffs.byvar$n == 0),
      nobs.shared, nrow(object$frame.summary$unique[[1]]), nrow(object$frame.summary$unique[[2]]),
      nobs.uneq, nobs.shared - nobs.uneq,
      n.diffs(object)
    ),
    stringsAsFactors = FALSE
  )

  #### start with differences in variables first ####
  get.vars.not.shared <- function(a, b)
  {
    var.diff.a <- object$vars.summary[is.na(object$vars.summary[[paste0("var.", b)]]), paste0(c("var.", "pos.", "class."), a)]
    var.diff.a2 <- cbind(version = rep(a, times = nrow(var.diff.a)), var.diff.a, stringsAsFactors = FALSE)
    colnames(var.diff.a2) <- c("version", "variable", "position", "class")
    var.diff.a2
  }

  vars.ns <- rbind(get.vars.not.shared("x", "y"), get.vars.not.shared("y", "x"))

  #### report variables not compared ####

  vars.nc <- as.data.frame(object$vars.summary[idx_var_sum(object, "vars.not.compared"),
                                               c("var.x", "pos.x", "class.x", "var.y", "pos.y", "class.y"), drop = FALSE])

  #### Now for the observations which aren't shared ####

  obs.ns <- diffs(object, what = "observations")

  #### Now for the actual differences ####

  # done in the structure statement

  #### Now for attributes ####

  attrs.tmp <- as.data.frame(object$vars.summary[idx_var_sum(object, "non.identical.attributes"), c("var.x", "var.y", "attrs"), drop = FALSE])
  attrs.diffs <- do.call(rbind, Map(cbind, var.x = attrs.tmp$var.x, var.y = attrs.tmp$var.y, attrs.tmp$attrs,
                                    MoreArgs = list(stringsAsFactors = FALSE)))
  if(is.null(attrs.diffs))
  {
    attrs.diffs <- data.frame(var.x = character(0), var.y = character(0), name = character(0), stringsAsFactors = FALSE)
  } else if(!show.attrs) attrs.diffs <- attrs.diffs[c("var.x", "var.y", "name")]

  structure(list(
    frame.summary.table = frame.summary,
    comparison.summary.table = comparison.summary,
    vars.ns.table = vars.ns, vars.nc.table = vars.nc, obs.table = obs.ns,
    diffs.byvar.table = diffs.byvar, diffs.table = diffs.tab,
    attrs.table = attrs.diffs,
    control = control
  ), class = "summary.comparedf")
}

#' @rdname summary.comparedf
#' @export
print.summary.comparedf <- function(x, ..., format = "pandoc")
{
  orig <- x
  sumdiffs <- sum(x$diffs.byvar.table$n)
  ctrl <- x$control
  ctrl$max.print.vars.ns <- ctrl$max.print.vars
  ctrl$max.print.vars.nc <- ctrl$max.print.vars

  if(is.null(ctrl$max.print.diffs.per.var) || is.na(ctrl$max.print.diffs.per.var)) ctrl$max.print.diffs.per.var <- sumdiffs
  if(nrow(x$diffs.table) > 0)
  {
    x$diffs.table <- do.call(rbind, by(x$diffs.table, factor(x$diffs.table$var.x, levels = unique(x$diffs.table$var.x)),
                                       utils::head, ctrl$max.print.diffs.per.var))

    # Need this for knitr to output list-cols of factors and dates correctly
    as_char <- function(x) if(is.factor(x) || is.Date(x)) x <- as.character(x) else x
    x$diffs.table$values.x <- lapply(x$diffs.table$values.x, as_char)
    x$diffs.table$values.y <- lapply(x$diffs.table$values.y, as_char)
  }

  for(v in c("frame.summary", "comparison.summary", "vars.ns", "vars.nc", "obs", "diffs.byvar", "diffs", "attrs"))
  {
    obj <- x[[paste0(v, ".table")]]
    nprint <- ctrl[[paste0("max.print.", v)]]

    # there is purposefully no max.print.frame.summary or max.print.comparison.summary
    if(is.null(nprint) || is.na(nprint)) nprint <- nrow(obj)

    caption <- switch(
      v,
      frame.summary = "Summary of data.frames",
      comparison.summary = "Summary of overall comparison",
      vars.ns = "Variables not shared",
      vars.nc = "Other variables not compared",
      obs = "Observations not shared",
      diffs.byvar = "Differences detected by variable",
      diffs = "Differences detected",
      attrs = "Non-identical attributes"
    )
    if(nrow(obj) > 0)
    {
      if(v == "diffs" && sumdiffs > min(nprint, nrow(obj)))
      {
        caption <- paste0(caption, " (", sumdiffs - min(nprint, nrow(obj)), " not shown)")
      } else if(nrow(obj) > nprint)
      {
        caption <- paste0(caption, " (", nrow(obj) - nprint, " not shown)")
      }
      print(knitr::kable(utils::head(obj, nprint), format = format, caption = caption, row.names = FALSE, ...))
    } else
    {
      nocaption <- paste0("No ", tolower(caption))
      print(knitr::kable(data.frame(x = nocaption), format = format, caption = caption, row.names = FALSE, col.names = "", ...))
    }
    cat("\n")
  }
  invisible(orig)
}

#' Helper functions for \code{write2}
#'
#' Helper functions for \code{\link{write2}}.
#'
#' @param ... For \code{verbatim}, objects to print out monospaced (as if in the terminal). For \code{code.chunk},
#'   either expressions or single character strings to paste into the code chunk.
#' @param chunk.opts A single character string giving the code chunk options. Make sure to specify the engine!
#' @details
#' The \code{"verbatim"} class is to tell \code{\link{write2}} to print the object inside
#'   a section surrounded by three back ticks. The results will look like it would in the terminal (monospaced).
#'
#' \code{code.chunk()} is to write explicit code chunks in the \code{.Rmd} file; it captures the call and writes it to the
#'   file, to execute upon knitting.
#' @name write2.internal
NULL
#> NULL

#' @export
print.verbatim <- function(x, ...)
{
  for(i in seq_along(x))
  {
    cat("```\n")
    print(x[[i]], ...)
    cat("\n```\n\n")
  }
  invisible(x)
}

#' @rdname write2.internal
#' @export
verbatim <- function(...)
{
  structure(list(...), class = "verbatim")
}

#' @rdname write2.internal
#' @export
code.chunk <- function(..., chunk.opts = "r")
{
  if(!is.character(chunk.opts) || length(chunk.opts) != 1) stop("'chunk.opts' should be a single character string")
  Call <- match.call()
  Call[[1]] <- NULL
  Call$chunk.opts <- NULL
  if(length(Call) == 0) Call[[1]] <- ""
  set_attr(set_attr(Call, "chunk.opts", chunk.opts), "class", c("code.chunk", class(Call)))
}

#' @export
print.code.chunk <- function(x, ...)
{
  cat("```{", attr(x, "chunk.opts"), "}\n", sep = "")
  lapply(x, function(elt) if(is.character(elt)) cat(elt, "\n", sep = "") else print(elt))
  cat("```\n\n")
}

#' formulize
#'
#' A shortcut to generate one-, two-, or many-sided formulas from vectors of variable names.
#'
#' @param y,x,... Character vectors, names, or calls to be collapsed (by \code{"+"}) and put left-to-right in the formula.
#'   If \code{data} is supplied, these can also be numeric, denoting which column name to use. See examples.
#' @param data An R object with non-null column names.
#' @param collapse How should terms be collapsed? Default is addition.
#' @param collapse.y How should the y-terms be collapsed? Default is addition. Also accepts the special string "list",
#'   which combines them into a multiple-left-hand-side formula, for use in other functions.
#' @param escape A logical indicating whether character vectors should be coerced to names (that is, whether names with spaces should
#'   be surrounded with backticks or not)
#' @seealso \code{\link[stats:delete.response]{reformulate}}
#' @author Ethan Heinzen
#' @examples
#' ## two-sided formula
#' f1 <- formulize("y", c("x1", "x2", "x3"))
#'
#' ## one-sided formula
#' f2 <- formulize(x = c("x1", "x2", "x3"))
#'
#' ## multi-sided formula
#' f3 <- formulize("y", c("x1", "x2", "x3"), c("z1", "z2"), "w1")
#'
#' ## can use numerics for column names
#' data(mockstudy)
#' f4 <- formulize(y = 1, x = 2:4, data = mockstudy)
#'
#' ## mix and match
#' f5 <- formulize(1, c("x1", "x2", "x3"), data = mockstudy)
#'
#' ## get an interaction
#' f6 <- formulize("y", c("x1*x2", "x3"))
#'
#' ## get only interactions
#' f7 <- formulize("y", c("x1", "x2", "x3"), collapse = "*")
#'
#' ## no intercept
#' f8 <- formulize("y", "x1 - 1")
#' f9 <- formulize("y", c("x1", "x2", "-1"))
#'
#' ## LHS as a list to use in arsenal functions
#' f10 <- formulize(c("y1", "y2", "y3"), c("x", "z"), collapse.y = "list")
#'
#' ## use in an lm
#' f11 <- formulize(2, 3:4, data = mockstudy)
#' summary(lm(f11, data = mockstudy))
#'
#' ## using non-syntactic names or calls (like reformulate example)
#' f12 <- formulize(as.name("+-"), c("`P/E`", "`% Growth`"))
#' f12 <- formulize("+-", c("P/E", "% Growth"), escape = TRUE)
#'
#' f <- Surv(ft, case) ~ a + b
#' f13 <- formulize(f[[2]], f[[3]])
#'
#' @export
formulize <- function(y = "", x, ..., data = NULL, collapse = "+", collapse.y = collapse, escape = FALSE)
{
  dots <- list(y = y, x = x, ...)
  if(!is.null(data))
  {
    if(is.null(colnames(data))) stop("colnames(data) is NULL")
    dots <- lapply(dots, function(elt, cn) if(is.numeric(elt)) lapply(cn[elt], as.name) else elt, cn = colnames(data))
  }
  name.or.call <- function(elt) is.name(elt) || is.call(elt)
  dots <- lapply(dots, function(elt) {
    if(name.or.call(elt)) return(list(elt))
    if(escape) {
      lapply(elt, function(elt2) if(identical(elt2, "")) "" else as.name(elt2))
    } else elt
  })
  is.ok <- function(x) is.character(x) || (is.list(x) && all(vapply(x, name.or.call, NA) | vapply(x, is.character, NA)))
  trash <- lapply(dots, function(elt) if(!is.ok(elt))
    stop("One or more argument isn't a character vector, numeric vector, list of names, or list of calls"))

  dots[[1]] <- if(collapse.y == "list")
  {
    paste0("list(", paste0(dots[[1]], collapse = ", "), ")")
  } else paste0(dots[[1]], collapse = collapse.y)

  elts <- vapply(dots, paste0, character(1), collapse = collapse)
  stats::as.formula(paste0(elts, collapse = " ~ "), env = parent.frame())
}

#' write2
#'
#' Functions to output tables to a single document. (Also the S3 backbone behind the \code{write2*} functions.)
#'
#' @param object An object.
#' @param file A single character string denoting the filename for the output document.
#' @param ... Additional arguments to be passed to \code{FUN}, \code{rmarkdown::render}, etc.
#'   One popular option is to use \code{quiet = TRUE} to suppress the command line output.
#' @param FUN The summary-like or print-like function to use to generate the markdown content. Can be passed as a function or a
#'   character string. It's expected that \code{FUN(object, ...)} looks "good" when put directly in a \code{.Rmd} file.
#' @param append. Logical, denoting whether (if a temporary \code{.Rmd} file of the same name already exists)
#'   to append on. Used mostly for \code{write2.list}.
#' @param render. Logical, denoting whether to render the temporary \code{.Rmd} file. Used mostly for \code{write2.list}.
#' @param keep.rmd Logical, denoting whether to keep the intermediate \code{.Rmd} file. Used mostly for \code{write2.list}.
#' @param output_format One of the following:
#'  \enumerate{
#'    \item{An output format object, e.g. \code{rmarkdown::\link[rmarkdown]{html_document}(...)}.}
#'    \item{A character string denoting such a format function, e.g. \code{"html_document"}. In this case, the \code{"..."} are NOT passed.}
#'    \item{The format function itself, e.g. \code{rmarkdown::html_document}. In this case, the \code{"..."} arguments are passed.}
#'    \item{One of \code{"html"}, \code{"pdf"}, and \code{"word"}, shortcuts implemented here. In this case, the \code{"..."} arguments are passed.}
#'    \item{\code{NULL}, in which the output is HTML by default.}
#'  }
#'  See \code{rmarkdown::\link[rmarkdown]{render}} for details.
#' @return \code{object} is returned invisibly, and \code{file} is written.
#' @details \code{write2} is an S3 method. The default prints the object (using \code{\link{print}})
#'    inside a section surrounded by three back ticks. See \code{\link{verbatim}} for details.
#'
#'    There are methods implemented for \code{\link{tableby}}, \code{\link{modelsum}}, and \code{\link{freqlist}}, all of which use the
#'    \code{summary} function. There are also methods compatible with \code{\link[knitr]{kable}}, \code{\link[xtable]{xtable}},
#'    and \code{\link[pander]{pander_return}}. Another option is to coerce an object using \code{\link{verbatim}()} to print out the
#'    results monospaced (as if they were in the terminal). To output multiple tables into a document, simply make a list of them
#'    and call the same function as before. Finally, to output code chunks to be evaluated, use \code{\link{code.chunk}}.
#'
#'    For more information, see \code{vignette("write2")}.
#' @seealso \code{\link{write2word}}, \code{\link{write2pdf}}, \code{\link{write2html}},
#'   \code{\link[rmarkdown]{render}}, \code{\link[rmarkdown]{word_document}}, \code{\link[rmarkdown]{html_document}}, \code{\link[rmarkdown]{pdf_document}},
#'   \code{\link[rmarkdown]{rtf_document}}, \code{\link[rmarkdown]{md_document}}, \code{\link[rmarkdown]{odt_document}}
#' @examples
#' \dontrun{
#' data(mockstudy)
#' # tableby example
#' tab1 <- tableby(arm ~ sex + age, data=mockstudy)
#' write2(tab1, tempfile(fileext = ".rtf"),
#'   toc = TRUE, # passed to rmarkdown::rtf_document, though in this case it's not practical
#'   quiet = TRUE, # passed to rmarkdown::render
#'   title = "My cool new title", # passed to summary.tableby
#'   output_format = rmarkdown::rtf_document)
#'
#' write2html(list(
#'   "# Header 1", # a header
#'   code.chunk(a <- 1, b <- 2, a + b), # a code chunk
#'   verbatim("hi there") # verbatim output
#' ),
#'   tempfile(fileext = ".html"),
#'   quite = TRUE)
#' }
#' @author Ethan Heinzen, adapted from code from Krista Goergen
#' @name write2
NULL
#> NULL


#' @rdname write2
#' @export
write2 <- function(object, file, ..., output_format)
{
  UseMethod("write2")
}

write2_using_summary <- function(object, file, ..., output_format = NULL)
{
  write2.default(object = object, file = file, FUN = summary, ..., output_format = output_format)
}

write2_using_print <- function(object, file, ..., output_format = NULL)
{
  write2.default(object = object, file = file, FUN = print, ..., output_format = output_format)
}

############################ write2 for arsenal objects ############################

#' @rdname write2
#' @export
write2.arsenal_table <- write2_using_summary

#' @rdname write2
#' @export
write2.summary.arsenal_table <- write2_using_print

#' @rdname write2
#' @export
write2.comparedf <- write2_using_summary

#' @rdname write2
#' @export
write2.summary.comparedf <- write2_using_print

#' @rdname write2
#' @export
write2.verbatim <- write2_using_print

#' @rdname write2
#' @export
write2.yaml <- write2_using_print

#' @rdname write2
#' @export
write2.code.chunk <- write2_using_print

############################ write2 for external objects ############################

#' @rdname write2
#' @export
write2.knitr_kable <- write2_using_print

#' @rdname write2
#' @export
write2.xtable <- write2_using_print

#' @rdname write2
#' @export
## this intended for use with pander_return()
write2.character <- function(object, file, ..., output_format = NULL)
{
  write2.default(object = object, file = file, FUN = cat, ..., sep = "\n", output_format = output_format)
}

############################ write2 for lists of objects ############################

#' @rdname write2
#' @export
write2.list <- function(object, file, ..., append. = FALSE, render. = TRUE, keep.rmd = !render., output_format = NULL)
{
  if(!is.character(file) || length(file) > 1) stop("'file' argument must be a single character string.")
  if(!is.logical(append.) || length(append.) > 1) stop("'append.' argument must be a single logical value.")

  filename <- paste0(file, ".Rmd")
  if(!append. || !file.exists(filename)) file.create(filename)

  # find any YAML specifications
  idx <- vapply(object, is.yaml, NA)
  if(any(idx))
  {
    yamls <- Reduce(c, object[idx])
    object <- object[!idx]
    write2(yamls, file = file, ..., keep.rmd = TRUE, append. = TRUE, render. = FALSE, output_format = output_format)
  }

  # separate the tables with a few blank lines, leading with the blank lines
  object2 <- c(object, as.list(rep("\n\n", times = length(object))))[order(c(seq_along(object), seq_along(object) - 0.5))]

  lapply(object2, write2, file = file, ..., keep.rmd = TRUE, append. = TRUE, render. = FALSE, output_format = output_format)

  write2("\n", file = file, ..., render. = render., append. = TRUE, keep.rmd = keep.rmd, output_format = output_format)

  invisible(object)
}

#' freqlist
#'
#' Approximate the output from SAS's \code{PROC FREQ} procedure when using the \code{/list} option of the \code{TABLE} statement.
#'
#' @param object An R object, usually of class \code{"table"} or class \code{"xtabs"}
#' @param na.options a character string indicating how to handling missing values: \code{"include"}
#'   (include values with NAs in counts and percentages),
#'   \code{"showexclude"} (show NAs but exclude from cumulative counts and all percentages),
#'   \code{"remove"} (remove values with NAs); default is \code{"include"}.
#' @param strata (formerly \code{groupBy}) an optional character string specifying a variable(s) to use for grouping when calculating cumulative
#'   counts and percentages. \code{\link{summary.freqlist}} will also separate by grouping variable for printing. Note that this is different
#'   from \code{modelsum} and \code{tableby}, which take bare column names (and only one, at that!)
#' @param labelTranslations an optional character string (or list) of labels to use for variable levels when summarizing.
#'   Names will be matched appropriately.
#' @param control control parameters to handle optional settings within \code{freqlist}. See \code{\link{freq.control}}
#' @param ... additional arguments. In the formula method, these are passed to the table method. These are also passed to
#'   \code{\link{freq.control}}
#' @param formula,data,subset,na.action,addNA,exclude,drop.unused.levels Arguments passed to \code{\link[stats]{xtabs}}.
#' @return An object of class \code{c("freqlist", "arsenal_table")}
#' @seealso \code{\link{arsenal_table}}, \code{\link{summary.freqlist}}, \code{\link{freq.control}}, \code{\link{freqlist.internal}},
#'   \code{\link[base]{table}}, \code{\link[stats]{xtabs}}
#'
#' @examples
#' # load mockstudy data
#' data(mockstudy)
#' tab.ex <- table(mockstudy[c("arm", "sex", "mdquality.s")], useNA = "ifany")
#' noby <- freqlist(tab.ex, na.options = "include")
#' summary(noby)
#'
#' # show the top 6 rows' frequencies and percents
#' head(summary(sort(noby, decreasing = TRUE)[c(1:4, 6)]))
#'
#' withby <- freqlist(tab.ex, strata = c("arm","sex"), na.options = "showexclude")
#' summary(withby)
#' @author Tina Gunderson, with revisions by Ethan Heinzen
#' @name freqlist
NULL
#> NULL

#' @rdname freqlist
#' @export
freqlist <- function(object, ...)
{
  UseMethod("freqlist")
}

#' @rdname freqlist
#' @export
freqlist.table <- function(object, na.options = c("include", "showexclude", "remove"), strata = NULL, labelTranslations = NULL, control = NULL, ...)
{
  control <- c(list(...), control)
  control <- do.call("freq.control", control[!duplicated(names(control))])

  Call <- match.call()
  na.options <- match.arg(na.options)
  if(min(dim(object)) < 1) stop("table object has dimension of 0")
  if("groupBy" %in% names(list(...)))
  {
    if(is.null(strata)) strata <- list(...)$groupBy
    .Deprecated(msg = "Using 'groupBy = ' is deprecated. Use 'strata = ' instead.")
  }

  hasStrata <- !is.null(strata)
  if(hasStrata && any(strata %nin% names(dimnames(object)))) stop("strata variable not found in table names")

  # all this just to keep non-syntactic names
  to_df <- function(x)
  {
    data.frame(
      do.call("expand.grid", c(dimnames(provideDimnames(x, sep = "", base = list(LETTERS))),
                               KEEP.OUT.ATTRS = FALSE, stringsAsFactors = TRUE)),
      Freq = c(x), row.names = NULL, check.names = FALSE
    )
  }

  tab.freq <- to_df(object)

  if(hasStrata && is.null(names(dimnames(object))) && is.null(names(labelTranslations)))
  {
    if(length(labelTranslations) != ncol(tab.freq) - 1) stop("'labelTranslations' doesn't appear to be the same length as 'object'")
    names(labelTranslations) <- utils::head(names(tab.freq), -1)
  }

  #### x variables (which might include strata) ####
  xTerms <- lapply(utils::head(names(tab.freq), -1), function(nm) list(variable=nm, label=nm, term=nm))
  names(xTerms) <- vapply(xTerms, "[[", NA_character_, "variable")

  #if a grouping factor is given, will add NA as a factor level so it is not dropped when using the by function
  if(hasStrata) {
    if(na.options != 'remove') tab.freq[strata] <- lapply(tab.freq[strata], function(x) if(anyNA(x)) addNA(x) else x)

    tableout <- unclass(by(tab.freq, tab.freq[rev(strata)], FUN = internalTable, na.options = na.options))
    tableout <- lapply(tableout, function(x) {
      x <- x[c(strata, colnames(x)[colnames(x) %nin% strata])]
      row.names(x) <- NULL
      x
    })
    xTerms <- xTerms[c(strata, names(xTerms)[names(xTerms) %nin% strata])]

    strata.levels <- ""
    strataLabel <- strata

  } else {
    tableout <- list(internalTable(tab.freq, na.options = na.options))
    strata <- strata.levels <- strataLabel <- ""
  }

  out.tables = list(
    list(
      y = list(term = "", label = ""),
      strata = list(term = strata, values = strata.levels, label = strataLabel, hasStrata = hasStrata),
      x = add_freqlist_xterms(xTerms),
      tables = unname(tableout),
      hasWeights = FALSE,
      na.options = na.options
    )
  )

  out <- structure(list(Call = Call, control = control, tables = out.tables), class = c("freqlist", "arsenal_table"))
  if(!is.null(labelTranslations)) labels(out) <- labelTranslations
  out
}

#' @rdname freqlist
#' @export
freqlist.formula <- function(formula, data, subset, na.action, na.options = c("include", "showexclude", "remove"),
                             strata = NULL, labelTranslations = NULL, control = NULL,
                             addNA, exclude, drop.unused.levels, ...)
{
  control <- c(list(...), control)
  control <- do.call("freq.control", control[!duplicated(names(control))])

  Call <- match.call()
  na.options <- match.arg(na.options)

  if("groupBy" %in% names(list(...)))
  {
    if(is.null(strata)) strata <- list(...)$groupBy
    .Deprecated(msg = "Using 'groupBy = ' is deprecated. Use 'strata = ' instead.")
  }

  indx <- match(c("formula", "data", "subset", "na.action", "addNA", "exclude", "drop.unused.levels"), names(Call), nomatch = 0)
  if(indx[1] == 0) stop("A formula argument is required.")
  formula.list <- as_list_formula(formula)
  out.tables <- list()
  for(FORM in formula.list)
  {
    temp.call <- Call[c(1, indx[1:4])]
    temp.call[[1L]] <- quote(stats::model.frame)
    temp.call$formula <- FORM

    if(!missing(data)) temp.call$data <- as.call(list(keep.labels, temp.call$data))
    modeldf <- loosen.labels(eval(temp.call, parent.frame()))
    Terms <- stats::terms(modeldf)
    hasStrata <- !is.null(strata)
    if(hasStrata && any(strata %nin% names(modeldf))) stop("strata variable not found in table names")

    #### Check for strata ####
    if(hasStrata)
    {
      strata.levels <- ""
      strata.terms <- strata
      strataLabel <- unname(vapply(strata, function(x) if(is.null(labelEff <- attr(modeldf[[x]], "label"))) x else labelEff, NA_character_))
    } else strata.terms <- strata.levels <- strataLabel <- ""

    strataList <- list(term = strata.terms, values = strata.levels, label = strataLabel, hasStrata = hasStrata)

    #### Check for weights ####

    if(hasWeights <- attributes(Terms)$response != 0)
    {
      termBy <- names(modeldf)[1]
      if(is.null(labelBy <- attr(modeldf[[1]], "label"))) labelBy <- termBy
      yList <- list(term = termBy, label = labelBy)
      modeldf[[1]] <- NULL
    } else yList <- list(term = "", label = "")

    #### x variables (which might include strata) ####
    xTerms <- Map(modeldf, names(modeldf), f = function(col, nm) {
      if(is.null(labelEff <- attr(col, "label"))) labelEff <- nm
      list(variable=nm, label=labelEff, term=nm)
    })
    names(xTerms) <- vapply(xTerms, "[[", NA_character_, "variable")
    if(hasStrata) xTerms <- xTerms[c(strata, names(xTerms)[names(xTerms) %nin% strata])]

    ####

    temp.call <- Call[c(1, indx)]
    temp.call[[1L]] <- quote(stats::xtabs)
    temp.call$formula <- FORM
    if(indx[5] == 0) temp.call$addNA <- TRUE

    tab <- freqlist(eval(temp.call, parent.frame()), strata = strata, na.options = na.options, ...)$tables[[1]]
    tab$hasWeights <- hasWeights
    tab$y <- yList
    tab$strata <- strataList
    tab$x <- add_freqlist_xterms(xTerms)

    # this should still work even if there's multiple LHS -- test that with list(, , y) ~ x
    out.tables[[yList$term]] <- tab
  }

  out <- structure(list(Call = Call, control = control, tables = out.tables), class = c("freqlist", "arsenal_table"))
  if(!is.null(labelTranslations)) labels(out) <- labelTranslations
  out
}

tweakcolnames <- function(by.x, by.y, cn.x, cn.y, control)
{
  if(any(by.x %in% setdiff(cn.y, by.y)))
  {
    stop("A by-variable for x appears in non-by-variables for y. base::merge() will not work in this function.")
  }
  if(anyDuplicated(cn.x) || anyDuplicated(cn.y)) stop("Sorry, there are duplicate colnames.")

  cn.y[match(by.y, cn.y)] <- by <- by.x
  rm(by.x, by.y) # just to make sure we don't use it again

  rn <- "..row.names.." %in% by

  # don't replace the characters for "..row.names.."
  if(rn)
  {
    if(cn.x[length(cn.x)] != "..row.names.." || cn.y[length(cn.y)] != "..row.names..")
    {
      stop("Something went wrong with the row.names.")
    }
    cn.x <- cn.x[- length(cn.x)]
    cn.y <- cn.y[- length(cn.y)]
  }

  tv <- control$tol.vars
  if(!is.null(names(tv)))
  {
    for(i in seq_along(tv))
    {
      if(bad1 <- names(tv)[i] %nin% cn.x)
        warning("Variable tolerance '", names(tv)[i], "' not found in colnames of x")
      if(bad2 <- tv[i] %nin% cn.y)
        warning("Variable tolerance '", tv[i], "' not found in colnames of y")
      if(!bad1 && !bad2) cn.y[cn.y == tv[i]] <- names(tv)[i]
    }
    # no need to do anything with the by-variables, since we've already set those to x
  } else if("none" %nin% tv)
  {
    for(elt in strsplit(tv, "", fixed = TRUE))
    {
      if(length(elt) == 1)
      {
        cn.x <- gsub(elt, "", cn.x, fixed = TRUE)
        cn.y <- gsub(elt, "", cn.y, fixed = TRUE)
        if(!rn) by <- gsub(elt, "", by, fixed = TRUE)
      } else
      {
        for(elt2 in elt[-1])
        {
          # I know I could do some fancy regex work here, but for readibility, the for-loop is easiest.
          cn.x <- gsub(elt2, elt[1], cn.x, fixed = TRUE)
          cn.y <- gsub(elt2, elt[1], cn.y, fixed = TRUE)
          if(!rn) by <- gsub(elt2, elt[1], by, fixed = TRUE)
        }
      }
    }
    if(anyDuplicated(cn.x) || anyDuplicated(cn.y)) stop("'tol.vars' resulted in duplicate colnames.")
  }

  if(rn)
  {
    cn.x <- c(cn.x, "..row.names..")
    cn.y <- c(cn.y, "..row.names..")
  }

  return(list(by = by, cn.x = cn.x, cn.y = cn.y))
}

cleanup.null.na <- function(x) if(is.null(x) || allNA(x)) NA_character_ else x

compare_values <- function(i, v, df, byvars, contr)
{
  if(is.na(v$var.x[i]) || is.na(v$var.y[i])) return(NULL)
  if(v$tmp[i] %in% byvars) return("by-variable")

  var <- v$tmp[i]
  var.x <- df[[paste0(var, ".x")]]
  var.y <- df[[paste0(var, ".y")]]

  int.num <- function(vr) is.integer(vr) || is.numeric(vr)
  fac.chr <- function(vr) is.factor(vr)  || is.character(vr)

  if(length(intersect(v$class.x[[i]], v$class.y[[i]])) == 0 &&
     !(contr$int.as.num && int.num(var.x) && int.num(var.y)) &&
     !(contr$factor.as.char && fac.chr(var.x) && fac.chr(var.y))) return("Not compared")

  ## Technically, we can compare these as-is, but let's be explicit about it.
  if(!identical(v$class.x[i], v$class.y[i]) && contr$int.as.num && int.num(var.x) && int.num(var.y))
  {
    var.x <- as.numeric(var.x)
    var.y <- as.numeric(var.y)
  }

  if(!identical(v$class.x[i], v$class.y[i]) && contr$factor.as.char && fac.chr(var.x) && fac.chr(var.y))
  {
    var.x <- as.character(var.x)
    var.y <- as.character(var.y)
  }

  find_tol <- function(whch)
  {
    opts <- contr[[whch]]
    if(v$var.x[i] %in% names(opts)) return(opts[[v$var.x[i]]])
    if(is.null(names(opts))) return(opts[[1]])
    if("" %nin% names(opts)) stop("No default tolerance specified for ", whch)
    opts[[which(names(opts) == "")]]
  }

  if(is.logical(var.x) && is.logical(var.y))
  {
    idx <- find_tol("tol.logical")(var.x, var.y)
  } else if(is.numeric(var.x) && is.numeric(var.y)) # this covers integers, too
  {
    idx <- find_tol("tol.num")(var.x, var.y, contr$tol.num.val)
  } else if(is.factor(var.x) && is.factor(var.y))
  {
    idx <- find_tol("tol.factor")(var.x, var.y)

  } else if(is.character(var.x) && is.character(var.y))
  {
    idx <- find_tol("tol.char")(var.x, var.y)
  } else if(is.Date(var.x) && is.Date(var.y))
  {
    idx <- find_tol("tol.date")(var.x, var.y, contr$tol.date.val)
  } else
  {
    idx <- find_tol("tol.other")(var.x, var.y)
  }

  out <- data.frame(values.x = I(var.x[idx]), # just in case list-column
                    values.y = I(var.y[idx]),
                    row.x = df[["..row.x.."]][idx],
                    row.y = df[["..row.y.."]][idx])

  return(cbind(df[idx, byvars, drop = FALSE], out))
}

compare_attrs <- function(i, v, x_, y_)
{
  if(is.na(v$var.x[i]) || is.na(v$var.y[i])) return(NULL)
  attr.x <- attributes(x_[[v$var.x[i]]])
  attr.y <- attributes(y_[[v$var.y[i]]])
  if(is.null(attr.x) && is.null(attr.y)) return(NULL)

  empty <- data.frame(name = character(0), attr = I(list()), stringsAsFactors = FALSE)

  out <- merge(if(!is.null(attr.x)) data.frame(name = names(attr.x), attr = I(attr.x), stringsAsFactors = FALSE) else empty,
               if(!is.null(attr.y)) data.frame(name = names(attr.y), attr = I(attr.y), stringsAsFactors = FALSE) else empty,
               by = "name", all = TRUE)
  out$attr.x <- lapply(out$attr.x, cleanup.null.na)
  out$attr.y <- lapply(out$attr.y, cleanup.null.na)

  if(nrow(out) > 0)
  {
    out <- out[order(out$name), , drop = FALSE]
    out <- out[vapply(seq_len(nrow(out)), function(i) !identical(out$attr.x[[i]], out$attr.y[[i]]), logical(1)), , drop = FALSE]
  }
  out
}


####################################################################################################
####################################################################################################
####################################################################################################

idx_var_sum <- function(object, which = c("vars.not.shared", "nonby.vars.shared", "vars.compared", "vars.not.compared",
                                          "differences.found", "non.identical.attributes", "by.variables"))
{
  which <- match.arg(which, several.ok = FALSE)
  if(which == "vars.not.shared")
  {
    vapply(object$vars.summary$values, is.null, logical(1))
  } else if(which == "nonby.vars.shared")
  {
    vapply(object$vars.summary$values, function(elt) !is.null(elt) && !identical(elt, "by-variable"), logical(1))
  } else if(which == "vars.not.compared")
  {
    vapply(object$vars.summary$values, identical, logical(1), y = "Not compared")
  } else if(which == "by.variables")
  {
    vapply(object$vars.summary$values, identical, logical(1), y = "by-variable")
  } else if(which == "vars.compared")
  {
    vapply(object$vars.summary$values, function(elt) is.data.frame(elt), logical(1))
  } else if(which == "differences.found")
  {
    vapply(object$vars.summary$values, function(elt) is.data.frame(elt) && nrow(elt) > 0, logical(1))
  } else if(which == "non.identical.attributes")
  {
    vapply(object$vars.summary$attrs, function(elt) is.data.frame(elt) && nrow(elt) > 0, logical(1))
  }
}

#' Extract differences
#'
#' Extract differences (\code{diffs()}), number of differences (\code{n.diffs()}),
#' or number of not-shared observations (\code{n.diff.obs()}) from a \code{comparedf} object.
#'
#' @param object An object of class \code{comparedf} or \code{summary.comparedf}.
#' @param what Should differences or the not-shared observations be returned?
#' @param vars A character vector of variable names to subset the results to.
#' @param ... Other arguments (not in use at this time).
#' @param by.var Logical: should the number of differences by variable be reported, or should
#'   all differences be reported (the default).
#' @author Ethan Heinzen
#' @seealso \code{\link{comparedf}} \code{\link{summary.comparedf}}
#' @name diffs
NULL
#> NULL


#' @rdname diffs
#' @export
n.diff.obs <- function(object, ...)
{
  UseMethod("n.diff.obs")
}

#' @rdname diffs
#' @export
n.diff.obs.comparedf <- function(object, ...)
{
  nrow(object$frame.summary$unique[[1]]) + nrow(object$frame.summary$unique[[2]])
}

#' @rdname diffs
#' @export
n.diff.obs.summary.comparedf <- function(object, ...)
{
  nrow(object$obs.table)
}

#' @rdname diffs
#' @export
n.diffs <- function(object, ...)
{
  UseMethod("n.diffs")
}

#' @rdname diffs
#' @export
n.diffs.comparedf <- function(object, ...)
{
  sum(vapply(object$vars.summary$values, function(elt) if(is.data.frame(elt)) nrow(elt) else 0, numeric(1)))
}

#' @rdname diffs
#' @export
n.diffs.summary.comparedf <- function(object, ...)
{
  nrow(object$diffs.table)
}

#' @rdname diffs
#' @export
diffs <- function(object, ...)
{
  UseMethod("diffs")
}

#' @rdname diffs
#' @export
diffs.comparedf <- function(object, what = c("differences", "observations"), vars = NULL, ..., by.var = FALSE)
{
  what <- match.arg(what)
  if(what == "observations")
  {
    get.obs.not.shared <- function(n)
    {
      obs.ns <- object$frame.summary$unique[[n]]
      cbind(version = rep(object$frame.summary$version[[n]], times = nrow(obs.ns)), obs.ns, stringsAsFactors = FALSE)
    }
    return(rbind(get.obs.not.shared(1), get.obs.not.shared(2)))
  }

  if(!is.logical(by.var) || length(by.var) != 1) stop("'by.var' must be a single logical value.")
  diffs <- as.data.frame(object$vars.summary[idx_var_sum(object, "vars.compared"), c("var.x", "var.y", "values")])
  diffs$n <- vapply(diffs$values, nrow, numeric(1))
  sumNA <- function(df) sum(is.na(df$values.x) | is.na(df$values.y))
  diffs$NAs <- vapply(diffs$values, sumNA, numeric(1))

  if(is.null(vars)) vars <- unique(c(diffs$var.x, diffs$var.y)) else if(!is.character(vars)) stop("'vars' should be NULL or a character vector.")

  rownames(diffs) <- NULL
  if(by.var) return(diffs[diffs$var.x %in% vars | diffs$var.y %in% vars, c("var.x", "var.y", "n", "NAs"), drop = FALSE])

  tolist <- function(df)
  {
    df$values.x <- I(as.list(df$values.x)) # need the I() for factors and dates to show up right
    df$values.y <- I(as.list(df$values.y))
    df
  }

  diffs1 <- diffs[diffs$n > 0, , drop = FALSE]
  if(nrow(diffs1) > 0)
  {
    diffs.table <- do.call(rbind, lapply(Map(cbind, var.x = diffs1$var.x, var.y = diffs1$var.y, diffs1$values,
                                             MoreArgs = list(stringsAsFactors = FALSE)), tolist))
  } else
  {
    diffs.table <- data.frame(var.x = character(0), var.y = character(0), stringsAsFactors = FALSE)
    diffs.table[object$frame.summary$by[[1]]] <- rep(list(character(0)), length(object$frame.summary$by[[1]]))
    diffs.table$values.y <- diffs.table$values.x <- I(list())
    diffs.table$row.y <- diffs.table$row.x <- integer(0)
  }

  rownames(diffs.table) <- NULL
  diffs.table[diffs.table$var.x %in% vars | diffs.table$var.y %in% vars, , drop = FALSE]
}

#' @rdname diffs
#' @export
diffs.summary.comparedf <- function(object, what = c("differences", "observations"), vars = NULL, ..., by.var = FALSE)
{
  what <- match.arg(what)
  if(what == "observations") return(object$obs.table)

  tmp <- if(by.var) object$diffs.byvar.table else object$diffs.table

  if(is.null(vars)) vars <- unique(tmp$var.x, tmp$var.y) else if(!is.character(vars)) stop("'vars' should be NULL or a character vector.")

  tmp[tmp$var.x %in% vars | tmp$var.y %in% vars, , drop = FALSE]
}

## Tests for formulize


context("Testing the comparedf output")

data(mockstudy)
mockstudy2 <- muck_up_mockstudy()

# a far simpler example

df1 <- data.frame(id = paste0("person", 1:3), a = c("a", "b", "c"), b = c(1, 3, 4), c = c("f", "e", "d"),
                  row.names = paste0("rn", 1:3), stringsAsFactors = FALSE)
df2 <- data.frame(id = paste0("person", 3:1), a = c("c", "b", "a"), b = c(1, 3, 4), d = paste0("rn", 1:3),
                  row.names = paste0("rn", c(1,3,2)), stringsAsFactors = FALSE)


###########################################################################################################
#### Just test that it ran right
###########################################################################################################

test_that("Basic comparison works: by row", {
  expect_identical(
    capture.output(comparedf(df1, df2)),
    c("Compare Object"                                     ,
      ""                                                   ,
      "Function Call: "                                    ,
      "comparedf(x = df1, y = df2)"                        ,
      ""                                                   ,
      "Shared: 3 non-by variables and 3 observations."     ,
      "Not shared: 2 variables and 0 observations."        ,
      ""                                                   ,
      "Differences found in 2/3 variables compared."       ,
      "0 variables compared have non-identical attributes."
    )
  )
  expect_true(n.diffs(comparedf(df1, df1)) == 0)
})

test_that("Basic comparison works: by id", {
  expect_identical(
    capture.output(comparedf(df1, df2, by = "id")),
    c("Compare Object"                                     ,
      ""                                                   ,
      "Function Call: "                                    ,
      "comparedf(x = df1, y = df2, by = \"id\")"           ,
      ""                                                   ,
      "Shared: 2 non-by variables and 3 observations."     ,
      "Not shared: 2 variables and 0 observations."        ,
      ""                                                   ,
      "Differences found in 1/2 variables compared."       ,
      "0 variables compared have non-identical attributes."
    )
  )
  expect_true(n.diffs(comparedf(df1, df1, by = "id")) == 0)
})

test_that("Basic comparison works: by row.names", {
  expect_identical(
    capture.output(comparedf(df1, df2, by = "row.names")),
    c("Compare Object"                                          ,
      ""                                                        ,
      "Function Call: "                                         ,
      "comparedf(x = df1, y = df2, by = \"row.names\")"         ,
      ""                                                        ,
      "Shared: 3 non-by variables and 3 observations."          ,
      "Not shared: 2 variables and 0 observations."             ,
      ""                                                        ,
      "Differences found in 3/3 variables compared."            ,
      "0 variables compared have non-identical attributes."
    )
  )
  expect_true(n.diffs(comparedf(df1, df1, by = "row.names")) == 0)
})

test_that("Basic comparison works: by row.names for x and something else for y", {
  expect_identical(
    capture.output(comparedf(df1, df2, by.x = "row.names", by.y = "d")),
    c("Compare Object"                                                          ,
      ""                                                                        ,
      "Function Call: "                                                         ,
      "comparedf(x = df1, y = df2, by.x = \"row.names\", by.y = \"d\")"         ,
      ""                                                                        ,
      "Shared: 3 non-by variables and 3 observations."                          ,
      "Not shared: 1 variables and 0 observations."                             ,
      ""                                                                        ,
      "Differences found in 2/3 variables compared."                            ,
      "0 variables compared have non-identical attributes."
    )
  )
})

df1$listcol <- list(1:3, "hi there", FALSE)
df2$listcol <- list(FALSE, "bye now", 1:2)

test_that("List-column comparison works: by id", {
  expect_identical(
    capture.output(comparedf(df1, df2, by = "id")),
    c("Compare Object"                                     ,
      ""                                                   ,
      "Function Call: "                                    ,
      "comparedf(x = df1, y = df2, by = \"id\")"           ,
      ""                                                   ,
      "Shared: 3 non-by variables and 3 observations."     ,
      "Not shared: 2 variables and 0 observations."        ,
      ""                                                   ,
      "Differences found in 2/3 variables compared."       ,
      "0 variables compared have non-identical attributes."
    )
  )
  expect_true(n.diffs(comparedf(df1, df1, by = "id")) == 0)
})

df1$testdate <- as.Date(c("2017-07-09", "2017-08-08", "2017-09-07"))
df2$testdate <- as.Date(c("2017-09-07", "2017-08-08", "2017-09-07"))

test_that("Dates comparison works: by id", {
  expect_identical(
    capture.output(comparedf(df1, df2, by = "id")),
    c("Compare Object"                                     ,
      ""                                                   ,
      "Function Call: "                                    ,
      "comparedf(x = df1, y = df2, by = \"id\")"           ,
      ""                                                   ,
      "Shared: 4 non-by variables and 3 observations."     ,
      "Not shared: 2 variables and 0 observations."        ,
      ""                                                   ,
      "Differences found in 3/4 variables compared."       ,
      "0 variables compared have non-identical attributes."
    )
  )
  expect_true(n.diffs(comparedf(df1, df1, by = "id")) == 0)
})

test_that("Basic mockstudy comparison works: by id", {
  expect_identical(
    capture.output(comparedf(mockstudy, mockstudy2, by = 'case')),
    c("Compare Object"                                                  ,
      ""                                                                ,
      "Function Call: "                                                 ,
      "comparedf(x = mockstudy, y = mockstudy2, by = \"case\")"         ,
      ""                                                                ,
      "Shared: 9 non-by variables and 1495 observations."               ,
      "Not shared: 7 variables and 4 observations."                     ,
      ""                                                                ,
      "Differences found in 3/7 variables compared."                    ,
      "3 variables compared have non-identical attributes."
    )
  )
})


test_that("Comparison with empty data.frames works", {
  mck1 <- mockstudy[0, , drop = FALSE]
  mck2 <- mockstudy2[0, , drop = FALSE]
  expect_identical(
    capture.output(comparedf(mockstudy, mck2, by = 'case')),
    c("Compare Object"                                            ,
      ""                                                          ,
      "Function Call: "                                           ,
      "comparedf(x = mockstudy, y = mck2, by = \"case\")"         ,
      ""                                                          ,
      "Shared: 9 non-by variables and 0 observations."            ,
      "Not shared: 7 variables and 1499 observations."            ,
      ""                                                          ,
      "Differences found in 0/7 variables compared."              ,
      "3 variables compared have non-identical attributes."
    )
  )
  expect_true(n.diff.obs(comparedf(mockstudy, mck2, by = "case")) == 1499)
  expect_true(n.diffs(comparedf(mockstudy, mck2, by = "case")) == 0)

  expect_identical(
    capture.output(comparedf(mockstudy, mck2, by = 'row.names')),
    c("Compare Object"                                                 ,
      ""                                                               ,
      "Function Call: "                                                ,
      "comparedf(x = mockstudy, y = mck2, by = \"row.names\")"         ,
      ""                                                               ,
      "Shared: 10 non-by variables and 0 observations."                ,
      "Not shared: 7 variables and 1499 observations."                 ,
      ""                                                               ,
      "Differences found in 0/8 variables compared."                   ,
      "3 variables compared have non-identical attributes."
    )
  )
  expect_true(n.diff.obs(comparedf(mockstudy, mck2, by = "row.names")) == 1499)
  expect_true(n.diffs(comparedf(mockstudy, mck2, by = "row.names")) == 0)

  expect_identical(
    capture.output(comparedf(mck1, mck2, by = "case")),
    c("Compare Object"                                       ,
      ""                                                     ,
      "Function Call: "                                      ,
      "comparedf(x = mck1, y = mck2, by = \"case\")"         ,
      ""                                                     ,
      "Shared: 9 non-by variables and 0 observations."       ,
      "Not shared: 7 variables and 0 observations."          ,
      ""                                                     ,
      "Differences found in 0/7 variables compared."         ,
      "2 variables compared have non-identical attributes."
    )
  )
  expect_true(n.diff.obs(comparedf(mck1, mck2, by = "case")) == 0)
  expect_true(n.diffs(comparedf(mck1, mck2, by = "case")) == 0)
})

###########################################################################################################
#### Check for certain errors
###########################################################################################################

test_that("Different by-variables with overlap with non-by-variables throws an error", {
  expect_error(comparedf(df1, df2, by.x = "id", by.y = "b"), "non-by-variables")
  expect_error(comparedf(df1, df2, by.x = "c", by.y = "id"), NA)
})

test_that("Using forbidden names throws an error", {
  expect_error(comparedf(df1, cbind(df2, ..row.names.. = 1:3)), "reserved colnames")
})

###########################################################################################################
#### Using comparedf.control
###########################################################################################################

test_that("tol.vars is working correctly", {
  expect_identical(
    capture.output(comparedf(mockstudy, mockstudy2, by = 'case', tol.vars = "._ ")),
    c("Compare Object"                                                   ,
      ""                                                                 ,
      "Function Call: "                                                  ,
      "comparedf(x = mockstudy, y = mockstudy2, by = \"case\", tol.vars = \"._ \")",
      ""                                                                 ,
      "Shared: 11 non-by variables and 1495 observations."               ,
      "Not shared: 3 variables and 4 observations."                      ,
      ""                                                                 ,
      "Differences found in 3/9 variables compared."                     ,
      "3 variables compared have non-identical attributes."
    )
  )

  expect_identical(
    capture.output(comparedf(mockstudy, mockstudy2, by = 'case', tol.vars = c("._ ", "case"))),
    c("Compare Object"                                                   ,
      ""                                                                 ,
      "Function Call: "                                                  ,
      "comparedf(x = mockstudy, y = mockstudy2, by = \"case\", tol.vars = c(\"._ \", ",
      "    \"case\"))"                                                   ,
      ""                                                                 ,
      "Shared: 12 non-by variables and 1495 observations."               ,
      "Not shared: 1 variables and 4 observations."                      ,
      ""                                                                 ,
      "Differences found in 3/10 variables compared."                    ,
      "4 variables compared have non-identical attributes."
    )
  )

  tolvars <- c(arm = "Arm", fu.time = "fu_time", fu.stat = "fu stat")
  expect_identical(
    comparedf(mockstudy, mockstudy2, by = 'case', tol.vars = tolvars)[1:2],
    comparedf(mockstudy, mockstudy2, by = 'case', tol.vars = c("._ ", "case"))[1:2]
  )
  expect_warning(comparedf(mockstudy, mockstudy2, by = 'case', tol.vars = c("hi" = "Arm")),
                 "'hi' not found in colnames of x")
  expect_warning(comparedf(mockstudy, mockstudy2, by = 'case', tol.vars = c(arm = "hi")),
                 "'hi' not found in colnames of y")
})

tol <- comparedf.control(
  tol.vars = c("._ ", "case"), # dots=underscores=spaces, ignore case
  int.as.num = TRUE,           # compare integers and numerics
  tol.num.val = 10,            # allow absolute differences <= 10
  tol.factor = "labels",       # match only factor labels
  factor.as.char = TRUE,       # compare factors and characters
  tol.char = "case"            # ignore case in character vectors
)

test_that("tolerances are working correctly", {
  expect_identical(
    capture.output(comparedf(mockstudy, mockstudy2, by = "case", control = tol)),
    c("Compare Object"                                                        ,
      ""                                                                      ,
      "Function Call: "                                                       ,
      "comparedf(x = mockstudy, y = mockstudy2, by = \"case\", control = tol)",
      ""                                                                      ,
      "Shared: 12 non-by variables and 1495 observations."                    ,
      "Not shared: 1 variables and 4 observations."                           ,
      ""                                                                      ,
      "Differences found in 3/12 variables compared."                         ,
      "4 variables compared have non-identical attributes."
    )
  )
})

tol.minus9 <- function(x, y, tol)
{
  idx1 <- is.na(x) & !is.na(y) & y == -9
  idx2 <- tol.num.absolute(x, y, tol) # find other absolute differences
  return(!idx1 & idx2)
}

tol$tol.num <- tol.minus9 # ignore NA -> -9 changes

test_that("custom tolerances are working correctly", {
  expect_identical(
    capture.output(comparedf(mockstudy, mockstudy2, by = "case", control = tol)),
    c("Compare Object"                                                        ,
      ""                                                                      ,
      "Function Call: "                                                       ,
      "comparedf(x = mockstudy, y = mockstudy2, by = \"case\", control = tol)",
      ""                                                                      ,
      "Shared: 12 non-by variables and 1495 observations."                    ,
      "Not shared: 1 variables and 4 observations."                           ,
      ""                                                                      ,
      "Differences found in 2/12 variables compared."                         ,
      "4 variables compared have non-identical attributes."
    )
  )
})



tols <- comparedf.control(
  tol.vars = c("._ ", "case"), # dots=underscores=spaces; match up Arm and arm
  int.as.num = TRUE,           # compare integers and numerics
  tol.factor = "labels",       # match only factor labels
  factor.as.char = TRUE,       # compare factors and characters
  tol.char = "case"            # ignore case in character vectors
)

test_that("Summary numbers are reported correctly", {
  expect_equal(
    summary(comparedf(mockstudy, mockstudy2, by = "case", control = tols))$comparison.summary.table$value,
    c(1, 12, 12, 1, 0, 3, 9, 1495, 4, 0, 269, 1226, 270)
  )
})

tols$tol.vars <- "._ " # don't match arm and Arm anymore

test_that("Summary numbers are still reported correctly", {
  expect_equal(
    summary(comparedf(mockstudy, mockstudy2, by = "case", control = tols))$comparison.summary.table$value,
    c(1, 11, 11, 2, 1, 3, 8, 1495, 4, 0, 269, 1226, 270)
  )
})

###########################################################################################################
#### Using custom tolerances
###########################################################################################################

test_that("Custom tolerances work specified by variable", {
  dat1 <- dat2 <- data.frame(
    x1 = rep(c("A", "B", "C"), each = 10),
    x2 = rep(c("D", "E", "F"), each = 10),
    x3 = 1:30 + 0.5,
    x4 = 1:30 + 0,
    stringsAsFactors = FALSE
  )
  dat2$x1 <- tolower(dat2$x1)
  dat2$x2 <- tolower(dat2$x2)
  dat2$x3 <- dat2$x3 + rep(c(0, -0.5, 1), each = 10)
  dat2$x4 <- dat2$x4 * rep(c(1, 1.1, 1.6), each = 10)

  expect_equal(
    summary(comparedf(dat1, dat2, tol.num = list("absolute", x4 = "percent"), tol.num.val = 0.5,
                      tol.char = list(x1 = "case", x2 = function(x, y) x != y & x %nin% c("D", "E"))))$comparison.summary.table$value,
    c(0, 4, 4, 0, 0, 3, 1, 30, 0, 0, 10, 20, 30)
  )
})

###########################################################################################################
#### Using helper functions
###########################################################################################################

test_that("helper functions are working correctly", {

  expect_true(n.diffs(comparedf(mockstudy, mockstudy2, by = "case")) ==
                n.diffs(summary(comparedf(mockstudy, mockstudy2, by = "case"))))

  expect_true(n.diffs(comparedf(df1, df2, by = "id")) == n.diffs(summary(comparedf(df1, df2, by = "id"))))
  expect_identical(diffs(comparedf(df1, df2, by = "id")), diffs(summary(comparedf(df1, df2, by = "id"))))
  expect_identical(diffs(comparedf(df1, df2, by = "id"), by.var = TRUE), diffs(summary(comparedf(df1, df2, by = "id")), by.var = TRUE))

  expect_identical(diffs(comparedf(df1, df2, by = "id"), vars = "a"), diffs(summary(comparedf(df1, df2, by = "id")), vars = "a"))
  expect_identical(diffs(comparedf(df1, df2, by = "id"), vars = "b"), diffs(summary(comparedf(df1, df2, by = "id")), vars = "b"))
  expect_identical(diffs(comparedf(df1, df2, by = "id"), vars = "a", by.var = TRUE), diffs(summary(comparedf(df1, df2, by = "id")), vars = "a", by.var = TRUE))

})

test_that("diff.obs() works (#305)", {
  expect_identical(diffs(comparedf(df1, df2, by = "id"), what = "observations"), diffs(summary(comparedf(df1, df2, by = "id")), what = "observations"))
})

###########################################################################################################
#### Summary output
###########################################################################################################

test_that("Summary output looks right (i.e. for factors)", {
  expect_identical(
    capture.kable(summary(comparedf(mockstudy, mockstudy2, by = "case"))),
    c("Table: Summary of data.frames"                                                 ,
      ""                                                                              ,
      "version   arg           ncol   nrow"                                           ,
      "--------  -----------  -----  -----"                                           ,
      "x         mockstudy       14   1499"                                           ,
      "y         mockstudy2      13   1495"                                           ,
      ""                                                                              ,
      ""                                                                              ,
      ""                                                                              ,
      "Table: Summary of overall comparison"                                          ,
      ""                                                                              ,
      "statistic                                                      value"          ,
      "------------------------------------------------------------  ------"          ,
      "Number of by-variables                                             1"          ,
      "Number of non-by variables in common                               9"          ,
      "Number of variables compared                                       7"          ,
      "Number of variables in x but not y                                 4"          ,
      "Number of variables in y but not x                                 3"          ,
      "Number of variables compared with some values unequal              3"          ,
      "Number of variables compared with all values equal                 4"          ,
      "Number of observations in common                                1495"          ,
      "Number of observations in x but not y                              4"          ,
      "Number of observations in y but not x                              0"          ,
      "Number of observations with some compared variables unequal     1495"          ,
      "Number of observations with all compared variables equal           0"          ,
      "Number of values unequal                                        1762"          ,
      ""                                                                              ,
      ""                                                                              ,
      ""                                                                              ,
      "Table: Variables not shared"                                                   ,
      ""                                                                              ,
      "version   variable    position  class     "                                    ,
      "--------  ---------  ---------  ----------"                                    ,
      "x         age                2  integer   "                                    ,
      "x         arm                3  character "                                    ,
      "x         fu.time            6  integer   "                                    ,
      "x         fu.stat            7  integer   "                                    ,
      "y         fu_time           11  integer   "                                    ,
      "y         fu stat           12  integer   "                                    ,
      "y         Arm               13  character "                                    ,
      ""                                                                              ,
      ""                                                                              ,
      ""                                                                              ,
      "Table: Other variables not compared"                                           ,
      ""                                                                              ,
      "var.x    pos.x  class.x     var.y    pos.y  class.y "                          ,
      "------  ------  ----------  ------  ------  --------"                          ,
      "race         5  character   race         3  factor  "                          ,
      "ast         12  integer     ast          8  numeric "                          ,
      ""                                                                              ,
      ""                                                                              ,
      ""                                                                              ,
      "Table: Observations not shared"                                                ,
      ""                                                                              ,
      "version      case   observation"                                               ,
      "--------  -------  ------------"                                               ,
      "x           88989             9"                                               ,
      "x           90158             8"                                               ,
      "x           99508             7"                                               ,
      "x          112263             5"                                               ,
      ""                                                                              ,
      ""                                                                              ,
      ""                                                                              ,
      "Table: Differences detected by variable"                                       ,
      ""                                                                              ,
      "var.x         var.y             n   NAs"                                       ,
      "------------  ------------  -----  ----"                                       ,
      "sex           sex            1495     0"                                       ,
      "ps            ps                1     1"                                       ,
      "hgb           hgb             266   266"                                       ,
      "bmi           bmi               0     0"                                       ,
      "alk.phos      alk.phos          0     0"                                       ,
      "mdquality.s   mdquality.s       0     0"                                       ,
      "age.ord       age.ord           0     0"                                       ,
      ""                                                                              ,
      ""                                                                              ,
      ""                                                                              ,
      "Table: Differences detected (1741 not shown)"                                  ,
      ""                                                                              ,
      "var.x   var.y     case  values.x   values.y    row.x   row.y"                  ,
      "------  ------  ------  ---------  ---------  ------  ------"                  ,
      "sex     sex      76170  Male       Male           26      20"                  ,
      "sex     sex      76240  Male       Male           27      21"                  ,
      "sex     sex      76431  Female     Female         28      22"                  ,
      "sex     sex      76712  Male       Male           29      23"                  ,
      "sex     sex      76780  Female     Female         30      24"                  ,
      "sex     sex      77066  Female     Female         31      25"                  ,
      "sex     sex      77316  Male       Male           32      26"                  ,
      "sex     sex      77355  Male       Male           33      27"                  ,
      "sex     sex      77591  Male       Male           34      28"                  ,
      "sex     sex      77851  Male       Male           35      29"                  ,
      "ps      ps       86205  0          NA              6       3"                  ,
      "hgb     hgb      88714  NA         -9            192     186"                  ,
      "hgb     hgb      88955  NA         -9            204     198"                  ,
      "hgb     hgb      89549  NA         -9            229     223"                  ,
      "hgb     hgb      89563  NA         -9            231     225"                  ,
      "hgb     hgb      89584  NA         -9            237     231"                  ,
      "hgb     hgb      89591  NA         -9            238     232"                  ,
      "hgb     hgb      89595  NA         -9            239     233"                  ,
      "hgb     hgb      89647  NA         -9            243     237"                  ,
      "hgb     hgb      89665  NA         -9            244     238"                  ,
      "hgb     hgb      89827  NA         -9            255     249"                  ,
      ""                                                                              ,
      ""                                                                              ,
      ""                                                                              ,
      "Table: Non-identical attributes"                                               ,
      ""                                                                              ,
      "var.x   var.y   name   "                                                       ,
      "------  ------  -------"                                                       ,
      "sex     sex     label  "                                                       ,
      "sex     sex     levels "                                                       ,
      "race    race    class  "                                                       ,
      "race    race    label  "                                                       ,
      "race    race    levels "                                                       ,
      "bmi     bmi     label  "
    )
  )
})


test_that("Summary output with attributes and max.print options", {
  expect_identical(
    capture.kable(summary(comparedf(mockstudy, mockstudy2, by = "case"), show.attrs = TRUE,
                          max.print.vars = 2, max.print.obs = 3, max.print.diffs.per.var = 3,
                          max.print.diffs = NA, max.print.attrs = 3)),
    c("Table: Summary of data.frames"                                                ,
      ""                                                                             ,
      "version   arg           ncol   nrow"                                          ,
      "--------  -----------  -----  -----"                                          ,
      "x         mockstudy       14   1499"                                          ,
      "y         mockstudy2      13   1495"                                          ,
      ""                                                                             ,
      ""                                                                             ,
      ""                                                                             ,
      "Table: Summary of overall comparison"                                         ,
      ""                                                                             ,
      "statistic                                                      value"         ,
      "------------------------------------------------------------  ------"         ,
      "Number of by-variables                                             1"         ,
      "Number of non-by variables in common                               9"         ,
      "Number of variables compared                                       7"         ,
      "Number of variables in x but not y                                 4"         ,
      "Number of variables in y but not x                                 3"         ,
      "Number of variables compared with some values unequal              3"         ,
      "Number of variables compared with all values equal                 4"         ,
      "Number of observations in common                                1495"         ,
      "Number of observations in x but not y                              4"         ,
      "Number of observations in y but not x                              0"         ,
      "Number of observations with some compared variables unequal     1495"         ,
      "Number of observations with all compared variables equal           0"         ,
      "Number of values unequal                                        1762"         ,
      ""                                                                             ,
      ""                                                                             ,
      ""                                                                             ,
      "Table: Variables not shared (5 not shown)"                                    ,
      ""                                                                             ,
      "version   variable    position  class     "                                   ,
      "--------  ---------  ---------  ----------"                                   ,
      "x         age                2  integer   "                                   ,
      "x         arm                3  character "                                   ,
      ""                                                                             ,
      ""                                                                             ,
      ""                                                                             ,
      "Table: Other variables not compared"                                          ,
      ""                                                                             ,
      "var.x    pos.x  class.x     var.y    pos.y  class.y "                         ,
      "------  ------  ----------  ------  ------  --------"                         ,
      "race         5  character   race         3  factor  "                         ,
      "ast         12  integer     ast          8  numeric "                         ,
      ""                                                                             ,
      ""                                                                             ,
      ""                                                                             ,
      "Table: Observations not shared (1 not shown)"                                 ,
      ""                                                                             ,
      "version     case   observation"                                               ,
      "--------  ------  ------------"                                               ,
      "x          88989             9"                                               ,
      "x          90158             8"                                               ,
      "x          99508             7"                                               ,
      ""                                                                             ,
      ""                                                                             ,
      ""                                                                             ,
      "Table: Differences detected by variable"                                      ,
      ""                                                                             ,
      "var.x         var.y             n   NAs"                                      ,
      "------------  ------------  -----  ----"                                      ,
      "sex           sex            1495     0"                                      ,
      "ps            ps                1     1"                                      ,
      "hgb           hgb             266   266"                                      ,
      "bmi           bmi               0     0"                                      ,
      "alk.phos      alk.phos          0     0"                                      ,
      "mdquality.s   mdquality.s       0     0"                                      ,
      "age.ord       age.ord           0     0"                                      ,
      ""                                                                             ,
      ""                                                                             ,
      ""                                                                             ,
      "Table: Differences detected (1755 not shown)"                                 ,
      ""                                                                             ,
      "var.x   var.y     case  values.x   values.y    row.x   row.y"                 ,
      "------  ------  ------  ---------  ---------  ------  ------"                 ,
      "sex     sex      76170  Male       Male           26      20"                 ,
      "sex     sex      76240  Male       Male           27      21"                 ,
      "sex     sex      76431  Female     Female         28      22"                 ,
      "ps      ps       86205  0          NA              6       3"                 ,
      "hgb     hgb      88714  NA         -9            192     186"                 ,
      "hgb     hgb      88955  NA         -9            204     198"                 ,
      "hgb     hgb      89549  NA         -9            229     223"                 ,
      ""                                                                             ,
      ""                                                                             ,
      ""                                                                             ,
      "Table: Non-identical attributes (3 not shown)"                                ,
      ""                                                                             ,
      "var.x   var.y   name     attr.x           attr.y       "                      ,
      "------  ------  -------  ---------------  -------------"                      ,
      "sex     sex     label    NA               Sex (M/F)    "                      ,
      "sex     sex     levels   Male  , Female   Female, Male "                      ,
      "race    race    class    NA               factor       "
    )
  )
})


###########################################################################################################
#### Reported bugs for comparedf
###########################################################################################################

test_that("2019/04/09: Percent tolerances work when everything is zero (#206)", {
  expect_true(n.diffs(comparedf(data.frame(x = 0), data.frame(x = 0), tol.num = "percent")) == 0)
})

test_that("2019/04/10: summary breaks when no variables were compared (#207)", {

  check_it <- function(x, y, by = NULL, n = 0)
  {
    tmp <- diffs(comparedf(x, y, by = by))
    expect_true(nrow(tmp) == n)
    expect_identical(names(tmp), c("var.x", "var.y", if(is.null(by)) "..row.names.." else by, "values.x", "values.y", "row.x", "row.y"))
    expect_true(inherits(tmp$values.x, "AsIs"))
    expect_true(inherits(tmp$values.y, "AsIs"))
    expect_true(is.integer(tmp$row.x))
    expect_true(is.integer(tmp$row.y))
  }

  check_it(data.frame(x = 1, y = 1), data.frame(x = 1, z = 1))
  check_it(data.frame(x = 1, y = 1), data.frame(x = 1, z = 1), by = "x")
  check_it(data.frame(x = 1, y = 1), data.frame(x = 1, y = 1), by = "x")
  check_it(data.frame(x = 1, y = 1), data.frame(x = 1, y = 1.1), by = "x", n = 1)
})

test_that("2019/05/15: using row.names with other by-variables", {
  d <- data.frame(a = 1:3, b = 2:4, row.names = c("A", "B", "C"))
  f <- data.frame(a = 3:1, b = 2:4, row.names = c("C", "A", "B"))
  expect_identical(
    capture.output(comparedf(d, f, by = c("a", "row.names"))),
    c("Compare Object"                                       ,
      ""                                                     ,
      "Function Call: "                                      ,
      "comparedf(x = d, y = f, by = c(\"a\", \"row.names\"))",
      ""                                                     ,
      "Shared: 1 non-by variables and 1 observations."       ,
      "Not shared: 0 variables and 4 observations."          ,
      ""                                                     ,
      "Differences found in 1/1 variables compared."         ,
      "0 variables compared have non-identical attributes."
    )
  )
})

test_that("2019/05/15: empty by-variable isn't counted", {
  expect_equal(summary(comparedf(mockstudy, mockstudy))$comparison.summary.table$value, c(0, 14, 14, 0, 0, 0, 14, 1499, 0, 0, 0, 1499, 0))
})

test_that("2019/05/16: multiple by-variables are counted", {
  expect_equal(summary(comparedf(mockstudy, mockstudy, by = c("case", "sex", "arm")))$comparison.summary.table$value,
               c(3, 11, 11, 0, 0, 0, 11, 1499, 0, 0, 0, 1499, 0))
})

test_that("2019/05/22: vectors that share a class are still compared (#216)", {
  dat2 <- dat <- data.frame(x = 1:5)
  class(dat2$x) <- c("myclass", class(dat2$x))
  cmp <- summary(comparedf(dat, dat2))
  expect_true(nrow(cmp$vars.nc.table) == 0)
  expect_true(nrow(cmp$diffs.table) == 0)
})

test_that("Inf (#306)", {
  dat <- data.frame(x = c(2, Inf, -Inf, Inf, NA))
  dat2 <- data.frame(x = c(2, Inf, -Inf, -Inf, Inf))
  cmp <- summary(comparedf(dat, dat2))
  expect_true(nrow(cmp$diffs.table) == 2)
  cmp <- summary(comparedf(dat, dat2, tol.num = "pct"))
  expect_true(nrow(cmp$diffs.table) == 2)
})

#' Summary Statistics of a Set of Independent Variables by a Categorical Variable
#'
#' Summarize one or more variables (x) by a categorical variable (y). Variables
#'   on the right side of the formula, i.e. independent variables, are summarized by the
#'   levels of a categorical variable on the left of the formula. Optionally, an appropriate test is performed to test the
#'   distribution of the independent variables across the levels of the categorical variable.
#'
#' @param formula an object of class \code{\link{formula}}; a symbolic description of the variables to be summarized by the group,
#'   or categorical variable, of interest. See "Details" for more information. To only view overall summary
#'   statistics, a one-sided formula can be used.
#' @param data an optional data frame, list or environment (or object coercible by \code{\link{as.data.frame}} to a data frame)
#'   containing the variables in the model. If not found in data, the variables are taken from \code{environment(formula)},
#'   typically the environment from which the function is called.
#' @param na.action a function which indicates what should happen when the data contain \code{NA}s.
#'   The default is \code{na.tableby(TRUE)} if there is a by-variable, and \code{na.tableby(FALSE)} if there is not.
#'   This schema thus includes observations with \code{NA}s in x variables,
#'   but removes those with \code{NA} in the categorical group variable and strata (if used).
#' @param subset an optional vector specifying a subset of observations (rows of data) to be used in the results.
#'   Works as vector of logicals or an index.
#' @param weights a vector of weights. Using weights will disable statistical tests.
#' @param strata a vector of strata to separate summaries by an additional group.
#' @param control control parameters to handle optional settings within \code{tableby}.
#'   Two aspects of \code{tableby} are controlled with these: test options of RHS variables across levels of the categorical
#'   grouping variable, and x variable summaries within the grouping variable. Arguments for \code{tableby.control}
#'   can be passed to \code{tableby} via the \code{...} argument, but if a control object and \code{...} arguments are both supplied,
#'   the latter are used. See \code{\link{tableby.control}} for more details.
#' @param ... additional arguments to be passed to internal \code{tableby} functions or \code{\link{tableby.control}}.
#' @details
#' The group variable (if any) is categorical, which could be an integer, character,
#' factor, or ordered factor. \code{tableby} makes a simple summary of
#' the counts within the k-levels of the independent variables on the
#' right side of the formula. Note that unused levels are dropped.
#'
#' The \code{data} argument allows data.frames with label attributes for the columns, and those
#' labels will be used in the summary methods for the \code{tableby} class.
#'
#' The independent variables are a mixture of types: categorical (discrete),
#' numeric (continuous), and time to event (survival). These variables
#' are split by the levels of the group variable (if any), then summarized within
#' those levels, specific to the variable type. A statistical test is
#' performed to compare the distribution of the independent variables across the
#' levels of the grouping variable.
#'
#' The tests differ by the independent variable type, but can be specified
#' explicitly in the formula statement or in the control function.
#' These tests are accepted:
#' \itemize{
#'   \item{
#'     \code{anova}: analysis of variance test; the default test for continuous variables. When
#'     LHS variable has two levels, equivalent to two-sample t-test.
#'   }
#'   \item{
#'     \code{kwt}: Kruskal-Wallis Rank Test, optional test for continuous
#'     variables. When LHS variable has two levels, equivalent to Wilcoxon test.
#'   }
#'   \item{
#'     \code{wt}: An explicit Wilcoxon test.
#'   }
#'   \item{
#'     \code{medtest}: A median test.
#'   }
#'   \item{
#'     \code{chisq}: chi-square goodness of fit test for equal counts of a
#'     categorical variable across categories; the default for categorical
#'     or factor variables
#'   }
#'   \item{
#'     \code{fe}: Fisher's exact test for categorical variables
#'   }
#'   \item{
#'     \code{trend}: trend test for equal distribution of an ordered variable
#'     across a categorical variable; the default for ordered factor variables
#'   }
#'   \item{
#'     \code{logrank}: log-rank, the default for time-to-event variables
#'   }
#'   \item{
#'     \code{notest}: no test is performed.
#'   }
#' }
#'
#' To perform a mixture of asymptotic and rank-based tests on two
#' different continuous variables, an example formula is:
#' \code{formula = group ~ anova(age) + kwt(height)}. The test settings
#' in \code{tableby.control} apply to all independent variables of a given type.
#'
#' The summary statistics reported for each independent variable within the
#' group variable can be set in \code{\link{tableby.control}}.
#'
#' Finally, multiple by-variables can be set using \code{list()}. See the examples for more details.
#'
#' @return An object with class \code{c("tableby", "arsenal_table")}
#' @seealso \code{\link{arsenal_table}}, \code{\link[stats]{anova}}, \code{\link[stats]{chisq.test}}, \code{\link{tableby.control}},
#'   \code{\link{summary.tableby}}, \code{\link{tableby.internal}}, \code{\link{formulize}}, \code{\link{selectall}}
#'
#' @examples
#' data(mockstudy)
#' tab1 <- tableby(arm ~ sex + age, data=mockstudy)
#' summary(tab1, text=TRUE)
#'
#' mylabels <- list(sex = "SEX", age ="Age, yrs")
#' summary(tab1, labelTranslations = mylabels, text=TRUE)
#'
#' tab3 <- tableby(arm ~ sex + age, data=mockstudy, test=FALSE, total=FALSE,
#'                 numeric.stats=c("median","q1q3"), numeric.test="kwt")
#' summary(tab3, text=TRUE)
#'
#' # multiple LHS
#' summary(tableby(list(arm, sex) ~ age, data = mockstudy, strata = ps), text = TRUE)
#'
#' tab.test <- tableby(arm ~ kwt(age) + anova(bmi) + kwt(ast), data=mockstudy)
#' tests(tab.test)
#'
#' @author Jason Sinnwell, Beth Atkinson, Gregory Dougherty, and Ethan Heinzen, adapted from SAS Macros written by Paul Novotny and Ryan Lennon
#' @name tableby
NULL
#> NULL

#' @rdname tableby
#' @export
tableby <- function(formula, data, na.action, subset=NULL, weights=NULL, strata, control = NULL, ...)
{
  control <- c(list(...), control)
  control <- do.call("tableby.control", control[!duplicated(names(control))])

  Call <- match.call()
  ## Tell user if they passed an argument that was not expected, either here or in control
  expectArgs <- c("formula", "data", "na.action", "subset", "weights", "strata", "control", names(control), "times","survconf.type")
  match.idx <- match(names(Call)[-1], expectArgs)
  if(anyNA(match.idx)) warning("unused arguments: ", paste(names(Call)[1+which(is.na(match.idx))], collapse=", "), "\n")

  indx <- match(c("formula", "data", "subset", "weights", "na.action", "strata"), names(Call), nomatch = 0)
  if(indx[1] == 0) stop("A formula argument is required")

  special <- c("anova", "kwt", "wt", "medtest", "chisq", "fe", "logrank", "trend", "notest", "stddiff")

  out.tables <- list()
  formula.list <- as_list_formula(formula)
  for(FORM in formula.list)
  {
    temp.call <- Call[c(1, indx)]
    temp.call[[1]] <- as.name("model.frame")

    if(is.null(temp.call$na.action)) {
      temp.call$na.action <- if(length(FORM) == 2) na.tableby(FALSE) else na.tableby(TRUE)
    } else if(length(FORM) == 2 && identical(na.action, na.tableby(TRUE), ignore.environment = TRUE)) {
      # purposely using na.action instead of temp.call$na.action here
      warning("It appears you're using na.tableby(TRUE) with a one-sided formula... Results may not be what you expect.")
    }

    if (missing(data)) {
      temp.call$formula <- stats::terms(FORM, special)
    } else {
      # instead of call("keep.labels", ...), which breaks when arsenal isn't loaded (Can't find "keep.labels")
      temp.call$data <- as.call(list(keep.labels, temp.call$data))
      temp.call$formula <- stats::terms(FORM, special, data = keep.labels(data))
    }

    ##  set up new environment for
    ## if specials, assign dummy versions of those functions
    tabenv <- new.env(parent = environment(formula))
    for(sp in special)
    {
      if(!is.null(attr(temp.call$formula, "specials")[[sp]])) assign(sp, inline_tableby_stat_test, envir = tabenv)
    }

    ## set tabenv as environment in which to evalulate formula
    environment(temp.call$formula) <- tabenv

    ## evaluate the formula with env set for it
    modeldf <- loosen.labels(eval.parent(temp.call))
    if(nrow(modeldf) == 0) stop("No (non-missing) observations")

    Terms <- stats::terms(modeldf)

    ###### Check for weights ######

    if(hasWeights <- "(weights)" %in% colnames(modeldf)) {
      weights <- as.vector(stats::model.weights(modeldf))
      if(!is.numeric(weights) || anyNA(weights) || any(weights < 0)) stop("'weights' must be a numeric vector and must be non-missing and non-negative")
      modeldf[["(weights)"]] <- NULL
      control$test <- FALSE
    } else weights <- NULL

    ###### Check for strata ######

    if(hasStrata <- "(strata)" %in% colnames(modeldf))
    {
      strata.col <- modeldf[["(strata)"]]
      strataTerm <- paste(deparse(Call$strata), collapse = " ")
      if(is.null(strataLabel <- attr(strata.col, "label"))) strataLabel <- strataTerm
      if(is.factor(strata.col))
      {
        strata.col <- droplevels(strata.col)
        strata.levels <- levels(strata.col)
      } else strata.levels <- sort(unique(strata.col))

      modeldf[["(strata)"]] <- NULL
    } else
    {
      strata.col <- rep("", nrow(modeldf))
      strataTerm <- strataLabel <- strata.levels <- ""
    }

    ###### Check for by-variable ######

    if(attributes(Terms)$response != 0)
    {
      by.col <- modeldf[[1]]
      termBy <- names(modeldf)[1]
      if(is.null(labelBy <- attr(by.col, "label"))) labelBy <- termBy
      if(is.factor(by.col)) {
        by.col <- droplevels(by.col)
        by.levels <- levels(by.col)
      } else by.levels <- sort(unique(by.col))
      by.col <- as.character(by.col)
      by.levels <- as.character(by.levels)

      if(any(by.levels == ""))
      {
        warning('Empty string detected in by-variable is not allowed; converting to " ".')
        by.col[by.col == ""] <- " "
        by.levels <- unique(replace(by.levels, by.levels == "", " "))
      }
      reserved <- c("group.term", "group.label", "strata.term", "strata.value", "variable", "term",
                    "label", "variable.type", "test", "p.value")
      if(any(by.levels %in% reserved))
      {
        stop("One or more reserved word found in by-variable: ",
             paste0(intersect(by.levels, reserved), collapse = ", "),
             ". Please use a different word")
      }

      if(length(by.levels) < 2 && control$test)
      {
        warning("The by-variable has fewer than two levels; statistical tests are ignored")
        control$test <- FALSE
      }
      modeldf[[1]] <- NULL
    } else
    {
      ## no response, create a dummy one
      if(is.null(overall <- control$stats.labels$overall)) overall <- "Overall"
      by.col <- rep(overall, nrow(modeldf))
      termBy <- labelBy <- by.levels <- overall
      control$total <- FALSE
      control$test <- FALSE
    }

    if(is.null(totallab <- control$stats.labels$total)) totallab <- "Total"
    ystats <- if(hasWeights)
    {
      c(stats::xtabs(weights ~ factor(by.col, levels=by.levels), exclude = NA), stats::setNames(sum(weights[!is.na(by.col)]), totallab))
    } else c(table(factor(by.col, levels=by.levels), exclude=NA), stats::setNames(sum(!is.na(by.col)), totallab))
    yList <- list(stats=ystats, label=labelBy, term=termBy)

    ## find which columnss of modeldf have specials assigned to known specials
    specialIndices <- unlist(attr(Terms, "specials")) - attributes(Terms)$response
    specialTests <- rep("", ncol(modeldf))
    ## If a special shows up multiple times, the unlist assigned a number at the end. Strip it off.
    ## This disallows functions with a number at the end
    specialTests[specialIndices] <- gsub("\\d+$", "", names(specialIndices))

    xTerms <- Map(modeldf, names(modeldf), f = function(col, nm) {
      if(is.null(nameEff <- attr(col, "name"))) nameEff <- nm
      if(is.null(labelEff <- attr(col, "label"))) labelEff <- nameEff
      if(is.null(termEff <- attr(col, "term"))) termEff <- nm
      list(variable=nameEff, label=labelEff, term=termEff, term.orig = nm)
    })
    names(xTerms) <- vapply(xTerms, "[[", NA_character_, "variable")

    control.list <- lapply(modeldf, attr, "control.list")
    names(control.list) <- names(xTerms)

    strataList <- vector("list", length(strata.levels))
    if(hasStrata) names(strataList) <- paste0("(", strataTerm, ") == ", strata.levels)

    for(strat in strata.levels)
    {
      ## list of x variables
      xList <- vector("list", ncol(modeldf))
      names(xList) <- names(xTerms)

      bycol <- by.col[strata.col == strat]
      weightscol <- weights[strata.col == strat]

      for(eff in seq_along(modeldf)) {

        currcol <- modeldf[[eff]]

        ############################################################

        if(is.ordered(currcol) || is.logical(currcol) || is.factor(currcol) || is.character(currcol)) {
          ######## ordinal or categorical variable (character or factor) ###############

          ## convert logicals and characters to factor
          if(is.character(currcol))
          {
            currcol <- factor(currcol, levels = sort(unique(currcol[!is.na(currcol)])))
          } else if(is.logical(currcol)) currcol <- factor(currcol, levels=c(FALSE, TRUE))

          ## to make sure all levels of cat variable are counted, need to pass values along
          xlevels <- levels(currcol)

          if(length(xlevels) == 0) stop(paste0("Zero-length levels found for ", names(xTerms)[eff]))

          ## get stats funs from either formula  or control
          if(is.ordered(currcol))
          {
            currstats <-  control$ordered.stats
            currtest <- control$ordered.test
            vartype <- "ordinal"
          } else
          {
            currstats <- control$cat.stats
            currtest <- control$cat.test
            vartype <- "categorical"
          }

        } else if(is.Date(currcol)) {
          ######## Date variable ###############
          xlevels <- sort(unique(currcol))

          ## get stats funs from either formula  or control
          currstats <- control$date.stats
          currtest <- control$date.test
          vartype <- "Date"

        } else if(is.selectall(currcol)) {
          xlevels <- colnames(currcol)

          currstats <- control$selectall.stats
          currtest <- control$selectall.test
          vartype <- "selectall"

        } else if(inherits(currcol, "Surv")) {
          ##### Survival (time to event) #######
          xlevels <- NULL
          if(any(currcol[, 2] %nin% c(0:1, NA))) stop("Survival endpoint may not be coded 0/1.")

          currstats <- control$surv.stats
          currtest <- control$surv.test
          vartype <- "survival"

        } else if(is.numeric(currcol) || inherits(currcol, "difftime")) {
          ######## Continuous variable (numeric) ###############

          ## for difftime, convert to numeric
          if(inherits(currcol, "difftime")) currcol <- as.numeric(currcol)
          xlevels <- sort(unique(currcol))

          ## if no missings, and control says not to show missings,
          ## remove Nmiss stat fun
          currstats <- control$numeric.stats
          currtest <- control$numeric.test
          vartype <- "numeric"
        } else stop("Variable ", names(xTerms), " has unknown class(es): ", paste0(class(currcol)[-1], collapse = ", "))
        ############################################################

        ## if no missings, and control says not to show missings,
        ## remove Nmiss stat fun
        if(!is.null(attrstats <- attr(modeldf[[eff]], "stats"))) currstats <- attrstats

        # now finally subset
        currcol <- currcol[strata.col == strat]

        if(vartype == "categorical") {
          tmpdl <- control.list[[eff]]$cat.droplevels
          if(is.null(tmpdl)) tmpdl <- control$cat.droplevels
          if(tmpdl) {
            currcol <- droplevels(currcol)
            xlevels <- levels(currcol)
            if(length(xlevels) == 0) stop(paste0("Zero-length levels found for ", names(xTerms)[eff]))
            if(!control$test.always) specialTests[eff] <- "notest"
          }
        }

        if(!anyNA(currcol)) {
          if("Nmiss" %in% currstats) currstats <- currstats[currstats != "Nmiss"]
          if("Nmisspct" %in% currstats) currstats <- currstats[currstats != "Nmisspct"]
        }
        statList <- list()
        for(statfun2 in currstats) {
          statfun <- get_stat_function(statfun2)
          tmp <- get0(statfun, mode = "function")
          statfun <- if(is.null(tmp)) get(statfun, parent.frame(), mode = "function") else tmp

          bystatlist <- list()
          if(statfun2 %in% c("countrowpct", "countcellpct", "rowbinomCI", "Nrowpct", "rowpct"))
          {
            bystatlist <- do.call(statfun, list(
              currcol, levels = xlevels,
              by = bycol, by.levels = by.levels,
              weights = weightscol,
              na.rm = TRUE,
              totallab = totallab
            ))
          } else
          {
            for(bylev in by.levels) {
              idx <- bycol == bylev
              bystatlist[[bylev]] <- do.call(statfun, list(currcol[idx], levels=xlevels, na.rm=TRUE,
                                                           weights=weightscol[idx], conf.level=control$conf.level, times=control$times, survconf.type=control$survconf.type))
            }
            ## add Total
            bystatlist[[totallab]] <- do.call(statfun, list(currcol, levels=xlevels, na.rm=TRUE,
                                                            weights=weightscol, conf.level=control$conf.level, times=control$times, survconf.type=control$survconf.type))
          }
          statList[[statfun2]] <- bystatlist
        }
        if(length(statList) == 0) stop(paste0("Nothing to show for variable '", names(xTerms)[eff], "'"))

        currtest <- if(nchar(specialTests[eff]) > 0) specialTests[eff] else currtest
        testout <- if(control$test) {
          eval(call(currtest, currcol, factor(bycol, levels = by.levels),
                    chisq.correct=control$chisq.correct, simulate.p.value=control$simulate.p.value, B=control$B,
                    wilcox.correct = control$wilcox.correct, wilcox.exact = control$wilcox.exact,
                    test.always=control$test.always))
        } else notest()

        xList[[eff]] <- list(stats=statList, test=testout, type=vartype)
      }

      strataList[[if(!hasStrata) 1 else paste0("(", strataTerm, ") == ", strat)]] <- xList
    }
    out.tables[[termBy]] <- list(y = yList, strata = list(term = strataTerm, values = strata.levels, label = strataLabel, hasStrata = hasStrata),
                                 x = xTerms, tables = strataList, control.list = control.list, hasWeights = hasWeights)
  }
  structure(list(Call = Call, control = control, tables = out.tables), class = c("tableby", "arsenal_table"))
}

#' \code{arsenal} tables with common structure
#'
#' @param x,y,object An object of class \code{"arsenal_table"}
#' @param i,j A vector to index \code{x} with: either names of variables, a numeric vector, or a logical vector of appropriate length.
#'   \code{i} indexes the x-variables, and \code{j} indexes the by-variables.
#' @param value A list of new labels.
#' @param all,all.x,all.y Logicals, denoting which terms to keep if not all are in common.
#' @param ... Other arguments (only used in \code{print.summary.arsenal_table})
#' @param format Passed to \code{\link[knitr]{kable}}: the format for the table. The default here is "markdown".
#'   To use the default in \code{kable}, pass \code{NULL}. If \code{x$text} specifies LaTeX or HTML formatting,
#'   that format is used in the table.
#' @param escape Passed to \code{\link[knitr]{kable}}: should special characters be escaped when printed?
#' @param width,min.split Passed to \code{\link{smart.split}} for formatting of the "term" column.
#' @seealso \code{\link{merge}}, \code{\link{labels}}
#' @name arsenal_table
NULL
#> NULL

#' @rdname arsenal_table
#' @export
has_strata <- function(x) vapply(x$tables, function(x) x$strata$hasStrata, NA)

na_lhs_strata <- function(object, ...) {
  omit <- is.na(object[[1]])
  if("(strata)" %in% names(object)) omit <- omit | is.na(object[["(strata)"]])

  xx <- object[!omit, , drop = FALSE]
  if(any(omit)) {
    temp <- stats::setNames(seq_along(omit)[omit], attr(object, "row.names")[omit])
    attr(temp, "class") <- "omit"
    attr(xx, "na.action") <- temp
  }
  xx
}

#' @rdname arsenal_table
#' @export
`[.arsenal_table` <- function(x, i, j, ...) {
  if(missing(i) && missing(j)) return(x)
  newx <- x

  give_warn <- function(vec) warning(paste0("Some indices not found in object: ", paste0(vec, collapse = ", ")), call. = FALSE)
  if(!missing(j))
  {
    if(is.character(j))
    {
      if(any(tmp <- j %nin% names(newx$tables)))
      {
        give_warn(j[tmp])
        j <- j[!tmp]
      }
      # we could leave "j" alone here and use the names, but for when names are missing ("") we'll do this
      j <- match(j, names(newx$tables), nomatch = 0)
    } else if(is.numeric(j) && any(tmp <- j %nin% seq_along(newx$tables)))
    {
      give_warn(j[tmp])
      j <- j[!tmp]
    } else if(is.logical(j) && length(j) != length(newx$tables))
    {
      stop("Logical vector index not the right length")
    }
    if(length(j) == 0 || anyNA(j)) stop("Indices must have nonzero length and no NAs.")
    newx$tables <- newx$tables[j]
  }

  if(!missing(i))
  {
    newx$tables <- lapply(newx$tables, function(yList) {
      if(is.character(i) && any(tmp <- i %nin% names(yList$x)))
      {
        give_warn(i[tmp])
        i <- i[!tmp]
        # we expect the names to be non-missing (""), unlike y
      } else if(is.numeric(i) && any(tmp <- i %nin% seq_along(yList$x)))
      {
        give_warn(i[tmp])
        i <- i[!tmp]
      } else if(is.logical(i) && length(i) != length(yList$x))
      {
        stop("Logical vector index not the right length")
      }
      if(length(i) == 0 || anyNA(i)) stop("Indices must have nonzero length and no NAs.")
      yList$x <- yList$x[i]
      yList$tables <- lapply(yList$tables, "[", i)
      yList
    })
  }
  newx
}

#' @rdname arsenal_table
#' @export
`[.tableby` <- function(x, i, j, ...) {
  give_warn <- function(vec) warning(paste0("Some indices not found in object: ", paste0(vec, collapse = ", ")), call. = FALSE)
  if(!missing(j) && is.list(j))
  {
    if(is.null(names(j))) stop("A list used to index must have names")
    for(nm in names(j)) {
      k <- j[[nm]]
      if(nm %nin% names(x$tables)) stop("Can't find term '", nm, "'")
      jj <- match(nm, names(x$tables), nomatch = 0)

      if(is.character(k)) {
        if(any(tmp <- k %nin% names(x$tables[[jj]]$y$stats)))
        {
          give_warn(k[tmp])
          k <- k[!tmp]
        }
        # we could leave "k" alone here and use the names, but for when names are missing ("") we'll do this
        k <- match(k, names(x$tables[[jj]]$y$stats), nomatch = 0)
      } else if(is.numeric(k) && any(tmp <- k %nin% seq_along(x$tables[[jj]]$y$stats)))
      {
        give_warn(k[tmp])
        k <- k[!tmp]
      } else if(is.logical(k) && length(k) != length(x$tables[[jj]]$y$stats))
      {
        stop("Logical vector index not the right length")
      } else if(is.logical(k) && !any(k)) stop("Logical vector must have at least one TRUE element")

      if(length(k) == 0 || anyNA(k)) stop("Indices must have nonzero length and no NAs.")
      x$tables[[jj]]$y$stats <- x$tables[[jj]]$y$stats[k]
      for(ss in seq_along(x$tables[[jj]]$tables)) {
        for(xx in seq_along(x$tables[[jj]]$tables[[ss]])) {
          for(tt in seq_along(x$tables[[jj]]$tables[[ss]][[xx]]$stats)) {
            x$tables[[jj]]$tables[[ss]][[xx]]$stats[[tt]] <- x$tables[[jj]]$tables[[ss]][[xx]]$stats[[tt]][k]
          }
        }
      }
    }
    j <- names(j)
  }
  NextMethod()
}

#' @rdname arsenal_table
#' @export
labels.arsenal_table <- function(object, ...) {

  get_lab <- function(x)
  {
    xLabs <- vapply(x$x, "[[", NA_character_, "label")
    aLabs <- if(is.null(x$adjust)) NULL else vapply(x$adjust, "[[", NA_character_, "label")
    c(stats::setNames(x$y$label, x$y$term), xLabs, aLabs)
  }

  labs <- unlist(unname(lapply(object$tables, get_lab)), recursive = FALSE)
  labs[!duplicated(labs) | !duplicated(names(labs))]
}


make_ms_labs <- function(x)
{
  tmp <- x$term
  for(i in seq_along(x$varterm2))
  {
    colon <- if(i > 1) ":" else ""
    space <- if(all(grepl(paste0(x$varterm2[i], ":"), tmp, fixed = TRUE))) "" else " "

    tmp <- sub(paste0(colon, x$varterm2[i]), paste0(colon, x$varlabel[i], space), tmp, fixed = TRUE)
  }
  x$label <- trimws(tmp)
  x
}

#' @rdname arsenal_table
#' @export
`labels<-.arsenal_table` <- function(x, value) {
  ## if the value vector is named, then assign the labels to
  ## those names that match those in x and y
  if(is.list(value)) value <- unlist(value)
  if(is.null(value))
  {
    for(i in seq_along(x$tables))
    {
      mk_lab <- function(elt)
      {
        if(is.null(elt$variable2) || is.null(elt$varlabel) || is.null(elt$varterm))
        { # for tableby
          elt$label <- elt$term
        } else
        {
          elt$varlabel <- elt$varterm
          elt <- make_ms_labs(elt)
        }
        elt
      }
      for(j in seq_along(x$tables[[i]]$x)) x$tables[[i]]$x[[j]] <- mk_lab(x$tables[[i]]$x[[j]])
      for(j in seq_along(x$tables[[i]]$adjust)) x$tables[[i]]$adjust[[j]] <- mk_lab(x$tables[[i]]$adjust[[j]])
      x$tables[[i]]$y$label <- x$tables[[i]]$y$term
      x$tables[[i]]$strata$label <- x$tables[[i]]$strata$term

    }
  } else if(!is.null(names(value))) {
    for(L in seq_along(value))
    {
      for(i in seq_along(x$tables))
      {
        nm <- names(value)[L]
        val <- unname(value[L])
        for(j in seq_along(x$tables[[i]]$x))
        {
          if(nm %in% x$tables[[i]]$x[[j]]$term)
          {
            x$tables[[i]]$x[[j]]$label[x$tables[[i]]$x[[j]]$term == nm] <- val
          } else if(nm %in% x$tables[[i]]$x[[j]]$term.orig)
          {
            # for backwards-compatibility
            x$tables[[i]]$x[[j]]$label[x$tables[[i]]$x[[j]]$term.orig == nm] <- val
          }
        }
        for(j in seq_along(x$tables[[i]]$adjust))
        {
          if(nm %in% x$tables[[i]]$adjust[[j]]$term) x$tables[[i]]$adjust[[j]]$label[x$tables[[i]]$adjust[[j]]$term == nm] <- val
        }

        if(nm == x$tables[[i]]$y$term) x$tables[[i]]$y$label <- val
        if(nm %in% x$tables[[i]]$strata$term) x$tables[[i]]$strata$label[x$tables[[i]]$strata$term == nm] <- val
      }
    }
  } else stop("Unnamed label assignments are no longer supported")
  x
}

#' @rdname arsenal_table
#' @export
print.arsenal_table <- function(x, ...)
{
  cat(class(x)[1], "Object\n\nFunction Call:\n")
  print(x$Call)
  cat("\nVariable(s):\n")
  lapply(x$tables, function(tab) {
    cat(tab$y$term, " ~ ", paste0(unlist(lapply(tab$x, "[[", "term"), use.names = FALSE), collapse = ", "),
        if(tab$strata$hasStrata) paste0(" (strata = ", paste0(tab$strata$term, collapse = ", "), ")"), "\n", sep = "")
  })
  invisible(x)
}

#' @rdname arsenal_table
#' @export
merge.arsenal_table <- function(x, y, all = FALSE, all.x = all, all.y = all, ...) {
  Call <- match.call()

  nms.x <- names(x$tables)
  nms.y <- names(y$tables)
  nms <- if(all.x && all.y) union(nms.x, nms.y) else if(all.x) nms.x else if(all.y) nms.y else intersect(nms.x, nms.y)
  if(length(nms) == 0) stop("No terms in common. Do you need 'all=TRUE'?")
  x <- x[, nms.x %in% nms] # could use names themselves with warn=FALSE, but for freqlist, let's do this instead
  y <- y[, nms.y %in% nms]
  nms.x <- names(x$tables)
  nms.y <- names(y$tables)

  for(i in seq_along(y$tables))
  {
    ytrm <- names(y$tables)[i]
    if(ytrm == "")
    {
      x$tables[[length(x$tables) + 1]] <- y$tables[[i]] # this is why we need 'i' instead of just 'ytrm'
      next
    } else if(ytrm %nin% names(x$table))
    {
      x$tables[[ytrm]] <- y$tables[[ytrm]]
      next
    }
    if(!identical(x$tables[[ytrm]]$y, y$tables[[ytrm]]$y)) stop("By-variables not identical for term ", ytrm)
    if(!identical(x$tables[[ytrm]]$strata, y$tables[[ytrm]]$strata)) stop("Strata not identical for term ", ytrm)
    if(!identical(x$tables[[ytrm]]$adjust, y$tables[[ytrm]]$adjust)) stop("Adjust not identical for term ", ytrm)
    if(x$tables[[ytrm]]$hasWeights != y$tables[[ytrm]]$hasWeights) stop("Weights not present in both objects for term ", ytrm)
    if(!identical(x$tables[[ytrm]]$family, y$tables[[ytrm]]$family)) stop("'family' not the same in both objects for term ", ytrm)

    xtrms <- names(y$tables[[ytrm]]$x)
    x$tables[[ytrm]]$x[xtrms] <- y$tables[[ytrm]]$x
    if(!is.null(x$tables[[ytrm]]$control.list) && !is.null(y$tables[[ytrm]]$control.list))
      x$tables[[ytrm]]$control.list[xtrms] <- y$tables[[ytrm]]$control.list

    for(j in seq_along(x$tables[[ytrm]]$tables))
    {
      x$tables[[ytrm]]$tables[[j]][xtrms] <- y$tables[[ytrm]]$tables[[j]]
    }
  }

  x$Call <- Call
  x
}

#' @rdname arsenal_table
#' @export
merge.freqlist <- function(x, y, all = TRUE, ...)
{
  nms.x <- setdiff(names(x$tables), "")
  nms.y <- setdiff(names(y$tables), "")
  # this check should be okay even if both nms.x and nms.y are empty
  if(any(nms.x %in% nms.y)) stop("Can only merge freqlist objects with different left-hand sides")
  NextMethod(all = TRUE)
}

#' @rdname arsenal_table
#' @export
print.summary.arsenal_table <- function(x, ..., format = if(!is.null(x$text) && x$text %in% c("html", "latex")) x$text else "markdown",
                                        escape = x$text %nin% c("html", "latex"), width = NULL, min.split = NULL)
{
  df <- as.data.frame(x, ..., width = width, min.split = min.split, list.ok = TRUE)

  #### finally print it out ####
  shown <- FALSE
  for(i in seq_along(df))
  {
    if(nrow(df[[i]]) == 0) next
    shown <- TRUE
    cap <- if(length(x$title) < i) NULL else x$title[[i]]
    print(knitr::kable(df[[i]], caption = cap, align = attr(df[[i]], "align"), format = format, row.names = FALSE, escape = escape, ...))
    if(!is.null(attr(df[[i]], "tests"))) cat(paste0(attr(df[[i]], "tests"), "\n", collapse = ""))
  }
  if(!shown) stop("There wasn't anything to summarize! (All of the tables have 0 rows)")
  cat("\n")

  invisible(x)
}

#' Control settings for \code{freqlist} function
#'
#' Control test and summary settings for the \code{\link{freqlist}} function.
#'
#' @param sparse a logical value indicating whether to keep rows with counts of zero.
#'   The default is \code{FALSE} (drop zero-count rows).
#' @param single logical, indicating whether to collapse results created using a strata variable into a single table for printing
#' @param dupLabels logical: should labels which are the same as the row above be printed? The default (\code{FALSE}) more
#'   closely approximates \code{PROC FREQ} output from SAS, where a label carried down from the row above is left blank.
#' @param digits.count Number of decimal places for count values.
#' @param digits.pct Number of decimal places for percents.
#' @param ... additional arguments.
#' @param digits A deprecated argument
#' @return A list with settings to be used within the \code{freqlist} function.
#'
#' @seealso \code{\link{freqlist}}, \code{\link{summary.freqlist}}, \code{\link{freqlist.internal}}
#' @author Ethan Heinzen
#' @export
freq.control <- function(sparse = FALSE, single = FALSE, dupLabels = FALSE, digits.count = 0L, digits.pct = 2L, ..., digits = NULL)
{
  if(!is.null(digits))
  {
    .Deprecated(msg = "Using 'digits = ' is deprecated. Use 'digits.pct = ' instead.")
    digits.pct <- digits
  }

  # digits are OK to be NULL. See ?format
  if(!is.null(digits.count) && digits.count < 0L)
  {
    warning("digits.count must be >= 0. Set to default.")
    digits.count <- 0L
  }
  if(!is.null(digits.pct) && digits.pct < 0L)
  {
    warning("digits.pct must be >= 0. Set to default.")
    digits.pct <- 1L
  }

  list(sparse = sparse, single = single, dupLabels = dupLabels, digits.count = digits.count, digits.pct = digits.pct)
}

### Test functions #######

notest <- function(x, x.by, ...)
{
  list(p.value = NA_real_, method = "No test")
}

stddiff <- function(x, x.by, ..., test.always = FALSE) {
  tab <- table(is.na(x), x.by)
  cs <- colSums(tab)
  if(!test.always && (any(tab[1, ] == 0) || any(colSums(tab) == 0))) {
    return(list(p.value=NA_real_, method="Standardized difference"))
  }
  if(sum(cs > 0) != 2) stop("Standardized differences require exactly two by-groups", call. = FALSE)
  tab <- tab[, cs > 0]
  trt <- levels(x.by)[cs > 0][2]
  x.by <- +(x.by == trt)
  check_pkg("stddiff")
  if(is.ordered(x) || inherits(x, "Surv") || is.selectall(x)) {
    stop("Can't run standardized differences unless the object is factor-ish or numeric-ish", call. = FALSE)
  } else if(is.factor(x)) {
    if(length(levels(x)) < 2) stop("At least two levels are required to run standardized differences")
    if(length(levels(x)) == 2) {
      s <- stddiff::stddiff.binary(data.frame(x, x.by), 2, 1)
    } else {
      s <- stddiff::stddiff.category(data.frame(x, x.by), 2, 1)
    }
  } else if(is.numeric(x) || inherits(x, "difftime") || is.Date(x)) {
    s <- stddiff::stddiff.numeric(data.frame(x, x.by), 2, 1)
  } else stop("Can't run standardized differences unless the object is categorical (but not ordered), numeric, or date", call. = FALSE)

  list(p.value=s[1, "stddiff"], method="Standardized difference")
}


## continuous tests:
## 1. anova  (parametric)
## consider allowing glm, for now just lm with gaussian errors
## Would like to just use either "anova" or "aov", anova needs lm(group~x first,
## aov does not return p-value. Could add it after.
## For now, just write our own to avoid over-writing anova R-base function
## also, nice to keep same format to call, eval(call(function, x, x,by)), as other tests
anova <- function(x, x.by, ..., test.always = FALSE) {
  tab <- table(is.na(x), x.by)
  if(!test.always && (any(tab[1, ] == 0) || any(colSums(tab) == 0))) {
    return(list(p.value=NA_real_, statistic.F=NA_real_, method="Linear Model ANOVA"))
  }
  aov.out <- stats::lm(x~x.by)
  test <- stats::anova(aov.out)
  list(p.value = test[1,ncol(test)],
       statistic.F = test[1,ncol(test)-1],
       method = "Linear Model ANOVA")
}
## 2. kruskal-wallis (non-parametric)
kwt <- function(x, x.by, ..., test.always = FALSE) {
  tab <- table(is.na(x), x.by)
  if(!test.always && (any(tab[1, ] == 0) || any(colSums(tab) == 0))) {
    return(list(p.value=NA_real_, statistic.F=NA_real_, method="Kruskal-Wallis rank sum test"))
  }
  stats::kruskal.test(x, as.factor(x.by))
}

## 2. wilcoxon (non-parametric)
wt <- function(x, x.by, ..., wilcox.correct = FALSE, wilcox.exact = NULL, test.always = FALSE) {
  tab <- table(is.na(x), x.by)
  if(ncol(tab) != 2) stop("The Wilcoxon Rank Sum test must have exactly two groups")

  if(!test.always && (any(tab[1, ] == 0) || any(colSums(tab) == 0))) {
    return(list(p.value=NA_real_, statistic.F=NA_real_, method="Wilcoxon rank sum test"))
  }
  stats::wilcox.test(x ~ as.factor(x.by), correct = wilcox.correct, exact = wilcox.exact)
}

## median test
medtest <- function(x, x.by, ..., test.always = FALSE) {
  if(!requireNamespace("coin", quietly = TRUE))
  {
    warning("The \"coin\" package is required to run a median test.", call. = FALSE)
    return(notest(x, x.by, ...))
  }

  tab <- table(is.na(x), x.by)
  if(!test.always && (any(tab[1, ] == 0) || any(colSums(tab) == 0))) {
    return(list(p.value=NA_real_, method = "Median test"))
  }
  ## should be taken care of with coin::
  check_pkg("coin")
  mtest <- coin::median_test(x~as.factor(x.by), teststat="quad")
  list(p.value=coin::pvalue(mtest), method="Median test", statistic=mtest@statistic@teststatistic)
}

## two tests for categorical,
## 1. chisq goodness of fit, equal proportions across table cells
chisq <- function(x, x.by, ..., chisq.correct=FALSE, simulate.p.value=FALSE, B=2000, test.always = FALSE) {
  tab <- table(x, x.by, exclude=NA)
  rs <- rowSums(tab)
  cs <- colSums(tab)

  if(!test.always && (any(rs == 0) || any(cs == 0)) && ncol(tab) > 1 && nrow(tab) > 1) {
    return(list(p.value=NA_real_, method="Pearson's Chi-squared test"))
  }
  if(length(cs) > 1) tab <- tab[rs > 0, , drop = FALSE]
  if(length(rs) > 1) tab <- tab[, cs > 0, drop = FALSE]
  suppressWarnings(stats::chisq.test(tab, correct=chisq.correct, simulate.p.value=simulate.p.value, B=B))
}

## 2. Fisher's exact test for prob of as or more extreme table
fe <- function(x, x.by, ..., simulate.p.value=FALSE, B=2000, test.always = FALSE) {
  tab <- table(x, x.by, exclude=NA)
  rs <- rowSums(tab)
  cs <- colSums(tab)

  if((!test.always && (any(rs == 0) || any(cs == 0))) || ncol(tab) == 1 || nrow(tab) == 1) {
    return(list(p.value=NA_real_, method = "Fisher's Exact Test for Count Data"))
  }
  # this already subsets out rows and cols with all 0's
  stats::fisher.test(tab, simulate.p.value=simulate.p.value, B=B)
}

## trend test for ordinal data
trend <- function(x, x.by, ..., test.always = FALSE) {
  if(!requireNamespace("coin", quietly = TRUE))
  {
    warning("The \"coin\" package is required to run a trend test.", call. = FALSE)
    return(notest(x, x.by, ...))
  }

  tab <- table(x, x.by, exclude=NA)
  rs <- rowSums(tab)
  cs <- colSums(tab)

  if(!test.always && (any(rs == 0) || any(cs == 0))) {
    return(list(p.value=NA_real_, method = "Trend test for ordinal variables"))
  }
  ## should be taken care of with coin::
  check_pkg("coin")
  indtest <- coin::independence_test(x~as.factor(x.by), teststat="quad")
  list(p.value=coin::pvalue(indtest), method="Trend test for ordinal variables", statistic=indtest@statistic@teststatistic)
}

## ' logrank
## '
## ' survdiff logrank test
## ' @param x  surv variable
## ' @param x.by  by, categorical variable
## ' @return   test output with $method and $p.value
logrank <- function(x, x.by, ..., test.always = FALSE) {
  tab <- table(is.na(x), x.by, exclude=NA)
  if(!test.always && (any(tab[1, ] == 0) || any(colSums(tab) == 0))) {
    return(list(p.value=NA_real_, method="survdiff logrank"))
  }
  out <- survival::survdiff(x ~ x.by)
  out$p.value <- 1-stats::pchisq(out$chisq, df=sum(tab[1,] != 0)-1)
  out$method <- "survdiff logrank"
  out
}

capture.kable <- function(x)
{
  y <- capture.output(x)
  stopifnot(length(y) > 3)
  if(grepl("^Table:", y[2])) y <- c("", y)
  if(y[1] != "" || y[2] != "" || tail(y, 1) != "") stop("Leading and/or trailing elements aren't blank")
  utils::head(utils::tail(y, -2), -1)
}

#' Summary Statistics of a Set of Independent Variables Paired Across Two Timepoints
#'
#' Summarize one or more variables (x) by a paired time variable (y). Variables
#'   on the right side of the formula, i.e. independent variables, are summarized by the
#'   two time points on the left of the formula. Optionally, an appropriate test is performed to test the
#'   distribution of the independent variables across the time points.
#' @param formula an object of class \code{\link{formula}} of the form \code{time ~ var1 + ...}.
#'   See "Details" for more information.
#' @inheritParams tableby
#' @param id The vector giving IDs to match up data for the same subject across two timepoints.
#' @param na.action a function which indicates what should happen when the data contain \code{NA}s.
#'   The default is \code{na.paired("in.both")}. See \code{\link{na.paired}} for more details
#' @param control control parameters to handle optional settings within \code{paired}.
#'   Two aspects of \code{paired} are controlled with these: test options of RHS variables and x variable summaries.
#'   Arguments for \code{paired.control} can be passed to \code{paired} via the \code{...} argument,
#'   but if a control object and \code{...} arguments are both supplied,
#'   the latter are used. See \code{\link{paired.control}} for more details.
#' @param ... additional arguments to be passed to internal \code{paired} functions or \code{\link{paired.control}}.
#' @return An object with class \code{c("paired", "tableby", "arsenal_table")}
#' @details
#' Do note that this function piggybacks off of \code{\link{tableby}} quite heavily, so there is no
#' \code{summary.paired} function (for instance).
#'
#' These tests are accepted:
#' \itemize{
#'   \item{
#'     \code{paired.t}: a paired \code{\link[stats:t.test]{t-test}}.
#'   }
#'   \item{
#'     \code{mcnemar}: \link[stats:mcnemar.test]{McNemar's test}.
#'   }
#'   \item{
#'     \code{signed.rank}: a \link[stats:wilcox.test]{signed rank test}.
#'   }
#'   \item{
#'     \code{signtest}: a sign test.
#'   }
#'   \item{
#'     \code{notest}: no test is performed.
#'   }
#' }
#' @seealso \code{\link{arsenal_table}}, \code{\link{paired.control}}, \code{\link{tableby}}, \code{\link{formulize}}, \code{\link{selectall}}
#' @author Jason Sinnwell, Beth Atkinson, Ryan Lennon, and Ethan Heinzen
#' @name paired
NULL
#> NULL

#' @rdname paired
#' @export
paired <- function(formula, data, id, na.action, subset=NULL, strata, control = NULL, ...) {
  control <- c(list(...), control)
  control <- do.call("paired.control", control[!duplicated(names(control))])

  Call <- match.call()

  ## Tell user if they passed an argument that was not expected, either here or in control
  expectArgs <- c("formula", "data", "na.action", "subset", "strata", "control", names(control), "id")
  match.idx <- match(names(Call)[-1], expectArgs)
  if(anyNA(match.idx)) warning("unused arguments: ", paste(names(Call)[1+which(is.na(match.idx))], collapse=", "), "\n")

  indx <- match(c("formula", "data", "subset", "na.action", "id", "strata"), names(Call), nomatch = 0)
  if(indx[1] == 0) stop("A formula argument is required")
  if(length(formula) != 3) stop("'formula' must be two-sided.")
  if(indx[5] == 0) stop("An id argument is required")

  special <- c("paired.t", "mcnemar", "signed.rank", "signtest", "sign.test", "notest")

  out.tables <- list()
  formula.list <- as_list_formula(formula)
  for(FORM in formula.list)
  {
    temp.call <- Call[c(1, indx)]
    temp.call[[1]] <- as.name("model.frame")

    if(is.null(temp.call$na.action)) temp.call$na.action <- na.paired("in.both")

    if (missing(data)) {
      temp.call$formula <- stats::terms(FORM, special)
    } else {
      # instead of call("keep.labels", ...), which breaks when arsenal isn't loaded (Can't find "keep.labels")
      temp.call$data <- as.call(list(keep.labels, temp.call$data))
      temp.call$formula <- stats::terms(FORM, special, data = keep.labels(data))
    }

    ##  set up new environment for
    ## if specials, assign dummy versions of those functions
    tabenv <- new.env(parent = environment(formula))
    for(sp in special)
    {
      if(!is.null(attr(temp.call$formula, "specials")[[sp]])) assign(sp, inline_tableby_stat_test, envir = tabenv)
    }

    ## set tabenv as environment in which to evalulate formula
    environment(temp.call$formula) <- tabenv

    ## evaluate the formula with env set for it
    modeldf <- loosen.labels(eval.parent(temp.call))
    if(nrow(modeldf) == 0) stop("No (non-missing) observations")

    Terms <- stats::terms(modeldf)

    ###### Check for strata ######

    if(hasStrata <- "(strata)" %in% colnames(modeldf))
    {
      strata.col <- modeldf[["(strata)"]]
      strataTerm <- paste(deparse(Call$strata), collapse = " ")
      if(is.null(strataLabel <- attr(strata.col, "label"))) strataLabel <- strataTerm
      if(is.factor(strata.col))
      {
        strata.col <- droplevels(strata.col)
        strata.levels <- levels(strata.col)
      } else strata.levels <- sort(unique(strata.col))

      modeldf[["(strata)"]] <- NULL
    } else
    {
      strata.col <- rep("", nrow(modeldf))
      strataTerm <- strataLabel <- strata.levels <- ""
    }

    ###### Check for by-variable ######

    if(attributes(Terms)$response != 0)
    {
      by.col <- modeldf[[1]]
      termBy <- names(modeldf)[1]
      if(is.null(labelBy <- attr(by.col, "label"))) labelBy <- termBy
      if(is.factor(by.col)) {
        by.col <- droplevels(by.col)
        by.levels <- levels(by.col)
      } else by.levels <- sort(unique(by.col))
      by.col <- as.character(by.col)
      by.levels <- as.character(by.levels)

      if(any(by.levels == ""))
      {
        warning('Empty string detected in by-variable is not allowed; converting to " ".')
        by.col[by.col == ""] <- " "
        by.levels <- unique(replace(by.levels, by.levels == "", " "))
      }
      modeldf[[1]] <- NULL
    }

    if(length(by.levels) != 2) stop("Please specify exactly 2 time points")
    ids <- modeldf$`(id)`
    tab <- table(ids, by.col)
    if(sum(tab > 1) > 0)
      stop("At least one person has multiple observations for at least one time point")
    if(sum(rowSums(tab) == 2) == 0)
      stop("No one appears to have data on both time points")

    ids.both <- intersect(ids[by.col == by.levels[1]], ids[by.col == by.levels[2]])

    strata.col1 <- strata.col[by.col == by.levels[1]]
    TP1 <- modeldf[by.col == by.levels[1], , drop = FALSE]
    strata.col2 <- strata.col[by.col == by.levels[2]]
    TP2 <- modeldf[by.col == by.levels[2], , drop = FALSE]

    cn <- colnames(modeldf)
    cn <- cn[cn != "(id)"]

    idx1 <- match(ids.both, TP1$`(id)`, nomatch = 0)
    idx2 <- match(ids.both, TP2$`(id)`, nomatch = 0)
    strata.col1 <- strata.col1[idx1]
    strata.col2 <- strata.col2[idx2]
    TP1 <- TP1[idx1, cn, drop = FALSE]
    TP2 <- TP2[idx2, cn, drop = FALSE]
    modeldf[["(id)"]] <- NULL

    if(is.null(difflab <- control$stats.labels$difference)) difflab <- "Difference"
    ystats <- c(table(factor(by.col, levels=by.levels), exclude=NA), Difference=length(ids.both))
    names(ystats)[names(ystats) == "Difference"] <- difflab
    yList <- list(stats=ystats, label=labelBy, term=termBy)


    ## find which columnss of modeldf have specials assigned to known specials
    specialIndices <- unlist(attr(Terms, "specials")) - attributes(Terms)$response
    specialTests <- rep("", ncol(modeldf))
    ## If a special shows up multiple times, the unlist assigned a number at the end. Strip it off.
    ## This disallows functions with a number at the end
    specialTests[specialIndices] <- gsub("\\d+$", "", names(specialIndices))

    xTerms <- Map(modeldf, names(modeldf), f = function(col, nm) {
      if(is.null(nameEff <- attr(col, "name"))) nameEff <- nm
      if(is.null(labelEff <- attr(col, "label"))) labelEff <- nameEff
      if(is.null(termEff <- attr(col, "term"))) termEff <- nm
      list(variable=nameEff, label=labelEff, term=termEff, term.orig = nm)
    })
    names(xTerms) <- vapply(xTerms, "[[", NA_character_, "variable")

    control.list <- lapply(modeldf, attr, "control.list")
    names(control.list) <- names(xTerms)

    strataList <- vector("list", length(strata.levels))
    if(hasStrata) names(strataList) <- paste0("(", strataTerm, ") == ", strata.levels)

    for(strat in strata.levels)
    {
      ## list of x variables
      xList <- vector("list", ncol(modeldf))
      names(xList) <- names(xTerms)

      bycol <- by.col[strata.col == strat]

      for(eff in seq_along(modeldf)) {

        currcol <- modeldf[[eff]]
        TP1.eff <- TP1[[eff]]
        TP2.eff <- TP2[[eff]]

        ############################################################

        if(is.ordered(currcol) || is.logical(currcol) || is.factor(currcol) || is.character(currcol)) {
          ######## ordinal or categorical variable (character or factor) ###############

          ## convert logicals and characters to factor
          if(is.character(currcol))
          {
            lvl <- sort(unique(currcol[!is.na(currcol)]))
            currcol <- factor(currcol, levels = lvl)
            TP1.eff <- factor(TP1.eff, levels = lvl)
            TP2.eff <- factor(TP2.eff, levels = lvl)
          } else if(is.logical(currcol))
          {
            lvl <- c(FALSE, TRUE)
            currcol <- factor(currcol, levels=lvl)
            TP1.eff <- factor(TP1.eff, levels = lvl)
            TP2.eff <- factor(TP2.eff, levels = lvl)
          }


          ## to make sure all levels of cat variable are counted, need to pass values along
          xlevels <- levels(currcol)

          if(length(xlevels) == 0) stop(paste0("Zero-length levels found for ", names(xTerms)[eff]))

          ## get stats funs from either formula  or control
          if(is.ordered(currcol))
          {
            currstats <-  control$ordered.stats
            currtest <- control$ordered.test
            vartype <- "ordinal"
          } else
          {
            currstats <- control$cat.stats
            currtest <- control$cat.test
            vartype <- "categorical"
          }

        } else if(is.Date(currcol)) {
          ######## Date variable ###############
          xlevels <- sort(unique(currcol))

          ## get stats funs from either formula  or control
          currstats <- control$date.stats
          currtest <- control$date.test
          vartype <- "Date"

        } else if(is.selectall(currcol)) {
          xlevels <- colnames(currcol)

          currstats <- control$selectall.stats
          currtest <- control$selectall.test
          vartype <- "selectall"

        } else if(inherits(currcol, "Surv")) {
          ##### Survival (time to event) #######
          stop("Sorry, survival objects don't work in this function.")

        } else if(is.numeric(currcol) || inherits(currcol, "difftime")) {
          ######## Continuous variable (numeric) ###############

          ## for difftime, convert to numeric
          if(inherits(currcol, "difftime")) currcol <- as.numeric(currcol)
          xlevels <- sort(unique(currcol))

          ## if no missings, and control says not to show missings,
          ## remove Nmiss stat fun
          currstats <- control$numeric.stats
          currtest <- control$numeric.test
          vartype <- "numeric"
        } else stop("Variable ", names(xTerms), " has unknown class(es): ", paste0(class(currcol)[-1], collapse = ", "))
        ############################################################

        ## if no missings, and control says not to show missings,
        ## remove Nmiss stat fun
        if(!is.null(attrstats <- attr(modeldf[[eff]], "stats"))) currstats <- attrstats

        # now finally subset
        currcol <- currcol[strata.col == strat]
        TP1.eff <- TP1.eff[strata.col1 == strat]
        TP2.eff <- TP2.eff[strata.col2 == strat]
        if(!anyNA(currcol) && "Nmiss" %in% currstats) currstats <- currstats[currstats != "Nmiss"]
        statList <- list()
        for(statfun2 in currstats) {
          statfun <- get_stat_function(statfun2)
          tmp <- get0(statfun, mode = "function")
          statfun <- if(is.null(tmp)) get(statfun, parent.frame(), mode = "function") else tmp

          bystatlist <- list()
          if(statfun2 %in% c("countrowpct", "countcellpct", "rowbinomCI", "Nrowpct"))
          {
            bystatlist <- do.call(statfun, list(currcol, levels = xlevels,
                                                by = by.col, by.levels = by.levels, na.rm = TRUE))
            bystatlist$Total <- NULL
          } else if(statfun2 == "Nsigntest") {
            bystatlist <- as.tbstat(NA_real_, which.count = 1L)
          } else
          {
            for(bylev in by.levels) {
              idx <- bycol == bylev
              bystatlist[[bylev]] <- do.call(statfun, list(currcol[idx], levels=xlevels, na.rm=TRUE, conf.level = control$conf.level))
            }
          }

          if(is.selectall(currcol))
          {
            tmp <- as.selectall(TP1.eff != TP2.eff)
            bystatlist[[difflab]] <- do.call(statfun, list(tmp, levels=xlevels, na.rm=TRUE, conf.level = control$conf.level))
          } else if(statfun2 %in% c("countpct", "countrowpct", "countcellpct"))
          {
            # countrowpct to get the right percentages
            bystatlist[[difflab]] <- countrowpct(TP1.eff, levels = xlevels, by = TP1.eff == TP2.eff,
                                                 by.levels = c(TRUE, FALSE), na.rm = TRUE)[[2]]
          } else if(statfun2 %in% c("rowpct", "pct"))
          {
            # rowpct to get the right percentages
            bystatlist[[difflab]] <- rowpct(TP1.eff, levels = xlevels, by = TP1.eff == TP2.eff,
                                                 by.levels = c(TRUE, FALSE), na.rm = TRUE)[[2]]
          } else if(statfun2 == "count")
          {
            # this doesn't have percentages
            bystatlist[[difflab]] <- count(replace(TP1.eff, TP1.eff == TP2.eff, NA), levels = xlevels, na.rm = TRUE)
          } else if(statfun2 %in% c("binomCI", "rowbinomCI"))
          {
            bystatlist[[difflab]] <- rowbinomCI(TP1.eff, levels = xlevels, by = TP1.eff == TP2.eff,
                                                by.levels = c(TRUE, FALSE), na.rm = TRUE, conf.level = control$conf.level)[[2]]
          } else if(statfun2 == "Nrowpct")
          {
            # get the right percentages
            bystatlist[[difflab]] <- Nrowpct(TP1.eff, levels = xlevels, by = TP1.eff == TP2.eff, by.levels = c(TRUE, FALSE), na.rm = TRUE)[[2]]
          } else
          {
            bystatlist[[difflab]] <- do.call(statfun, list(as.numeric(TP2.eff) - as.numeric(TP1.eff),
                                                           levels=xlevels, na.rm=TRUE, conf.level = control$conf.level))
          }
          statList[[statfun2]] <- bystatlist
        }
        if(length(statList) == 0) stop(paste0("Nothing to show for variable '", names(xTerms)[eff], "'"))

        currtest <- if(nchar(specialTests[eff]) > 0) specialTests[eff] else currtest
        if(currtest == "sign.test") currtest <- "signtest"
        testout <- if(control$test) {
          eval(call(currtest, TP1.eff, TP2.eff, mcnemar.correct=control$mcnemar.correct,
                    signed.rank.exact = control$signed.rank.exact, signed.rank.correct = control$signed.rank.correct,
                    test.always=control$test.always))
        } else notest()

        xList[[eff]] <- list(stats=statList, test=testout, type=vartype)
      }

      strataList[[if(!hasStrata) 1 else paste0("(", strataTerm, ") == ", strat)]] <- xList
    }
    out.tables[[termBy]] <- list(formula = FORM, y = yList, strata = list(term = strataTerm, values = strata.levels, label = strataLabel, hasStrata = hasStrata),
                                 x = xTerms, tables = strataList, control.list = control.list, hasWeights = FALSE)
  }
  structure(list(Call = Call, control = control, tables = out.tables), class = c("paired", "tableby", "arsenal_table"))
}

#' Adjust P-values for Multiple Comparisons
#'
#' @param p An object.
#' @inheritParams stats::p.adjust
#' @param suffix A suffix to add to the footnotes indicating that the tests were adjusted.
#' @param ... Other arguments.
#' @seealso \code{\link[stats]{p.adjust}}, \code{\link{modpval.tableby}}, \code{\link{tests.tableby}}
#' @name padjust
NULL
#> NULL

#' @rdname padjust
#' @export
padjust <- function(p, method, n, ...) UseMethod("padjust")

#' @rdname padjust
#' @export
padjust.default <- function(p, method, n, ...)
{
  Call <- match.call()
  indx <- match(c("p", "method", "n"), names(Call), nomatch = 0)
  temp.call <- Call[c(1, indx)]
  temp.call[[1L]] <- quote(stats::p.adjust)

  eval(temp.call, parent.frame())
}

#' @rdname padjust
#' @export
padjust.tableby <- function(p, method, n, suffix = " (adjusted for multiple comparisons)", ...)
{
  Call <- match.call()
  if(any(has_strata(p)) || length(p$tables) > 1) stop("Can't adjust p-values on tables with strata or multiple by-variables.")
  if(!p$control$test) stop("Can't adjust p-values when no tests were run")
  indx <- match(c("p", "method", "n"), names(Call), nomatch = 0)
  temp.call <- Call[c(1, indx)]
  temp.call[[1L]] <- quote(stats::p.adjust)

  pvals <- tests(p)
  temp.call$p <- pvals$p.value
  pvals$p.value <- eval(temp.call, parent.frame())
  pvals$Method <- paste0(pvals$Method, suffix)
  modpval.tableby(p, pvals)
}

#' @rdname padjust
#' @export
padjust.summary.tableby <- function(p, method, n, suffix = " (adjusted for multiple comparisons)", ...)
{
  Call <- match.call()
  if(any(p$hasStrata) || length(p$object) > 1) stop("Can't adjust p-values on tables with strata or multiple by-variables")
  if(!p$control$test)
  {
    warning("No tests run on tableby object")
    return(p)
  }
  indx <- match(c("p", "method", "n"), names(Call), nomatch = 0)
  temp.call <- Call[c(1, indx)]
  temp.call[[1L]] <- quote(stats::p.adjust)
  pvals <- unique(p$object[[1]][c("variable", "p.value")]) # find unique variable-pval combos
  temp.call$p <- pvals$p.value
  pvals$p.value <- eval(temp.call, parent.frame())
  p$object[[1]][["p.value"]] <- pvals$p.value[match(p$object[[1]][["variable"]], pvals$variable)] # "merge" them back in
  p$object[[1]][["test"]] <- paste0(p$object[[1]][["test"]], suffix)
  p
}

magic8 <- function(question=NULL){
  sample(c("There is an 'apply' function for that",
         'It is decidedly so',
         'Without a doubt',
         "Need to install another bioconductor package for that",
         'Better off asking Siri',
         'You may rely on it',
         'As I see it, yes',
         'Most likely',
         "It might be time for a walking break",
         'Just bootstrap it',
         'May need to perform simulations',
         'Signs point to yes',
         "You need to go home and rethink your life",
         'Reply hazy; try again',
         'Insufficient sample size',
         'The answer is 7',
         "Bazinga!",
         'Cannot predict right now',
         'Concentrate and ask again',
         "Don't count on it",
         'Maybe, p-value 0.06',
         'Hierarchical clustering always gives an answer',
         'My sources say no',
         'Make a 3-D pie chart',
         "Yes, if the figure is colorful",
         'Very doubtful',
         "PC Load Letter",
         "Not Sure",
         "The possibility of successfully answering your question is approximately three thousand seven hundred and twenty to one!",
         "Depends...did you want the two-tailed probability or the one-tailed?",
         "Error: 404 Not Found",
         "You index is off by one (we won't tell)",
         "Correlation does not equal causation"), 1)
}

#' \code{comparedf} tolerances
#'
#' Internal functions defining tolerances for the \code{\link{comparedf.control}} function.
#'   To create your own tolerance definitions, see the vignette.
#'
#' @param x,y vectors of the appropriate lengths and types.
#' @param tol A numeric tolerance
#' @param idx A logical vector of appropriate length.
#' @return A logical vector of length equal to that of \code{x} and \code{y}, where \code{TRUE} denotes a
#'   difference between \code{x} and \code{y}, and \code{FALSE} denotes no difference between \code{x} and \code{y}.
#' @details
#' \code{tol.NA} takes as differences between two vectors any elements which are NA in one but not the other,
#'   or which are non-NA in both and \code{TRUE} in \code{idx}. It is useful for handling NAs in custom tolerance functions.
#' @author Ethan Heinzen
#' @seealso \code{\link{comparedf.control}}, \code{\link{comparedf}}
#' @name comparedf.tolerances
NULL
#> NULL


#' @rdname comparedf.tolerances
#' @export
tol.NA <- function(x, y, idx)
{
  (is.na(x) & !is.na(y)) | (is.na(y) & !is.na(x)) | (!is.na(x) & !is.na(y) & idx)
}

#' @rdname comparedf.tolerances
#' @export
tol.num.absolute <- function(x, y, tol)
{
  both.inf <- is.infinite(x) & is.infinite(y)
  tol.NA(x, y, (both.inf & x != y) | (!both.inf & abs(x - y) > tol))
}

#' @rdname comparedf.tolerances
#' @export
tol.num.percent <- tol.num.pct <- function(x, y, tol)
{
  both.inf <- is.infinite(x) & is.infinite(y)
  tol.NA(x, y, (x == 0 & y != 0) | (both.inf & x != y) | (!both.inf & x != 0 & abs((x - y)/x) > tol))
}

#' @rdname comparedf.tolerances
#' @export
tol.num.pct <- tol.num.percent

#' @rdname comparedf.tolerances
#' @export
tol.factor.none <- function(x, y)
{
  tol.NA(x, y, (as.character(x) != as.character(y)) | (as.numeric(x) != as.numeric(y)))
}

#' @rdname comparedf.tolerances
#' @export
tol.factor.levels <- function(x, y)
{
  tol.NA(x, y, as.numeric(x) != as.numeric(y))
}

#' @rdname comparedf.tolerances
#' @export
tol.factor.labels <- function(x, y)
{
  tol.NA(x, y, as.character(x) != as.character(y))
}

#' @rdname comparedf.tolerances
#' @export
tol.char.both <- function(x, y)
{
  tol.NA(x, y, tolower(trimws(x)) != tolower(trimws(y)))
}

#' @rdname comparedf.tolerances
#' @export
tol.char.case <- function(x, y)
{
  tol.NA(x, y, tolower(x) != tolower(y))
}

#' @rdname comparedf.tolerances
#' @export
tol.char.trim <- function(x, y)
{
  tol.NA(x, y, trimws(x) != trimws(y))
}

#' @rdname comparedf.tolerances
#' @export
tol.char.none <- function(x, y)
{
  tol.NA(x, y, x != y)
}

#' @rdname comparedf.tolerances
#' @export
tol.date.absolute <- tol.num.absolute

#' @rdname comparedf.tolerances
#' @export
tol.logical.none <- tol.char.none

#' @rdname comparedf.tolerances
#' @export
tol.other.none <- function(x, y)
{
  unlist(Map(Negate(identical), x, y))
}

get_tb_strata_part <- function(tbList, sValue, xList, ...)
{
  Map(get_tb_part, tbList, xList, MoreArgs = list(sValue = sValue, ...))
}


get_tb_part <- function(tbList, xList, yList, sList, sValue, cntrl)
{
  statLabs <- cntrl$stats.labels
  f <- function(x, nm, lab = FALSE)
  {
    if(inherits(x[[1]], "tbstat_multirow")) return(if(lab) names(x[[1]]) else rep(nm, length(x[[1]])))
    if(lab && nm %in% names(statLabs)) statLabs[[nm]] else nm
  }

  out <- data.frame(
    group.term = yList$term,
    group.label = yList$label,
    strata.term = if(!sList$hasStrata) "" else paste0("(", sList$term, ") == ", sValue),
    strata.value = if(!sList$hasStrata) "" else sValue,
    variable = xList$variable,
    term = c(xList$term, unlist(Map(f, tbList$stats, names(tbList$stats)), use.names = FALSE)),
    label = c(xList$label, unlist(Map(f, tbList$stats, names(tbList$stats), lab = TRUE), use.names = FALSE)),
    variable.type = tbList$type,
    stringsAsFactors = FALSE
  )
  if(!sList$hasStrata) out$strata.value <- NULL else names(out)[4] <- sList$label

  f2 <- function(x, lv)
  {
    if(inherits(x[[1]], "tbstat_multirow")) x[[lv]] else x[lv]
  }
  for(lvl in names(yList$stats))
  {
    out[[lvl]] <- c("", unlist(lapply(tbList$stats, f2, lv = lvl), recursive = FALSE, use.names = FALSE))
  }
  if(cntrl$test)
  {
    out$test <- tbList$test$method
    out$p.value <- tbList$test$p.value
  }
  out
}

#' as.data.frame.tableby
#'
#' Coerce a \code{\link{tableby}} object to a \code{data.frame}.
#'
#' @param x A \code{\link{tableby}} object.
#' @param ... Arguments to pass to \code{\link{tableby.control}}.
#' @inheritParams summary.tableby
#' @seealso \code{\link{tableby}}, \code{\link{tableby}}
#' @return A \code{data.frame}.
#' @author Ethan Heinzen, based on code originally by Greg Dougherty
#' @export
as.data.frame.tableby <- function(x, ..., labelTranslations = NULL, list.ok = FALSE)
{
  if(!is.null(labelTranslations)) labels(x) <- labelTranslations

  control <- c(list(...), x$control)
  control <- do.call("tableby.control", control[!duplicated(names(control))])

  out <- lapply(x$tables, as_data_frame_tableby, control = control)

  if(!list.ok)
  {
    if(length(out) == 1) out <- out[[1]] else warning("as.data.frame.tableby is returning a list of data.frames")
  }

  set_attr(out, "control", control)
}


as_data_frame_tableby <- function(byList, control)
{
  stopifnot(length(byList$tables) == length(byList$strata$values))
  tabs <- Map(get_tb_strata_part, tbList = byList$tables, sValue = byList$strata$values,
              MoreArgs = list(yList = byList$y, sList = byList$strata, xList = byList$x, cntrl = control))
  out <- do.call(rbind_chr, unlist(tabs, recursive = FALSE, use.names = FALSE))

  f <- function(elt, whch) if(is.null(elt[[whch]])) control[[whch]] else elt[[whch]]
  simp.num <- vapply(byList$control.list, f, NA, "numeric.simplify")
  simp.cat <- lapply(byList$control.list, f, "cat.simplify")
  simp.ord <- lapply(byList$control.list, f, "ordered.simplify")
  simp.dat <- vapply(byList$control.list, f, NA, "date.simplify")

  if(!all(vapply(simp.cat, identical, NA, FALSE)) || any(simp.num) || !all(vapply(simp.ord, identical, NA, FALSE)) || any(simp.dat))
  {
    simplify <- function(x)
    {
      ## make sure there's only two lines of (the same) summary statistic, and that it's categorical
      if(!identical(simp.cat[[x$variable[1]]], FALSE) && all(x$variable.type == "categorical") &&
         (nrow(x) == 2 || nrow(x) == 3 && x$term[2] == x$term[3]))
      {
        y <- x[nrow(x), , drop = FALSE]
        y$term[1] <- x$term[1]
        y$label[1] <- paste0(x$label[1], if(identical(simp.cat[[x$variable[1]]], "label")) paste0(" (", y$label[1], ")"))

      } else if(!identical(simp.ord[[x$variable[1]]], FALSE) && all(x$variable.type == "ordinal") &&
                (nrow(x) == 2 || nrow(x) == 3 && x$term[2] == x$term[3]))
      {
        y <- x[nrow(x), , drop = FALSE]
        y$term[1] <- x$term[1]
        y$label[1] <- paste0(x$label[1], if(identical(simp.ord[[x$variable[1]]], "label")) paste0(" (", y$label[1], ")"))

      } else if((simp.num[x$variable[1]] && all(x$variable.type == "numeric") ||
                 simp.dat[x$variable[1]] && all(x$variable.type == "Date")) && nrow(x) == 2)
      {
        y <- x[2, , drop = FALSE]
        y$term[1] <- x$term[1]
        y$label[1] <- x$label[1]
      } else y <- x
      y
    }
    bylst <- list(factor(out$variable, levels = unique(out$variable)))
    if(byList$strata$hasStrata) bylst[[2]] <- factor(out[[4]], levels = unique(out[[4]]))
    out <- do.call(rbind_chr, by(out, bylst, simplify, simplify = FALSE))
  }
  set_attr(set_attr(out, "control.list", byList$control.list), "ylabel", byList$y$label)
}

#' Some functions to handle NAs
#'
#' \code{allNA} tests if all elements are NA, and \code{includeNA} sets the
#'   \code{NA}s in a character vector or factor to an explicit label.
#'
#' @param x An object
#' @param label A character string denoting the label to set \code{NA}s to.
#' @param first Logical; should the new label be the first level?
#' @param ... Other arguments (not in use at this time).
#'
#' @seealso \code{\link{is.na}}, \code{\link{anyNA}}
#' @author Ethan Heinzen
#' @name NA.operations
NULL
#> NULL

#' @rdname NA.operations
#' @export
allNA <- function(x) all(is.na(x))

#' @rdname NA.operations
#' @export
includeNA <- function(x, label, ...)
{
  UseMethod("includeNA")
}

#' @rdname NA.operations
#' @export
includeNA.factor <- function(x, label = "(Missing)", first = FALSE, ...)
{
  lvl <- levels(x)
  if(label %in% lvl)
  {
    warning('"', label, '" already appears in levels(x).')
  } else if(first)
  {
    levels(x) <- c(label, lvl)
    x[] <- levels(x)[as.integer(x) + 1L]

  } else levels(x) <- c(lvl, label) # don't use factor() here, in case you lose attributes
  x[is.na(x)] <- label
  x
}

#' @rdname NA.operations
#' @export
includeNA.default <- function(x, label = "(Missing)", ...)
{
  if(label %in% x) warning('"', label, '" already appears in x.')
  x[is.na(x)] <- label
  x
}

#' @rdname write2
#' @export
write2.default <- function(object, file, FUN = NULL, ..., append. = FALSE, render. = TRUE, keep.rmd = !render., output_format = NULL)
{
  check_pkg("rmarkdown")
  if(!is.character(file) || length(file) > 1) stop("'file' argument must be a single character string.")
  if(!is.logical(append.) || length(append.) > 1) stop("'append.' argument must be a single logical value.")
  if(!is.logical(render.) || length(render.) > 1) stop("'render.' argument must be a single logical value.")
  if(!is.logical(keep.rmd) || length(keep.rmd) > 1) stop("'keep.rmd' argument must be a single logical value.")

  if(is.null(FUN))
  {
    object <- verbatim(object)
    FUN <- print
  }
  FUN <- match.fun(FUN)

  if(is.character(output_format) && length(output_format) > 1)
  {
    warning("At this point, write2() only supports one output type.")
    output_format <- output_format[1]
  }


  output_format <- if(is.null(output_format) || identical(output_format, "html"))
  {
    rmarkdown::html_document
  } else if(identical(output_format, "pdf"))
  {
    rmarkdown::pdf_document
  } else if(identical(output_format, "word"))
  {
    rmarkdown::word_document
  } else output_format

  filename <- paste0(file, ".Rmd")
  if(!append. || !file.exists(filename)) file.create(filename) # this will create a blank document when needed but allows the append. = TRUE case to work, too
  dots <- list(...)
  if(names(formals(FUN))[1] == "...") # this is when the FUN is, e.g., cat(). Any named arguments would still get cat'd, which we don't want
  {
    ARGS <- c(list(object), dots[names(dots) %in% names(formals(FUN))])
    utils::capture.output(do.call(FUN, ARGS), file = filename, append = append.)
  } else
  {
    utils::capture.output(FUN(object, ...), file = filename, append = append.)
  }


  if(render.)
  {
    render.args <- dots[names(dots) %in% names(formals(rmarkdown::render))]
    render.args$input <- filename
    render.args$output_file <- file

    # if output_format is a function, evaluate it with the ... arguments
    # otherwise, just return the character string or list which rmarkdown::render() will handle
    render.args$output_format <- if(is.function(output_format))
    {
      if("..." %in% names(formals(output_format)))
      {
        do.call(output_format, dots)
      } else do.call(output_format, dots[names(dots) %in% names(formals(output_format))])
    } else output_format

    do.call(rmarkdown::render, render.args)
  }

  # This short-circuits if they want to keep the intermediate files. Otherwise, file.remove() returns a logical about successful file removal
  if(!keep.rmd && !file.remove(filename)) warning("Something went wrong removing the temporary .Rmd file.")

  invisible(object)
}

#' write2word, write2html, write2pdf
#'
#' Functions to output tables to a single Word, HTML, or PDF document.
#'
#' @inheritParams write2
#' @return \code{object} is returned invisibly, and \code{file} is written.
#' @details
#'  To generate the appropriate file type, the \code{write2*} functions use one of \code{rmarkdown::word_document}, \code{rmarkdown::html_document},
#'    and \code{rmarkdown::pdf_document} to get the job done. \code{"..."} arguments are passed to these functions, too.
#' @seealso \code{\link{write2}}
#' @examples
#' \dontrun{
#' data(mockstudy)
#' # tableby example
#' tab1 <- tableby(arm ~ sex + age, data=mockstudy)
#' write2html(tab1, "~/trash.html")
#'
#' # freqlist example
#' tab.ex <- table(mockstudy[, c("arm", "sex", "mdquality.s")], useNA = "ifany")
#' noby <- freqlist(tab.ex, na.options = "include")
#' write2pdf(noby, "~/trash2.pdf")
#'
#' # A more complicated example
#' write2word(tab1, "~/trash.doc",
#'   keep.md = TRUE,
#'   reference_docx = mystyles.docx, # passed to rmarkdown::word_document
#'   quiet = TRUE, # passed to rmarkdown::render
#'   title = "My cool new title") # passed to summary.tableby
#' }
#' @author Ethan Heinzen, adapted from code from Krista Goergen
#' @name write2specific
NULL
#> NULL

#' @rdname write2specific
#' @export
write2word <- function(object, file, ...)
{
  write2(object, file, ..., output_format = "word")
}

#' @rdname write2specific
#' @export
write2pdf <- function(object, file, ...)
{
  write2(object, file, ..., output_format = "pdf")
}

#' @rdname write2specific
#' @export
write2html <- function(object, file, ...)
{
  write2(object, file, ..., output_format = "html")
}

## Tests for write2


context("Testing the write2 output")

data(mockstudy)

expect_write2_worked <- function(FUN, object, reference, ...)
{
  skip_on_ci()
  skip_on_cran()
  skip_on_os("mac")
  skip_on_os("windows")
  FUN <- match.fun(FUN)
  filename <- tempfile()
  # on.exit(expect_true(file.remove(paste0(filename, ".Rmd"))))
  if(!file.exists(reference)) skip("Couldn't find the reference file.")
  if(!file.create(paste0(filename, ".Rmd"))) skip("Couldn't create the temporary file.")

  ss <- options(show.signif.stars = FALSE)
  on.exit(options(ss), add = TRUE)
  expect_error(FUN(list(object, yaml(title = "Test title")), file = filename, ..., render. = TRUE, keep.rmd = TRUE, append. = FALSE, quiet = TRUE), NA)
  # on.exit(expect_true(file.remove(filename)), add = TRUE)

  generated <- readLines(paste0(filename, ".Rmd"))
  expect_output_file(cat(generated, sep = "\n"), reference)
}

###########################################################################################################
#### Internal output
###########################################################################################################

test_that("write2.tableby -> HTML", {
  expect_write2_worked(write2html, tableby(arm ~ sex + age, data=mockstudy, numeric.stats = c("meansd", "q1q3", "range")),
                       reference = "write2.tableby.html.Rmd",
                       title = "My test table", labelTranslations = list(sex = "SEX", age ="Age, yrs"), total = FALSE)
})

test_that("write2.modelsum -> HTML", {
  expect_write2_worked(write2html, modelsum(alk.phos ~ arm + ps + hgb, adjust= ~ age + sex, family = "gaussian", data = mockstudy),
                       reference = "write2.modelsum.html.Rmd",
                       title = "My test table", show.intercept = FALSE, digits = 5)
})

old.labs <- c(cumFreq = "cumFreq", freqPercent = "freqPercent", cumPercent = "cumPercent")

test_that("write2.freqlist -> HTML", {
  expect_write2_worked(write2html, freqlist(table(mockstudy[c("arm", "sex", "mdquality.s")], useNA = "ifany"), strata = c("arm", "sex")),
                       reference = "write2.freqlist.html.Rmd", single = TRUE, labelTranslations = old.labs)
})

test_that("write2.freqlist -> doc", {
  expect_write2_worked(write2word, freqlist(table(mockstudy[c("arm", "sex", "mdquality.s")], useNA = "ifany"), strata = c("arm", "sex")),
                       reference = "write2.freqlist.doc.Rmd", single = TRUE, title = "My cool title", labelTranslations = old.labs)
})

## From the vignette

test_that("write2.list (summary objects) -> PDF", {
  mylist6 <- list(
    summary(tableby(sex ~ age, data = mockstudy), title = "A Title for tableby"),
    summary(modelsum(age ~ sex, data = mockstudy), title = "A Title for modelsum"),
    summary(freqlist(~ sex, data = mockstudy, labelTranslations = old.labs), title = "A Title for freqlist")
  )
  expect_write2_worked(write2pdf, mylist6, reference = "write2.multititles.pdf.Rmd")
})

###########################################################################################################
#### External output, commented out on 11/9/17 because of external package changes
###########################################################################################################
#
# test_that("write2.knitr_kable -> HTML", {
#   if(require(knitr))
#   {
#     expect_write2_worked(write2html, knitr::kable(head(mockstudy)), reference = "write2.kable.html.Rmd")
#   } else skip("library(knitr) not available.")
# })
#
# test_that("write2.xtable -> HTML", {
#   if(require(xtable))
#   {
#     expect_write2_worked(write2html, xtable::xtable(head(mockstudy), caption = "My xtable"), reference = "write2.xtable.html.Rmd",
#                          type = "html", comment = FALSE, include.rownames = FALSE, caption.placement = 'top')
#   } else skip("library(xtable) not available.")
# })
#
# test_that("write2.character (pander) -> HTML", {
#   if(require(pander))
#   {
#     expect_write2_worked(write2html, pander::pander_return(head(mockstudy)), reference = "write2.pander.html.Rmd")
#   } else skip("library(pander) not available.")
# })
#
###########################################################################################################
#### List output
###########################################################################################################


mylist <- list(tableby(sex ~ age, data = mockstudy, numeric.stats = c("meansd", "q1q3", "range")),
               freqlist(table(mockstudy[, c("sex", "arm")]), labelTranslations = old.labs),
               knitr::kable(utils::head(mockstudy)))
mylist2 <- list("# Header 1",
                "This is a small paragraph.",
                tableby(sex ~ age, data = mockstudy, numeric.stats = c("meansd", "q1q3", "range")))

test_that("write2.list -> PDF", {
  expect_write2_worked(write2pdf, mylist, reference = "write2.mylist.pdf.Rmd")
})

test_that("write2.list -> Word", {
  expect_write2_worked(write2word, mylist2, reference = "write2.mylist2.doc.Rmd")
})

test_that("write2.list recursion -> PDF", {
  expect_write2_worked(write2word, list(mylist2, mylist),
                       reference = "write2.mylists.pdf.Rmd")
})


###########################################################################################################
#### verbatim output
###########################################################################################################

my.lm <- summary(lm(age ~ sex, data = mockstudy))
test_that("write2.default -> PDF", {
  expect_write2_worked(write2pdf, my.lm,
                       reference = "write2.lm.pdf.Rmd")
})

test_that("write2.verbatim -> html", {
  expect_write2_worked(write2pdf, verbatim(paste0("Hi.", 1:5)),
                       reference = "write2.char.html.Rmd")
})


test_that("Writing HTML from PDF works (#162)", {
  expect_write2_worked(write2pdf, list(
    "hi there",
    code.chunk(
      arsenal::write2html(list("hi there", yaml(title = "hi there")), "hi_there.html", clean = TRUE)
    )
  ), reference = "write2.render.html.Rmd")
})

test_that("verbatim(call()) works (#248)", {
  expect_identical(
    capture.output(print(verbatim(call("hi", x = 1), NULL, "hi"))),
    c("```"       ,
      "hi(x = 1)" ,
      ""          ,
      "```"       ,
      ""          ,
      "```"       ,
      "NULL"      ,
      ""          ,
      "```"       ,
      ""          ,
      "```"       ,
      "[1] \"hi\"",
      ""          ,
      "```"       ,
      ""
    )
  )
})

###########################################################################################################
#### YAML output
###########################################################################################################

mylist3 <- list(
  "# Header 1",
  "This is a small paragraph.",
  tableby(sex ~ age, data = mockstudy, numeric.stats = c("meansd", "q1q3", "range")),
  yaml(title = "My title"),
  my.lm,
  yaml(author = "Ethan P Heinzen"),
  yaml("header-includes" = list("\\usepackage[labelformat=empty]{caption}")),
  code.chunk(a <- 1, "b <- 2", a + b, "a - b", chunk.opts = "r echo = FALSE, eval = TRUE")
)

mylist4 <- list(
  yaml(title = "My title", author = "Ethan P Heinzen", "header-includes" = list("\\usepackage[labelformat=empty]{caption}")),
  "# Header 1",
  "This is a small paragraph.",
  tableby(sex ~ age, data = mockstudy, numeric.stats = c("meansd", "q1q3", "range")),
  my.lm,
  code.chunk(a <- 1, "b <- 2", a + b, "a - b", chunk.opts = "r echo = FALSE, eval = TRUE")
)

test_that("write2.yaml -> PDF", {
  expect_write2_worked(write2pdf, mylist3, reference = "write2.yaml.pdf.Rmd")
  expect_write2_worked(write2pdf, mylist4, reference = "write2.yaml.pdf.Rmd")
})


###########################################################################################################
#### Code used to generate the files
###########################################################################################################
#
# write2html(tableby(arm ~ sex + age, data=mockstudy, numeric.stats = c("meansd", "q1q3", "range")), "tests/testthat/write2.tableby.html",
#            title = "My test table", labelTranslations = list(sex = "SEX", age ="Age, yrs"), total = FALSE, render. = FALSE)
#
# write2html(modelsum(alk.phos ~ arm + ps + hgb, adjust= ~ age + sex, family = "gaussian", data = mockstudy),
#            "tests/testthat/write2.modelsum.html",
#            title = "My test table", show.intercept = FALSE, digits = 5, render. = FALSE)
#
# write2html(freqlist(table(mockstudy[, c("arm", "sex", "mdquality.s")], useNA = "ifany"), groupBy = c("arm", "sex")),
#            "tests/testthat/write2.freqlist.html", single = TRUE, render. = FALSE)
#
# write2word(freqlist(table(mockstudy[, c("arm", "sex", "mdquality.s")], useNA = "ifany"), groupBy = c("arm", "sex")),
#            "tests/testthat/write2.freqlist.doc", single = TRUE, title = "My cool title", render. = FALSE)
#
# write2pdf(mylist6, "tests/testthat/write2.multititles.pdf", render. = FALSE)
#
## write2html(knitr::kable(head(mockstudy)),
##            "tests/testthat/write2.kable.html", render. = FALSE)
##
## write2html(xtable::xtable(head(mockstudy), caption = "My xtable"),
##            "tests/testthat/write2.xtable.html",
##            type = "html", comment = FALSE, include.rownames = FALSE, caption.placement = "top", render. = FALSE)
##
## write2html(pander::pander_return(head(mockstudy)),
##            "tests/testthat/write2.pander.html", render. = FALSE)
#
#
# write2pdf(mylist, "tests/testthat/write2.mylist.pdf", render. = FALSE)
# write2word(mylist2, "tests/testthat/write2.mylist2.doc", render. = FALSE)
# write2pdf(list(mylist2, mylist), "tests/testthat/write2.mylists.pdf", render. = FALSE)
#
# write2pdf(my.lm, "tests/testthat/write2.lm.pdf", render. = FALSE)
# write2html(verbatim(paste0("Hi.", 1:5)),
#            "tests/testthat/write2.char.html", render. = FALSE)
# write2pdf(mylist3, "tests/testthat/write2.yaml.pdf", render. = FALSE)

###########################################################################################################
#### Reported bugs for write2
###########################################################################################################

## Development Tasks

The Makefile provides several useful commands:

```bash
make cython      # Build Cython extensions
make doc         # Generate documentation using Sphinx
make test        # Run all tests
make coverage    # Generate test coverage report
make clean       # Clean build artifacts and documentation
```

## Building and Publishing

Update: Building and publishing are handled by Github actions.

1. Install build tools:
    
```bash
pip install build twine
```

2. Clean old distributions:

```bash
rm -rf dist/ build/ *.egg-info
```

3. Create distribution packages:

```bash
make cython
python -m build
```

4. Check the distributions:

```bash
twine check dist/*
```

5. Upload to Test PyPI first:

```bash
twine upload --repository testpypi dist/*
```

6. If everything looks good, upload to PyPI:

```bash
twine upload dist/*
```

Note: You'll need a PyPI account and API token. Store your credentials in `~/.pypirc` or enter them when prompted.

---
title: "The tableby function"
author: "Beth Atkinson, Ethan Heinzen, Jason Sinnwell, Shannon McDonnell and Greg Dougherty"
output: 
  rmarkdown::html_vignette:
    toc: yes
    toc_depth: 3
vignette: |
  %\VignetteIndexEntry{The tableby function}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r echo = FALSE}
options(width = 100)
ge330 <- getRversion() >= "3.3.0"
```

# Introduction

One of the most common tables in medical literature includes summary statistics for a set of variables,
often stratified by some group (e.g. treatment arm).  Locally at Mayo, the SAS macros `%table` and `%summary` were
written to create summary tables with a single call. With the increasing interest in R, we have developed
the function `tableby` to create similar tables within the R environment.

In developing the `tableby()` function, the goal was to bring the best features of these macros into an R function.
However, the task was not simply to duplicate all the functionality, but rather to make use of R's strengths
(modeling, method dispersion, flexibility in function definition and output format) and make a tool that fits
the needs of R users.  Additionally, the results needed to fit within the general reproducible research framework
so the tables could be displayed within an R markdown report.

This report provides step-by-step directions for using the functions associated with `tableby()`.
All functions presented here are available within the `arsenal` package.  An assumption is made that users
are somewhat familiar with R Markdown documents.  For those who are new to the topic, a good initial resource
is available at [rmarkdown.rstudio.com](https://rmarkdown.rstudio.com/). 

# Simple Example

The first step when using the `tableby` function is to load the `arsenal` package.  All the examples in this report use
a dataset called `mockstudy` made available by Paul Novotny which includes a variety of types of variables (character,
numeric, factor, ordered factor, survival) to use as examples.

```{r, load-data}
library(arsenal)
require(knitr)
require(survival)
data(mockstudy) ##load data
dim(mockstudy)  ##look at how many subjects and variables are in the dataset 
# help(mockstudy) ##learn more about the dataset and variables
str(mockstudy) ##quick look at the data
```

To create a simple table stratified by treatment arm, use a formula statement to specify the variables that you want summarized.
The example below uses age (a continuous variable) and sex (a factor). 

```{r, simple1}
tab1 <- tableby(arm ~ sex + age, data=mockstudy)
```

If you want to take a quick look at the table, you can use `summary()` on your tableby object and the table
will print out as text in your R console window.  If you use `summary()` without any options you will see a
number of $\&nbsp;$ statements which translates to "space" in HTML.

## Pretty text version of table

If you want a nicer version in your console window then add the `text=TRUE` option.  

```{r, simple-text}
summary(tab1, text=TRUE)
```

## Pretty Rmarkdown version of table

In order for the report to look nice within an R markdown (knitr) report, you just need to specify
`results="asis"` when creating the R chunk. This changes the layout slightly (compresses it) and bolds the variable names. 

```{r, simple-markdown, results='asis'}
summary(tab1)
```

## Data frame version of table

If you want a data.frame version, simply use `as.data.frame`.

```{r}
as.data.frame(tab1)
```

## Summaries using standard R code

```{r}
## base R frequency example
tmp <- table(Gender=mockstudy$sex, "Study Arm"=mockstudy$arm)
tmp

# Note: The continuity correction is applied by default in R (not used in %table)
chisq.test(tmp)

## base R numeric summary example
tapply(mockstudy$age, mockstudy$arm, summary)
summary(aov(age ~ arm, data=mockstudy))

```

# Modifying Output

## Add labels

In the above example, age is shown with a label (Age in Years), but sex is listed "as is" with lower case letters.
This is because the data was created in SAS and in the SAS dataset, age had a label but sex did not.
The label is stored as an attribute within R.

```{r, check-labels}
## Look at one variable's label
attr(mockstudy$age,'label')

## See all the variables with a label
unlist(lapply(mockstudy,'attr','label'))
# Can also use labels(mockstudy)
```

If you want to add labels to other variables, there are a couple of options.  First, you could add labels to the variables in your dataset.

```{r, add-label, results='asis'}
attr(mockstudy$sex,'label')  <- 'Gender'

tab1 <- tableby(arm ~ sex + age, data=mockstudy)
summary(tab1)
```

You can also use the built-in `data.frame` method for `labels<-`:

```{r, results = 'asis'}
labels(mockstudy)  <- c(age = 'Age, yrs', sex = "Gender")

tab1 <- tableby(arm ~ sex + age, data=mockstudy)
summary(tab1)
```

Another option is to add labels after you have created the table

```{r, results='asis'}
mylabels <- list(sex = "SEX", age = "Age, yrs")
summary(tab1, labelTranslations = mylabels)
```

Alternatively, you can check the variable labels and manipulate them with a function called `labels`, which works on the `tableby` object.

```{r, assignlabels}
labels(tab1)
labels(tab1) <- c(arm="Treatment Assignment", age="Baseline Age (yrs)")
labels(tab1)
```

```{r, results='asis'}
summary(tab1)
```

## Change summary statistics globally

Currently the default behavior is to summarize continuous variables with: Number of missing values,
Mean (SD), 25th - 75th quantiles, and Minimum-Maximum values with an ANOVA (t-test with equal variances) p-value.
For categorical variables the default is to show: Number of missing values and count (column percent) with a
chi-square p-value.  This behavior can be modified using the tableby.control function. In fact, you can save
your standard settings and use that for future tables. Note that `test=FALSE` and `total=FALSE` results in the
total column and p-value column not being printed.  


```{r, results='asis'}
mycontrols  <- tableby.control(test=FALSE, total=FALSE,
                               numeric.test="kwt", cat.test="chisq",
                               numeric.stats=c("N", "median", "q1q3"),
                               cat.stats=c("countpct"),
                               stats.labels=list(N='Count', median='Median', q1q3='Q1,Q3'))
tab2 <- tableby(arm ~ sex + age, data=mockstudy, control=mycontrols)
summary(tab2)
```

You can also change these settings directly in the tableby call.  

```{r, results='asis'}
tab3 <- tableby(arm ~ sex + age, data=mockstudy, test=FALSE, total=FALSE, 
                numeric.stats=c("median","q1q3"), numeric.test="kwt")
summary(tab3)
```


## Change summary statistics within the formula

In addition to modifying summary options globally, it is possible to modify the test and summary statistics for
specific variables within the formula statement.  For example, both the kwt (Kruskal-Wallis rank-based) and anova
(asymptotic analysis of variance) tests apply to numeric variables, and we can use one for the variable "age",
another for the variable "bmi", and no test for the variable "ast". A list of all the options is shown at the end of the vignette.

The `tests` function can do a quick check on what tests were performed on each variable in tableby.

```{r, testformula}
tab.test <- tableby(arm ~ kwt(age) + anova(bmi) + notest(ast), data=mockstudy)
tests(tab.test)
```

```{r, results='asis'}
summary(tab.test)
```

Summary statistics for any individual variable can also be modified, but it must be done as secondary
arguments to the test function. The function names must be strings that are functions already written for tableby,
built-in R functions like mean and range, or user-defined functions.

```{r, testsAndStats, results='asis'}
tab.test <- tableby(arm ~ kwt(ast, "Nmiss2","median") + anova(age, "N","mean") +
                    notest(bmi, "Nmiss","median"), data=mockstudy)
summary(tab.test)
```

## Controlling Options for Categorical Tests (Chisq and Fisher's)

The formal tests for categorical variables against the levels of the by variable, chisq and fe, have
options to simulate p-values. We show how to turn on the simulations for these with 500 replicates
for the Fisher's test (fe).

```{r, simfe, results='asis'}
set.seed(100)
tab.catsim <- tableby(arm ~ sex + race, cat.test="fe", simulate.p.value=TRUE, B=500, data=mockstudy)
tests(tab.catsim)
```

The chi-square test on 2x2 tables applies Yates' continuity correction by default, so we provide an option
to turn off the correction. We show the results with and without the correction that is applied to
treatment arm by sex, if we use subset to ignore one of the three treatment arms.

```{r, chisqcorrect, results='asis'}
cat.correct <- tableby(arm ~ sex + race, cat.test="chisq", subset = !grepl("^F", arm), data=mockstudy)
tests(cat.correct)
cat.nocorrect <- tableby(arm ~ sex + race, cat.test="chisq", subset = !grepl("^F", arm),
     chisq.correct=FALSE, data=mockstudy)
tests(cat.nocorrect)
```

## Modifying the look & feel in Word documents 

You can easily create Word versions of `tableby` output via an Rmarkdown report and the default options will give you a reasonable table in Word - just select the "Knit Word" option in RStudio.

**The functionality listed in this next paragraph is coming soon but needs an upgraded version of RStudio**
If you want to modify fonts used for the table, then you'll need to add an extra line to your header at the beginning of your file.
You can take the `WordStylesReference01.docx` file and modify the fonts (storing the format preferences in your project directory).
To see how this works, run your report once using WordStylesReference01.docx and then WordStylesReference02.docx. 

```
output: word_document
  reference_docx: /projects/bsi/gentools/R/lib320/arsenal/doc/WordStylesReference01.docx 
```

For more information on changing the look/feel of your Word document, see the [Rmarkdown documentation](https://bookdown.org/yihui/rmarkdown/word-document.html) website.


# Additional Examples

Here are multiple examples showing how to use some of the different options.

## 1. Summarize without a group/by variable

```{r, nobyvar, results='asis'}
tab.noby <- tableby(~ bmi + sex + age, data=mockstudy)
summary(tab.noby)
```

## 2. Display footnotes indicating which "test" was used

```{r, results="asis"}
summary(tab.test, pfootnote=TRUE)
```

## 3. Summarize an ordered factor

When comparing groups of ordered data there are a couple of options.  The **default** uses a general independence test available from the `coin` package.
For two-group comparisons, this is essentially the Armitage trend test.  The other option is to specify the Kruskal Wallis test.
The example below shows both options.

```{r}
mockstudy$age.ordnew <- ordered(c("a",NA,as.character(mockstudy$age.ord[-(1:2)])))
table(mockstudy$age.ord, mockstudy$sex)
table(mockstudy$age.ordnew, mockstudy$sex)
class(mockstudy$age.ord)
```

```{r, results="asis", eval=requireNamespace("coin", quietly = TRUE)}
summary(tableby(sex ~ age.ordnew, data = mockstudy), pfootnote = TRUE)
summary(tableby(sex ~ age.ord, data = mockstudy), pfootnote = TRUE)
```

## 4. Summarize a survival variable

First look at the information that is presented by the `survfit()` function, then see how the same results can be seen with tableby.
The default is to show the median survival (time at which the probability of survival = 50%). 

```{r, eval=ge330}
survfit(Surv(fu.time, fu.stat)~sex, data=mockstudy)
survdiff(Surv(fu.time, fu.stat)~sex, data=mockstudy)
```

```{r, results='asis'}
summary(tableby(sex ~ Surv(fu.time, fu.stat), data=mockstudy))
```

It is also possible to obtain summaries of the % survival at certain time points (say the probability of surviving 1-year).

```{r, eval=ge330}
summary(survfit(Surv(fu.time/365.25, fu.stat)~sex, data=mockstudy), times=1:5)
```

```{r, results='asis', eval=ge330}
summary(tableby(sex ~ Surv(fu.time/365.25, fu.stat), data=mockstudy, times=1:5, surv.stats=c("NeventsSurv","NriskSurv")))
```

## 5. Summarize date variables

Date variables by default are summarized with the number of missing values, the median, and the range.
For example purposes we've created a random date.  Missing values are introduced for impossible February dates.

```{r, results='asis'}
set.seed(100)
N <- nrow(mockstudy)
mockstudy$dtentry <- mdy.Date(month=sample(1:12,N,replace=T), day=sample(1:29,N,replace=T), 
                              year=sample(2005:2009,N,replace=T))
summary(tableby(sex ~ dtentry, data=mockstudy))
```

## 6. Summarize multiple variables without typing them out

Often one wants to summarize a number of variables.  Instead of typing by hand each individual variable,
an alternative approach is to create a formula using the `paste` command with the `collapse="+"` option.  

```{r, results='asis'}
## create a vector specifying the variable names
myvars <- names(mockstudy)

## select the 8th through the last variables
## paste them together, separated by the + sign
RHS <- paste(myvars[8:10], collapse="+")
RHS

## create a formula using the as.formula function
as.formula(paste('arm ~ ', RHS))

## use the formula in the tableby function
summary(tableby(as.formula(paste('arm ~', RHS)), data=mockstudy))
```

These steps can also be done using the `formulize` function.

```{r, results='asis'}
## The formulize function does the paste and as.formula steps
tmp <- formulize('arm',myvars[8:10])
tmp

## More complex formulas could also be written using formulize
tmp2 <- formulize('arm',c('ps','hgb^2','bmi'))

## use the formula in the tableby function
summary(tableby(tmp, data=mockstudy))
```

To change summary statistics or statistical tests en masse, consider using `paste0()` together with `formulize()`:

```{r results='asis'}
varlist1 <- c('age','sex','hgb')
varlist2 <- paste0("anova(", c('bmi','alk.phos','ast'), ", 'meansd')")

summary(tableby(formulize("arm", c(varlist1, varlist2)),
                data = mockstudy, numeric.test = "kwt"), pfootnote = TRUE)
```

## 7. Subset the dataset used in the analysis

Here are two ways to get the same result (limit the analysis to subjects age>5 and in the F: FOLFOX treatment group). 

* The first approach uses the subset function applied to the dataset `mockstudy`.  This example also selects a subset of variables.
The `tableby` function is then applied to this subsetted data.


```{r}
newdata <- subset(mockstudy, subset=age>50 & arm=='F: FOLFOX', select = c(sex,ps:bmi))
dim(mockstudy)
table(mockstudy$arm)
dim(newdata)
names(newdata)
```

```{r, results='asis'}
summary(tableby(sex ~ ., data=newdata))
```

* The second approach does the same analysis but uses the subset
argument within `tableby` to subset the data.

```{r, results='asis'}
summary(tableby(sex ~ ps + hgb + bmi, subset=age>50 & arm=="F: FOLFOX", data=mockstudy))
```

## 8. Create combinations of variables on the fly

```{r}
## create a variable combining the levels of mdquality.s and sex
with(mockstudy, table(interaction(mdquality.s,sex)))
```

```{r, results='asis'}
summary(tableby(arm ~ interaction(mdquality.s,sex), data=mockstudy))
```

```{r, results='asis'}
## create a new grouping variable with combined levels of arm and sex
summary(tableby(interaction(mdquality.s, sex) ~  age + bmi, data=mockstudy, subset=arm=="F: FOLFOX"))
```

## 9. Transform variables on the fly

Certain transformations need to be surrounded by `I()` so that R knows to treat it as a variable transformation and
not some special model feature.  If the transformation includes any of the symbols `/ - + ^ *` then surround the new variable by `I()`.


```{r, maketrans, results='asis'}
trans <- tableby(arm ~ I(age/10) + log(bmi) + factor(mdquality.s, levels=0:1, labels=c('N','Y')),
                 data=mockstudy)
summary(trans)
```

The labels for these variables aren't exactly what we'd like, so we can change modify those after the fact.
Instead of typing out the very long variable names, you can modify specific labels by position. 

```{r, assignlabels2}
labels(trans)
labels(trans)[2:4] <- c('Age per 10 yrs', 'log(BMI)', 'MD Quality')
labels(trans)
```

```{r, transsummary, results='asis'}
summary(trans)
```

Note that if we had not changed `mdquality.s` to a factor, it would have been summarized as though it were a continuous variable.

```{r, results='asis'}
class(mockstudy$mdquality.s)
summary(tableby(arm~mdquality.s, data=mockstudy))
```

Another option would be to specify the test and summary statistics.
In fact, if I had a set of variables coded 0/1 and that was all I was summarizing, then I could change the global option
for continuous variables to use the chi-square test and show countpct.

```{r, results='asis'}
summary(tableby(arm ~ chisq(mdquality.s, "Nmiss","countpct"), data=mockstudy))
```


## 10. Subsetting (change the ordering of the variables, delete a variable, sort by p-value, filter by p-value)

```{r, results='asis'}
mytab <- tableby(arm ~ sex + alk.phos + age, data=mockstudy)
mytab2 <- mytab[c('age','sex','alk.phos')]
summary(mytab2)
summary(mytab[c('age','sex')], digits = 2)
summary(mytab[c(3,1)], digits = 3)
summary(sort(mytab, decreasing = TRUE))
summary(mytab[mytab < 0.5])
head(mytab, 1) # can also use tail()
```

## 11. Merge two `tableby` objects together 

It is possible to combine two tableby objects so that they print out together. Overlapping by-variables will have their x-variables
concatenated, and (if `all=TRUE`) non-overlapping by-variables will have their tables printed separately.

```{r, results="asis"}
## demographics
tab1 <- tableby(arm ~ sex + age, data=mockstudy,
                control=tableby.control(numeric.stats=c("Nmiss","meansd"), total=FALSE))
## lab data
tab2 <- tableby(arm ~ hgb + alk.phos, data=mockstudy,
                control=tableby.control(numeric.stats=c("Nmiss","median","q1q3"),
                                        numeric.test="kwt", total=FALSE))
tab12 <- merge(tab1, tab2)
class(tab12)
summary(tab12)
```

For tables with two different outcomes, consider the `all=TRUE` argument:

```{r, results='asis'}
summary(merge(
  tableby(sex ~ age, data = mockstudy),
  tableby(arm ~ bmi, data = mockstudy),
  all = TRUE
))
```

## 12. Add a title to the table

When creating a pdf the tables are automatically numbered and the title appears below the table.  In Word and HTML, the titles appear un-numbered and above the table.  

```{r, results='asis'}
t1 <- tableby(arm ~ sex + age, data=mockstudy)
summary(t1, title='Demographics')
```

With multiple left-hand sides, you can pass a vector or list to determine labels for each table:

```{r, results='asis'}
summary(tableby(list(arm, sex) ~ age, data = mockstudy), title = c("arm table", "sex table"))
```

## 13. Modify how missing values are displayed

Depending on the report you are writing you have the following options:

* Show how many subjects have each variable

* Show how many subjects are missing each variable

* Show how many subjects are missing each variable only if there are any missing values

* Don't indicate missing values at all

```{r}
## look at how many missing values there are for each variable
apply(is.na(mockstudy),2,sum)
```

```{r, results='asis'}
## Show how many subjects have each variable (non-missing)
summary(tableby(sex ~ ast + age, data=mockstudy,
                control=tableby.control(numeric.stats=c("N","median"), total=FALSE)))

## Always list the number of missing values
summary(tableby(sex ~ ast + age, data=mockstudy,
                control=tableby.control(numeric.stats=c("Nmiss2","median"), total=FALSE)))

## Only show the missing values if there are some (default)
summary(tableby(sex ~ ast + age, data=mockstudy, 
                control=tableby.control(numeric.stats=c("Nmiss","mean"),total=FALSE)))

## Don't show N at all
summary(tableby(sex ~ ast + age, data=mockstudy, 
                control=tableby.control(numeric.stats=c("mean"),total=FALSE)))
```

One might also consider the use of `includeNA()` to include NAs in the counts and percents for categorical variables.

```{r, results = 'asis'}
mockstudy$ps.cat <- factor(mockstudy$ps)
attr(mockstudy$ps.cat, "label") <- "ps"
summary(tableby(sex ~ includeNA(ps.cat), data = mockstudy, cat.stats = "countpct"))
```

## 14. Modify the number of digits used

Within tableby.control function there are 4 options for controlling the number of significant digits shown.  

* digits: controls the number of digits after the decimal place for continuous values

* digits.count: controls the number of digits after the decimal point for counts

* digits.pct: controls the number of digits after the decimal point for percents

* digits.p: controls the number of digits after the decimal point for p-values

```{r, results='asis'}
summary(tableby(arm ~ sex + age + fu.time, data=mockstudy), digits=4, digits.p=2, digits.pct=1)
```

With the exception of `digits.p`, all of these can be specified on a per-variable basis
using the in-formula functions that specify which tests are run:

```{r results='asis'}
summary(tableby(arm ~ chisq(sex, digits.pct=1) + anova(age, digits=4) +
                  anova(fu.time, digits = 1), data=mockstudy))
```


## 15. Create a user-defined summary statistic

For purposes of this example, the code below creates a trimmed mean function (trims 10%)
and use that to summarize the data. Note the use of the `...` which tells R to pass extra arguments on - this is required
for user-defined functions.  In this case, `na.rm=T` is passed to `myfunc`.  The *weights* argument is also required, even though
it isn't passed on to the internal function in this particular example.   

```{r, results='asis'}
trim10 <- function(x, weights=rep(1,length(x)), ...){
  mean(x, trim=.1, ...)
}

summary(tableby(sex ~ hgb, data=mockstudy, 
                control=tableby.control(numeric.stats=c("Nmiss","trim10"), numeric.test="kwt",
                    stats.labels=list(Nmiss='Missing values', trim10="Trimmed Mean, 10%"))))
```

For statistics to be formatted appropriately, you may want to use `as.tbstat()` or `as.countpct()`.
For example, suppose you want to create a trimmed mean function that trims by both 5 and 10 percent.
The first example shows them separated by a comma; the second puts the 10% trimmed mean in brackets

```{r, results='asis'}
trim510comma <- function(x, weights=rep(1,length(x)), ...){
  tmp <- c(mean(x, trim = 0.05, ...), mean(x, trim = 0.1, ...))
  as.tbstat(tmp, sep = ", ")
}
trim510bracket <- function(x, weights=rep(1,length(x)), ...){
  tmp <- c(mean(x, trim = 0.05, ...), mean(x, trim = 0.1, ...))
  as.tbstat(tmp, sep = " ", parens = c("[", "]"))
}

summary(tableby(sex ~ hgb, data=mockstudy, numeric.stats=c("Nmiss", "trim510comma"), test = FALSE))
summary(tableby(sex ~ hgb, data=mockstudy, numeric.stats=c("Nmiss", "trim510bracket"), test = FALSE))
```

Or perhaps it's useful to put the amount of trimming in parentheses. Since it is a percent, we can
flag it as such:

```{r results='asis'}
trim10pct <- function(x, weights=rep(1,length(x)), ...){
  tmp <- mean(x, trim = 0.05, ...)
  as.countpct(c(tmp, 10), sep = " ", parens = c("(", ")"), which.count = 0, which.pct = 2, pct = "%")
}
summary(tableby(sex ~ hgb, data=mockstudy, numeric.stats=c("Nmiss", "trim10pct"),
                digits = 2, digits.pct = 0, test = FALSE))

```

## 16. Use case-weights for creating summary statistics

When comparing groups, they are often unbalanced when it comes to nuisances such as age and sex.
The `tableby` function allows you to create weighted summary statistics.  If this option us used then p-values are not calculated (`test=FALSE`).

```{r}
##create fake group that is not balanced by age/sex 
set.seed(200)
mockstudy$fake_arm <- ifelse(mockstudy$age>60 & mockstudy$sex=='Female',sample(c('A','B'),replace=T, prob=c(.2,.8)),
                            sample(c('A','B'),replace=T, prob=c(.8,.4)))

mockstudy$agegp <- cut(mockstudy$age, breaks=c(18,50,60,70,90), right=FALSE)

## create weights based on agegp and sex distribution
tab1 <- with(mockstudy,table(agegp, sex))
tab2 <- with(mockstudy, table(agegp, sex, fake_arm))
tab2
gpwts <- rep(tab1, length(unique(mockstudy$fake_arm)))/tab2
gpwts[gpwts>50] <- 30

## apply weights to subjects
index <- with(mockstudy, cbind(as.numeric(agegp), as.numeric(sex), as.numeric(as.factor(fake_arm)))) 
mockstudy$wts <- gpwts[index]

## show weights by treatment arm group
tapply(mockstudy$wts,mockstudy$fake_arm, summary)
```

```{r, results='asis', eval=ge330}
orig <- tableby(fake_arm ~ age + sex + Surv(fu.time/365, fu.stat), data=mockstudy, test=FALSE)
summary(orig, title='No Case Weights used')
tab1 <- tableby(fake_arm ~ age + sex + Surv(fu.time/365, fu.stat), data=mockstudy, weights=wts)
summary(tab1, title='Case Weights used')
```
                
                
## 17. Create your own p-value and add it to the table

When using weighted summary statistics, it is often desirable to then show a p-value from a model that corresponds to the weighted analysis. 
It is possible to add your own p-value and modify the column title for that new p-value.  Another use for this would be to add standardized 
differences or confidence intervals instead of a p-value.

To add the p-value, you simply need to create a data frame and use the function `modpval.tableby()`.
The first few columns in the data.frame are required: (1) the by-variable, (2) the strata value (if
the table has a strata term), (3) the x-variable, and (4) the new p-value (or test statistic).
Another optional column can be used to indicate what method was used to calculate the p-value.
If you specify `use.pname=TRUE` then the column name indicating the p-value will be also be used in the tableby summary.


```{r, results='asis', eval=ge330}
mypval <- data.frame(
  byvar = "fake_arm",
  variable = c('age','sex','Surv(fu.time/365, fu.stat)'), 
  adj.pvalue = c(.953,.811,.01), 
  method = c('Age/Sex adjusted model results')
)
tab2 <- modpval.tableby(tab1, mypval, use.pname=TRUE)
summary(tab2, title='Case Weights used, p-values added', pfootnote=TRUE)
```

## 18. For two-level categorical variables or one-line numeric variables, simplify the output.

If the `cat.simplify` option is set to `TRUE`, then only the second level of two-level
categorical varialbes is shown.  In the example below, `sex` has two levels,
and "Female" is the second level, hence only the counts and percents for Female are shown.
Similarly, "mdquality.s" was turned to a factor, and "1" is the second level, but since
there are missings, the table ignores `cat.simplify` and displays all levels (since the
output can no longer be displayed on one line).

```{r, results='asis'}
table2 <- tableby(arm~sex + factor(mdquality.s), data=mockstudy, cat.simplify=TRUE)
summary(table2, labelTranslations=c(sex="Female", "factor(mdquality.s)"="MD Quality"))
```

Similarly, if `numeric.simplify` is set to `TRUE`, then any numerics which only have one
row of summary statistics are simplified into a single row. Note again that `ast` has
missing values and so is not simplified to a single row.

```{r results='asis'}
summary(tableby(arm ~ age + ast, data = mockstudy,
                numeric.simplify=TRUE, numeric.stats=c("Nmiss", "meansd")))
```

The in-formula functions to change which tests are run can also be used to specify these
options for each variable at a time.

```{r results='asis'}
summary(tableby(arm ~ anova(age, "meansd", numeric.simplify=TRUE) +
                  chisq(sex, cat.simplify=TRUE), data = mockstudy))
```

The `cat.simplify` and `ord.simplify` argument also accept the special string `"label"`, which appends the shown level's label to the overall label:

```{r results='asis'}
summary(tableby(arm ~ sex, cat.simplify = "label", data = mockstudy))
```


## 19. Use `tableby` within an Sweave document

For those users who wish to create tables within an Sweave document, the following code seems to work.

```
\documentclass{article}

\usepackage{longtable}
\usepackage{pdfpages}

\begin{document}

\section{Read in Data}
<<echo=TRUE>>=
require(arsenal)
require(knitr)
require(rmarkdown)
data(mockstudy)

tab1 <- tableby(arm~sex+age, data=mockstudy)
@

\section{Convert Summary.Tableby to LaTeX}
<<echo=TRUE, results='hide', message=FALSE>>=
capture.output(summary(tab1), file="Test.md")

## Convert R Markdown Table to LaTeX
render("Test.md", pdf_document(keep_tex=TRUE))
@ 

\includepdf{Test.pdf}

\end{document}
```

## 20. Export `tableby` object to a .CSV file

When looking at multiple variables it is sometimes useful to export the results to a csv file. The `as.data.frame` function creates a data frame object that can be exported or further manipulated within R.

```{r}
tab1 <- summary(tableby(arm~sex+age, data=mockstudy), text = NULL)
as.data.frame(tab1)

# write.csv(tab1, '/my/path/here/my_table.csv')
```

## 21. Write `tableby` object to a separate Word or HTML file

```{r eval = FALSE}
## write to an HTML document
tab1 <- tableby(arm ~ sex + age, data=mockstudy)
write2html(tab1, "~/trash.html")

## write to a Word document
write2word(tab1, "~/trash.doc", title="My table in Word")
```

## 22. Use `tableby` in R Shiny

The easiest way to output a `tableby()` object in an R Shiny app is to use the `tableOutput()` UI in combination with
the `renderTable()` server function and `as.data.frame(summary(tableby()))`:

```{r eval=FALSE}
# A standalone shiny app
library(shiny)
library(arsenal)
data(mockstudy)

shinyApp(
  ui = fluidPage(tableOutput("table")),
  server = function(input, output) {
    output$table <- renderTable({
      as.data.frame(summary(tableby(sex ~ age, data = mockstudy), text = "html"))
    }, sanitize.text.function = function(x) x)
  }
)
```

This can be especially powerful if you feed the selections from a `selectInput(multiple = TRUE)` into `formulize()` to make
the table dynamic!

## 23. Use `tableby` in bookdown

Since the backbone of `tableby()` is `knitr::kable()`, tables still render well in bookdown. However, `print.summary.tableby()` doesn't use
the `caption=` argument of `kable()`, so some tables may not have a properly numbered caption. To fix this, use the method described
[on the bookdown site](https://bookdown.org/yihui/bookdown/tables.html) to give the table a tag/ID.

```{r eval=FALSE}
summary(tableby(sex ~ age, data = mockstudy), title="(\\#tab:mytableby) Caption here")
```

## 24. Adjust `tableby` for multiple p-values

The `padjust()` function is a new S3 generic piggybacking off of `p.adjust()`. It works on both `tableby` and `summary.tableby` objects:

```{r results='asis'}
tab <- summary(tableby(sex ~ age + fu.time + bmi + mdquality.s, data = mockstudy))
tab
padjust(tab, method = "bonferroni")
```

## 25. Tabulate multiple endpoints

You can now use `list()` on the left-hand side of `tableby()` to give multiple endpoints.

```{r results='asis'}
summary(tableby(list(sex, mdquality.s, ps) ~ age + bmi, data = mockstudy))
```

To avoid confusion about which table is which endpoint, you can set `term.name=TRUE` in `summary()`. This takes the labels
for each by-variable and puts them in the top-left of the table.

```{r results='asis'}
summary(tableby(list(sex, mdquality.s, ps) ~ age + bmi, data = mockstudy), term.name = TRUE)
```

## 26. Tabulate data by a non-test group (strata)

You can also specify a second grouping variable that doesn't get tested (but instead separates results): a *strata* variable.

```{r results='asis'}
summary(tableby(list(sex, ps) ~ age + bmi, strata = arm, data = mockstudy))
```

# Available Function Options

## Summary statistics

The **default** summary statistics, by varible type, are:

* `numeric.stats`: Continuous variables will show by default `Nmiss, meansd, range`
* `cat.stats`: Categorical and factor variables will show by default `Nmiss, countpct`
* `ordered.stats`: Ordered factors will show by default `Nmiss, countpct`
* `surv.stats`: Survival variables will show by default `Nmiss, Nevents, medsurv`
* `date.stats`: Date variables will show by default `Nmiss, median, range`

There are a number of extra functions defined specifically for the tableby function.

* `N`: a count of the number of observations for a particular group
* `Nmiss`: only show the count of the number of missing values if there are some missing values
* `Nmiss2`: always show a count of the number of missing values for a variable within each group
* `meansd`: print the mean and standard deviation in the format `mean(sd)`
* `meanse`: print the mean and standard error in the format `mean(se)`
* `meanCI`: print the mean and a (t) confidence interval
* `count`: print the number of values in a category
* `countN`: print the number of values in a category plus the total N for the group in the format `N/Total`
* `countpct`: print the number of values in a category plus the column-percentage in the format `N (%)`
* `countrowpct`: print the number of values in a category plus the row-percentage in the format `N (%)`
* `countcellpct`: print the number of values in a category plus the cell-percentage in the format `N (%)`
* `binomCI`: print the proportion in a category plus a binomial confidence interval.
* `rowbinomCI`: print the row proportion in a category plus a binomial confidence interval.
* `medianq1q3`: print the median, 25th, and 75th quantiles `median (Q1, Q3)`
* `q1q3`: print the 25th and 75th quantiles `Q1, Q3`
* `iqr`: print the inter-quartile range.
* `medianrange`: print the median, minimum and maximum values `median (minimum, maximum)`
* `medianmad`: print the median and median absolute deviation (mad)
* `Nevents`: print number of events for a survival object within each grouping level
* `medSurv`: print the median survival
* `NeventsSurv`: print number of events and survival at given times
* `NriskSurv`: print the number still at risk and survival at given times
* `Nrisk`: print the number still at risk at given times
* `medTime`: print the median follow-up time
* `sum`
* `max`
* `min`
* `mean`
* `sd`
* `var`
* `median`
* `range`
* `gmean`, `gsd`, `gmeansd`, `gmeanCI`: geometric means, sds, and confidence intervals.

## Testing options

The tests used to calculate p-values differ by the variable type, but can be specified
explicitly in the formula statement or in the control function.

The following tests are accepted:

* `anova`: analysis of variance test; the default test for continuous variables. When
    the grouping variable has two levels, it is equivalent to the two-sample t-test with equal variance. 
    
* `kwt`: Kruskal-Wallis test, optional test for continuous
    variables. When the grouping variable has two levels, it is equivalent to the Wilcoxon Rank Sum test.

* `wt`: An explicit Wilcoxcon test.
    
* `medtest`: Median test test, optional test for continuous variables.

* `chisq`: chi-square goodness of fit test for equal counts of a
    categorical variable across categories; the default for categorical
    or factor variables

* `fe`: Fisher's exact test for categorical variables; optional 
  
* `logrank`: log-rank test, the default test for time-to-event
    variables

* `trend`: The `independence_test` function from the `coin` is used to test for trends.  Whenthe grouping variable has two levels,
  it is equivalent to the Armitage trend test.  This is the default for ordered factors
  
* `notest`: Don't perform a test.

## `tableby.control` settings

A quick way to see what arguments are possible to utilize in a function is to use the `args()` command. Settings involving the number of digits can be set in `tableby.control` or in `summary.tableby`.

```{r}
args(tableby.control)
```

## `summary.tableby` settings

The `summary.tableby` function has options that modify how the table appears (such as adding a title or modifying labels).  

```{r}
args(arsenal:::summary.tableby)
```

---
title: "The comparedf function"
author: "Ethan Heinzen, Ryan Lennon, Andrew Hanson"
output:
  rmarkdown::html_vignette:
    toc: yes
    toc_depth: 3
vignette: |
  %\VignetteIndexEntry{The comparedf function}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r include = FALSE}
knitr::opts_chunk$set(eval = TRUE, message = FALSE, results = 'asis', comment='')
options(width = 120)
```

# Introduction

The `comparedf()` function can be used to
determine and report differences between two data.frames. It was written in the spirit of replacing `PROC COMPARE`
from SAS.

```{r results = 'asis'}
library(arsenal)
```

Why "comparedf"? We originally called this function `compare.data.frame()`, using `testthat::compare()` as our S3 generic,
but that ended up getting us in trouble because of conflicting object structures. Why this didn't occur to us at the time remains a mystery.
To replace it, we brainstormed several ideas (`comparedf()`, `dfcompare()`, `collate()`, `comparison()`) but settled on the former for three
reasons:

1. There were no other objects with that generic or class (see `testthat::compare()` and `compare::compare()`).

2. It is mnemonically easy to remember (we "compare data.frames", not "data.frames compare").

3. It tab auto-completes from the original "compare".

# Basic examples

We first build two similar data.frames to compare.

```{r}
df1 <- data.frame(id = paste0("person", 1:3),
                  a = c("a", "b", "c"),
                  b = c(1, 3, 4),
                  c = c("f", "e", "d"),
                  row.names = paste0("rn", 1:3),
                  stringsAsFactors = FALSE)
df2 <- data.frame(id = paste0("person", 3:1),
                  a = c("c", "b", "a"),
                  b = c(1, 3, 4),
                  d = paste0("rn", 1:3),
                  row.names = paste0("rn", c(1,3,2)),
                  stringsAsFactors = FALSE)
```

To compare these datasets, simply pass them to the `comparedf()` function:

```{r results='markup'}
comparedf(df1, df2)
```

Use `summary()` to get a more detailed summary

```{r}
summary(comparedf(df1, df2))
```

By default, the datasets are compared row-by-row. To change this, use the `by=` or `by.x=` and `by.y=` arguments:

```{r}
summary(comparedf(df1, df2, by = "id"))
```

# A larger example

Let's muck up the `mockstudy` data.

```{r}
data(mockstudy)
mockstudy2 <- muck_up_mockstudy()
```

We've changed row order, so let's compare by the case ID:

```{r}
summary(comparedf(mockstudy, mockstudy2, by = "case"))
```

# Column name comparison options

It is possible to change which column names are considered "the same variable".

## Ignoring case

For example, to ignore case in variable names (so that `Arm` and `arm` are considered the same), pass `tol.vars = "case"`.

You can do this using `comparedf.control()`

```{r eval = FALSE}
summary(comparedf(mockstudy, mockstudy2, by = "case", control = comparedf.control(tol.vars = "case")))
```

or pass it through the `...` arguments.

```{r}
summary(comparedf(mockstudy, mockstudy2, by = "case", tol.vars = "case"))
```

## Treating dots and underscores the same (equivalence classes)

It is possible to treat certain characters or sets of characters as the same by
passing a character vector of equivalence classes to the `tol.vars=` argument.

In short, each string in the vector is split into single characters, and the resulting set of characters is
replaced by the first character in the string. For example, passing `c("._")` would replace all
underscores with dots in the column names of both datasets. Similarly, passing `c("aA", "BbCc")` would
replace all instances of `"A"` with `"a"` and all instances of `"b"`, `"C"`, or `"c"` with `"B"`.
This is one way to ignore case for certain letters. Otherwise, it's possible to combine
the equivalence classes with ignoring case, by passing (e.g.) `c("._", "case")`.

Passing a single character as an element this vector will replace that character with the empty string.
For example, passing c(" ", ".") would remove all spaces and dots from the column names.

For mockstudy, let's treat dots, underscores, and spaces as the same, and ignore case:

```{r}
summary(comparedf(mockstudy, mockstudy2, by = "case",
                tol.vars = c("._ ", "case") # dots=underscores=spaces, ignore case
))
```

## Manually specifying columns to match together

If you pass a named vector to the `tol.vars=` argument, `comparedf()` will line up the names of that
vector to the column names of `x` and the values of that vector to the column names of `y`. In this
way, you can manually specify which non-identically-named columns to compare.

For mockstudy, let's specify our variables manually in this way:

```{r}
summary(comparedf(mockstudy, mockstudy2, by = "case",
                tol.vars = c(arm = "Arm", fu.stat = "fu stat", fu.time = "fu_time")
))
```

# Column comparison options

## Logical tolerance

Use the `tol.logical=` argument to change how logicals are compared. By default, they're expected to be equal to each other.

## Numeric tolerance

To allow numeric differences of a certain tolerance, use the `tol.num=` and `tol.num.val=` options.
`tol.num.val=` determines the maximum (unsigned) difference tolerated if `tol.num="absolute"` (default),
and determines the maximum (unsigned) percent difference tolerated if `tol.num="percent"`.

Also note the option `int.as.num=`, which determines whether integers and numerics should be compared despite
their class difference. If `TRUE`, the integers are coerced to numeric.
Note that `mockstudy$ast` is integer, while `mockstudy2$ast` is numeric:

```{r}
summary(comparedf(mockstudy, mockstudy2, by = "case",
                tol.vars = c("._ ", "case"), # dots=underscores=spaces, ignore case
                int.as.num = TRUE            # compare integers and numerics
))
```

Suppose a tolerance of up to 10 is allowed for `ast`:

```{r}
summary(comparedf(mockstudy, mockstudy2, by = "case",
                tol.vars = c("._ ", "case"), # dots=underscores=spaces, ignore case
                int.as.num = TRUE,           # compare integers and numerics
                tol.num.val = 10             # allow absolute differences <= 10
))
```

## Factor tolerance

By default, factors are compared to each other based on both the labels and the underlying
numeric levels. Set `tol.factor="levels"` to match only the numeric levels, or set
`tol.factor="labels"` to match only the labels.

```{r}
summary(comparedf(mockstudy, mockstudy2, by = "case",
                tol.vars = c("._ ", "case"), # dots=underscores=spaces, ignore case
                int.as.num = TRUE,           # compare integers and numerics
                tol.num.val = 10,            # allow absolute differences <= 10
                tol.factor = "labels"        # match only factor labels
))
```

Also note the option `factor.as.char=`, which determines whether factors and characters should be compared despite
their class difference. If `TRUE`, the factors are coerced to characters.
Note that `mockstudy$race` is a character, while `mockstudy2$race` is a factor:

```{r}
summary(comparedf(mockstudy, mockstudy2, by = "case",
                tol.vars = c("._ ", "case"), # dots=underscores=spaces, ignore case
                int.as.num = TRUE,           # compare integers and numerics
                tol.num.val = 10,            # allow absolute differences <= 10
                tol.factor = "labels",       # match only factor labels
                factor.as.char = TRUE        # compare factors and characters
))
```

## Character tolerance

Use the `tol.char=` argument to change how character variables are compared.
By default, they are compared as-is, but they can be compared after ignoring case
or trimming whitespace or both.

```{r}
summary(comparedf(mockstudy, mockstudy2, by = "case",
                tol.vars = c("._ ", "case"), # dots=underscores=spaces, ignore case
                int.as.num = TRUE,           # compare integers and numerics
                tol.num.val = 10,            # allow absolute differences <= 10
                tol.factor = "labels",       # match only factor labels
                factor.as.char = TRUE,       # compare factors and characters
                tol.char = "case"            # ignore case in character vectors
))
```

## Date tolerance

Use the `tol.date=` argument to change how dates are compared. By default, they're expected to be equal to each other.

## Other data type tolerances

Use the `tol.other=` argument to change how other objects are compared. By default,
they're expected to be `identical()`.

## Specifying tolerances for each variable

You can also provide a list of tolerance functions to `comparedf()`:

```{r eval=FALSE}
comparedf.control(tol.char = list(
  "none",      # the default
  x1 = "case", # be case-insensitive for the variable "x1"
  x2 = function(x, y) tol.NA(x, y, x != y | y == "NA") # a custom-defined tolerance
))
```

## User-defined tolerance functions

### Details

The `comparedf.control()` function accepts functions for any of the tolerance arguments in addition
to the short-hand character strings. This allows the user to create custom tolerance functions to suit his/her needs.

Any custom tolerance function must accept two vectors as arguments and return a logical vector of the same length. The `TRUE`s in
the results should correspond to elements which are deemed "different". Note that the numeric and date tolerance functions should also
include a third argument for tolerance size (even if it's not used).

CAUTION: the results should not include NAs, since the logical vector is used to subset the input data.frames. The `tol.NA()` function
is useful for considering any NAs in the two vectors (but not both) as differences, in addition to other criteria.

The `tol.NA()` function is used in all default tolerance functions to help handle NAs.

### Example 1

Suppose we want to ignore any dates which are later in the second dataset than the first. We define a custom tolerance function.

```{r results = 'markup'}
my.tol <- function(x, y, tol)
{
  tol.NA(x, y, x > y)
}

date.df1 <- data.frame(dt = as.Date(c("2017-09-07", "2017-08-08", "2017-07-09", NA)))
date.df2 <- data.frame(dt = as.Date(c("2017-10-01", "2017-08-08", "2017-07-10", "2017-01-01")))
n.diffs(comparedf(date.df1, date.df2)) # default finds any differences
n.diffs(comparedf(date.df1, date.df2, tol.date = my.tol)) # our function identifies only the NA as different...
n.diffs(comparedf(date.df2, date.df1, tol.date = my.tol)) # ... until we change the argument order

```

### Example 2

(Continuing our mockstudy example)

Suppose we're okay with NAs getting replaced by -9.

```{r}
tol.minus9 <- function(x, y, tol)
{
  idx1 <- is.na(x) & !is.na(y) & y == -9
  idx2 <- tol.num.absolute(x, y, tol) # find other absolute differences
  return(!idx1 & idx2)
}

summary(comparedf(mockstudy, mockstudy2, by = "case",
                tol.vars = c("._ ", "case"), # dots=underscores=spaces, ignore case
                int.as.num = TRUE,           # compare integers and numerics
                tol.num.val = 10,            # allow absolute differences <= 10
                tol.factor = "labels",       # match only factor labels
                factor.as.char = TRUE,       # compare factors and characters
                tol.char = "case",           # ignore case in character vectors
                tol.num = tol.minus9         # ignore NA -> -9 changes
))
```

# Extract Differences

Differences can be easily extracted using the `diffs()` function. If you only want to determine how many differences
were found, use the `n.diffs()` function.

```{r results = 'markup'}
cmp <- comparedf(mockstudy, mockstudy2, by = "case", tol.vars = c("._ ", "case"), int.as.num = TRUE)
n.diffs(cmp)
head(diffs(cmp))
```

Differences can also be summarized by variable.

```{r results = 'markup'}
diffs(cmp, by.var = TRUE)
```

To report differences from only a few variables, one can pass a list of variable names to `diffs()`.

```{r results = 'markup'}
diffs(cmp, vars = c("ps", "ast"), by.var = TRUE)
diffs(cmp, vars = c("ps", "ast"))
```

# Appendix

## Stucture of the Object

(This section is just as much for my use as for yours!)

```{r}
obj <- comparedf(mockstudy, mockstudy2, by = "case")
```

There are two main objects in the `"comparedf"` object, each with its own print method.

The `frame.summary` contains:

- the substituted-deparsed arguments

- information about the number of columns and rows in each dataset

- the by-variables for each dataset (which may not be the same)

- the attributes for each dataset (which get counted in the print method)

- a data.frame of by-variables and row numbers of observations not shared between datasets

- the number of shared observations

```{r results='markup'}
print(obj$frame.summary)
```

The `vars.summary` contains:

- variable name, column number, and class vector (with possibly more than one element) for each x and y.
  These are all `NA` if there isn't a match in both datasets.
  
- values, a list-column of the text string `"by-variable"` for the by-variables,
  `NULL` for columns that aren't compared, or a data.frame containing:

    - The by-variables for differences found
    
    - The values which are different for x and y
    
    - The row numbers for differences found
    
- attrs, a list-column of `NULL` if there are no attributes, or
  a data.frame containing:
  
    - The name of the attributes
    
    - The attributes for x and y, set to `NA` if non-existant
    
    - The actual attributes (if `show.attr=TRUE`).

```{r results='markup'}
print(obj$vars.summary)
```

---
title: "The modelsum function"
author: "Beth Atkinson, Ethan Heinzen, Pat Votruba, Jason Sinnwell, Shannon McDonnell and Greg Dougherty"
output:
  rmarkdown::html_vignette:
    toc: yes
    toc_depth: 3
vignette: |
  %\VignetteIndexEntry{The modelsum function}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r, echo=FALSE, message=FALSE, results='hide', warning=FALSE}
require(knitr)
require(broom)
require(MASS)
require(pROC)
require(rpart)
 
opts_chunk$set(comment = NA, echo=TRUE, prompt=TRUE, collapse=TRUE)

```


# Introduction

Very often we are asked to summarize model results from multiple fits into a nice table.
The endpoint might be of different types (e.g., survival, case/control, continuous) and there
may be several independent variables that we want to examine univariately or adjusted for certain
variables such as age and sex. Locally at Mayo, the SAS macros `%modelsum`, `%glmuniv`, and `%logisuni`
were written to create such summary tables. With the increasing interest in R, we have developed the
function `modelsum` to create similar tables within the R environment.  

In developing the `modelsum` function, the goal was to bring the best features of these macros into an R function.
However, the task was not simply to duplicate all the functionality, but rather to make use of R's strengths
(modeling, method dispersion, flexibility in function definition and output format) and make a tool that fits
the needs of R users.  Additionally, the results needed to fit within the general reproducible research framework
so the tables could be displayed within an R markdown report.

This report provides step-by-step directions for using the functions associated with `modelsum`.
All functions presented here are available within the `arsenal` package.  An assumption is made that users
are somewhat familiar with R markdown documents.  For those who are new to the topic, a good initial
resource is available at [rmarkdown.rstudio.com](https://rmarkdown.rstudio.com/). 

# Simple Example

The first step when using the `modelsum` function is to load the `arsenal` package.  All the examples in this report
use a dataset called `mockstudy` made available by Paul Novotny which includes a variety of types of variables
(character, numeric, factor, ordered factor, survival) to use as examples.

```{r, load-data}
require(arsenal)
data(mockstudy) # load data
dim(mockstudy)  # look at how many subjects and variables are in the dataset 
# help(mockstudy) # learn more about the dataset and variables
str(mockstudy) # quick look at the data
```

To create a simple linear regression table (the default), use a formula statement to specify the variables
that you want summarized.  The example below predicts BMI with the variables sex and age. 

```{r simple1}
tab1 <- modelsum(bmi ~ sex + age, data=mockstudy)
```

If you want to take a quick look at the table, you can use `summary` on your modelsum object and the table will
print out as text in your R console window.  If you use `summary` without any options you will see a number of
$\&nbsp;$ statements which translates to "space" in HTML.

## Pretty text version of table

If you want a nicer version in your console window then adding the `text=TRUE` option.  

```{r simple-text}
summary(tab1, text=TRUE)
```

## Pretty Rmarkdown version of table

In order for the report to look nice within an R markdown (knitr) report, you just need to specify
`results="asis"` when creating the r chunk. This changes the layout slightly (compresses it) and bolds
the variable names.

```{r simple-markdown, results='asis'}
summary(tab1)
```

## Data frame version of table

If you want a data.frame version, simply use `as.data.frame`.  

```{r}
as.data.frame(tab1)
```

## Add an adjustor to the model

The argument `adjust` allows the user to indicate that all the variables should be adjusted for these terms. To adjust each model
for age and sex (for instance), we use `adjust = ~ age + sex`:

```{r adjust, results="asis"}
tab2 <- modelsum(alk.phos ~ arm + ps + hgb, adjust= ~age + sex, data=mockstudy)
summary(tab2)
```

# Models for each endpoint type

To make sure the correct model is run you need to specify "family".  The options available right
now are : gaussian, binomial, survival, and poisson.  If there is enough interest, additional models can be added.  

## Gaussian

### Fit and summarize linear regression model

Look at whether there is any evidence that AlkPhos values vary by study arm after adjusting for sex and age (assuming a linear age relationship).

```{r}
fit <- lm(alk.phos ~ arm + age + sex, data=mockstudy)
summary(fit)
plot(fit)
```

The results suggest that the endpoint may need to be transformed.  Calculating the Box-Cox transformation suggests a log transformation.

```{r}
require(MASS)
boxcox(fit)
```

```{r}
fit2 <- lm(log(alk.phos) ~ arm + age + sex, data=mockstudy)
summary(fit2)
plot(fit2)
```

Finally, look to see whether there there is a non-linear relationship with age.

```{r}
require(splines)
fit3 <- lm(log(alk.phos) ~ arm + ns(age, df=2) + sex, data=mockstudy)

# test whether there is a difference between models 
stats::anova(fit2,fit3)

# look at functional form of age
termplot(fit3, term=2, se=T, rug=T)
```

In this instance it looks like there isn't enough evidence to say that the relationship is non-linear.

### Extract data using the `broom` package

The `broom` package makes it easy to extract information from the fit.

```{r}
tmp <- tidy(fit3) # coefficients, p-values
class(tmp)
tmp

glance(fit3)
```

### Create a summary table using modelsum

```{r, results="asis"}
ms.logy <- modelsum(log(alk.phos) ~ arm + ps + hgb, data=mockstudy, adjust= ~age + sex, 
                    family=gaussian,  
                    gaussian.stats=c("estimate","CI.lower.estimate","CI.upper.estimate","p.value"))
summary(ms.logy)
```

## Binomial

### Fit and summarize logistic regression model

```{r}
boxplot(age ~ mdquality.s, data=mockstudy, ylab=attr(mockstudy$age,'label'), xlab='mdquality.s')

fit <- glm(mdquality.s ~ age + sex, data=mockstudy, family=binomial)
summary(fit)

# create Odd's ratio w/ confidence intervals
tmp <- data.frame(summary(fit)$coef)
tmp

tmp$OR <- round(exp(tmp[,1]),2)
tmp$lower.CI <- round(exp(tmp[,1] - 1.96* tmp[,2]),2)
tmp$upper.CI <- round(exp(tmp[,1] + 1.96* tmp[,2]),2)
names(tmp)[4] <- 'P-value'

kable(tmp[,c('OR','lower.CI','upper.CI','P-value')])

# Assess the predictive ability of the model

# code using the pROC package
require(pROC)
pred <- predict(fit, type='response')
tmp <- pROC::roc(mockstudy$mdquality.s[!is.na(mockstudy$mdquality.s)]~ pred, plot=TRUE, percent=TRUE)
tmp$auc

```

### Extract data using `broom` package

The `broom` package makes it easy to extract information from the fit.

```{r}
tidy(fit, exp=T, conf.int=T) # coefficients, p-values, conf.intervals

glance(fit) # model summary statistics
```

### Create a summary table using modelsum

```{r, results="asis"}
summary(modelsum(mdquality.s ~ age + bmi, data=mockstudy, adjust=~sex, family=binomial))

fitall <- modelsum(mdquality.s ~ age, data=mockstudy, family=binomial,
                   binomial.stats=c("Nmiss2","OR","p.value"))
summary(fitall)
```


## Survival

### Fit and summarize a Cox regression model 

```{r survival}
require(survival)

# multivariable model with all 3 terms
fit  <- coxph(Surv(fu.time, fu.stat) ~ age + sex + arm, data=mockstudy)
summary(fit)

# check proportional hazards assumption
fit.z <- cox.zph(fit)
fit.z
plot(fit.z[1], resid=FALSE) # makes for a cleaner picture in this case
abline(h=coef(fit)[1], col='red')

# check functional form for age using pspline (penalized spline)
# results are returned for the linear and non-linear components
fit2 <- coxph(Surv(fu.time, fu.stat) ~ pspline(age) + sex + arm, data=mockstudy)
fit2

# plot smoothed age to visualize why significant
termplot(fit2, se=T, terms=1)
abline(h=0)

# The c-statistic comes out in the summary of the fit
summary(fit2)$concordance

# It can also be calculated using the survConcordance function
survConcordance(Surv(fu.time, fu.stat) ~ predict(fit2), data=mockstudy)
```

### Extract data using `broom` package

The `broom` package makes it easy to extract information from the fit.

```{r}
tidy(fit) # coefficients, p-values

glance(fit) # model summary statistics
```

### Create a summary table using modelsum

```{r results="asis"}
##Note: You must use quotes when specifying family="survival" 
##      family=survival will not work
summary(modelsum(Surv(fu.time, fu.stat) ~ arm, 
                 adjust=~age + sex, data=mockstudy, family="survival"))

##Note: the pspline term is not working yet
#summary(modelsum(Surv(fu.time, fu.stat) ~ arm, 
#                adjust=~pspline(age) + sex, data=mockstudy, family='survival'))
```


## Poisson

Poisson regression is useful when predicting an outcome variable representing counts.
It can also be useful when looking at survival data.  Cox models and Poisson models are very closely
related and survival data can be summarized using Poisson regression. If you have overdispersion (see
if the residual deviance is much larger than degrees of freedom), you may want to use `quasipoisson()`
instead of `poisson()`.  Some of these diagnostics need to be done outside of `modelsum`. 

### Example 1: fit and summarize a Poisson regression model 

For the first example, use the solder dataset available in the `rpart` package.  The endpoint `skips` has a definite Poisson look.

```{r poisson}
require(rpart) ##just to get access to solder dataset
data(solder)
hist(solder$skips)

fit <- glm(skips ~ Opening + Solder + Mask , data=solder, family=poisson)
stats::anova(fit, test='Chi')
summary(fit)
```

Overdispersion is when the Residual deviance is larger than the degrees of freedom.  This can be tested, approximately using the following code.  The goal is to have a p-value that is $>0.05$.

```{r}
1-pchisq(fit$deviance, fit$df.residual)
```

One possible solution is to use the quasipoisson family instead of the poisson family.  This adjusts for the overdispersion.

```{r}
fit2 <- glm(skips ~ Opening + Solder + Mask, data=solder, family=quasipoisson)
summary(fit2)
```

### Extract data using `broom` package

The `broom` package makes it easy to extract information from the fit.

```{r}
tidy(fit) # coefficients, p-values

glance(fit) # model summary statistics
```


### Create a summary table using modelsum

```{r results='asis'}
summary(modelsum(skips~Opening + Solder + Mask, data=solder, family="quasipoisson"))
summary(modelsum(skips~Opening + Solder + Mask, data=solder, family="poisson"))
```

### Example 2: fit and summarize a Poisson regression model 

This second example uses the survival endpoint available in the `mockstudy` dataset.  There is a close
relationship between survival and Poisson models, and often it is easier to fit the model using Poisson
regression, especially if you want to present absolute risk.

```{r}
# add .01 to the follow-up time (.01*1 day) in order to keep everyone in the analysis
fit <- glm(fu.stat ~ offset(log(fu.time+.01)) + age + sex + arm, data=mockstudy, family=poisson)
summary(fit)
1-pchisq(fit$deviance, fit$df.residual)

coef(coxph(Surv(fu.time,fu.stat) ~ age + sex + arm, data=mockstudy))
coef(fit)[-1]

# results from the Poisson model can then be described as risk ratios (similar to the hazard ratio)
exp(coef(fit)[-1])

# As before, we can model the dispersion which alters the standard error
fit2 <- glm(fu.stat ~ offset(log(fu.time+.01)) + age + sex + arm, 
            data=mockstudy, family=quasipoisson)
summary(fit2)
```

### Extract data using `broom` package

The `broom` package makes it easy to extract information from the fit.

```{r}
tidy(fit) ##coefficients, p-values

glance(fit) ##model summary statistics
```


### Create a summary table using `modelsum`

Remember that the result from `modelsum` is different from the `fit` above.  The `modelsum`
summary shows the results for `age + offset(log(fu.time+.01))` then `sex + offset(log(fu.time+.01))`
instead of `age + sex + arm + offset(log(fu.time+.01))`.

```{r results="asis", eval=TRUE}
summary(modelsum(fu.stat ~ age, adjust=~offset(log(fu.time+.01))+ sex + arm, 
                 data=mockstudy, family=poisson))
                 
```


# Additional Examples


Here are multiple examples showing how to use some of the different options.

## 1. Change summary statistics globally

There are standard settings for each type of model regarding what information is summarized in the table.
This behavior can be modified using the modelsum.control function. In fact, you can save your standard
settings and use that for future tables.  


```{r, results='asis'}
mycontrols  <- modelsum.control(gaussian.stats=c("estimate","std.error","adj.r.squared","Nmiss"),
                                show.adjust=FALSE, show.intercept=FALSE)                            
tab2 <- modelsum(bmi ~ age, adjust=~sex, data=mockstudy, control=mycontrols)
summary(tab2)
```

You can also change these settings directly in the modelsum call.  

```{r, results='asis'}
tab3 <- modelsum(bmi ~  age, adjust=~sex, data=mockstudy,
                 gaussian.stats=c("estimate","std.error","adj.r.squared","Nmiss"), 
                 show.intercept=FALSE, show.adjust=FALSE)
summary(tab3)
```

## 2. Add labels to independent variables

In the above example, age is shown with a label (Age in Years), but sex is listed "as is".
This is because the data was created in SAS and in the SAS dataset, age had a label but sex did not.
The label is stored as an attribute within R.

```{r check-labels}
## Look at one variable's label
attr(mockstudy$age,'label')

## See all the variables with a label
unlist(lapply(mockstudy,'attr','label'))

## or
cbind(sapply(mockstudy,attr,'label'))
```

If you want to add labels to other variables, there are a couple of options.  First, you could add labels to the variables in your dataset.

```{r add-label, results='asis'}
attr(mockstudy$age,'label')  <- 'Age, yrs'

tab1 <- modelsum(bmi ~  age, adjust=~sex, data=mockstudy)
summary(tab1)
```

You can also use the built-in `data.frame` method for `labels<-`:

```{r, results = 'asis'}
labels(mockstudy)  <- c(age = 'Age, yrs')

tab1 <- modelsum(bmi ~  age, adjust=~sex, data=mockstudy)
summary(tab1)
```

Another option is to add labels after you have created the table

```{r, results='asis'}
mylabels <- list(sexFemale = "Female", age ="Age, yrs")
summary(tab1, labelTranslations = mylabels)
```

Alternatively, you can check the variable labels and manipulate them with a function called `labels`, which works on the `modelsum` object.

```{r, eval=TRUE}
labels(tab1)
labels(tab1) <- c(sexFemale="Female", age="Baseline Age (yrs)")
labels(tab1)
```

```{r, results='asis'}
summary(tab1)
```

## 3. Don't show intercept values

```{r, results='asis'}
summary(modelsum(age~mdquality.s+sex, data=mockstudy), show.intercept=FALSE)
```

## 4. Don't show results for adjustment variables

```{r, results='asis'}
summary(modelsum(mdquality.s ~ age + bmi, data=mockstudy, adjust=~sex, family=binomial),
        show.adjust=FALSE)  
```

## 5. Summarize multiple variables without typing them out

Often one wants to summarize a number of variables.  Instead of typing by hand each individual variable,
an alternative approach is to create a formula using the `paste` command with the `collapse="+"` option.  

```{r, results='asis'}
# create a vector specifying the variable names
myvars <- names(mockstudy)

# select the 8th through the 12th
# paste them together, separated by the + sign
RHS <- paste(myvars[8:12], collapse="+")
RHS

# create a formula using the as.formula function
as.formula(paste('mdquality.s ~ ', RHS))

# use the formula in the modelsum function
summary(modelsum(as.formula(paste('mdquality.s ~', RHS)), family=binomial, data=mockstudy))
```

These steps can also be done using the `formulize` function.

```{r, results='asis'}
## The formulize function does the paste and as.formula steps
tmp <- formulize('mdquality.s',myvars[8:10])
tmp

## More complex formulas could also be written using formulize
tmp2 <- formulize('mdquality.s',c('ps','hgb','sqrt(bmi)'))

## use the formula in the modelsum function
summary(modelsum(tmp, data=mockstudy, family=binomial))
```


## 6. Subset the dataset used in the analysis

Here are two ways to get the same result (limit the analysis to subjects age>50 and in the F: FOLFOX treatment group). 

* The first approach uses the subset function applied to the dataset `mockstudy`.
This example also selects a subset of variables.  The `modelsum` function is then applied to this subsetted data.


```{r}
newdata <- subset(mockstudy, subset=age>50 & arm=='F: FOLFOX', select = c(age,sex, bmi:alk.phos))
dim(mockstudy)
table(mockstudy$arm)
dim(newdata)
names(newdata)
```

```{r, results='asis'}
summary(modelsum(alk.phos ~ ., data=newdata))
```

* The second approach does the same analysis but uses the subset
argument within `modelsum` to subset the data.

```{r, results='asis', eval=TRUE}
summary(modelsum(log(alk.phos) ~ sex + ps + bmi, subset=age>50 & arm=="F: FOLFOX", data=mockstudy))
summary(modelsum(alk.phos ~ ps + bmi, adjust=~sex, subset = age>50 & bmi<24, data=mockstudy))
summary(modelsum(alk.phos ~ ps + bmi, adjust=~sex, subset=1:30, data=mockstudy))
```

## 7. Create combinations of variables on the fly

```{r}
## create a variable combining the levels of mdquality.s and sex
with(mockstudy, table(interaction(mdquality.s,sex)))
```

```{r, results='asis'}
summary(modelsum(age ~ interaction(mdquality.s,sex), data=mockstudy))
```

## 8. Transform variables on the fly

Certain transformations need to be surrounded by `I()` so that R knows to treat it as a variable
transformation and not some special model feature.  If the transformation includes any of the
symbols `/ - + ^ *` then surround the new variable by `I()`.


```{r, results='asis'}
summary(modelsum(arm=="F: FOLFOX" ~ I(age/10) + log(bmi) + mdquality.s,
                 data=mockstudy, family=binomial))
```


## 9. Change the ordering of the variables or delete a variable

```{r, results='asis'}
mytab <- modelsum(bmi ~ sex + alk.phos + age, data=mockstudy)
mytab2 <- mytab[c('age','sex','alk.phos')]
summary(mytab2)
summary(mytab[c('age','sex')])
summary(mytab[c(3,1)])
```

## 10. Merge two `modelsum` objects together 

It is possible to merge two modelsum objects so that they print out together, however you need to pay
attention to the columns that are being displayed.  It is sometimes easier to combine two models of the same
family (such as two sets of linear models). Overlapping y-variables will have their x-variables
concatenated, and (if `all=TRUE`) non-overlapping y-variables will have their tables printed separately.

```{r, results="asis"}
## demographics
tab1 <- modelsum(bmi ~ sex + age, data=mockstudy)
## lab data
tab2 <- modelsum(mdquality.s ~ hgb + alk.phos, data=mockstudy, family=binomial)
                
tab12 <- merge(tab1, tab2, all = TRUE)
class(tab12)
summary(tab12)
```

## 11. Add a title to the table

When creating a pdf the tables are automatically numbered and the title appears below the table.
In Word and HTML, the titles appear un-numbered and above the table.  

```{r, results='asis'}
t1 <- modelsum(bmi ~ sex + age, data=mockstudy)
summary(t1, title='Demographics')
```

## 12. Modify how missing values are treated

Depending on the report you are writing you have the following options: 

* Use all values available for each variable

* Use only those subjects who have measurements available for all the variables

```{r}
## look at how many missing values there are for each variable
apply(is.na(mockstudy),2,sum)
```

```{r, results='asis'}
## Show how many subjects have each variable (non-missing)
summary(modelsum(bmi ~ ast + age, data=mockstudy,
                control=modelsum.control(gaussian.stats=c("N","estimate"))))

## Always list the number of missing values
summary(modelsum(bmi ~ ast + age, data=mockstudy,
                control=modelsum.control(gaussian.stats=c("Nmiss2","estimate"))))

## Only show the missing values if there are some (default)
summary(modelsum(bmi ~ ast + age, data=mockstudy, 
                control=modelsum.control(gaussian.stats=c("Nmiss","estimate"))))

## Don't show N at all
summary(modelsum(bmi ~ ast + age, data=mockstudy, 
                control=modelsum.control(gaussian.stats=c("estimate"))))
```

## 13. Modify the number of digits used

Within modelsum.control function there are 3 options for controlling the number of significant digits shown.  

* digits: controls the number of digits after the decimal point for continuous values

* digits.ratio: controls the number of digits after the decimal point for continuous values

* digits.p: controls the number of digits after the decimal point for continuous values

```{r, results='asis'}
summary(modelsum(bmi ~ sex + age + fu.time, data=mockstudy), digits=4, digits.test=2)
```

## 14. Use case-weights in the models

Occasionally it is of interest to fit models using case weights.
The `modelsum` function allows you to pass on the weights to the models and it will do the appropriate fit.

```{r}
mockstudy$agegp <- cut(mockstudy$age, breaks=c(18,50,60,70,90), right=FALSE)

## create weights based on agegp and sex distribution
tab1 <- with(mockstudy,table(agegp, sex))
tab1
tab2 <- with(mockstudy, table(agegp, sex, arm))
gpwts <- rep(tab1, length(unique(mockstudy$arm)))/tab2

## apply weights to subjects
index <- with(mockstudy, cbind(as.numeric(agegp), as.numeric(sex), as.numeric(as.factor(arm)))) 
mockstudy$wts <- gpwts[index]

## show weights by treatment arm group
tapply(mockstudy$wts,mockstudy$arm, summary)
```

```{r results='asis'}
mockstudy$newvarA <- as.numeric(mockstudy$arm=='A: IFL')
tab1 <- modelsum(newvarA ~ ast + bmi + hgb, data=mockstudy, subset=(arm !='G: IROX'), 
                 family=binomial)
summary(tab1, title='No Case Weights used')

suppressWarnings({
tab2 <- modelsum(newvarA ~ ast + bmi + hgb, data=mockstudy, subset=(arm !='G: IROX'), 
                 weights=wts, family=binomial)
summary(tab2, title='Case Weights used')
})
```
          
## 15. Use `modelsum` within an Sweave document

For those users who wish to create tables within an Sweave document, the following code seems to work.

```
\documentclass{article}

\usepackage{longtable}
\usepackage{pdfpages}

\begin{document}

\section{Read in Data}
<<echo=TRUE>>=
require(arsenal)
require(knitr)
require(rmarkdown)
data(mockstudy)

tab1 <- modelsum(bmi~sex+age, data=mockstudy)
@

\section{Convert Summary.modelsum to LaTeX}
<<echo=TRUE, results='hide', message=FALSE>>=
capture.output(summary(tab1), file="Test.md")

## Convert R Markdown Table to LaTeX
render("Test.md", pdf_document(keep_tex=TRUE))
@ 

\includepdf{Test.pdf}

\end{document}
```
## 16. Export `modelsum` results to a .CSV file

When looking at multiple variables it is sometimes useful to export the results to a csv file.
The `as.data.frame` function creates a data frame object that can be exported or further manipulated within R.


```{r}
summary(tab2, text=T)
tmp <- as.data.frame(summary(tab2, text = TRUE))
tmp
# write.csv(tmp, '/my/path/here/mymodel.csv')
```

## 17. Write `modelsum` object to a separate Word or HTML file

```{r eval = FALSE}
## write to an HTML document
write2html(tab2, "~/ibm/trash.html")

## write to a Word document
write2word(tab2, "~/ibm/trash.doc", title="My table in Word")
```

## 18. Use `modelsum` in R Shiny

The easiest way to output a `modelsum()` object in an R Shiny app is to use the `tableOutput()` UI in combination with
the `renderTable()` server function and `as.data.frame(summary(modelsum()))`:

```{r eval=FALSE}
# A standalone shiny app
library(shiny)
library(arsenal)
data(mockstudy)

shinyApp(
  ui = fluidPage(tableOutput("table")),
  server = function(input, output) {
    output$table <- renderTable({
      as.data.frame(summary(modelsum(age ~ sex, data = mockstudy), text = "html"))
    }, sanitize.text.function = function(x) x)
  }
)
```

This can be especially powerful if you feed the selections from a `selectInput(multiple = TRUE)` into `formulize()` to make
the table dynamic!

## 23. Use `modelsum` in bookdown

Since the backbone of `modelsum()` is `knitr::kable()`, tables still render well in bookdown. However, `print.summary.modelsum()` doesn't use
the `caption=` argument of `kable()`, so some tables may not have a properly numbered caption. To fix this, use the method described
[on the bookdown site](https://pkg.yihui.org/bookdown/tables.html) to give the table a tag/ID.

```{r eval=FALSE}
summary(modelsum(age ~ sex, data = mockstudy), title="(\\#tab:mytableby) Caption here")
```

## 24. Model multiple endpoints

You can now use `list()` on the left-hand side of `modelsum()` to give multiple endpoints.
Note that only one "family" can be specified this way (use `merge()` instead if you want multiple families).

```{r results='asis'}
summary(modelsum(list(age, hgb) ~ bmi + sex, adjust = ~ arm, data = mockstudy))
```

To avoid confusion about which table is which endpoint, you can set `term.name=TRUE` in `summary()`. This takes the labels
for each endpoint and puts them in the top-left of the table.

```{r results='asis'}
summary(modelsum(list(age, hgb) ~ bmi + sex, adjust = ~ arm, data = mockstudy), term.name = TRUE)
```

## 25. Model data by a non-test group (strata)

You can also specify a grouping variable that doesn't get tested (but instead separates results): a *strata* variable.

```{r results='asis'}
summary(modelsum(list(age, hgb) ~ bmi + sex, strata = arm, data = mockstudy))
```

## 26. Add multiple sets of adjustors to the model

By putting multiple formulas into a list, you can use multiple sets of adjustors. Use `~ 1` or `NULL` for an "unadjusted" model. By using the 
`adjustment.names=TRUE` argument and giving names to your adjustor sets in the list, you can name the various analyses.

```{r}
adj.list <- list(
  Unadjusted = ~ 1, # can also specify NULL here
  "Adjusted for Arm" = ~ arm
)
multi.adjust <- modelsum(list(age, bmi) ~ fu.time + ast, adjust = adj.list, data = mockstudy)
summary(multi.adjust, adjustment.names = TRUE)
summary(multi.adjust, adjustment.names = TRUE, show.intercept = FALSE, show.adjust = FALSE)
```


# Available Function Options

## Summary statistics

The available summary statistics, by varible type, are:

* `ordinal`: Ordinal logistic regression models
  + default: `Nmiss, OR, CI.lower.OR, CI.upper.OR, p.value`
  + optional: `estimate, CI.OR, CI.estimate, CI.lower.estimate, CI.upper.estimate,`
       `N, Nmiss2, endpoint, std.error, statistic, logLik, AIC, BIC, edf, deviance, df.residual, p.value.lrt`
* `binomial`,`quasibinomial`: Logistic regression models 
  +  default:  `OR, CI.lower.OR, CI.upper.OR, p.value, concordance, Nmiss`
  +  optional: `estimate, CI.OR, CI.estimate, CI.lower.estimate, CI.upper.estimate,`
        `CI.wald, CI.lower.wald, CI.upper.wald, CI.OR.wald, CI.lower.OR.wald, CI.upper.OR.wald,`
        `N, Nmiss2, Nevents, endpoint, std.error, statistic, logLik, AIC, BIC, null.deviance, deviance, df.residual, df.null, p.value.lrt`
* `gaussian`: Linear regression models 
  +  default:  `estimate, std.error, p.value, adj.r.squared, Nmiss`
  +  optional: `CI.estimate, CI.lower.estimate, CI.upper.estimate, N, Nmiss2, statistic,`
        `standard.estimate, endpoint, r.squared, AIC, BIC, logLik, statistic.F, p.value.F, p.value.lrt`
* `poisson`, `quasipoisson`: Poisson regression models 
  +  default: `RR, CI.lower.RR, CI.upper.RR, p.value, Nmiss`
  +  optional: `CI.RR, CI.estimate, CI.lower.estimate, CI.upper.estimate, CI.RR, Nmiss2, std.error,`
        `estimate, statistic, endpoint, AIC, BIC, logLik, dispersion, null.deviance, deviance, df.residual, df.null, p.value.lrt`
* `negbin`: Negative binomial regression models 
  +  default: `RR, CI.lower.RR, CI.upper.RR, p.value, Nmiss`
  +  optional: `CI.RR, CI.estimate, CI.lower.estimate, CI.upper.estimate, CI.RR, Nmiss2, std.error, estimate,`
        `statistic, endpoint, AIC, BIC, logLik, dispersion, null.deviance, deviance, df.residual, df.null, theta, SE.theta, p.value.lrt`
* `clog`: Conditional Logistic models
  + default: `OR, CI.lower.OR, CI.upper.OR, p.value, concordance, Nmiss`
  + optional: `CI.OR, CI.estimate, CI.lower.estimate, CI.upper.estimate, N, Nmiss2, estimate, std.error, endpoint, Nevents, statistic,`
        `r.squared, r.squared.max, logLik, AIC, BIC, statistic.log, p.value.log, statistic.sc, p.value.sc,`
        `statistic.wald, p.value.wald, N, std.error.concordance, p.value.lrt`
* `survival`: Cox models
  +  default: `HR, CI.lower.HR, CI.upper.HR, p.value, concordance, Nmiss`
  +  optional: `CI.HR, CI.estimate, CI.lower.estimate, CI.upper.estimate, N, Nmiss2, estimate, std.error, endpoint,`
        `Nevents, statistic, r.squared, r.squared.max, logLik, AIC, BIC, statistic.log, p.value.log, statistic.sc, p.value.sc,`
        `statistic.wald, p.value.wald, N, std.error.concordance, p.value.lrt`

The full description of these parameters that can be shown for models include:

* `N`: a count of the number of observations used in the analysis
* `Nmiss`: only show the count of the number of missing values if there are some missing values 
* `Nmiss2`: always show a count of the number of missing values for a model 
* `endpoint`: dependent variable used in the model
* `std.err`: print the standard error
* `statistic`: test statistic
* `statistic.F`: test statistic (F test)
* `p.value`: print the p-value
* `p.value.lrt`: print the likelihood ratio p-value for *the main effect only* (not the adjustors)
* `r.squared`: print the model R-square 
* `adj.r.squared`: print the model adjusted R-square 
* `r.squared.max`: print the model R-square
* `concordance`: print the model C statistic (which is the AUC for logistic models)
* `logLik`: print the loglikelihood value
* `p.value.log`: print the p-value for the overall model likelihood test
* `p.value.wald`: print the p-value for the overall model wald test
* `p.value.sc`: print the p-value for overall model score test
* `AIC`: print the Akaike information criterion
* `BIC`: print the Bayesian information criterion
* `null.deviance`: null deviance
* `deviance`: model deviance
* `df.residual`: degrees of freedom for the residual
* `df.null`: degrees of freedom for the null model
* `dispersion`: This is used in Poisson models and is defined as the deviance/df.residual
* `statistic.sc`: overall model score statistic
* `statistic.wald`: overall model score statistic
* `statistic.log`: overall model score statistic
* `std.error.concordance`: standard error for the C statistic
* `HR`: print the hazard ratio (for survival models), i.e. exp(beta)
* `CI.lower.HR, CI.upper.HR`: print the confidence interval for the HR 
* `OR`: print the odd's ratio (for logistic models), i.e. exp(beta)
* `CI.lower.OR, CI.upper.OR`: print the confidence interval for the OR
* `CI.lower.OR.wald, CI.upper.OR.wald`: print the Wald confidence interval for the OR
* `RR`: print the risk ratio (for poisson models), i.e. exp(beta)
* `CI.lower.RR, CI.upper.RR`: print the confidence interval for the RR
* `estimate`: print beta coefficient
* `standardized.estimate`: print the standardized beta coefficient 
* `CI.lower.estimate, CI.upper.estimate`: print the confidence interval for the beta coefficient
* `CI.lower.wald, CI.upper.wald`: print the Wald confidence interval for the beta coefficient
* `edf`: print the effective degrees of freedom.
* `theta`: print the estimate of theta.
* `SE.theta`: print the estimate of theta's standard error.

## `modelsum.control` settings

A quick way to see what arguments are possible to utilize in a function is to use the `args()`
command. Settings involving the number of digits can be set in `modelsum.control` or in `summary.modelsum`.

```{r}
args(modelsum.control)
```

## `summary.modelsum` settings

The summary.modelsum function has options that modify how the table appears (such as adding a title or modifying labels).  

```{r}
args(arsenal:::summary.modelsum)
```

---
title: "The freqlist function"
author: "Tina Gunderson and Ethan Heinzen"
output:
  rmarkdown::html_vignette:
    toc: yes
    toc_depth: 3
vignette: |
  %\VignetteIndexEntry{The freqlist function}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, tidy.opts=list(width.cutoff=80), tidy=TRUE, comment=NA)
options(width=80, max.print=1000)
```

# Overview 

`freqlist()` is a function meant to produce output similar to SAS's `PROC FREQ` procedure when using the `/list` option of the `TABLE` statement.
`freqlist()` provides options for handling missing or sparse data and can provide cumulative counts and percentages based on subgroups.
It depends on the `knitr` package for printing. 

```{r message = FALSE}
require(arsenal)
```

## Sample dataset

For our examples, we'll load the `mockstudy` data included with this package and use it to create a basic table.
Because they have fewer levels, for brevity, we'll use the variables arm, sex, and mdquality.s to create the example table.
We'll retain NAs in the table creation.
See the appendix for notes regarding default NA handling and other useful information regarding tables in R. 

```{r loading.data}
# load the data
data(mockstudy)

# retain NAs when creating the table using the useNA argument
tab.ex <- table(mockstudy[, c("arm", "sex", "mdquality.s")], useNA="ifany")
```
 
# The `freqlist` object

The `freqlist()` function is an S3 generic (with methods for tables and formulas) which returns an object of class `"freqlist"`.

```{r console.output}
example1 <- freqlist(tab.ex)

str(example1)

# view the data frame portion of freqlist output
head(as.data.frame(example1)) ## or use as.data.frame(example1)
```

# Basic output using `summary()` 

The `summary` method for `freqlist()` relies on the `kable()` function (in the `knitr` package) for printing.
`knitr::kable()` converts the output to markdown which can be printed in the console or easily rendered in
Word, PDF, or HTML documents.

Note that you must supply `results="asis"` to properly format the markdown output.

```{r, results = 'asis'}
summary(example1)
```

You can print a title for the table using the `title=` argument.

```{r, results = 'asis'}
summary(example1, title="Basic freqlist output")
```

You can also easily pull out the `freqlist` data frame for more complicated formatting or manipulation
(e.g. with another function such as `xtable()` or `pander()`) using `as.data.frame(summary())`:

```{r}
head(as.data.frame(summary(example1)))
```

# Using a formula with `freqlist`

Instead of passing a pre-computed table to `freqlist()`, you can instead pass a formula, which will be
in turn passed to the `xtabs()` function. Additional `freqlist()` arguments are passed through the `...`
to the `freqlist()` table method.

Note that `freqlist()` sets the `addNA=TRUE` argument by default:

```{r results='asis'}
summary(freqlist(~ arm + sex + mdquality.s, data = mockstudy))
```

One can also set NAs to an explicit value using `includeNA()`.

```{r results='asis'}
summary(freqlist(~ arm + sex + includeNA(mdquality.s, "Missing"), data = mockstudy))
```

In fact, since `xtabs()` allows for left-hand-side weights, so does `freqlist()`!

```{r results='asis'}
mockstudy$weights <- c(10000, rep(1, nrow(mockstudy) - 1))
summary(freqlist(weights ~ arm + sex + addNA(mdquality.s), data = mockstudy))
```

You can also specify multiple weights:

```{r results='asis'}
mockstudy$weights2 <- c(rep(1, nrow(mockstudy) - 1), 10000)
summary(freqlist(list(weights, weights2) ~ arm + sex + addNA(mdquality.s), data = mockstudy))
```

# Rounding percentage digits or changing variable names for printing

The `digits.pct=` argument takes a single numeric value and controls the number of digits of percentages in the output.
The `digits.count=` argument takes a similar argument and controls the number of digits of the count columns.
The `labelTranslations=` argument is a named character vector or list.
Both options are applied in the following example. 

```{r labelTranslations, results = 'asis'}
example2 <- freqlist(tab.ex, labelTranslations = c(arm = "Treatment Arm", sex = "Gender", mdquality.s = "LASA QOL"),
                      digits.pct = 1, digits.count = 1)
summary(example2)
```

# Additional examples

## Including combinations with frequencies of zero

The `sparse=` argument takes a single logical value as input. The default option is `FALSE`.
If set to `TRUE`, the sparse option will include combinations with frequencies of zero in the list of results.
As our initial table did not have any such levels, we create a second table to use in our example.

```{r sparse, results = 'asis'}
summary(freqlist(~ race + sex + arm, data = mockstudy, sparse = TRUE, digits.pct=1))
```

## Options for NA handling

The various `na.options=` allow you to include or exclude data with missing values for one or more factor
levels in the counts and percentages, as well as show the missing data but exclude it from the cumulative
counts and percentages. The default option is to include all combinations with missing values. 

```{r na.options, results = 'asis'}
summary(freqlist(tab.ex, na.options="include"))
summary(freqlist(tab.ex, na.options="showexclude"))
summary(freqlist(tab.ex, na.options="remove"))
```

## Frequency counts and percentages subset by factor levels 

The `strata=` argument internally subsets the data by the specified factor prior to calculating cumulative counts and percentages.
By default, when used each subset will print in a separate table. Using the `single = TRUE` option when printing will collapse
the subsetted result into a single table.

```{r freq.counts, results='asis'}
example3 <- freqlist(tab.ex, strata = c("arm","sex"))
summary(example3)

#using the single = TRUE argument will collapse results into a single table for printing
summary(example3, single = TRUE)

```

## Show only the "n" most common combinations in each table (`head()` and `sort()`)

You can now sort `freqlist()` objects, and, by taking the `head()` of the summary, output the most common frequencies. This looks
the prettiest with `dupLabels=TRUE`.

```{r}
head(summary(sort(example1, decreasing = TRUE), dupLabels = TRUE))
```

## Change labels on the fly

```{r changelabs, results = 'asis'}
labs <- c(arm = "Arm", sex = "Sex", mdquality.s = "QOL", freqPercent = "%")
labels(example1) <- labs
summary(example1)
```

You can also supply `labelTranslations=` to `summary()`.

```{r, results = 'asis'}
summary(example1, labelTranslations = labs)
```

## Using `xtable()` to format and print `freqlist()` results

Fair warning: `xtable()` has kind of a steep learning curve. These examples are given without explanation, for more advanced users.

```{r results='asis'}
require(xtable)

# set up custom function for xtable text
italic <- function(x) paste0('<i>', x, '</i>')

xftbl <- xtable(as.data.frame(summary(example1)), 
  caption = "xtable formatted output of freqlist data frame", align="|r|r|r|r|c|c|c|r|")

# change the column names
names(xftbl)[1:3] <- c("Arm", "Gender", "LASA QOL")

print(xftbl, sanitize.colnames.function = italic, include.rownames = FALSE, type = "html", comment = FALSE)
```

## Use `freqlist` in bookdown

Since the backbone of `freqlist()` is `knitr::kable()`, tables still render well in bookdown. However, `print.summary.freqlist()` doesn't use
the `caption=` argument of `kable()`, so some tables may not have a properly numbered caption. To fix this, use the method described
[on the bookdown site](https://bookdown.org/yihui/bookdown/tables.html) to give the table a tag/ID.

```{r eval=FALSE}
summary(freqlist(~ sex + age, data = mockstudy), title="(\\#tab:mytableby) Caption here")
```

# Appendix: Notes regarding table options in R

## NAs

There are several widely used options for basic tables in R. The `table()` function in base R is probably the most common;
by default it excludes NA values. You can change NA handling in `base::table()` using the `useNA=` or `exclude=` arguments.

```{r}
# base table default removes NAs
tab.d1 <- base::table(mockstudy[, c("arm", "sex", "mdquality.s")], useNA="ifany")
tab.d1
```

`xtabs()` is similar to `table()`, but uses a formula-based syntax. However, NAs must be explicitly added to each factor
using the `addNA()` function or using the argument `addNA = TRUE`.

```{r}
# without specifying addNA
tab.d2 <- xtabs(formula = ~ arm + sex + mdquality.s, data = mockstudy)
tab.d2

# now with addNA
tab.d3 <- xtabs(~ arm + sex + addNA(mdquality.s), data = mockstudy)
tab.d3

```

Since the formula method of `freqlist()` uses `xtabs()`, NAs should be treated in the same way.
`includeNA()` can also be helpful here for setting explicit NA values.

## Table dimname names (dnn)

Supplying a data.frame to the `table()` function without giving columns individually will create a contingency table
using all variables in the data.frame.

However, if the columns of a data.frame or matrix are supplied separately (i.e., as vectors),
column names will not be preserved.

```{r}
# providing variables separately (as vectors) drops column names
table(mockstudy$arm, mockstudy$sex, mockstudy$mdquality.s)
```

If desired, you can use the `dnn=` argument to pass variable names.

```{r}
# add the column name labels back using dnn option in base::table
table(mockstudy$arm, mockstudy$sex, mockstudy$mdquality.s, dnn=c("Arm", "Sex", "QOL"))
```

You can also name the arguments to `table()`:

```{r}
table(Arm = mockstudy$arm, Sex = mockstudy$sex, QOL = mockstudy$mdquality.s)
```

If using `freqlist()`, you can provide the labels directly to `freqlist()` or to `summary()` using `labelTranslations=`.

---
title: "The paired function"
author: "Ethan Heinzen, Beth Atkinson, Jason Sinnwell"
output: 
  rmarkdown::html_vignette:
    toc: yes
    toc_depth: 3
vignette: |
  %\VignetteIndexEntry{The paired function}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r echo = FALSE}
options(width = 100)
```

# Introduction

Another one of the most common tables in medical literature includes summary statistics for a set of variables paired across
two time points.  Locally at Mayo, the SAS macro `%paired` was written to create summary tables with a single call.
With the increasing interest in R, we have developed the function `paired()` to create similar tables within the R environment.  

This vignette is light on purpose; `paired()` piggybacks off of tableby, so most documentation there applies here, too.

# Simple Example

The first step when using the `paired()` function is to load the `arsenal` package.  We can't use `mockstudy` here because
we need a dataset with paired observations, so we'll create our own dataset.

```{r, load-data}
library(arsenal)
dat <- data.frame(
  tp = paste0("Time Point ", c(1, 2, 1, 2, 1, 2, 1, 2, 1, 2)),
  id = c(1, 1, 2, 2, 3, 3, 4, 4, 5, 6),
  Cat = c("A", "A", "A", "B", "B", "B", "B", "A", NA, "B"),
  Fac = factor(c("A", "B", "C", "A", "B", "C", "A", "B", "C", "A")),
  Num = c(1, 2, 3, 4, 4, 3, 3, 4, 0, NA),
  Ord = ordered(c("I", "II", "II", "III", "III", "III", "I", "III", "II", "I")),
  Lgl = c(TRUE, TRUE, FALSE, TRUE, FALSE, TRUE, TRUE, FALSE, FALSE, FALSE),
  Dat = as.Date("2018-05-01") + c(1, 1, 2, 2, 3, 4, 5, 6, 3, 4),
  stringsAsFactors = FALSE
)
```

To create a simple table stratified by time point, use a `formula=` statement to specify the variables that you want summarized and
the `id=` argument to specify the paired observations.

```{r results = 'asis'}
p <- paired(tp ~ Cat + Fac + Num + Ord + Lgl + Dat, data = dat, id = id, signed.rank.exact = FALSE)
summary(p)
```

The third column shows the difference between time point 1 and time point 2. For categorical variables, it reports
the percent of observations from time point 1 which changed in time point 2.

# NAs

Note that by default, observations which do not have both timepoints are removed.
This is easily changed using the `na.action = na.paired("<arg>")` argument. For example:

```{r results = 'asis'}
p <- paired(tp ~ Cat + Fac + Num + Ord + Lgl + Dat, data = dat, id = id,
            signed.rank.exact = FALSE, na.action = na.paired("fill"))
summary(p)
```

For more details, see the help page for `na.paired()`.

# Available Function Options

## Testing options

The tests used to calculate p-values differ by the variable type, but can be specified
explicitly in the formula statement or in the control function.

The following tests are accepted:

* `paired.t`: A paired t-test. 

* `mcnemar`: McNemar's test.

* `signed.rank`: the signed-rank test.

* `sign.test`: the sign test.

* `notest`: Don't perform a test.

## `paired.control` settings

A quick way to see what arguments are possible to utilize in a function is to use the `args()` command. Settings involving the number of digits can be set in `paired.control` or in `summary.tableby`.

```{r}
args(paired.control)
```

## `summary.tableby` settings

Since the "paired" object inherits "tableby", the `summary.tableby` function is what's actually used
to format and print the table.

```{r}
args(arsenal:::summary.tableby)
```

---
title: "A Few Notes on Labels"
author: "Ethan Heinzen"
output:
  rmarkdown::html_vignette:
    toc: true
vignette: |
  %\VignetteIndexEntry{A Few Notes on Labels}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r include = FALSE}
knitr::opts_chunk$set(message = FALSE, results = 'asis')
```

# Introduction

The `arsenal` package relies somewhat heavily on variable labels to make output more "pretty".
A `label` here is understood to be a single character string with "pretty" text (i.e., not an "ugly" variable name).
Three of the main `arsenal` function use labels in their `summary()` output. There are several ways to set these labels.

We'll use the `mockstudy` dataset for all examples here:

```{r}
library(arsenal)
data(mockstudy)
library(magrittr)

# for 'freqlist' examples
tab.ex <- table(mockstudy[c("arm", "sex", "mdquality.s")], useNA="ifany")
```

# Examples

## Set labels in the function call

The `summary()` method for `tableby()`, `modelsum()`, and `freqlist()` objects contains a `labelTranslations = ` argument to specify labels
in the function call. Note that the `freqlist()` function matches labels in order, whereas the other two match labels by name. The labels
can be input as a list or a character vector.

```{r}
summary(freqlist(tab.ex),
        labelTranslations = c(arm = "Treatment Arm", sex = "Gender", mdquality.s = "LASA QOL"))
summary(tableby(arm ~ sex + age, data = mockstudy),
        labelTranslations = c(sex = "SEX", age = "Age, yrs"))
summary(modelsum(bmi ~ age, adjust = ~sex, data = mockstudy),
        labelTranslations = list(sexFemale = "Female", age = "Age, yrs"))
```

## Modify labels after the fact

Another option is to add labels after you have created the object. To do this, you can use the form
`labels(x) <- value` or use the pipe-able version, `set_labels()`.

```{r}
# the non-pipe version; somewhat clunky
tmp <- freqlist(tab.ex)
labels(tmp) <- c(arm = "Treatment Arm", sex = "Gender", mdquality.s = "LASA QOL")
summary(tmp)

# piped--much cleaner
mockstudy %>% 
  tableby(arm ~ sex + age, data = .) %>% 
  set_labels(c(sex = "SEX", age = "Age, yrs")) %>% 
  summary()

mockstudy %>% 
  modelsum(bmi ~ age, adjust = ~ sex, data = .) %>% 
  set_labels(list(sexFemale = "Female", age = "Age, yrs")) %>% 
  summary()
```

## Add labels to a `data.frame`

`tableby()` and `modelsum()` also allow you to have label attributes on the data. Note
that by default these attributes usually get dropped upon subsetting, but `tableby()` and
`modelsum()` use the `keep.labels()` function to retain them.

```{r}
mockstudy.lab <- keep.labels(mockstudy)
class(mockstudy$age)
class(mockstudy.lab$age)
```

To undo this, simply `loosen.labels()`:

```{r}
class(loosen.labels(mockstudy.lab)$age)
```

You can set attributes one at a time in two ways:

```{r}
attr(mockstudy.lab$sex, "label") <- "Sex"
labels(mockstudy.lab$age) <- "Age, yrs"
```

...or all at once:

```{r}
labels(mockstudy.lab) <- list(sex = "Sex", age = "Age, yrs")
summary(tableby(arm ~ sex + age, data = mockstudy.lab))
```

You can pipe this, too.

```{r}
mockstudy %>% 
  set_labels(list(sex = "SEX", age = "Age, yrs")) %>% 
  modelsum(bmi ~ age, adjust = ~ sex, data = .) %>% 
  summary()
```

To extract labels from a `data.frame`, simply use the `labels()` function:

```{r results='markdown'}
labels(mockstudy.lab)
```

## When labels get long

`tableby()` and `modelsum()` both support the wrapping of long labels. Consider the `width=` argument in the `print()` function:

```{r}
mockstudy %>% 
  set_labels(list(age = "This is a really long label for the arm variable")) %>% 
  tableby(sex ~ age, data = .) %>% 
  summary() %>% 
  print(width = 20)
```

---
title: "The modelsum function"
author: "Beth Atkinson, Ethan Heinzen, Pat Votruba, Jason Sinnwell, Shannon McDonnell and Greg Dougherty"
output:
  rmarkdown::html_vignette:
    toc: yes
    toc_depth: 3
vignette: |
  %\VignetteIndexEntry{The modelsum function}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r, echo=FALSE, message=FALSE, results='hide', warning=FALSE}
require(knitr)
require(broom)
require(MASS)
require(pROC)
require(rpart)
 
opts_chunk$set(comment = NA, echo=TRUE, prompt=TRUE, collapse=TRUE)

```


# Introduction

Very often we are asked to summarize model results from multiple fits into a nice table.
The endpoint might be of different types (e.g., survival, case/control, continuous) and there
may be several independent variables that we want to examine univariately or adjusted for certain
variables such as age and sex. Locally at Mayo, the SAS macros `%modelsum`, `%glmuniv`, and `%logisuni`
were written to create such summary tables. With the increasing interest in R, we have developed the
function `modelsum` to create similar tables within the R environment.  

In developing the `modelsum` function, the goal was to bring the best features of these macros into an R function.
However, the task was not simply to duplicate all the functionality, but rather to make use of R's strengths
(modeling, method dispersion, flexibility in function definition and output format) and make a tool that fits
the needs of R users.  Additionally, the results needed to fit within the general reproducible research framework
so the tables could be displayed within an R markdown report.

This report provides step-by-step directions for using the functions associated with `modelsum`.
All functions presented here are available within the `arsenal` package.  An assumption is made that users
are somewhat familiar with R markdown documents.  For those who are new to the topic, a good initial
resource is available at [rmarkdown.rstudio.com](https://rmarkdown.rstudio.com/). 

# Simple Example

The first step when using the `modelsum` function is to load the `arsenal` package.  All the examples in this report
use a dataset called `mockstudy` made available by Paul Novotny which includes a variety of types of variables
(character, numeric, factor, ordered factor, survival) to use as examples.

```{r, load-data}
require(arsenal)
data(mockstudy) # load data
dim(mockstudy)  # look at how many subjects and variables are in the dataset 
# help(mockstudy) # learn more about the dataset and variables
str(mockstudy) # quick look at the data
```

To create a simple linear regression table (the default), use a formula statement to specify the variables
that you want summarized.  The example below predicts BMI with the variables sex and age. 

```{r simple1}
tab1 <- modelsum(bmi ~ sex + age, data=mockstudy)
```

If you want to take a quick look at the table, you can use `summary` on your modelsum object and the table will
print out as text in your R console window.  If you use `summary` without any options you will see a number of
$\&nbsp;$ statements which translates to "space" in HTML.

## Pretty text version of table

If you want a nicer version in your console window then adding the `text=TRUE` option.  

```{r simple-text}
summary(tab1, text=TRUE)
```

## Pretty Rmarkdown version of table

In order for the report to look nice within an R markdown (knitr) report, you just need to specify
`results="asis"` when creating the r chunk. This changes the layout slightly (compresses it) and bolds
the variable names.

```{r simple-markdown, results='asis'}
summary(tab1)
```

## Data frame version of table

If you want a data.frame version, simply use `as.data.frame`.  

```{r}
as.data.frame(tab1)
```

## Add an adjustor to the model

The argument `adjust` allows the user to indicate that all the variables should be adjusted for these terms. To adjust each model
for age and sex (for instance), we use `adjust = ~ age + sex`:

```{r adjust, results="asis"}
tab2 <- modelsum(alk.phos ~ arm + ps + hgb, adjust= ~age + sex, data=mockstudy)
summary(tab2)
```

# Models for each endpoint type

To make sure the correct model is run you need to specify "family".  The options available right
now are : gaussian, binomial, survival, and poisson.  If there is enough interest, additional models can be added.  

## Gaussian

### Fit and summarize linear regression model

Look at whether there is any evidence that AlkPhos values vary by study arm after adjusting for sex and age (assuming a linear age relationship).

```{r}
fit <- lm(alk.phos ~ arm + age + sex, data=mockstudy)
summary(fit)
plot(fit)
```

The results suggest that the endpoint may need to be transformed.  Calculating the Box-Cox transformation suggests a log transformation.

```{r}
require(MASS)
boxcox(fit)
```

```{r}
fit2 <- lm(log(alk.phos) ~ arm + age + sex, data=mockstudy)
summary(fit2)
plot(fit2)
```

Finally, look to see whether there there is a non-linear relationship with age.

```{r}
require(splines)
fit3 <- lm(log(alk.phos) ~ arm + ns(age, df=2) + sex, data=mockstudy)

# test whether there is a difference between models 
stats::anova(fit2,fit3)

# look at functional form of age
termplot(fit3, term=2, se=T, rug=T)
```

In this instance it looks like there isn't enough evidence to say that the relationship is non-linear.

### Extract data using the `broom` package

The `broom` package makes it easy to extract information from the fit.

```{r}
tmp <- tidy(fit3) # coefficients, p-values
class(tmp)
tmp

glance(fit3)
```

### Create a summary table using modelsum

```{r, results="asis"}
ms.logy <- modelsum(log(alk.phos) ~ arm + ps + hgb, data=mockstudy, adjust= ~age + sex, 
                    family=gaussian,  
                    gaussian.stats=c("estimate","CI.lower.estimate","CI.upper.estimate","p.value"))
summary(ms.logy)
```

## Binomial

### Fit and summarize logistic regression model

```{r}
boxplot(age ~ mdquality.s, data=mockstudy, ylab=attr(mockstudy$age,'label'), xlab='mdquality.s')

fit <- glm(mdquality.s ~ age + sex, data=mockstudy, family=binomial)
summary(fit)

# create Odd's ratio w/ confidence intervals
tmp <- data.frame(summary(fit)$coef)
tmp

tmp$OR <- round(exp(tmp[,1]),2)
tmp$lower.CI <- round(exp(tmp[,1] - 1.96* tmp[,2]),2)
tmp$upper.CI <- round(exp(tmp[,1] + 1.96* tmp[,2]),2)
names(tmp)[4] <- 'P-value'

kable(tmp[,c('OR','lower.CI','upper.CI','P-value')])

# Assess the predictive ability of the model

# code using the pROC package
require(pROC)
pred <- predict(fit, type='response')
tmp <- pROC::roc(mockstudy$mdquality.s[!is.na(mockstudy$mdquality.s)]~ pred, plot=TRUE, percent=TRUE)
tmp$auc

```

### Extract data using `broom` package

The `broom` package makes it easy to extract information from the fit.

```{r}
tidy(fit, exp=T, conf.int=T) # coefficients, p-values, conf.intervals

glance(fit) # model summary statistics
```

### Create a summary table using modelsum

```{r, results="asis"}
summary(modelsum(mdquality.s ~ age + bmi, data=mockstudy, adjust=~sex, family=binomial))

fitall <- modelsum(mdquality.s ~ age, data=mockstudy, family=binomial,
                   binomial.stats=c("Nmiss2","OR","p.value"))
summary(fitall)
```


## Survival

### Fit and summarize a Cox regression model 

```{r survival}
require(survival)

# multivariable model with all 3 terms
fit  <- coxph(Surv(fu.time, fu.stat) ~ age + sex + arm, data=mockstudy)
summary(fit)

# check proportional hazards assumption
fit.z <- cox.zph(fit)
fit.z
plot(fit.z[1], resid=FALSE) # makes for a cleaner picture in this case
abline(h=coef(fit)[1], col='red')

# check functional form for age using pspline (penalized spline)
# results are returned for the linear and non-linear components
fit2 <- coxph(Surv(fu.time, fu.stat) ~ pspline(age) + sex + arm, data=mockstudy)
fit2

# plot smoothed age to visualize why significant
termplot(fit2, se=T, terms=1)
abline(h=0)

# The c-statistic comes out in the summary of the fit
summary(fit2)$concordance

# It can also be calculated using the survConcordance function
survConcordance(Surv(fu.time, fu.stat) ~ predict(fit2), data=mockstudy)
```

### Extract data using `broom` package

The `broom` package makes it easy to extract information from the fit.

```{r}
tidy(fit) # coefficients, p-values

glance(fit) # model summary statistics
```

### Create a summary table using modelsum

```{r results="asis"}
##Note: You must use quotes when specifying family="survival" 
##      family=survival will not work
summary(modelsum(Surv(fu.time, fu.stat) ~ arm, 
                 adjust=~age + sex, data=mockstudy, family="survival"))

##Note: the pspline term is not working yet
#summary(modelsum(Surv(fu.time, fu.stat) ~ arm, 
#                adjust=~pspline(age) + sex, data=mockstudy, family='survival'))
```


## Poisson

Poisson regression is useful when predicting an outcome variable representing counts.
It can also be useful when looking at survival data.  Cox models and Poisson models are very closely
related and survival data can be summarized using Poisson regression. If you have overdispersion (see
if the residual deviance is much larger than degrees of freedom), you may want to use `quasipoisson()`
instead of `poisson()`.  Some of these diagnostics need to be done outside of `modelsum`. 

### Example 1: fit and summarize a Poisson regression model 

For the first example, use the solder dataset available in the `rpart` package.  The endpoint `skips` has a definite Poisson look.

```{r poisson}
require(rpart) ##just to get access to solder dataset
data(solder)
hist(solder$skips)

fit <- glm(skips ~ Opening + Solder + Mask , data=solder, family=poisson)
stats::anova(fit, test='Chi')
summary(fit)
```

Overdispersion is when the Residual deviance is larger than the degrees of freedom.  This can be tested, approximately using the following code.  The goal is to have a p-value that is $>0.05$.

```{r}
1-pchisq(fit$deviance, fit$df.residual)
```

One possible solution is to use the quasipoisson family instead of the poisson family.  This adjusts for the overdispersion.

```{r}
fit2 <- glm(skips ~ Opening + Solder + Mask, data=solder, family=quasipoisson)
summary(fit2)
```

### Extract data using `broom` package

The `broom` package makes it easy to extract information from the fit.

```{r}
tidy(fit) # coefficients, p-values

glance(fit) # model summary statistics
```


### Create a summary table using modelsum

```{r results='asis'}
summary(modelsum(skips~Opening + Solder + Mask, data=solder, family="quasipoisson"))
summary(modelsum(skips~Opening + Solder + Mask, data=solder, family="poisson"))
```

### Example 2: fit and summarize a Poisson regression model 

This second example uses the survival endpoint available in the `mockstudy` dataset.  There is a close
relationship between survival and Poisson models, and often it is easier to fit the model using Poisson
regression, especially if you want to present absolute risk.

```{r}
# add .01 to the follow-up time (.01*1 day) in order to keep everyone in the analysis
fit <- glm(fu.stat ~ offset(log(fu.time+.01)) + age + sex + arm, data=mockstudy, family=poisson)
summary(fit)
1-pchisq(fit$deviance, fit$df.residual)

coef(coxph(Surv(fu.time,fu.stat) ~ age + sex + arm, data=mockstudy))
coef(fit)[-1]

# results from the Poisson model can then be described as risk ratios (similar to the hazard ratio)
exp(coef(fit)[-1])

# As before, we can model the dispersion which alters the standard error
fit2 <- glm(fu.stat ~ offset(log(fu.time+.01)) + age + sex + arm, 
            data=mockstudy, family=quasipoisson)
summary(fit2)
```

### Extract data using `broom` package

The `broom` package makes it easy to extract information from the fit.

```{r}
tidy(fit) ##coefficients, p-values

glance(fit) ##model summary statistics
```


### Create a summary table using `modelsum`

Remember that the result from `modelsum` is different from the `fit` above.  The `modelsum`
summary shows the results for `age + offset(log(fu.time+.01))` then `sex + offset(log(fu.time+.01))`
instead of `age + sex + arm + offset(log(fu.time+.01))`.

```{r results="asis", eval=TRUE}
summary(modelsum(fu.stat ~ age, adjust=~offset(log(fu.time+.01))+ sex + arm, 
                 data=mockstudy, family=poisson))
                 
```


# Additional Examples


Here are multiple examples showing how to use some of the different options.

## 1. Change summary statistics globally

There are standard settings for each type of model regarding what information is summarized in the table.
This behavior can be modified using the modelsum.control function. In fact, you can save your standard
settings and use that for future tables.  


```{r, results='asis'}
mycontrols  <- modelsum.control(gaussian.stats=c("estimate","std.error","adj.r.squared","Nmiss"),
                                show.adjust=FALSE, show.intercept=FALSE)                            
tab2 <- modelsum(bmi ~ age, adjust=~sex, data=mockstudy, control=mycontrols)
summary(tab2)
```

You can also change these settings directly in the modelsum call.  

```{r, results='asis'}
tab3 <- modelsum(bmi ~  age, adjust=~sex, data=mockstudy,
                 gaussian.stats=c("estimate","std.error","adj.r.squared","Nmiss"), 
                 show.intercept=FALSE, show.adjust=FALSE)
summary(tab3)
```

## 2. Add labels to independent variables

In the above example, age is shown with a label (Age in Years), but sex is listed "as is".
This is because the data was created in SAS and in the SAS dataset, age had a label but sex did not.
The label is stored as an attribute within R.

```{r check-labels}
## Look at one variable's label
attr(mockstudy$age,'label')

## See all the variables with a label
unlist(lapply(mockstudy,'attr','label'))

## or
cbind(sapply(mockstudy,attr,'label'))
```

If you want to add labels to other variables, there are a couple of options.  First, you could add labels to the variables in your dataset.

```{r add-label, results='asis'}
attr(mockstudy$age,'label')  <- 'Age, yrs'

tab1 <- modelsum(bmi ~  age, adjust=~sex, data=mockstudy)
summary(tab1)
```

You can also use the built-in `data.frame` method for `labels<-`:

```{r, results = 'asis'}
labels(mockstudy)  <- c(age = 'Age, yrs')

tab1 <- modelsum(bmi ~  age, adjust=~sex, data=mockstudy)
summary(tab1)
```

Another option is to add labels after you have created the table

```{r, results='asis'}
mylabels <- list(sexFemale = "Female", age ="Age, yrs")
summary(tab1, labelTranslations = mylabels)
```

Alternatively, you can check the variable labels and manipulate them with a function called `labels`, which works on the `modelsum` object.

```{r, eval=TRUE}
labels(tab1)
labels(tab1) <- c(sexFemale="Female", age="Baseline Age (yrs)")
labels(tab1)
```

```{r, results='asis'}
summary(tab1)
```

## 3. Don't show intercept values

```{r, results='asis'}
summary(modelsum(age~mdquality.s+sex, data=mockstudy), show.intercept=FALSE)
```

## 4. Don't show results for adjustment variables

```{r, results='asis'}
summary(modelsum(mdquality.s ~ age + bmi, data=mockstudy, adjust=~sex, family=binomial),
        show.adjust=FALSE)  
```

## 5. Summarize multiple variables without typing them out

Often one wants to summarize a number of variables.  Instead of typing by hand each individual variable,
an alternative approach is to create a formula using the `paste` command with the `collapse="+"` option.  

```{r, results='asis'}
# create a vector specifying the variable names
myvars <- names(mockstudy)

# select the 8th through the 12th
# paste them together, separated by the + sign
RHS <- paste(myvars[8:12], collapse="+")
RHS

# create a formula using the as.formula function
as.formula(paste('mdquality.s ~ ', RHS))

# use the formula in the modelsum function
summary(modelsum(as.formula(paste('mdquality.s ~', RHS)), family=binomial, data=mockstudy))
```

These steps can also be done using the `formulize` function.

```{r, results='asis'}
## The formulize function does the paste and as.formula steps
tmp <- formulize('mdquality.s',myvars[8:10])
tmp

## More complex formulas could also be written using formulize
tmp2 <- formulize('mdquality.s',c('ps','hgb','sqrt(bmi)'))

## use the formula in the modelsum function
summary(modelsum(tmp, data=mockstudy, family=binomial))
```


## 6. Subset the dataset used in the analysis

Here are two ways to get the same result (limit the analysis to subjects age>50 and in the F: FOLFOX treatment group). 

* The first approach uses the subset function applied to the dataset `mockstudy`.
This example also selects a subset of variables.  The `modelsum` function is then applied to this subsetted data.


```{r}
newdata <- subset(mockstudy, subset=age>50 & arm=='F: FOLFOX', select = c(age,sex, bmi:alk.phos))
dim(mockstudy)
table(mockstudy$arm)
dim(newdata)
names(newdata)
```

```{r, results='asis'}
summary(modelsum(alk.phos ~ ., data=newdata))
```

* The second approach does the same analysis but uses the subset
argument within `modelsum` to subset the data.

```{r, results='asis', eval=TRUE}
summary(modelsum(log(alk.phos) ~ sex + ps + bmi, subset=age>50 & arm=="F: FOLFOX", data=mockstudy))
summary(modelsum(alk.phos ~ ps + bmi, adjust=~sex, subset = age>50 & bmi<24, data=mockstudy))
summary(modelsum(alk.phos ~ ps + bmi, adjust=~sex, subset=1:30, data=mockstudy))
```

## 7. Create combinations of variables on the fly

```{r}
## create a variable combining the levels of mdquality.s and sex
with(mockstudy, table(interaction(mdquality.s,sex)))
```

```{r, results='asis'}
summary(modelsum(age ~ interaction(mdquality.s,sex), data=mockstudy))
```

## 8. Transform variables on the fly

Certain transformations need to be surrounded by `I()` so that R knows to treat it as a variable
transformation and not some special model feature.  If the transformation includes any of the
symbols `/ - + ^ *` then surround the new variable by `I()`.


```{r, results='asis'}
summary(modelsum(arm=="F: FOLFOX" ~ I(age/10) + log(bmi) + mdquality.s,
                 data=mockstudy, family=binomial))
```


## 9. Change the ordering of the variables or delete a variable

```{r, results='asis'}
mytab <- modelsum(bmi ~ sex + alk.phos + age, data=mockstudy)
mytab2 <- mytab[c('age','sex','alk.phos')]
summary(mytab2)
summary(mytab[c('age','sex')])
summary(mytab[c(3,1)])
```

## 10. Merge two `modelsum` objects together 

It is possible to merge two modelsum objects so that they print out together, however you need to pay
attention to the columns that are being displayed.  It is sometimes easier to combine two models of the same
family (such as two sets of linear models). Overlapping y-variables will have their x-variables
concatenated, and (if `all=TRUE`) non-overlapping y-variables will have their tables printed separately.

```{r, results="asis"}
## demographics
tab1 <- modelsum(bmi ~ sex + age, data=mockstudy)
## lab data
tab2 <- modelsum(mdquality.s ~ hgb + alk.phos, data=mockstudy, family=binomial)
                
tab12 <- merge(tab1, tab2, all = TRUE)
class(tab12)
summary(tab12)
```

## 11. Add a title to the table

When creating a pdf the tables are automatically numbered and the title appears below the table.
In Word and HTML, the titles appear un-numbered and above the table.  

```{r, results='asis'}
t1 <- modelsum(bmi ~ sex + age, data=mockstudy)
summary(t1, title='Demographics')
```

## 12. Modify how missing values are treated

Depending on the report you are writing you have the following options: 

* Use all values available for each variable

* Use only those subjects who have measurements available for all the variables

```{r}
## look at how many missing values there are for each variable
apply(is.na(mockstudy),2,sum)
```

```{r, results='asis'}
## Show how many subjects have each variable (non-missing)
summary(modelsum(bmi ~ ast + age, data=mockstudy,
                control=modelsum.control(gaussian.stats=c("N","estimate"))))

## Always list the number of missing values
summary(modelsum(bmi ~ ast + age, data=mockstudy,
                control=modelsum.control(gaussian.stats=c("Nmiss2","estimate"))))

## Only show the missing values if there are some (default)
summary(modelsum(bmi ~ ast + age, data=mockstudy, 
                control=modelsum.control(gaussian.stats=c("Nmiss","estimate"))))

## Don't show N at all
summary(modelsum(bmi ~ ast + age, data=mockstudy, 
                control=modelsum.control(gaussian.stats=c("estimate"))))
```

## 13. Modify the number of digits used

Within modelsum.control function there are 3 options for controlling the number of significant digits shown.  

* digits: controls the number of digits after the decimal point for continuous values

* digits.ratio: controls the number of digits after the decimal point for continuous values

* digits.p: controls the number of digits after the decimal point for continuous values

```{r, results='asis'}
summary(modelsum(bmi ~ sex + age + fu.time, data=mockstudy), digits=4, digits.test=2)
```

## 14. Use case-weights in the models

Occasionally it is of interest to fit models using case weights.
The `modelsum` function allows you to pass on the weights to the models and it will do the appropriate fit.

```{r}
mockstudy$agegp <- cut(mockstudy$age, breaks=c(18,50,60,70,90), right=FALSE)

## create weights based on agegp and sex distribution
tab1 <- with(mockstudy,table(agegp, sex))
tab1
tab2 <- with(mockstudy, table(agegp, sex, arm))
gpwts <- rep(tab1, length(unique(mockstudy$arm)))/tab2

## apply weights to subjects
index <- with(mockstudy, cbind(as.numeric(agegp), as.numeric(sex), as.numeric(as.factor(arm)))) 
mockstudy$wts <- gpwts[index]

## show weights by treatment arm group
tapply(mockstudy$wts,mockstudy$arm, summary)
```

```{r results='asis'}
mockstudy$newvarA <- as.numeric(mockstudy$arm=='A: IFL')
tab1 <- modelsum(newvarA ~ ast + bmi + hgb, data=mockstudy, subset=(arm !='G: IROX'), 
                 family=binomial)
summary(tab1, title='No Case Weights used')

suppressWarnings({
tab2 <- modelsum(newvarA ~ ast + bmi + hgb, data=mockstudy, subset=(arm !='G: IROX'), 
                 weights=wts, family=binomial)
summary(tab2, title='Case Weights used')
})
```
          
## 15. Use `modelsum` within an Sweave document

For those users who wish to create tables within an Sweave document, the following code seems to work.

```
\documentclass{article}

\usepackage{longtable}
\usepackage{pdfpages}

\begin{document}

\section{Read in Data}
<<echo=TRUE>>=
require(arsenal)
require(knitr)
require(rmarkdown)
data(mockstudy)

tab1 <- modelsum(bmi~sex+age, data=mockstudy)
@

\section{Convert Summary.modelsum to LaTeX}
<<echo=TRUE, results='hide', message=FALSE>>=
capture.output(summary(tab1), file="Test.md")

## Convert R Markdown Table to LaTeX
render("Test.md", pdf_document(keep_tex=TRUE))
@ 

\includepdf{Test.pdf}

\end{document}
```
## 16. Export `modelsum` results to a .CSV file

When looking at multiple variables it is sometimes useful to export the results to a csv file.
The `as.data.frame` function creates a data frame object that can be exported or further manipulated within R.


```{r}
summary(tab2, text=T)
tmp <- as.data.frame(summary(tab2, text = TRUE))
tmp
# write.csv(tmp, '/my/path/here/mymodel.csv')
```

## 17. Write `modelsum` object to a separate Word or HTML file

```{r eval = FALSE}
## write to an HTML document
write2html(tab2, "~/ibm/trash.html")

## write to a Word document
write2word(tab2, "~/ibm/trash.doc", title="My table in Word")
```

## 18. Use `modelsum` in R Shiny

The easiest way to output a `modelsum()` object in an R Shiny app is to use the `tableOutput()` UI in combination with
the `renderTable()` server function and `as.data.frame(summary(modelsum()))`:

```{r eval=FALSE}
# A standalone shiny app
library(shiny)
library(arsenal)
data(mockstudy)

shinyApp(
  ui = fluidPage(tableOutput("table")),
  server = function(input, output) {
    output$table <- renderTable({
      as.data.frame(summary(modelsum(age ~ sex, data = mockstudy), text = "html"))
    }, sanitize.text.function = function(x) x)
  }
)
```

This can be especially powerful if you feed the selections from a `selectInput(multiple = TRUE)` into `formulize()` to make
the table dynamic!

## 23. Use `modelsum` in bookdown

Since the backbone of `modelsum()` is `knitr::kable()`, tables still render well in bookdown. However, `print.summary.modelsum()` doesn't use
the `caption=` argument of `kable()`, so some tables may not have a properly numbered caption. To fix this, use the method described
[on the bookdown site](https://bookdown.org/yihui/bookdown/tables.html) to give the table a tag/ID.

```{r eval=FALSE}
summary(modelsum(age ~ sex, data = mockstudy), title="(\\#tab:mytableby) Caption here")
```

## 24. Model multiple endpoints

You can now use `list()` on the left-hand side of `modelsum()` to give multiple endpoints.
Note that only one "family" can be specified this way (use `merge()` instead if you want multiple families).

```{r results='asis'}
summary(modelsum(list(age, hgb) ~ bmi + sex, adjust = ~ arm, data = mockstudy))
```

To avoid confusion about which table is which endpoint, you can set `term.name=TRUE` in `summary()`. This takes the labels
for each endpoint and puts them in the top-left of the table.

```{r results='asis'}
summary(modelsum(list(age, hgb) ~ bmi + sex, adjust = ~ arm, data = mockstudy), term.name = TRUE)
```

## 25. Model data by a non-test group (strata)

You can also specify a grouping variable that doesn't get tested (but instead separates results): a *strata* variable.

```{r results='asis'}
summary(modelsum(list(age, hgb) ~ bmi + sex, strata = arm, data = mockstudy))
```

## 26. Add multiple sets of adjustors to the model

By putting multiple formulas into a list, you can use multiple sets of adjustors. Use `~ 1` or `NULL` for an "unadjusted" model. By using the 
`adjustment.names=TRUE` argument and giving names to your adjustor sets in the list, you can name the various analyses.

```{r}
adj.list <- list(
  Unadjusted = ~ 1, # can also specify NULL here
  "Adjusted for Arm" = ~ arm
)
multi.adjust <- modelsum(list(age, bmi) ~ fu.time + ast, adjust = adj.list, data = mockstudy)
summary(multi.adjust, adjustment.names = TRUE)
summary(multi.adjust, adjustment.names = TRUE, show.intercept = FALSE, show.adjust = FALSE)
```


# Available Function Options

## Summary statistics

The available summary statistics, by varible type, are:

* `ordinal`: Ordinal logistic regression models
  + default: `Nmiss, OR, CI.lower.OR, CI.upper.OR, p.value`
  + optional: `estimate, CI.OR, CI.estimate, CI.lower.estimate, CI.upper.estimate,`
       `N, Nmiss2, endpoint, std.error, statistic, logLik, AIC, BIC, edf, deviance, df.residual, p.value.lrt`
* `binomial`,`quasibinomial`: Logistic regression models 
  +  default:  `OR, CI.lower.OR, CI.upper.OR, p.value, concordance, Nmiss`
  +  optional: `estimate, CI.OR, CI.estimate, CI.lower.estimate, CI.upper.estimate,`
        `CI.wald, CI.lower.wald, CI.upper.wald, CI.OR.wald, CI.lower.OR.wald, CI.upper.OR.wald,`
        `N, Nmiss2, Nevents, endpoint, std.error, statistic, logLik, AIC, BIC, null.deviance, deviance, df.residual, df.null, p.value.lrt`
* `gaussian`: Linear regression models 
  +  default:  `estimate, std.error, p.value, adj.r.squared, Nmiss`
  +  optional: `CI.estimate, CI.lower.estimate, CI.upper.estimate, N, Nmiss2, statistic,`
        `standard.estimate, endpoint, r.squared, AIC, BIC, logLik, statistic.F, p.value.F, p.value.lrt`
* `poisson`, `quasipoisson`: Poisson regression models 
  +  default: `RR, CI.lower.RR, CI.upper.RR, p.value, Nmiss`
  +  optional: `CI.RR, CI.estimate, CI.lower.estimate, CI.upper.estimate, CI.RR, Nmiss2, std.error,`
        `estimate, statistic, endpoint, AIC, BIC, logLik, dispersion, null.deviance, deviance, df.residual, df.null, p.value.lrt`
* `negbin`: Negative binomial regression models 
  +  default: `RR, CI.lower.RR, CI.upper.RR, p.value, Nmiss`
  +  optional: `CI.RR, CI.estimate, CI.lower.estimate, CI.upper.estimate, CI.RR, Nmiss2, std.error, estimate,`
        `statistic, endpoint, AIC, BIC, logLik, dispersion, null.deviance, deviance, df.residual, df.null, theta, SE.theta, p.value.lrt`
* `clog`: Conditional Logistic models
  + default: `OR, CI.lower.OR, CI.upper.OR, p.value, concordance, Nmiss`
  + optional: `CI.OR, CI.estimate, CI.lower.estimate, CI.upper.estimate, N, Nmiss2, estimate, std.error, endpoint, Nevents, statistic,`
        `r.squared, r.squared.max, logLik, AIC, BIC, statistic.log, p.value.log, statistic.sc, p.value.sc,`
        `statistic.wald, p.value.wald, N, std.error.concordance, p.value.lrt`
* `survival`: Cox models
  +  default: `HR, CI.lower.HR, CI.upper.HR, p.value, concordance, Nmiss`
  +  optional: `CI.HR, CI.estimate, CI.lower.estimate, CI.upper.estimate, N, Nmiss2, estimate, std.error, endpoint,`
        `Nevents, statistic, r.squared, r.squared.max, logLik, AIC, BIC, statistic.log, p.value.log, statistic.sc, p.value.sc,`
        `statistic.wald, p.value.wald, N, std.error.concordance, p.value.lrt`

The full description of these parameters that can be shown for models include:

* `N`: a count of the number of observations used in the analysis
* `Nmiss`: only show the count of the number of missing values if there are some missing values 
* `Nmiss2`: always show a count of the number of missing values for a model 
* `endpoint`: dependent variable used in the model
* `std.err`: print the standard error
* `statistic`: test statistic
* `statistic.F`: test statistic (F test)
* `p.value`: print the p-value
* `p.value.lrt`: print the likelihood ratio p-value for *the main effect only* (not the adjustors)
* `r.squared`: print the model R-square 
* `adj.r.squared`: print the model adjusted R-square 
* `r.squared.max`: print the model R-square
* `concordance`: print the model C statistic (which is the AUC for logistic models)
* `logLik`: print the loglikelihood value
* `p.value.log`: print the p-value for the overall model likelihood test
* `p.value.wald`: print the p-value for the overall model wald test
* `p.value.sc`: print the p-value for overall model score test
* `AIC`: print the Akaike information criterion
* `BIC`: print the Bayesian information criterion
* `null.deviance`: null deviance
* `deviance`: model deviance
* `df.residual`: degrees of freedom for the residual
* `df.null`: degrees of freedom for the null model
* `dispersion`: This is used in Poisson models and is defined as the deviance/df.residual
* `statistic.sc`: overall model score statistic
* `statistic.wald`: overall model score statistic
* `statistic.log`: overall model score statistic
* `std.error.concordance`: standard error for the C statistic
* `HR`: print the hazard ratio (for survival models), i.e. exp(beta)
* `CI.lower.HR, CI.upper.HR`: print the confidence interval for the HR 
* `OR`: print the odd's ratio (for logistic models), i.e. exp(beta)
* `CI.lower.OR, CI.upper.OR`: print the confidence interval for the OR
* `CI.lower.OR.wald, CI.upper.OR.wald`: print the Wald confidence interval for the OR
* `RR`: print the risk ratio (for poisson models), i.e. exp(beta)
* `CI.lower.RR, CI.upper.RR`: print the confidence interval for the RR
* `estimate`: print beta coefficient
* `standardized.estimate`: print the standardized beta coefficient 
* `CI.lower.estimate, CI.upper.estimate`: print the confidence interval for the beta coefficient
* `CI.lower.wald, CI.upper.wald`: print the Wald confidence interval for the beta coefficient
* `edf`: print the effective degrees of freedom.
* `theta`: print the estimate of theta.
* `SE.theta`: print the estimate of theta's standard error.

## `modelsum.control` settings

A quick way to see what arguments are possible to utilize in a function is to use the `args()`
command. Settings involving the number of digits can be set in `modelsum.control` or in `summary.modelsum`.

```{r}
args(modelsum.control)
```

## `summary.modelsum` settings

The summary.modelsum function has options that modify how the table appears (such as adding a title or modifying labels).  

```{r}
args(arsenal:::summary.modelsum)
```

---
title: "The tableby function"
author: "Beth Atkinson, Ethan Heinzen, Jason Sinnwell, Shannon McDonnell and Greg Dougherty"
output: 
  rmarkdown::html_vignette:
    toc: yes
    toc_depth: 3
vignette: |
  %\VignetteIndexEntry{The tableby function}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r echo = FALSE}
options(width = 100)
ge330 <- getRversion() >= "3.3.0"
```

# Introduction

One of the most common tables in medical literature includes summary statistics for a set of variables,
often stratified by some group (e.g. treatment arm).  Locally at Mayo, the SAS macros `%table` and `%summary` were
written to create summary tables with a single call. With the increasing interest in R, we have developed
the function `tableby` to create similar tables within the R environment.

In developing the `tableby()` function, the goal was to bring the best features of these macros into an R function.
However, the task was not simply to duplicate all the functionality, but rather to make use of R's strengths
(modeling, method dispersion, flexibility in function definition and output format) and make a tool that fits
the needs of R users.  Additionally, the results needed to fit within the general reproducible research framework
so the tables could be displayed within an R markdown report.

This report provides step-by-step directions for using the functions associated with `tableby()`.
All functions presented here are available within the `arsenal` package.  An assumption is made that users
are somewhat familiar with R Markdown documents.  For those who are new to the topic, a good initial resource
is available at [rmarkdown.rstudio.com](https://rmarkdown.rstudio.com/). 

# Simple Example

The first step when using the `tableby` function is to load the `arsenal` package.  All the examples in this report use
a dataset called `mockstudy` made available by Paul Novotny which includes a variety of types of variables (character,
numeric, factor, ordered factor, survival) to use as examples.

```{r, load-data}
library(arsenal)
require(knitr)
require(survival)
data(mockstudy) ##load data
dim(mockstudy)  ##look at how many subjects and variables are in the dataset 
# help(mockstudy) ##learn more about the dataset and variables
str(mockstudy) ##quick look at the data
```

To create a simple table stratified by treatment arm, use a formula statement to specify the variables that you want summarized.
The example below uses age (a continuous variable) and sex (a factor). 

```{r, simple1}
tab1 <- tableby(arm ~ sex + age, data=mockstudy)
```

If you want to take a quick look at the table, you can use `summary()` on your tableby object and the table
will print out as text in your R console window.  If you use `summary()` without any options you will see a
number of $\&nbsp;$ statements which translates to "space" in HTML.

## Pretty text version of table

If you want a nicer version in your console window then add the `text=TRUE` option.  

```{r, simple-text}
summary(tab1, text=TRUE)
```

## Pretty Rmarkdown version of table

In order for the report to look nice within an R markdown (knitr) report, you just need to specify
`results="asis"` when creating the R chunk. This changes the layout slightly (compresses it) and bolds the variable names. 

```{r, simple-markdown, results='asis'}
summary(tab1)
```

## Data frame version of table

If you want a data.frame version, simply use `as.data.frame`.

```{r}
as.data.frame(tab1)
```

## Summaries using standard R code

```{r}
## base R frequency example
tmp <- table(Gender=mockstudy$sex, "Study Arm"=mockstudy$arm)
tmp

# Note: The continuity correction is applied by default in R (not used in %table)
chisq.test(tmp)

## base R numeric summary example
tapply(mockstudy$age, mockstudy$arm, summary)
summary(aov(age ~ arm, data=mockstudy))

```

# Modifying Output

## Add labels

In the above example, age is shown with a label (Age in Years), but sex is listed "as is" with lower case letters.
This is because the data was created in SAS and in the SAS dataset, age had a label but sex did not.
The label is stored as an attribute within R.

```{r, check-labels}
## Look at one variable's label
attr(mockstudy$age,'label')

## See all the variables with a label
unlist(lapply(mockstudy,'attr','label'))
# Can also use labels(mockstudy)
```

If you want to add labels to other variables, there are a couple of options.  First, you could add labels to the variables in your dataset.

```{r, add-label, results='asis'}
attr(mockstudy$sex,'label')  <- 'Gender'

tab1 <- tableby(arm ~ sex + age, data=mockstudy)
summary(tab1)
```

You can also use the built-in `data.frame` method for `labels<-`:

```{r, results = 'asis'}
labels(mockstudy)  <- c(age = 'Age, yrs', sex = "Gender")

tab1 <- tableby(arm ~ sex + age, data=mockstudy)
summary(tab1)
```

Another option is to add labels after you have created the table

```{r, results='asis'}
mylabels <- list(sex = "SEX", age = "Age, yrs")
summary(tab1, labelTranslations = mylabels)
```

Alternatively, you can check the variable labels and manipulate them with a function called `labels`, which works on the `tableby` object.

```{r, assignlabels}
labels(tab1)
labels(tab1) <- c(arm="Treatment Assignment", age="Baseline Age (yrs)")
labels(tab1)
```

```{r, results='asis'}
summary(tab1)
```

## Change summary statistics globally

Currently the default behavior is to summarize continuous variables with: Number of missing values,
Mean (SD), 25th - 75th quantiles, and Minimum-Maximum values with an ANOVA (t-test with equal variances) p-value.
For categorical variables the default is to show: Number of missing values and count (column percent) with a
chi-square p-value.  This behavior can be modified using the tableby.control function. In fact, you can save
your standard settings and use that for future tables. Note that `test=FALSE` and `total=FALSE` results in the
total column and p-value column not being printed.  


```{r, results='asis'}
mycontrols  <- tableby.control(test=FALSE, total=FALSE,
                               numeric.test="kwt", cat.test="chisq",
                               numeric.stats=c("N", "median", "q1q3"),
                               cat.stats=c("countpct"),
                               stats.labels=list(N='Count', median='Median', q1q3='Q1,Q3'))
tab2 <- tableby(arm ~ sex + age, data=mockstudy, control=mycontrols)
summary(tab2)
```

You can also change these settings directly in the tableby call.  

```{r, results='asis'}
tab3 <- tableby(arm ~ sex + age, data=mockstudy, test=FALSE, total=FALSE, 
                numeric.stats=c("median","q1q3"), numeric.test="kwt")
summary(tab3)
```


## Change summary statistics within the formula

In addition to modifying summary options globally, it is possible to modify the test and summary statistics for
specific variables within the formula statement.  For example, both the kwt (Kruskal-Wallis rank-based) and anova
(asymptotic analysis of variance) tests apply to numeric variables, and we can use one for the variable "age",
another for the variable "bmi", and no test for the variable "ast". A list of all the options is shown at the end of the vignette.

The `tests` function can do a quick check on what tests were performed on each variable in tableby.

```{r, testformula}
tab.test <- tableby(arm ~ kwt(age) + anova(bmi) + notest(ast), data=mockstudy)
tests(tab.test)
```

```{r, results='asis'}
summary(tab.test)
```

Summary statistics for any individual variable can also be modified, but it must be done as secondary
arguments to the test function. The function names must be strings that are functions already written for tableby,
built-in R functions like mean and range, or user-defined functions.

```{r, testsAndStats, results='asis'}
tab.test <- tableby(arm ~ kwt(ast, "Nmiss2","median") + anova(age, "N","mean") +
                    notest(bmi, "Nmiss","median"), data=mockstudy)
summary(tab.test)
```

These can also be passed to the `stats=` argument.

```{r, eval=FALSE}
tab.test <- tableby(arm ~ kwt(ast, stats = c("Nmiss2", "median")) + anova(age, stats = c("N","mean")) +
                    notest(bmi, stats = c("Nmiss","median")), data=mockstudy)
summary(tab.test)
```


## Controlling Options for Categorical Tests (Chisq and Fisher's)

The formal tests for categorical variables against the levels of the by variable, chisq and fe, have
options to simulate p-values. We show how to turn on the simulations for these with 500 replicates
for the Fisher's test (fe).

```{r, simfe, results='asis'}
set.seed(100)
tab.catsim <- tableby(arm ~ sex + race, cat.test="fe", simulate.p.value=TRUE, B=500, data=mockstudy)
tests(tab.catsim)
```

The chi-square test on 2x2 tables applies Yates' continuity correction by default, so we provide an option
to turn off the correction. We show the results with and without the correction that is applied to
treatment arm by sex, if we use subset to ignore one of the three treatment arms.

```{r, chisqcorrect, results='asis'}
cat.correct <- tableby(arm ~ sex + race, cat.test="chisq", subset = !grepl("^F", arm), data=mockstudy)
tests(cat.correct)
cat.nocorrect <- tableby(arm ~ sex + race, cat.test="chisq", subset = !grepl("^F", arm),
     chisq.correct=FALSE, data=mockstudy)
tests(cat.nocorrect)
```

## Modifying the look & feel in Word documents 

You can easily create Word versions of `tableby` output via an Rmarkdown report and the default options will give you a reasonable table in Word - just select the "Knit Word" option in RStudio.

**The functionality listed in this next paragraph is coming soon but needs an upgraded version of RStudio**
If you want to modify fonts used for the table, then you'll need to add an extra line to your header at the beginning of your file.
You can take the `WordStylesReference01.docx` file and modify the fonts (storing the format preferences in your project directory).
To see how this works, run your report once using WordStylesReference01.docx and then WordStylesReference02.docx. 

```
output: word_document
  reference_docx: /projects/bsi/gentools/R/lib320/arsenal/doc/WordStylesReference01.docx 
```

For more information on changing the look/feel of your Word document, see the [Rmarkdown documentation](https://pkg.yihui.org/rmarkdown-book/word-document.html) website.


# Additional Examples

Here are multiple examples showing how to use some of the different options.

## 1. Summarize without a group/by variable

```{r, nobyvar, results='asis'}
tab.noby <- tableby(~ bmi + sex + age, data=mockstudy)
summary(tab.noby)
```

## 2. Display footnotes indicating which "test" was used

```{r, results="asis"}
summary(tab.test, pfootnote=TRUE)
```

## 3. Summarize an ordered factor

When comparing groups of ordered data there are a couple of options.  The **default** uses a general independence test available from the `coin` package.
For two-group comparisons, this is essentially the Armitage trend test.  The other option is to specify the Kruskal Wallis test.
The example below shows both options.

```{r}
mockstudy$age.ordnew <- ordered(c("a",NA,as.character(mockstudy$age.ord[-(1:2)])))
table(mockstudy$age.ord, mockstudy$sex)
table(mockstudy$age.ordnew, mockstudy$sex)
class(mockstudy$age.ord)
```

```{r, results="asis", eval=requireNamespace("coin", quietly = TRUE)}
summary(tableby(sex ~ age.ordnew, data = mockstudy), pfootnote = TRUE)
summary(tableby(sex ~ age.ord, data = mockstudy), pfootnote = TRUE)
```

## 4. Summarize a survival variable

First look at the information that is presented by the `survfit()` function, then see how the same results can be seen with tableby.
The default is to show the median survival (time at which the probability of survival = 50%). 

```{r, eval=ge330}
survfit(Surv(fu.time, fu.stat)~sex, data=mockstudy)
survdiff(Surv(fu.time, fu.stat)~sex, data=mockstudy)
```

```{r, results='asis'}
summary(tableby(sex ~ Surv(fu.time, fu.stat), data=mockstudy))
```

It is also possible to obtain summaries of the % survival at certain time points (say the probability of surviving 1-year).

```{r, eval=ge330}
summary(survfit(Surv(fu.time/365.25, fu.stat)~sex, data=mockstudy), times=1:5)
```

```{r, results='asis', eval=ge330}
summary(tableby(sex ~ Surv(fu.time/365.25, fu.stat), data=mockstudy, times=1:3, surv.stats=c("NeventsSurv")))

summary(tableby(sex ~ Surv(fu.time/365.25, fu.stat), data=mockstudy, times=1:5, surv.stats=c("NriskSurv")))

summary(tableby(sex ~ Surv(fu.time/365.25, fu.stat), data=mockstudy, surv.stats="medSurvCI", survconf.type='log-log'), digits=2)

summary(tableby(sex ~ Surv(fu.time/365.25, fu.stat), data=mockstudy, surv.stats="medSurvQuant"), digits=2)


```

## 5. Summarize date variables

Date variables by default are summarized with the number of missing values, the median, and the range.
For example purposes we've created a random date.  Missing values are introduced for impossible February dates.

```{r, results='asis'}
set.seed(100)
N <- nrow(mockstudy)
mockstudy$dtentry <- mdy.Date(month=sample(1:12,N,replace=T), day=sample(1:29,N,replace=T), 
                              year=sample(2005:2009,N,replace=T))
summary(tableby(sex ~ dtentry, data=mockstudy))
```

## 6. Summarize multiple variables without typing them out

Often one wants to summarize a number of variables.  Instead of typing by hand each individual variable,
an alternative approach is to create a formula using the `paste` command with the `collapse="+"` option.  

```{r, results='asis'}
## create a vector specifying the variable names
myvars <- names(mockstudy)

## select the 8th through the last variables
## paste them together, separated by the + sign
RHS <- paste(myvars[8:10], collapse="+")
RHS

## create a formula using the as.formula function
as.formula(paste('arm ~ ', RHS))

## use the formula in the tableby function
summary(tableby(as.formula(paste('arm ~', RHS)), data=mockstudy))
```

These steps can also be done using the `formulize` function.

```{r, results='asis'}
## The formulize function does the paste and as.formula steps
tmp <- formulize('arm',myvars[8:10])
tmp

## More complex formulas could also be written using formulize
tmp2 <- formulize('arm',c('ps','hgb^2','bmi'))

## use the formula in the tableby function
summary(tableby(tmp, data=mockstudy))
```

To change summary statistics or statistical tests en masse, consider using `paste0()` together with `formulize()`:

```{r results='asis'}
varlist1 <- c('age','sex','hgb')
varlist2 <- paste0("anova(", c('bmi','alk.phos','ast'), ", 'meansd')")

summary(tableby(formulize("arm", c(varlist1, varlist2)),
                data = mockstudy, numeric.test = "kwt"), pfootnote = TRUE)
```

## 7. Subset the dataset used in the analysis

Here are two ways to get the same result (limit the analysis to subjects age>5 and in the F: FOLFOX treatment group). 

* The first approach uses the subset function applied to the dataset `mockstudy`.  This example also selects a subset of variables.
The `tableby` function is then applied to this subsetted data.


```{r}
newdata <- subset(mockstudy, subset=age>50 & arm=='F: FOLFOX', select = c(sex,ps:bmi))
dim(mockstudy)
table(mockstudy$arm)
dim(newdata)
names(newdata)
```

```{r, results='asis'}
summary(tableby(sex ~ ., data=newdata))
```

* The second approach does the same analysis but uses the subset
argument within `tableby` to subset the data.

```{r, results='asis'}
summary(tableby(sex ~ ps + hgb + bmi, subset=age>50 & arm=="F: FOLFOX", data=mockstudy))
```

## 8. Create combinations of variables on the fly

```{r}
## create a variable combining the levels of mdquality.s and sex
with(mockstudy, table(interaction(mdquality.s,sex)))
```

```{r, results='asis'}
summary(tableby(arm ~ interaction(mdquality.s,sex), data=mockstudy))
```

```{r, results='asis'}
## create a new grouping variable with combined levels of arm and sex
summary(tableby(interaction(mdquality.s, sex) ~  age + bmi, data=mockstudy, subset=arm=="F: FOLFOX"))
```

## 9. Transform variables on the fly

Certain transformations need to be surrounded by `I()` so that R knows to treat it as a variable transformation and
not some special model feature.  If the transformation includes any of the symbols `/ - + ^ *` then surround the new variable by `I()`.


```{r, maketrans, results='asis'}
trans <- tableby(arm ~ I(age/10) + log(bmi) + factor(mdquality.s, levels=0:1, labels=c('N','Y')),
                 data=mockstudy)
summary(trans)
```

The labels for these variables aren't exactly what we'd like, so we can change modify those after the fact.
Instead of typing out the very long variable names, you can modify specific labels by position. 

```{r, assignlabels2}
labels(trans)
labels(trans)[2:4] <- c('Age per 10 yrs', 'log(BMI)', 'MD Quality')
labels(trans)
```

```{r, transsummary, results='asis'}
summary(trans)
```

Note that if we had not changed `mdquality.s` to a factor, it would have been summarized as though it were a continuous variable.

```{r, results='asis'}
class(mockstudy$mdquality.s)
summary(tableby(arm~mdquality.s, data=mockstudy))
```

Another option would be to specify the test and summary statistics.
In fact, if I had a set of variables coded 0/1 and that was all I was summarizing, then I could change the global option
for continuous variables to use the chi-square test and show countpct.

```{r, results='asis'}
summary(tableby(arm ~ chisq(mdquality.s, "Nmiss","countpct"), data=mockstudy))
```


## 10. Subsetting (change the ordering of the variables, delete a variable, sort by p-value, filter by p-value, show only certain by-levels)

```{r, results='asis'}
mytab <- tableby(arm ~ sex + alk.phos + age, data=mockstudy)
mytab2 <- mytab[c('age','sex','alk.phos')]
summary(mytab2)
summary(mytab[c('age','sex')], digits = 2)
summary(mytab[c(3,1)], digits = 3)
summary(sort(mytab, decreasing = TRUE))
summary(mytab[mytab < 0.5])
head(mytab, 1) # can also use tail()

summary(tableby(list(arm, sex) ~ sex + alk.phos + age, data=mockstudy)[, "sex"])
summary(tableby(list(arm, sex) ~ sex + alk.phos + age, data=mockstudy)[, list(sex = "Female", arm = c("F: FOLFOX", "Total"))])
```

## 11. Merge two `tableby` objects together 

It is possible to combine two tableby objects so that they print out together. Overlapping by-variables will have their x-variables
concatenated, and (if `all=TRUE`) non-overlapping by-variables will have their tables printed separately.

```{r, results="asis"}
## demographics
tab1 <- tableby(arm ~ sex + age, data=mockstudy,
                control=tableby.control(numeric.stats=c("Nmiss","meansd"), total=FALSE))
## lab data
tab2 <- tableby(arm ~ hgb + alk.phos, data=mockstudy,
                control=tableby.control(numeric.stats=c("Nmiss","median","q1q3"),
                                        numeric.test="kwt", total=FALSE))
tab12 <- merge(tab1, tab2)
class(tab12)
summary(tab12)
```

For tables with two different outcomes, consider the `all=TRUE` argument:

```{r, results='asis'}
summary(merge(
  tableby(sex ~ age, data = mockstudy),
  tableby(arm ~ bmi, data = mockstudy),
  all = TRUE
))
```

## 12. Add a title to the table

When creating a pdf the tables are automatically numbered and the title appears below the table.  In Word and HTML, the titles appear un-numbered and above the table.  

```{r, results='asis'}
t1 <- tableby(arm ~ sex + age, data=mockstudy)
summary(t1, title='Demographics')
```

With multiple left-hand sides, you can pass a vector or list to determine labels for each table:

```{r, results='asis'}
summary(tableby(list(arm, sex) ~ age, data = mockstudy), title = c("arm table", "sex table"))
```

## 13. Modify how missing values are displayed

Depending on the report you are writing you have the following options:

* Show how many subjects have each variable

* Show how many subjects are missing each variable

* Show how many subjects are missing each variable only if there are any missing values

* Don't indicate missing values at all

```{r}
## look at how many missing values there are for each variable
apply(is.na(mockstudy),2,sum)
```

```{r, results='asis'}
## Show how many subjects have each variable (non-missing)
summary(tableby(sex ~ ast + age, data=mockstudy,
                control=tableby.control(numeric.stats=c("N","median"), total=FALSE)))

## Always list the number of missing values
summary(tableby(sex ~ ast + age, data=mockstudy,
                control=tableby.control(numeric.stats=c("Nmiss2","median"), total=FALSE)))

## Only show the missing values if there are some (default)
summary(tableby(sex ~ ast + age, data=mockstudy, 
                control=tableby.control(numeric.stats=c("Nmiss","mean"),total=FALSE)))

## Don't show N at all
summary(tableby(sex ~ ast + age, data=mockstudy, 
                control=tableby.control(numeric.stats=c("mean"),total=FALSE)))
```

One might also consider the use of `includeNA()` to include NAs in the counts and percents for categorical variables.

```{r, results = 'asis'}
mockstudy$ps.cat <- factor(mockstudy$ps)
attr(mockstudy$ps.cat, "label") <- "ps"
summary(tableby(sex ~ includeNA(ps.cat), data = mockstudy, cat.stats = "countpct"))
```

## 14. Modify the number of digits used

Within tableby.control function there are 4 options for controlling the number of significant digits shown.  

* digits: controls the number of digits after the decimal place for continuous values

* digits.count: controls the number of digits after the decimal point for counts

* digits.pct: controls the number of digits after the decimal point for percents

* digits.p: controls the number of digits after the decimal point for p-values

```{r, results='asis'}
summary(tableby(arm ~ sex + age + fu.time, data=mockstudy), digits=4, digits.p=2, digits.pct=1)
```

All of these can be specified on a per-variable basis
using the in-formula functions that specify which tests are run:

```{r results='asis'}
summary(tableby(arm ~ chisq(sex, digits.pct=1) + anova(age, digits=4) +
                  anova(fu.time, digits = 1, digits.p = 6, format.p = FALSE), data=mockstudy))
```


## 15. Create a user-defined summary statistic

For purposes of this example, the code below creates a trimmed mean function (trims 10%)
and use that to summarize the data. Note the use of the `...` which tells R to pass extra arguments on - this is required
for user-defined functions.  In this case, `na.rm=T` is passed to `myfunc`.  The *weights* argument is also required, even though
it isn't passed on to the internal function in this particular example.   

```{r, results='asis'}
trim10 <- function(x, weights=NULL, ...){
  mean(x, trim=.1, ...)
}

summary(tableby(sex ~ hgb, data=mockstudy, 
                control=tableby.control(numeric.stats=c("Nmiss","trim10"), numeric.test="kwt",
                    stats.labels=list(Nmiss='Missing values', trim10="Trimmed Mean, 10%"))))
```

For statistics to be formatted appropriately, you may want to use `as.tbstat()`.
For example, suppose you want to create a trimmed mean function that trims by both 5 and 10 percent.
The first example shows them separated by a comma; the second puts the 10% trimmed mean in brackets.
`as.tbstat()` takes a `fmt=` argument with a `glue` string specification, where the current value is exposed as
`x` and a formatted value (using `tbfmt()`) is exposed as `y`. See `?as.tbstat` for details.

```{r, results='asis'}
trim510comma <- function(x, ...){
  tmp <- c(mean(x, trim = 0.05, ...), mean(x, trim = 0.1, ...))
  as.tbstat(tmp, fmt = "{y[1]}, {y[2]}")
}
trim510bracket <- function(x, ...){
  tmp <- c(mean(x, trim = 0.05, ...), mean(x, trim = 0.1, ...))
  as.tbstat(tmp, fmt = "{y[1]} [{y[2]}]")
}

summary(tableby(sex ~ hgb, data=mockstudy, numeric.stats=c("Nmiss", "trim510comma", "trim510bracket"), test = FALSE))
```

Or perhaps it's useful to put the amount of trimming in parentheses. Since it is a percent, we can
flag it as such:

```{r results='asis'}
trim10pct <- function(x, ...){
  tmp <- mean(x, trim = 0.05, ...)
  as.tbstat(c(tmp, 10), fmt = "{y[1]} ({y[2]}%)", which.pct = 2L)
}
summary(tableby(sex ~ hgb, data=mockstudy, numeric.stats=c("Nmiss", "trim10pct"),
                digits = 2, digits.pct = 0, test = FALSE))

```

Finally, if there's a pre-computed summary statistic, you can easily change the formatting, like in `meanpmsd()`:

```{r}
meanpmsd
```

For example:

```{r}
dollarmean <- function(x, na.rm = TRUE, ...) {
  if(na.rm && allNA(x)) {
    as.tbstat(NA_real_)
  } else {
    out <- meansd(x, na.rm = na.rm, ...)
    attr(out, "fmt") <- "${y[1]}"
    out
  }
}

summary(tableby(sex ~ notest(age, "dollarmean", digits = 2), data = mockstudy))
```

## 16. Use case-weights for creating summary statistics

When comparing groups, they are often unbalanced when it comes to nuisances such as age and sex.
The `tableby` function allows you to create weighted summary statistics.  If this option us used then p-values are not calculated (`test=FALSE`).

```{r}
##create fake group that is not balanced by age/sex 
set.seed(200)
mockstudy$fake_arm <- ifelse(mockstudy$age>60 & mockstudy$sex=='Female',sample(c('A','B'),replace=T, prob=c(.2,.8)),
                            sample(c('A','B'),replace=T, prob=c(.8,.4)))

mockstudy$agegp <- cut(mockstudy$age, breaks=c(18,50,60,70,90), right=FALSE)

## create weights based on agegp and sex distribution
tab1 <- with(mockstudy,table(agegp, sex))
tab2 <- with(mockstudy, table(agegp, sex, fake_arm))
tab2
gpwts <- rep(tab1, length(unique(mockstudy$fake_arm)))/tab2
gpwts[gpwts>50] <- 30

## apply weights to subjects
index <- with(mockstudy, cbind(as.numeric(agegp), as.numeric(sex), as.numeric(as.factor(fake_arm)))) 
mockstudy$wts <- gpwts[index]

## show weights by treatment arm group
tapply(mockstudy$wts,mockstudy$fake_arm, summary)
```

```{r, results='asis', eval=ge330}
orig <- tableby(fake_arm ~ age + sex + Surv(fu.time/365, fu.stat), data=mockstudy, test=FALSE)
summary(orig, title='No Case Weights used')
tab1 <- tableby(fake_arm ~ age + sex + Surv(fu.time/365, fu.stat), data=mockstudy, weights=wts)
summary(tab1, title='Case Weights used')
```
                
                
## 17. Create your own p-value and add it to the table

When using weighted summary statistics, it is often desirable to then show a p-value from a model that corresponds to the weighted analysis. 
It is possible to add your own p-value and modify the column title for that new p-value.  Another use for this would be to add standardized 
differences or confidence intervals instead of a p-value.

To add the p-value, you simply need to create a data frame and use the function `modpval.tableby()`.
The first few columns in the data.frame are required: (1) the by-variable, (2) the strata value (if
the table has a strata term), (3) the x-variable, and (4) the new p-value (or test statistic).
Another optional column can be used to indicate what method was used to calculate the p-value.
If you specify `use.pname=TRUE` then the column name indicating the p-value will be also be used in the tableby summary.


```{r, results='asis', eval=ge330}
mypval <- data.frame(
  byvar = "fake_arm",
  variable = c('age','sex','Surv(fu.time/365, fu.stat)'), 
  adj.pvalue = c(.953,.811,.01), 
  method = c('Age/Sex adjusted model results')
)
tab2 <- modpval.tableby(tab1, mypval, use.pname=TRUE)
summary(tab2, title='Case Weights used, p-values added', pfootnote=TRUE)
```

## 18. For two-level categorical variables or one-line numeric variables, simplify the output.

If the `cat.simplify` option is set to `TRUE`, then only the second level of two-level
categorical varialbes is shown.  In the example below, `sex` has two levels,
and "Female" is the second level, hence only the counts and percents for Female are shown.
Similarly, "mdquality.s" was turned to a factor, and "1" is the second level, but since
there are missings, the table ignores `cat.simplify` and displays all levels (since the
output can no longer be displayed on one line).

```{r, results='asis'}
table2 <- tableby(arm~sex + factor(mdquality.s), data=mockstudy, cat.simplify=TRUE)
summary(table2, labelTranslations=c(sex="Female", "factor(mdquality.s)"="MD Quality"))
```

Similarly, if `numeric.simplify` is set to `TRUE`, then any numerics which only have one
row of summary statistics are simplified into a single row. Note again that `ast` has
missing values and so is not simplified to a single row.

```{r results='asis'}
summary(tableby(arm ~ age + ast, data = mockstudy,
                numeric.simplify=TRUE, numeric.stats=c("Nmiss", "meansd")))
```

The in-formula functions to change which tests are run can also be used to specify these
options for each variable at a time.

```{r results='asis'}
summary(tableby(arm ~ anova(age, "meansd", numeric.simplify=TRUE) +
                  chisq(sex, cat.simplify=TRUE), data = mockstudy))
```

The `cat.simplify` and `ord.simplify` argument also accept the special string `"label"`, which appends the shown level's label to the overall label:

```{r results='asis'}
summary(tableby(arm ~ sex, cat.simplify = "label", data = mockstudy))
```


## 19. Use `tableby` within an Sweave document

For those users who wish to create tables within an Sweave document, the following code seems to work.

```
\documentclass{article}

\usepackage{longtable}
\usepackage{pdfpages}

\begin{document}

\section{Read in Data}
<<echo=TRUE>>=
require(arsenal)
require(knitr)
require(rmarkdown)
data(mockstudy)

tab1 <- tableby(arm~sex+age, data=mockstudy)
@

\section{Convert Summary.Tableby to LaTeX}
<<echo=TRUE, results='hide', message=FALSE>>=
capture.output(summary(tab1), file="Test.md")

## Convert R Markdown Table to LaTeX
render("Test.md", pdf_document(keep_tex=TRUE))
@ 

\includepdf{Test.pdf}

\end{document}
```

## 20. Export `tableby` object to a .CSV file

When looking at multiple variables it is sometimes useful to export the results to a csv file. The `as.data.frame` function creates a data frame object that can be exported or further manipulated within R.

```{r}
tab1 <- summary(tableby(arm~sex+age, data=mockstudy), text = NULL)
as.data.frame(tab1)

# write.csv(tab1, '/my/path/here/my_table.csv')
```

## 21. Write `tableby` object to a separate Word or HTML file

```{r eval = FALSE}
## write to an HTML document
tab1 <- tableby(arm ~ sex + age, data=mockstudy)
write2html(tab1, "~/trash.html")

## write to a Word document
write2word(tab1, "~/trash.doc", title="My table in Word")
```

## 22. Use `tableby` in R Shiny

The easiest way to output a `tableby()` object in an R Shiny app is to use the `tableOutput()` UI in combination with
the `renderTable()` server function and `as.data.frame(summary(tableby()))`:

```{r eval=FALSE}
# A standalone shiny app
library(shiny)
library(arsenal)
data(mockstudy)

shinyApp(
  ui = fluidPage(tableOutput("table")),
  server = function(input, output) {
    output$table <- renderTable({
      as.data.frame(summary(tableby(sex ~ age, data = mockstudy), text = "html"))
    }, sanitize.text.function = function(x) x)
  }
)
```

This can be especially powerful if you feed the selections from a `selectInput(multiple = TRUE)` into `formulize()` to make
the table dynamic!

## 23. Use `tableby` in bookdown

Since the backbone of `tableby()` is `knitr::kable()`, tables still render well in bookdown. However, `print.summary.tableby()` doesn't use
the `caption=` argument of `kable()`, so some tables may not have a properly numbered caption. To fix this, use the method described
[on the bookdown site](https://pkg.yihui.org/bookdown/tables.html) to give the table a tag/ID.

```{r eval=FALSE}
summary(tableby(sex ~ age, data = mockstudy), title="(\\#tab:mytableby) Caption here")
```

## 24. Adjust `tableby` for multiple p-values

The `padjust()` function is a new S3 generic piggybacking off of `p.adjust()`. It works on both `tableby` and `summary.tableby` objects:

```{r results='asis'}
tab <- summary(tableby(sex ~ age + fu.time + bmi + mdquality.s, data = mockstudy))
tab
padjust(tab, method = "bonferroni")
```

## 25. Tabulate multiple endpoints

You can now use `list()` on the left-hand side of `tableby()` to give multiple endpoints.

```{r results='asis'}
summary(tableby(list(sex, mdquality.s, ps) ~ age + bmi, data = mockstudy))
```

To avoid confusion about which table is which endpoint, you can set `term.name=TRUE` in `summary()`. This takes the labels
for each by-variable and puts them in the top-left of the table.

```{r results='asis'}
summary(tableby(list(sex, mdquality.s, ps) ~ age + bmi, data = mockstudy), term.name = TRUE)
```

## 26. Tabulate data by a non-test group (strata)

You can also specify a second grouping variable that doesn't get tested (but instead separates results): a *strata* variable.

```{r results='asis'}
summary(tableby(list(sex, ps) ~ age + bmi, strata = arm, data = mockstudy))
```

# Available Function Options

## Summary statistics

The **default** summary statistics, by varible type, are:

* `numeric.stats`: Continuous variables will show by default `Nmiss, meansd, range`
* `cat.stats`: Categorical and factor variables will show by default `Nmiss, countpct`
* `ordered.stats`: Ordered factors will show by default `Nmiss, countpct`
* `surv.stats`: Survival variables will show by default `Nmiss, Nevents, medsurv`
* `date.stats`: Date variables will show by default `Nmiss, median, range`

There are a number of extra functions defined specifically for the tableby function.

* `N`: a count of the non-missing number of observations for a particular group
* `Npct`: a count of the non-missing number of observations and the percentage of the column total (missing + non-missing) in the format `N (%)`
* `Nrowpct`: a count of the non-missing number of observations and the row-percentage (of non-missings) in the format `N (%)`
* `Nmiss`: only show the count of the number of missing values if there are some missing values
* `Nmiss2`: always show a count of the number of missing values for a variable within each group
* `Nmisspct`: show the count of the number of missing values and the percentage of the column total (missing+non-missing) if there are some missing values
* `Nmisspct2`: The same as `Nmisspct`, but always show.
* `meanse`: print the mean and standard error in the format `mean(se)`
* `meanCI`: print the mean and a (t) confidence interval
* `count`: print the number of values in a category
* `countN`: print the number of values in a category plus the total N for the group in the format `N/Total`
* `countpct`: print the number of values in a category plus the column-percentage in the format `N (%)`
* `pct`: print the column-percentage
* `countrowpct`: print the number of values in a category plus the row-percentage in the format `N (%)`
* `rowpct`: print the row-percentage
* `countcellpct`: print the number of values in a category plus the cell-percentage in the format `N (%)`
* `binomCI`: print the proportion in a category plus a binomial confidence interval.
* `rowbinomCI`: print the row proportion in a category plus a binomial confidence interval.
* `medianq1q3`: print the median, 25th, and 75th quantiles `median (Q1, Q3)`
* `q1q3`: print the 25th and 75th quantiles `Q1, Q3`
* `iqr`: print the inter-quartile range.
* `medianrange`: print the median, minimum and maximum values `median (minimum, maximum)`
* `medianmad`: print the median and median absolute deviation (mad)
* `Nevents`: print number of events for a survival object within each grouping level
* `medSurv`: print the median survival
* `NeventsSurv`: print number of events and survival at given times
* `NriskSurv`: print the number still at risk and survival at given times
* `Nrisk`: print the number still at risk at given times
* `medTime`: print the median follow-up time
* `sum`
* `max`
* `min`
* `mean`
* `sd`
* `var`
* `median`
* `range`
* `gmean`, `gsd`, `gmeansd`, `gmeanCI`: geometric means, sds, and confidence intervals.

## Testing options

The tests used to calculate p-values differ by the variable type, but can be specified
explicitly in the formula statement or in the control function.

The following tests are accepted:

* `anova`: analysis of variance test; the default test for continuous variables. When
    the grouping variable has two levels, it is equivalent to the two-sample t-test with equal variance. 
    
* `kwt`: Kruskal-Wallis test, optional test for continuous
    variables. When the grouping variable has two levels, it is equivalent to the Wilcoxon Rank Sum test.

* `wt`: An explicit Wilcoxcon test.
    
* `medtest`: Median test test, optional test for continuous variables.

* `chisq`: chi-square goodness of fit test or Pearson chi-squared test; the default for categorical
    or factor variables

* `fe`: Fisher's exact test for categorical variables; optional 
  
* `logrank`: log-rank test, the default test for time-to-event
    variables

* `trend`: The `independence_test` function from the `coin` is used to test for trends.  Whenthe grouping variable has two levels,
  it is equivalent to the Armitage trend test.  This is the default for ordered factors
  
* `stddiff`: perform standardized differences.
  
* `notest`: Don't perform a test.

## `tableby.control` settings

A quick way to see what arguments are possible to utilize in a function is to use the `args()` command. Settings involving the number of digits can be set in `tableby.control` or in `summary.tableby`.

```{r}
args(tableby.control)
```

## `summary.tableby` settings

The `summary.tableby` function has options that modify how the table appears (such as adding a title or modifying labels).  

```{r}
args(arsenal:::summary.tableby)
```

---
title: "The write2 function"
author: "Ethan Heinzen"
output:
  rmarkdown::html_vignette:
    toc: yes
    toc_depth: 3
vignette: |
  %\VignetteIndexEntry{The write2 function}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r include = FALSE}
knitr::opts_chunk$set(eval = FALSE, message = FALSE)
```

# Introduction

The `write2*()` functions were designed as an alternative to SAS's `ODS` procedure for useRs who want to save R Markdown tables
to separate Word, HTML, or PDF files without needing separate R Markdown programs.

There are three shortcut functions for the most common output types: HTML, PDF, and Word. Each of these three functions calls
`write2()`, an S3 function which accepts many file output types (see the help pages for `rmarkdown::render()`). Methods have been implemented for
`tableby()`, `modelsum()`, and `freqlist()`, but also `knitr::kable()`, `xtable::xtable()`, and `pander::pander_return()`.

The two most important things to recognize with `write2()` are the following:

1. Which function is being used to output the object. Sometimes the `write2` functions use `summary()`,
   while other times they will use `print()`. The details for each object specifically are described below.
   
2. How the `...` arguments are passed. To change the options for the summary-like or print-like function,
   you can pass named arguments which will in turn get passed to the appropriate function.
   Details for each object specifically are described below.
   
# A note on piping

`arsenal` is piping-compatible!

The `write2*()` functions are probably the most useful place to take advantage of
the `magrittr` package's piping framework, since commands are often nested several functions deep in the context
of `write2*()`. Piping also allows the `arsenal` package to become a part of more standard analysis pipelines;
instead of needing to write separate R Markdown programs, intermediate analysis tables and output can be easily
incorporated into piped statements.

This vignette will sprinkle the foward pipe (`%>%`) throughout as a hint at the power and flexibility of `arsenal` and piping.

# Examples Using `arsenal` Objects

```{r}
library(arsenal)
library(magrittr)
data(mockstudy)
tmpdir <- tempdir()
```

## `tableby`

For `tableby` objects, the output function in `write2()` is `summary()`.
For `summary.tableby` objects, the output function is `print()`.
For available arguments, see the help pages for `summary.tableby()`.
Don't use the option `text = TRUE` with the `write2` functions.

```{r}
mylabels <- list(sex = "SEX", age ="Age, yrs")
tab1 <- tableby(arm ~ sex + age, data=mockstudy)

write2html(
  tab1, paste0(tmpdir, "/test.tableby.html"), quiet = TRUE,
  title = "My test table",      # passed to summary.tableby
  labelTranslations = mylabels, # passed to summary.tableby
  total = FALSE                 # passed to summary.tableby
)
```

## `modelsum`

For `modelsum` objects, the output function in `write2()` is `summary()`.
For `summary.modelsum` objects, the output function is `print()`.
For available arguments, see the help pages for `summary.modelsum()`.
Don't use the option `text = TRUE` with the `write2` functions.

```{r}
tab2 <- modelsum(alk.phos ~ arm + ps + hgb, adjust= ~ age + sex, family = "gaussian", data = mockstudy)

write2pdf(
  tab2, paste0(tmpdir, "/test.modelsum.pdf"), quiet = TRUE,
  title = "My test table", # passed to summary.modelsum
  show.intercept = FALSE,  # passed to summary.modelsum
  digits = 5               # passed to summary.modelsum
)
```

## `freqlist`

For `freqlist` objects, the output function in `write2()` is `summary()`.
For `summary.freqlist` objects, the output function is `print()`.
For available arguments, see the help pages for `summary.freqlist()`.

```{r}
mockstudy[, c("arm", "sex", "mdquality.s")] %>% 
  table(useNA = "ifany") %>% 
  freqlist(groupBy = c("arm", "sex")) %>% 
  write2word(
    paste0(tmpdir, "/test.freqlist.doc"), quiet = TRUE,
    single = FALSE,         # passed to summary.freqlist
    title = "My cool title" # passed to summary.freqlist
  )
```

## `comparedf`

For `comparedf` objects, the output function in `write2()` is `summary()`.
For `summary.comparedf` objects, the output function is `print()`.

# Examples Using Other Objects

## `knitr::kable()`

For objects resulting from a call to `kable()`, the output function in `write2()` is `print()`. There aren't any arguments to the `print.knitr_kable()` function.

```{r}
mockstudy %>% 
  head() %>% 
  knitr::kable() %>% 
  write2html(paste0(tmpdir, "/test.kable.html"), quiet = TRUE)
```

## `xtable::xtable()`

For `xtable` objects, the output function in `write2()` is `print()`. For available arguments, see the help pages for `print.xtable()`.

```{r}
mockstudy %>% 
  head() %>% 
  xtable::xtable(caption = "My xtable") %>% 
  write2pdf(
    paste0(tmpdir, "/test.xtable.pdf"), quiet = TRUE,
    comment = FALSE, # passed to print.xtable to turn off the default message about xtable version
    include.rownames = FALSE, # passed to print.xtable
    caption.placement = "top" # passed to print.xtable
  )
```

To make an HTML document, use the `print.xtable()` option `type = "html"`.

```{r}
mockstudy %>% 
  head() %>% 
  xtable::xtable(caption = "My xtable") %>% 
  write2html(
    paste0(tmpdir, "/test.xtable.html"), quiet = TRUE,
    type = "html",            # passed to print.xtable
    comment = FALSE, # passed to print.xtable to turn off the default message about xtable version
    include.rownames = FALSE, # passed to print.xtable
    caption.placement = "top" # passed to print.xtable
  )
```

User beware! `xtable()` is not compatible with `write2word()`.

## `pander::pander_return()`

Pander is a little bit more tricky. Since `pander::pander()` doesn't return an object, the useR should instead use
`pander::pander_return()`. For this (and for all character vectors), the the output function in `write2()` is `cat(sep = '\n')`.

```{r}
write2word(pander::pander_return(head(mockstudy)), file = paste0(tmpdir, "/test.pander.doc"), quiet = TRUE)
```

# Output Multiple Tables to One Document

To output multiple tables into a document, simply make a list of them and call the same function as before.

```{r}
mylist <- list(
  tableby(sex ~ age, data = mockstudy),
  freqlist(table(mockstudy[, c("sex", "arm")])),
  knitr::kable(head(mockstudy))
)

write2pdf(mylist, paste0(tmpdir, "/test.mylist.pdf"), quiet = TRUE)

```

One neat side-effect of this function is that you can output text and headers, etc. The possibilities are endless!

```{r}
mylist2 <- list(
  "# Header 1",
  "This is a small paragraph introducing tableby.",
  tableby(sex ~ age, data = mockstudy),
  "<hr>",
  "# Header 2",
  "<font color='red'>I can change color of my text!</font>"
)
write2html(mylist2, paste0(tmpdir, "/test.mylist2.html"), quiet = TRUE)
```

In fact, you can even recurse on the lists!

```{r}
write2pdf(list(mylist2, mylist), paste0(tmpdir, "/test.mylists.pdf"), quiet = TRUE)
```

# Output Other Objects Monospaced (as if in a terminal)

It may be useful at times to write output that would normally be copied from the terminal. The default method for `write2()`
does this automatically. To output the results of `summary.lm()`, for example:

```{r}
lm(age ~ sex, data = mockstudy) %>% 
  summary() %>% 
  write2pdf(paste0(tmpdir, "/test.lm.pdf"), quiet = TRUE)
```

The `verbatim()` function is another option to explicitly alert `write2()` to do this. This
becomes particularly helpful to overrule existing S3 methods.

For example, suppose you wanted to just print a tableby object (as if it were to print in the terminal):

```{r}
tab4 <- tableby(arm ~ sex + age, data=mockstudy)
write2html(verbatim(tab4), paste0(tmpdir, "/test.print.tableby.html"), quiet = TRUE)
```

Or suppose you wanted to print a character vector (as if it were to print in the terminal):

```{r}
chr <- paste0("MyVector", 1:10)
write2pdf(verbatim(chr), paste0(tmpdir, "/test.character.pdf"), quiet = TRUE)
```

Note that you can combine multiple objects in one call:

```{r}
write2pdf(verbatim(tab4, chr), paste0(tmpdir, "/test.verbatim.pdf"), quiet = TRUE)
```

# Add a YAML Header to the Output

You can add a YAML header to `write2()` output using the `yaml()` function.

```{r}
mylist3 <- list(
  yaml(title = "Test YAML Title", author = "My cool author name"),
  "# Header 1",
  "This is a small paragraph introducing tableby.",
  tableby(sex ~ age, data = mockstudy)
)
write2html(mylist3, paste0(tmpdir, "/test.yaml.html"), quiet = TRUE)
```

In fact, all detected YAML pieces will be moved as the first output, so that the above code chunk
gives the same output as this one:

```{r}
mylist4 <- list(
  "# Header 1",
  "This is a small paragraph introducing tableby.",
  yaml(title = "Test YAML Title"),
  tableby(sex ~ age, data = mockstudy),
  yaml(author = "My cool author name")
)
write2html(mylist4, paste0(tmpdir, "/test.yaml2.html"), quiet = TRUE)
```

# Add a Code Chunk to the Output

It is now possible to add code chunks to the output `.Rmd`:

```{r}
mylist5 <- list(
  "# What is 1 + 2?",
  code.chunk(a <- 1, b <- 2),
  code.chunk(a + b, chunk.opts = "r echo=FALSE, eval=TRUE")
)
write2html(mylist5, paste0(tmpdir, "/test.code.chunk.html"), quiet = TRUE)
```

This allow flexibility to create objects on-the-fly, to read in saved objects to the temporary `.Rmd`, etc. The possibilities are endless!

# FAQs

## How do I suppress the note about my document getting rendered?

This is easily accomplished by using the argument `quiet = TRUE` (passed to the `rmarkdown::render()` function).

```{r}
write2html(
  knitr::kable(head(mockstudy)), paste0(tmpdir, "/test.kable.quiet.html"),
  quiet = TRUE # passed to rmarkdown::render
)
```

## How do I look at the temporary `.Rmd` file?

This is easily accomplished by using the option `keep.rmd = TRUE`.

```{r}
write2html(
  knitr::kable(head(mockstudy)), paste0(tmpdir, "/test.kable.keep.rmd.html"),
  quiet = TRUE, # passed to rmarkdown::render
  keep.rmd = TRUE
)
```


## How do I prevent my document from being rendered?

This is easily accomplished by using the option `render. = FALSE`. Note that this will then default to `keep.rmd = TRUE`.

```{r}
write2html(
  knitr::kable(head(mockstudy)), paste0(tmpdir, "/test.kable.dont.render.html"),
  render. = FALSE
)
```

## How do I output headers, raw HTML/LaTeX, paragraphs, etc.?

One can simply abuse the list S3 method for `write2()`!

```{r}
mylist2 <- list(
  "# Header 1",
  "This is a small paragraph introducing tableby.",
  tableby(sex ~ age, data = mockstudy),
  "<hr>",
  "# Header 2",
  "<font color='red'>I can change color of my text!</font>"
)
write2html(mylist2, paste0(tmpdir, "/test.mylist2.html"), quiet = TRUE)
```

## How do I tweak the default format from `write2word()`, `write2html()`, or `write2pdf()`?

You can pass arguments to the format functions used behind the scenes.

```{r}
write2html(
  knitr::kable(head(mockstudy)), paste0(tmpdir, "/test.kable.theme.html"),
  quiet = TRUE,  # passed to rmarkdown::render
  theme = "yeti" # passed to rmarkdown::html_document
)
```

See the help pages for `rmarkdown::word_document()`, `rmarkdown::html_document()`, and `rmarkdown::pdf_document()`.

## How do I output to a file format other than word, HTML, and PDF?

This can be done using the generic `write2()` function. The last argument in the function can be another format specification.
For details on the acceptable inputs, see the help page for `write2()`.

```{r}
write2(
  knitr::kable(head(mockstudy[, 1:4])), paste0(tmpdir, "/test.kable.rtf"),
  quiet = TRUE,  # passed to rmarkdown::render
  output_format = rmarkdown::rtf_document
)
```

## How do I avoid prefixes on my table captions in PDF?

You can do this pretty easily with the `yaml()` function:

```{r}
mylist5 <- list(
  yaml("header-includes" = list("\\usepackage[labelformat=empty]{caption}")),
  "# Header 1",
  "This is a small paragraph introducing tableby.",
  tableby(sex ~ age, data = mockstudy)
)
write2pdf(mylist5, paste0(tmpdir, "/test.noprefixes.pdf"), title = "My tableby")
```

## How do I output multiple tables with different titles?

There are now `write2()` methods for the summary objects of `arsenal` functions. This allows you to
specify a title for each table:

```{r}
mylist6 <- list(
  summary(tableby(sex ~ age, data = mockstudy), title = "A Title for tableby"),
  summary(modelsum(age ~ sex, data = mockstudy), title = "A Title for modelsum"),
  summary(freqlist(~ sex, data = mockstudy), title = "A Title for freqlist")
)
write2pdf(mylist6, paste0(tmpdir, "/test.multiple.titles.pdf"))
```

## Why is `write2()` not working in R Markdown/R Studio?

It's possible that a global option in R Studio is preventing the tables from rendering. Consider turning off (i.e., unchecking) the option
Tools > Global Options > R Markdown > Show output inline for all R Markdown documents.

---
title: "The freqlist function"
author: "Tina Gunderson and Ethan Heinzen"
output:
  rmarkdown::html_vignette:
    toc: yes
    toc_depth: 3
vignette: |
  %\VignetteIndexEntry{The freqlist function}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, tidy.opts=list(width.cutoff=80), tidy=TRUE, comment=NA)
options(width=80, max.print=1000)
```

# Overview 

`freqlist()` is a function meant to produce output similar to SAS's `PROC FREQ` procedure when using the `/list` option of the `TABLE` statement.
`freqlist()` provides options for handling missing or sparse data and can provide cumulative counts and percentages based on subgroups.
It depends on the `knitr` package for printing. 

```{r message = FALSE}
require(arsenal)
```

## Sample dataset

For our examples, we'll load the `mockstudy` data included with this package and use it to create a basic table.
Because they have fewer levels, for brevity, we'll use the variables arm, sex, and mdquality.s to create the example table.
We'll retain NAs in the table creation.
See the appendix for notes regarding default NA handling and other useful information regarding tables in R. 

```{r loading.data}
# load the data
data(mockstudy)

# retain NAs when creating the table using the useNA argument
tab.ex <- table(mockstudy[, c("arm", "sex", "mdquality.s")], useNA="ifany")
```
 
# The `freqlist` object

The `freqlist()` function is an S3 generic (with methods for tables and formulas) which returns an object of class `"freqlist"`.

```{r console.output}
example1 <- freqlist(tab.ex)

str(example1)

# view the data frame portion of freqlist output
head(as.data.frame(example1)) ## or use as.data.frame(example1)
```

# Basic output using `summary()` 

The `summary` method for `freqlist()` relies on the `kable()` function (in the `knitr` package) for printing.
`knitr::kable()` converts the output to markdown which can be printed in the console or easily rendered in
Word, PDF, or HTML documents.

Note that you must supply `results="asis"` to properly format the markdown output.

```{r, results = 'asis'}
summary(example1)
```

You can print a title for the table using the `title=` argument.

```{r, results = 'asis'}
summary(example1, title="Basic freqlist output")
```

You can also easily pull out the `freqlist` data frame for more complicated formatting or manipulation
(e.g. with another function such as `xtable()` or `pander()`) using `as.data.frame(summary())`:

```{r}
head(as.data.frame(summary(example1)))
```

# Using a formula with `freqlist`

Instead of passing a pre-computed table to `freqlist()`, you can instead pass a formula, which will be
in turn passed to the `xtabs()` function. Additional `freqlist()` arguments are passed through the `...`
to the `freqlist()` table method.

Note that `freqlist()` sets the `addNA=TRUE` argument by default:

```{r results='asis'}
summary(freqlist(~ arm + sex + mdquality.s, data = mockstudy))
```

One can also set NAs to an explicit value using `includeNA()`.

```{r results='asis'}
summary(freqlist(~ arm + sex + includeNA(mdquality.s, "Missing"), data = mockstudy))
```

In fact, since `xtabs()` allows for left-hand-side weights, so does `freqlist()`!

```{r results='asis'}
mockstudy$weights <- c(10000, rep(1, nrow(mockstudy) - 1))
summary(freqlist(weights ~ arm + sex + addNA(mdquality.s), data = mockstudy))
```

You can also specify multiple weights:

```{r results='asis'}
mockstudy$weights2 <- c(rep(1, nrow(mockstudy) - 1), 10000)
summary(freqlist(list(weights, weights2) ~ arm + sex + addNA(mdquality.s), data = mockstudy))
```

# Rounding percentage digits or changing variable names for printing

The `digits.pct=` argument takes a single numeric value and controls the number of digits of percentages in the output.
The `digits.count=` argument takes a similar argument and controls the number of digits of the count columns.
The `labelTranslations=` argument is a named character vector or list.
Both options are applied in the following example. 

```{r labelTranslations, results = 'asis'}
example2 <- freqlist(tab.ex, labelTranslations = c(arm = "Treatment Arm", sex = "Gender", mdquality.s = "LASA QOL"),
                      digits.pct = 1, digits.count = 1)
summary(example2)
```

# Additional examples

## Including combinations with frequencies of zero

The `sparse=` argument takes a single logical value as input. The default option is `FALSE`.
If set to `TRUE`, the sparse option will include combinations with frequencies of zero in the list of results.
As our initial table did not have any such levels, we create a second table to use in our example.

```{r sparse, results = 'asis'}
summary(freqlist(~ race + sex + arm, data = mockstudy, sparse = TRUE, digits.pct=1))
```

## Options for NA handling

The various `na.options=` allow you to include or exclude data with missing values for one or more factor
levels in the counts and percentages, as well as show the missing data but exclude it from the cumulative
counts and percentages. The default option is to include all combinations with missing values. 

```{r na.options, results = 'asis'}
summary(freqlist(tab.ex, na.options="include"))
summary(freqlist(tab.ex, na.options="showexclude"))
summary(freqlist(tab.ex, na.options="remove"))
```

## Frequency counts and percentages subset by factor levels 

The `strata=` argument internally subsets the data by the specified factor prior to calculating cumulative counts and percentages.
By default, when used each subset will print in a separate table. Using the `single = TRUE` option when printing will collapse
the subsetted result into a single table.

```{r freq.counts, results='asis'}
example3 <- freqlist(tab.ex, strata = c("arm","sex"))
summary(example3)

#using the single = TRUE argument will collapse results into a single table for printing
summary(example3, single = TRUE)

```

## Show only the "n" most common combinations in each table (`head()` and `sort()`)

You can now sort `freqlist()` objects, and, by taking the `head()` of the summary, output the most common frequencies. This looks
the prettiest with `dupLabels=TRUE`.

```{r}
head(summary(sort(example1, decreasing = TRUE), dupLabels = TRUE))
```

## Change labels on the fly

```{r changelabs, results = 'asis'}
labs <- c(arm = "Arm", sex = "Sex", mdquality.s = "QOL", freqPercent = "%")
labels(example1) <- labs
summary(example1)
```

You can also supply `labelTranslations=` to `summary()`.

```{r, results = 'asis'}
summary(example1, labelTranslations = labs)
```

## Using `xtable()` to format and print `freqlist()` results

Fair warning: `xtable()` has kind of a steep learning curve. These examples are given without explanation, for more advanced users.

```{r results='asis'}
require(xtable)

# set up custom function for xtable text
italic <- function(x) paste0('<i>', x, '</i>')

xftbl <- xtable(as.data.frame(summary(example1)), 
  caption = "xtable formatted output of freqlist data frame", align="|r|r|r|r|c|c|c|r|")

# change the column names
names(xftbl)[1:3] <- c("Arm", "Gender", "LASA QOL")

print(xftbl, sanitize.colnames.function = italic, include.rownames = FALSE, type = "html", comment = FALSE)
```

## Use `freqlist` in bookdown

Since the backbone of `freqlist()` is `knitr::kable()`, tables still render well in bookdown. However, `print.summary.freqlist()` doesn't use
the `caption=` argument of `kable()`, so some tables may not have a properly numbered caption. To fix this, use the method described
[on the bookdown site](https://pkg.yihui.org/bookdown/tables.html) to give the table a tag/ID.

```{r eval=FALSE}
summary(freqlist(~ sex + age, data = mockstudy), title="(\\#tab:mytableby) Caption here")
```

# Appendix: Notes regarding table options in R

## NAs

There are several widely used options for basic tables in R. The `table()` function in base R is probably the most common;
by default it excludes NA values. You can change NA handling in `base::table()` using the `useNA=` or `exclude=` arguments.

```{r}
# base table default removes NAs
tab.d1 <- base::table(mockstudy[, c("arm", "sex", "mdquality.s")], useNA="ifany")
tab.d1
```

`xtabs()` is similar to `table()`, but uses a formula-based syntax. However, NAs must be explicitly added to each factor
using the `addNA()` function or using the argument `addNA = TRUE`.

```{r}
# without specifying addNA
tab.d2 <- xtabs(formula = ~ arm + sex + mdquality.s, data = mockstudy)
tab.d2

# now with addNA
tab.d3 <- xtabs(~ arm + sex + addNA(mdquality.s), data = mockstudy)
tab.d3

```

Since the formula method of `freqlist()` uses `xtabs()`, NAs should be treated in the same way.
`includeNA()` can also be helpful here for setting explicit NA values.

## Table dimname names (dnn)

Supplying a data.frame to the `table()` function without giving columns individually will create a contingency table
using all variables in the data.frame.

However, if the columns of a data.frame or matrix are supplied separately (i.e., as vectors),
column names will not be preserved.

```{r}
# providing variables separately (as vectors) drops column names
table(mockstudy$arm, mockstudy$sex, mockstudy$mdquality.s)
```

If desired, you can use the `dnn=` argument to pass variable names.

```{r}
# add the column name labels back using dnn option in base::table
table(mockstudy$arm, mockstudy$sex, mockstudy$mdquality.s, dnn=c("Arm", "Sex", "QOL"))
```

You can also name the arguments to `table()`:

```{r}
table(Arm = mockstudy$arm, Sex = mockstudy$sex, QOL = mockstudy$mdquality.s)
```

If using `freqlist()`, you can provide the labels directly to `freqlist()` or to `summary()` using `labelTranslations=`.

---
title: "The paired function"
author: "Ethan Heinzen, Beth Atkinson, Jason Sinnwell"
output: 
  rmarkdown::html_vignette:
    toc: yes
    toc_depth: 3
vignette: |
  %\VignetteIndexEntry{The paired function}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r echo = FALSE}
options(width = 100)
```

# Introduction

Another one of the most common tables in medical literature includes summary statistics for a set of variables paired across
two time points.  Locally at Mayo, the SAS macro `%paired` was written to create summary tables with a single call.
With the increasing interest in R, we have developed the function `paired()` to create similar tables within the R environment.  

This vignette is light on purpose; `paired()` piggybacks off of tableby, so most documentation there applies here, too.

# Simple Example

The first step when using the `paired()` function is to load the `arsenal` package.  We can't use `mockstudy` here because
we need a dataset with paired observations, so we'll create our own dataset.

```{r, load-data}
library(arsenal)
dat <- data.frame(
  tp = paste0("Time Point ", c(1, 2, 1, 2, 1, 2, 1, 2, 1, 2)),
  id = c(1, 1, 2, 2, 3, 3, 4, 4, 5, 6),
  Cat = c("A", "A", "A", "B", "B", "B", "B", "A", NA, "B"),
  Fac = factor(c("A", "B", "C", "A", "B", "C", "A", "B", "C", "A")),
  Num = c(1, 2, 3, 4, 4, 3, 3, 4, 0, NA),
  Ord = ordered(c("I", "II", "II", "III", "III", "III", "I", "III", "II", "I")),
  Lgl = c(TRUE, TRUE, FALSE, TRUE, FALSE, TRUE, TRUE, FALSE, FALSE, FALSE),
  Dat = as.Date("2018-05-01") + c(1, 1, 2, 2, 3, 4, 5, 6, 3, 4),
  stringsAsFactors = FALSE
)
```

To create a simple table stratified by time point, use a `formula=` statement to specify the variables that you want summarized and
the `id=` argument to specify the paired observations.

```{r results = 'asis'}
p <- paired(tp ~ Cat + Fac + Num + Ord + Lgl + Dat, data = dat, id = id, signed.rank.exact = FALSE)
summary(p)
```

The third column shows the difference between time point 1 and time point 2. For categorical variables, it reports
the percent of observations from time point 1 which changed in time point 2.

# NAs

Note that by default, observations which do not have both timepoints are removed.
This is easily changed using the `na.action = na.paired("<arg>")` argument. For example:

```{r results = 'asis'}
p <- paired(tp ~ Cat + Fac + Num + Ord + Lgl + Dat, data = dat, id = id,
            signed.rank.exact = FALSE, na.action = na.paired("fill"))
summary(p)
```

For more details, see the help page for `na.paired()`.

# Available Function Options

## Testing options

The tests used to calculate p-values differ by the variable type, but can be specified
explicitly in the formula statement or in the control function.

The following tests are accepted:

* `paired.t`: A paired t-test. 

* `mcnemar`: McNemar's test.

* `signed.rank`: the signed-rank test.

* `signtest`: the sign test.

* `notest`: Don't perform a test.

## `paired.control` settings

A quick way to see what arguments are possible to utilize in a function is to use the `args()` command. Settings involving the number of digits can be set in `paired.control` or in `summary.tableby`.

```{r}
args(paired.control)
```

## `summary.tableby` settings

Since the "paired" object inherits "tableby", the `summary.tableby` function is what's actually used
to format and print the table.

```{r}
args(arsenal:::summary.tableby)
```

---
title: Test title
---



```

Call:
lm(formula = age ~ sex, data = mockstudy)

Residuals:
    Min      1Q  Median      3Q     Max 
-41.455  -7.455   0.753   8.545  28.753 

Coefficients:
            Estimate Std. Error t value Pr(>|t|)
(Intercept)  60.4552     0.3802 159.001   <2e-16
sexFemale    -1.2082     0.6097  -1.982   0.0477

Residual standard error: 11.51 on 1497 degrees of freedom
Multiple R-squared:  0.002617,	Adjusted R-squared:  0.00195 
F-statistic: 3.927 on 1 and 1497 DF,  p-value: 0.04769


```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{arsenal-deprecated}
\alias{arsenal-deprecated}
\title{Deprecated functions in \code{arsenal}}
\description{
Details about deprecated functions in \code{arsenal}
}
\seealso{
\code{\link{arsenal-defunct}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arsenal.R
\docType{package}
\name{arsenal-package}
\alias{arsenal}
\alias{arsenal-package}
\title{An Arsenal of 'R' Functions for Large-Scale Statistical Summaries}
\description{
An Arsenal of 'R' functions for large-scale statistical summaries,
  which are streamlined to work within the latest reporting tools in 'R' and 'RStudio' and
  which use formulas and versatile summary statistics for summary tables and models.
}
\details{
The package download, NEWS, and README are available on CRAN: \url{https://cran.r-project.org/package=arsenal}
}
\section{Functions}{


Below are listed some of the most widely used functions available in \code{arsenal}:

\code{\link{tableby}}: Summary statistics of a set of independent variables by a categorical variable.

\code{\link{paired}}: Summary statistics of a set of independent variables paired across two timepoints.

\code{\link{modelsum}}: Fit models over each of a set of independent variables with a response variable.

\code{\link{freqlist}}: Approximate the output from SAS's \code{PROC FREQ} procedure when using
 the \code{/list} option of the \code{TABLE} statement.

\code{\link{comparedf}}: Compare two data.frames and report any differences between them,
 much like SAS's \code{PROC COMPARE} procedure.

\code{\link{write2word}}, \code{\link{write2html}}, \code{\link{write2pdf}}: Functions to output
  tables to a single Word, HTML, or PDF document.

\code{\link{write2}}: Functions to output tables to a single document.
  (Also the S3 backbone behind the \code{write2*} functions.)

\code{\link{keep.labels}}: Keep the \code{'label'} attribute on an R object when subsetting.

\code{\link{formulize}}: A shortcut to generate one-, two-, or many-sided formulas.

\code{\link{mdy.Date}} and \code{\link{Date.mdy}}: Convert numeric dates for month, day, and year to Date object, and vice versa.

\code{\link{is.Date}}: Test if an object is a date.

\code{\link{\%nin\%}}: Test for "not in".

\code{\link{allNA}} and \code{\link{includeNA}}: some useful functions for dealing with NAs.
}

\section{Data}{


\code{\link{mockstudy}}: Mock study data for examples.
}

\examples{
library(arsenal)

}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/mayoverse/arsenal}
  \item \url{https://cran.r-project.org/package=arsenal}
  \item \url{https://mayoverse.github.io/arsenal/}
  \item Report bugs at \url{https://github.com/mayoverse/arsenal/issues}
}

}
\author{
\strong{Maintainer}: Ethan Heinzen \email{heinzen.ethan@mayo.edu}

Authors:
\itemize{
  \item Ethan Heinzen \email{heinzen.ethan@mayo.edu}
  \item Jason Sinnwell
  \item Elizabeth Atkinson
  \item Tina Gunderson
  \item Gregory Dougherty
}

Other contributors:
\itemize{
  \item Patrick Votruba [contributor]
  \item Ryan Lennon [contributor]
  \item Andrew Hanson [contributor]
  \item Krista Goergen [contributor]
  \item Emily Lundt [contributor]
  \item Brendan Broderick [contributor]
  \item Maddie McCullough [artist]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write2.internal.R
\name{write2.internal}
\alias{write2.internal}
\alias{verbatim}
\alias{code.chunk}
\title{Helper functions for \code{write2}}
\usage{
verbatim(...)

code.chunk(..., chunk.opts = "r")
}
\arguments{
\item{...}{For \code{verbatim}, objects to print out monospaced (as if in the terminal). For \code{code.chunk},
either expressions or single character strings to paste into the code chunk.}

\item{chunk.opts}{A single character string giving the code chunk options. Make sure to specify the engine!}
}
\description{
Helper functions for \code{\link{write2}}.
}
\details{
The \code{"verbatim"} class is to tell \code{\link{write2}} to print the object inside
  a section surrounded by three back ticks. The results will look like it would in the terminal (monospaced).

\code{code.chunk()} is to write explicit code chunks in the \code{.Rmd} file; it captures the call and writes it to the
  file, to execute upon knitting.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write2.R, R/write2.default.R
\name{write2}
\alias{write2}
\alias{write2.arsenal_table}
\alias{write2.summary.arsenal_table}
\alias{write2.comparedf}
\alias{write2.summary.comparedf}
\alias{write2.verbatim}
\alias{write2.yaml}
\alias{write2.code.chunk}
\alias{write2.knitr_kable}
\alias{write2.xtable}
\alias{write2.character}
\alias{write2.list}
\alias{write2.default}
\title{write2}
\usage{
write2(object, file, ..., output_format)

\method{write2}{arsenal_table}(object, file, ..., output_format = NULL)

\method{write2}{summary.arsenal_table}(object, file, ..., output_format = NULL)

\method{write2}{comparedf}(object, file, ..., output_format = NULL)

\method{write2}{summary.comparedf}(object, file, ..., output_format = NULL)

\method{write2}{verbatim}(object, file, ..., output_format = NULL)

\method{write2}{yaml}(object, file, ..., output_format = NULL)

\method{write2}{code.chunk}(object, file, ..., output_format = NULL)

\method{write2}{knitr_kable}(object, file, ..., output_format = NULL)

\method{write2}{xtable}(object, file, ..., output_format = NULL)

\method{write2}{character}(object, file, ..., output_format = NULL)

\method{write2}{list}(
  object,
  file,
  ...,
  append. = FALSE,
  render. = TRUE,
  keep.rmd = !render.,
  output_format = NULL
)

\method{write2}{default}(
  object,
  file,
  FUN = NULL,
  ...,
  append. = FALSE,
  render. = TRUE,
  keep.rmd = !render.,
  output_format = NULL
)
}
\arguments{
\item{object}{An object.}

\item{file}{A single character string denoting the filename for the output document.}

\item{...}{Additional arguments to be passed to \code{FUN}, \code{rmarkdown::render}, etc.
One popular option is to use \code{quiet = TRUE} to suppress the command line output.}

\item{output_format}{One of the following:
\enumerate{
  \item{An output format object, e.g. \code{rmarkdown::\link[rmarkdown]{html_document}(...)}.}
  \item{A character string denoting such a format function, e.g. \code{"html_document"}. In this case, the \code{"..."} are NOT passed.}
  \item{The format function itself, e.g. \code{rmarkdown::html_document}. In this case, the \code{"..."} arguments are passed.}
  \item{One of \code{"html"}, \code{"pdf"}, and \code{"word"}, shortcuts implemented here. In this case, the \code{"..."} arguments are passed.}
  \item{\code{NULL}, in which the output is HTML by default.}
}
See \code{rmarkdown::\link[rmarkdown]{render}} for details.}

\item{append.}{Logical, denoting whether (if a temporary \code{.Rmd} file of the same name already exists)
to append on. Used mostly for \code{write2.list}.}

\item{render.}{Logical, denoting whether to render the temporary \code{.Rmd} file. Used mostly for \code{write2.list}.}

\item{keep.rmd}{Logical, denoting whether to keep the intermediate \code{.Rmd} file. Used mostly for \code{write2.list}.}

\item{FUN}{The summary-like or print-like function to use to generate the markdown content. Can be passed as a function or a
character string. It's expected that \code{FUN(object, ...)} looks "good" when put directly in a \code{.Rmd} file.}
}
\value{
\code{object} is returned invisibly, and \code{file} is written.
}
\description{
Functions to output tables to a single document. (Also the S3 backbone behind the \code{write2*} functions.)
}
\details{
\code{write2} is an S3 method. The default prints the object (using \code{\link{print}})
   inside a section surrounded by three back ticks. See \code{\link{verbatim}} for details.

   There are methods implemented for \code{\link{tableby}}, \code{\link{modelsum}}, and \code{\link{freqlist}}, all of which use the
   \code{summary} function. There are also methods compatible with \code{\link[knitr]{kable}}, \code{\link[xtable]{xtable}},
   and \code{\link[pander]{pander_return}}. Another option is to coerce an object using \code{\link{verbatim}()} to print out the
   results monospaced (as if they were in the terminal). To output multiple tables into a document, simply make a list of them
   and call the same function as before. Finally, to output code chunks to be evaluated, use \code{\link{code.chunk}}.

   For more information, see \code{vignette("write2")}.
}
\examples{
\dontrun{
data(mockstudy)
# tableby example
tab1 <- tableby(arm ~ sex + age, data=mockstudy)
write2(tab1, tempfile(fileext = ".rtf"),
  toc = TRUE, # passed to rmarkdown::rtf_document, though in this case it's not practical
  quiet = TRUE, # passed to rmarkdown::render
  title = "My cool new title", # passed to summary.tableby
  output_format = rmarkdown::rtf_document)

write2html(list(
  "# Header 1", # a header
  code.chunk(a <- 1, b <- 2, a + b), # a code chunk
  verbatim("hi there") # verbatim output
),
  tempfile(fileext = ".html"),
  quite = TRUE)
}
}
\seealso{
\code{\link{write2word}}, \code{\link{write2pdf}}, \code{\link{write2html}},
  \code{\link[rmarkdown]{render}}, \code{\link[rmarkdown]{word_document}}, \code{\link[rmarkdown]{html_document}}, \code{\link[rmarkdown]{pdf_document}},
  \code{\link[rmarkdown]{rtf_document}}, \code{\link[rmarkdown]{md_document}}, \code{\link[rmarkdown]{odt_document}}
}
\author{
Ethan Heinzen, adapted from code from Krista Goergen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/not.in.R
\name{\%nin\%}
\alias{\%nin\%}
\alias{nin}
\title{Not in}
\usage{
x \%nin\% table
}
\arguments{
\item{x}{vector or \code{NULL}: the values to be matched.}

\item{table}{vector or \code{NULL}: the values to be matched against.}
}
\value{
The negation of \code{\link[base:match]{\%in\%}}.
}
\description{
The not-in operator for R.
}
\examples{
1 \%nin\% 2:10
c("a", "b") \%nin\% c("a", "c", "d")
}
\seealso{
\code{\link[base:match]{\%in\%}}
}
\author{
Raymond Moore
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.data.frame.tableby.R
\name{as.data.frame.tableby}
\alias{as.data.frame.tableby}
\title{as.data.frame.tableby}
\usage{
\method{as.data.frame}{tableby}(x, ..., labelTranslations = NULL, list.ok = FALSE)
}
\arguments{
\item{x}{A \code{\link{tableby}} object.}

\item{...}{Arguments to pass to \code{\link{tableby.control}}.}

\item{labelTranslations}{A named list (or vector) where the name is the label in the
output to be replaced in the pretty rendering by the character string
value for the named element of the list, e.g., \code{list(age = "Age(Years)", meansd = "Mean(SD)")}.}

\item{list.ok}{If the object has multiple by-variables, is it okay to return a list of data.frames instead of a single data.frame?
If \code{FALSE} but there are multiple by-variables, a warning is issued.}
}
\value{
A \code{data.frame}.
}
\description{
Coerce a \code{\link{tableby}} object to a \code{data.frame}.
}
\seealso{
\code{\link{tableby}}, \code{\link{tableby}}
}
\author{
Ethan Heinzen, based on code originally by Greg Dougherty
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/na.operations.R
\name{NA.operations}
\alias{NA.operations}
\alias{allNA}
\alias{includeNA}
\alias{includeNA.factor}
\alias{includeNA.default}
\title{Some functions to handle NAs}
\usage{
allNA(x)

includeNA(x, label, ...)

\method{includeNA}{factor}(x, label = "(Missing)", first = FALSE, ...)

\method{includeNA}{default}(x, label = "(Missing)", ...)
}
\arguments{
\item{x}{An object}

\item{label}{A character string denoting the label to set \code{NA}s to.}

\item{...}{Other arguments (not in use at this time).}

\item{first}{Logical; should the new label be the first level?}
}
\description{
\code{allNA} tests if all elements are NA, and \code{includeNA} sets the
  \code{NA}s in a character vector or factor to an explicit label.
}
\seealso{
\code{\link{is.na}}, \code{\link{anyNA}}
}
\author{
Ethan Heinzen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internal.functions.R
\name{internal.functions}
\alias{internal.functions}
\alias{smart.split}
\alias{replace2}
\title{Internal Functions}
\usage{
smart.split(string, width = Inf, min.split = -Inf)

replace2(x, list, values)
}
\arguments{
\item{string}{A character vector}

\item{width}{Either \code{Inf} or \code{NULL} to specify no splitting,
or a positive integer giving the largest allowed string length.}

\item{min.split}{Either \code{-Inf} or \code{NULL} to specify no
lower bound on the string length, or a positive integer giving the minimum string length.}

\item{x}{a vector.}

\item{list}{an index vector.}

\item{values}{replacement values.}
}
\value{
For \code{smart.split}, a list of the same length as \code{string}, with each element being
  the "intelligently" split string.

  For \code{replace2}, a vector with the proper values replaced.
}
\description{
Internal Functions
}
\seealso{
\code{\link[base]{replace}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/paired.control.R
\name{paired.control}
\alias{paired.control}
\title{Control settings for \code{paired} function}
\usage{
paired.control(
  diff = TRUE,
  numeric.test = "paired.t",
  cat.test = "mcnemar",
  ordered.test = "signed.rank",
  date.test = "paired.t",
  mcnemar.correct = TRUE,
  signed.rank.exact = NULL,
  signed.rank.correct = TRUE,
  ...
)
}
\arguments{
\item{diff}{logical, telling \code{paired} whether to calculate a column of differences between time points.}

\item{numeric.test}{name of test for numeric RHS variables in \code{paired}: paired.t, signed.rank, signtest.}

\item{cat.test}{name of test for categorical variables: mcnemar}

\item{ordered.test}{name of test for ordered variables: signed.rank, signtest}

\item{date.test}{name of test to perform for date variables: paired.t, signed.rank, signtest}

\item{mcnemar.correct, signed.rank.exact, signed.rank.correct}{Options for statistical tests. See \code{\link{wilcox.test}}
and \code{\link{mcnemar.test}} for details.}

\item{...}{Arguments passed to \code{\link{tableby.control}}}
}
\value{
A list with settings to be used within the \code{\link{paired}} function.
}
\description{
Control test and summary settings for the \code{\link{paired}} function.
}
\details{
Note that (with the exception of \code{total}) all arguments to \code{\link{tableby.control}} are accepted in
  this function (in fact, this function passes everything through to \code{\link{tableby.control}}).
  However, there are different defaults for the statistical tests (shown here). For details on the other arguments,
  please see the help page for \code{\link{tableby.control}}.
}
\seealso{
\code{\link{paired}}, \code{\link{tableby}}, \code{\link{tableby.control}}, \code{\link{summary.tableby}}
}
\author{
Ethan Heinzen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modelsum.internal.R
\name{modelsum.internal}
\alias{modelsum.internal}
\alias{is.modelsum}
\alias{is.summary.modelsum}
\alias{na.modelsum}
\title{Helper functions for modelsum}
\usage{
is.modelsum(x)

is.summary.modelsum(x)

na.modelsum(object, ...)
}
\arguments{
\item{x}{A \code{modelsum} object.}

\item{object}{A \code{data.frame} resulting from evaluating a \code{modelsum} formula.}

\item{...}{Other arguments, or a vector of indices for extracting.}
}
\value{
\code{na.modelsum} returns a subsetted version of \code{object} (with attributes).
}
\description{
A set of helper functions for \code{\link{modelsum}}.
}
\seealso{
\code{\link{arsenal_table}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/freqlist.R
\name{freqlist}
\alias{freqlist}
\alias{freqlist.table}
\alias{freqlist.formula}
\title{freqlist}
\usage{
freqlist(object, ...)

\method{freqlist}{table}(
  object,
  na.options = c("include", "showexclude", "remove"),
  strata = NULL,
  labelTranslations = NULL,
  control = NULL,
  ...
)

\method{freqlist}{formula}(
  formula,
  data,
  subset,
  na.action,
  na.options = c("include", "showexclude", "remove"),
  strata = NULL,
  labelTranslations = NULL,
  control = NULL,
  addNA,
  exclude,
  drop.unused.levels,
  ...
)
}
\arguments{
\item{object}{An R object, usually of class \code{"table"} or class \code{"xtabs"}}

\item{...}{additional arguments. In the formula method, these are passed to the table method. These are also passed to
\code{\link{freq.control}}}

\item{na.options}{a character string indicating how to handling missing values: \code{"include"}
(include values with NAs in counts and percentages),
\code{"showexclude"} (show NAs but exclude from cumulative counts and all percentages),
\code{"remove"} (remove values with NAs); default is \code{"include"}.}

\item{strata}{(formerly \code{groupBy}) an optional character string specifying a variable(s) to use for grouping when calculating cumulative
counts and percentages. \code{\link{summary.freqlist}} will also separate by grouping variable for printing. Note that this is different
from \code{modelsum} and \code{tableby}, which take bare column names (and only one, at that!)}

\item{labelTranslations}{an optional character string (or list) of labels to use for variable levels when summarizing.
Names will be matched appropriately.}

\item{control}{control parameters to handle optional settings within \code{freqlist}. See \code{\link{freq.control}}}

\item{formula, data, subset, na.action, addNA, exclude, drop.unused.levels}{Arguments passed to \code{\link[stats]{xtabs}}.}
}
\value{
An object of class \code{c("freqlist", "arsenal_table")}
}
\description{
Approximate the output from SAS's \code{PROC FREQ} procedure when using the \code{/list} option of the \code{TABLE} statement.
}
\examples{
# load mockstudy data
data(mockstudy)
tab.ex <- table(mockstudy[c("arm", "sex", "mdquality.s")], useNA = "ifany")
noby <- freqlist(tab.ex, na.options = "include")
summary(noby)

# show the top 6 rows' frequencies and percents
head(summary(sort(noby, decreasing = TRUE)[c(1:4, 6)]))

withby <- freqlist(tab.ex, strata = c("arm","sex"), na.options = "showexclude")
summary(withby)
}
\seealso{
\code{\link{arsenal_table}}, \code{\link{summary.freqlist}}, \code{\link{freq.control}}, \code{\link{freqlist.internal}},
  \code{\link[base]{table}}, \code{\link[stats]{xtabs}}
}
\author{
Tina Gunderson, with revisions by Ethan Heinzen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.data.frame.freqlist.R
\name{as.data.frame.freqlist}
\alias{as.data.frame.freqlist}
\title{as.data.frame.freqlist}
\usage{
\method{as.data.frame}{freqlist}(x, ..., labelTranslations = NULL, list.ok = FALSE)
}
\arguments{
\item{x}{An object of class \code{"freqlist"}.}

\item{...}{Arguments to pass to \code{\link{freq.control}}}

\item{labelTranslations}{A named list (or vector) where the name is the label in the
output to be replaced in the pretty rendering by the character string
value for the named element of the list, e.g., \code{list(age = "Age(Years)", meansd = "Mean(SD)")}.}

\item{list.ok}{If the object has multiple by-variables, is it okay to return a list of data.frames instead of a single data.frame?
If \code{FALSE} but there are multiple by-variables, a warning is issued.}
}
\value{
A data.frame corresponding to the \code{freqlist} object.
}
\description{
Convert \code{\link{freqlist}} object to a data.frame.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modelsum.R
\name{modelsum}
\alias{modelsum}
\title{Fit models over each of a set of independent variables with a response variable}
\usage{
modelsum(
  formula,
  family = "gaussian",
  data,
  adjust = NULL,
  na.action = NULL,
  subset = NULL,
  weights = NULL,
  id,
  strata,
  control = NULL,
  ...
)
}
\arguments{
\item{formula}{an object of class \code{\link{formula}}; a symbolic description of the variables to be modeled.  See "Details" for more information.}

\item{family}{similar mechanism to \code{\link[stats]{glm}}, where the model to be fit is driven by the family.
Options include: binomial, gaussian, survival, poisson, negbin, clog, and ordinal. These can be passed as a string, as a function,
or as a list resulting from a call to one of the functions. See \code{\link{modelsum.family}} for details on
survival, ordinal, negbin, and clog families.}

\item{data}{an optional data.frame, list or environment (or object coercible by \code{\link[base]{as.data.frame}} to a data frame) containing the
variables in the model. If not found in \code{data}, the variables are taken from \code{environment(formula)}, typically
the environment from which \code{modelsum} is called.}

\item{adjust}{an object of class \code{\link{formula}} or a list of formulas, listing variables to adjust by in all models.
Specify as a one-sided formula, like: \code{~Age+ Sex}. If a list, the names are used for the summary function. Unadjusted models
can be specified as \code{~ 1} or as a list: \code{list(Unadjusted = NULL)}.}

\item{na.action}{a function which indicates what should happen when the data contain \code{NA}s.
The default (\code{NULL}) is to use the defaults of \code{\link[stats]{lm}}, \code{\link[stats]{glm}}, or \code{\link[survival]{coxph}},
depending on the \code{family} specifications.}

\item{subset}{an optional vector specifying a subset of observations (rows of \code{data}) to be used in the results.
If \code{strata} is missing, this works as vector of logicals or an index; otherwise, it should be a logical vector.}

\item{weights}{an optional vector specifying the weights to apply to each data observation (rows of \code{data})}

\item{id}{A vector to identify clusters. Only used for \code{\link{relrisk}} at this time.}

\item{strata}{a vector of strata to separate model summaries by an additional group. Note that for families like "clog",
the "usual" strata term to indicate subject groupings should be given in the \code{adjust} argument.}

\item{control}{control parameters to handle optional settings within \code{modelsum}.  Arguments for \code{modelsum.control}
can be passed to \code{modelsum} via the \code{...} argument, but if a control object and \code{...} arguments are both supplied,
the latter are used. See \code{\link{modelsum.control}} for other details.}

\item{...}{additional arguments to be passed to internal \code{modelsum} functions.}
}
\value{
An object with class \code{c("modelsum", "arsenal_table")}
}
\description{
Fit and summarize models for each independent (x) variable with a response variable (y), with options to adjust by variables for each model.
}
\examples{

data(mockstudy)

tab1 <- modelsum(bmi ~ sex + age, data = mockstudy)
summary(tab1, text = TRUE)

tab2 <- modelsum(alk.phos ~ arm + ps + hgb, adjust = ~ age + sex,
                 family = "gaussian", data = mockstudy)
summary(tab2, text = TRUE)

summary(tab2, show.intercept = FALSE, text = TRUE)

tab2.df <- as.data.frame(tab2)

tab2.df[1:5,]
}
\seealso{
\code{\link{arsenal_table}}, \code{\link{modelsum.control}}, \code{\link{summary.modelsum}},
  \code{\link{modelsum.internal}}, \code{\link{formulize}}
}
\author{
Jason Sinnwell, Patrick Votruba, Beth Atkinson, Gregory Dougherty, and Ethan Heinzen, adapted from SAS Macro of the same name
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.freqlist.R
\name{summary.freqlist}
\alias{summary.freqlist}
\alias{as.data.frame.summary.freqlist}
\title{summary.freqlist}
\usage{
\method{summary}{freqlist}(object, ..., labelTranslations = NULL, title = NULL)

\method{as.data.frame}{summary.freqlist}(x, ..., list.ok = FALSE)
}
\arguments{
\item{object}{an object of class \code{\link{freqlist}}}

\item{...}{For \code{summary.freqlist}, these are passed to \code{\link{as.data.frame.freqlist}} (and hence to
\code{\link{freq.control}}). For the print method, these are
additional arguments passed to the \code{\link[knitr]{kable}} function.}

\item{labelTranslations}{A named list (or vector) where the name is the label in the
output to be replaced in the pretty rendering by the character string
value for the named element of the list, e.g., \code{list(age = "Age(Years)", meansd = "Mean(SD)")}.}

\item{title}{Title/caption for the table, defaulting to \code{NULL} (no title). Passed to \code{\link[knitr]{kable}}.
Can be length > 1 if the more than one table is being printed.}

\item{x}{An object of class \code{summary.freqlist}.}

\item{list.ok}{If the object has multiple by-variables, is it okay to return a list of data.frames instead of a single data.frame?
If \code{FALSE} but there are multiple by-variables, a warning is issued.}
}
\value{
An object of class \code{"summary.freqlist"} (invisibly for the print method).
}
\description{
Summarize the \code{freqlist} object.
}
\examples{
# load mockstudy data
data(mockstudy)
tab.ex <- table(mockstudy[c("arm", "sex", "mdquality.s")], useNA = "ifany")
noby <- freqlist(tab.ex, na.options = "include")
summary(noby)
withby <- freqlist(tab.ex, strata = c("arm","sex"), na.options = "showexclude")
summary(withby)
summary(withby, dupLabels = TRUE)
}
\seealso{
\code{\link{freqlist}}, \code{\link[base]{table}}, \code{\link[stats]{xtabs}}, \code{\link[knitr]{kable}},
  \code{\link{freq.control}}, \code{\link{freqlist.internal}}
}
\author{
Tina Gunderson, with major revisions by Ethan Heinzen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/labels.R
\name{labels}
\alias{labels}
\alias{labels.data.frame}
\alias{labels.keep_labels}
\alias{labels<-}
\alias{labels<-.keep_labels}
\alias{labels<-.default}
\alias{labels<-.data.frame}
\alias{set_labels}
\alias{set_attr}
\title{Labels}
\usage{
\method{labels}{data.frame}(object, ...)

\method{labels}{keep_labels}(object, ...)

labels(x) <- value

\method{labels}{keep_labels}(x) <- value

\method{labels}{default}(x) <- value

\method{labels}{data.frame}(x) <- value

set_labels(x, value)

set_attr(x, which, value)
}
\arguments{
\item{...}{Other arguments (not in use at this time).}

\item{x, object}{An R object.}

\item{value}{A vector or list containing labels to assign. Labels are assigned based on
names, if available; otherwise, they're assigned in order. Can pass \code{NULL}
to remove all labels.}

\item{which}{See \code{\link{attr<-}}}
}
\value{
The labels of \code{object}, or \code{object} with new labels.
}
\description{
Assign and extract the \code{'label'} attribute on an R object. \code{set_labels} is
  the same as \code{labels(x) <- value} but returns \code{x} for use in a pipe chain.
  \code{set_attr} is the same as \code{attr(x, which) <- value} but returns \code{x}
  for use in a pipe chain.
}
\details{
The \code{\link{data.frame}} methods put labels on and extract labels from
  the \emph{columns} of \code{object}.
}
\seealso{
\code{\link{keep.labels}}
}
\author{
Ethan Heinzen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.modelsum.R
\name{summary.modelsum}
\alias{summary.modelsum}
\alias{as.data.frame.summary.modelsum}
\title{Summarize a \code{modelsum} object.}
\usage{
\method{summary}{modelsum}(
  object,
  ...,
  labelTranslations = NULL,
  text = FALSE,
  title = NULL,
  term.name = "",
  adjustment.names = FALSE
)

\method{as.data.frame}{summary.modelsum}(
  x,
  ...,
  text = x$text,
  term.name = x$term.name,
  adjustment.names = x$adjustment.names,
  width = NULL,
  min.split = NULL,
  list.ok = FALSE
)
}
\arguments{
\item{object}{A \code{\link{modelsum}} object.}

\item{...}{For \code{summary.modelsum}, other arguments passed to \code{\link{as.data.frame.modelsum}}.
For \code{as.data.frame.summary.modelsum}, "width" and "min.split" are passed to \code{\link{smart.split}}.
For \code{print}ing the summary object, these are passed to both \code{as.data.frame.summary.modelsum} and
\code{\link[knitr]{kable}}.}

\item{labelTranslations}{A named list (or vector) where the name is the label in the
output to be replaced in the pretty rendering by the character string
value for the named element of the list, e.g., \code{list(age = "Age(Years)", meansd = "Mean(SD)")}.}

\item{text}{An argument denoting how to print the summary to the screen.
Default is \code{FALSE} (show markdown output). \code{TRUE} and \code{NULL} output a text-only version, with
the latter avoiding all formatting.
\code{"html"} uses the HTML tag \code{<strong>} instead of the markdown formatting, and \code{"latex"} uses
the LaTeX command \code{\\textbf}.}

\item{title}{Title/caption for the table, defaulting to \code{NULL} (no title). Passed to \code{\link[knitr]{kable}}.
Can be length > 1 if the more than one table is being printed.}

\item{term.name}{A character vector denoting the column name for the "terms" column. It should be the same length
as the number of tables or less (it will get recycled if needed). The special value \code{TRUE} will
use the y-variable's label for each table.}

\item{adjustment.names}{Logical, denoting whether the names of the adjustment models should be printed.}

\item{x}{An object of class \code{"summary.modelsum"}.}

\item{width, min.split}{Passed to \code{\link{smart.split}} for formatting of the "term" column.}

\item{list.ok}{If the object has multiple by-variables, is it okay to return a list of data.frames instead of a single data.frame?
If \code{FALSE} but there are multiple by-variables, a warning is issued.}
}
\value{
An object of class \code{"summary.modelsum"}
}
\description{
Format the information in \code{object} as a table using Pandoc coding or plain text, and cat it to stdout.
}
\seealso{
\code{\link{modelsum}}, \code{\link{as.data.frame.modelsum}}
}
\author{
Ethan Heinzen, based on code originally by Greg Dougherty
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tableby.control.R
\name{tableby.control}
\alias{tableby.control}
\title{Control settings for \code{tableby} function}
\usage{
tableby.control(
  test = TRUE,
  total = TRUE,
  total.pos = c("after", "before"),
  test.pname = NULL,
  numeric.simplify = FALSE,
  cat.simplify = FALSE,
  cat.droplevels = FALSE,
  ordered.simplify = FALSE,
  date.simplify = FALSE,
  numeric.test = "anova",
  cat.test = "chisq",
  ordered.test = "trend",
  surv.test = "logrank",
  date.test = "kwt",
  selectall.test = "notest",
  test.always = FALSE,
  numeric.stats = c("Nmiss", "meansd", "range"),
  cat.stats = c("Nmiss", "countpct"),
  ordered.stats = c("Nmiss", "countpct"),
  surv.stats = c("Nmiss", "Nevents", "medSurv"),
  date.stats = c("Nmiss", "median", "range"),
  selectall.stats = c("Nmiss", "countpct"),
  stats.labels = list(),
  digits = 3L,
  digits.count = 0L,
  digits.pct = 1L,
  digits.p = 3L,
  format.p = TRUE,
  digits.n = 0L,
  conf.level = 0.95,
  wilcox.correct = FALSE,
  wilcox.exact = NULL,
  chisq.correct = FALSE,
  simulate.p.value = FALSE,
  B = 2000,
  times = 1:5,
  ...
)
}
\arguments{
\item{test}{logical, telling \code{tableby} whether to perform tests of x variables across levels of the group variable.}

\item{total}{logical, telling \code{tableby} whether to calculate a column of totals across group variable.}

\item{total.pos}{One of \code{"before"} or \code{"after"}, denoting where to put the total column relative to the by-variable columns.}

\item{test.pname}{character string denoting the p-value column name in \code{\link{summary.tableby}}.
Modifiable also with \code{\link{modpval.tableby}}.}

\item{numeric.simplify, date.simplify}{logical, tell \code{tableby} whether to condense numeric/date output to a single line.
NOTE: this only simplifies to one line if there is only one statistic reported, such as \code{meansd}.
In particular, if \code{Nmiss} is specified and there are missings, then the output is not simplified.}

\item{cat.simplify, ordered.simplify}{logical, tell \code{tableby} whether to remove the first level of the categorical/ordinal variable if binary.
If \code{TRUE}, only the summary stats of the second level are reported (unless there's only one level, in which case it's reported).
If \code{"label"}, the second level's label is additionally appended to the label.
NOTE: this only simplifies to one line if there is only one statistic reported, such as \code{countpct}.
In particular, if \code{Nmiss} is specified and there are missings, then the output is not simplified.}

\item{cat.droplevels}{Should levels be dropped for categorical variables? If set to true, p-values will not be displayed
unless \code{test.always = TRUE} as well.}

\item{numeric.test}{name of test for numeric RHS variables in \code{tableby}: anova, kwt (Kruskal-Wallis), medtest (median test).
If no LHS variable exists, then a mean is required for a univariate test.}

\item{cat.test}{name of test for categorical variables: chisq, fe (Fisher's Exact)}

\item{ordered.test}{name of test for ordered variables: trend}

\item{surv.test}{name of test for survival variables: logrank}

\item{date.test}{name of test for date variables: kwt}

\item{selectall.test}{name of test for date variables: notest}

\item{test.always}{Should the test be performed even if one or more by-group has 0 observations? Relevant
for kwt and anova.}

\item{numeric.stats, cat.stats, ordered.stats, surv.stats, date.stats, selectall.stats}{summary statistics to include for the respective class of RHS variables
within the levels of the group LHS variable.}

\item{stats.labels}{A named list of labels for all the statistics function names, where the function name is the named element in the list
and the value that goes with it is a string containing the formal name that will be printed in all printed renderings of the output,
e.g., \code{list(countpct="Count (Pct)")}. Any unnamed elements will be ignored. Passing \code{NULL} will disable labels.}

\item{digits}{Number of decimal places for numeric values.}

\item{digits.count}{Number of decimal places for count values.}

\item{digits.pct}{Number of decimal places for percents.}

\item{digits.p}{Number of decimal places for p-values.}

\item{format.p}{Logical, denoting whether to format p-values, or character, a \code{\link[glue]{glue}} specification for how to format. See "Details", below.}

\item{digits.n}{Number of decimal places for N's in the header. Set it to NA to suppress the N's.}

\item{conf.level}{Numeric, denoting what confidence level to use for confidence intervals.
(See, e.g., \code{\link{binomCI}})}

\item{wilcox.correct, wilcox.exact}{See \code{\link[stats]{wilcox.test}}}

\item{chisq.correct}{logical, correction factor for chisq.test}

\item{simulate.p.value}{logical, simulate p-value for categorical tests (fe and chisq)}

\item{B}{number of simulations to perform for simulation-based p-value}

\item{times}{A vector of times to use for survival summaries.}

\item{...}{additional arguments.}
}
\value{
A list with settings to be used within the \code{tableby} function.
}
\description{
Control test and summary settings for the \code{\link{tableby}} function.
}
\details{
All tests can be turned off by setting \code{test} to FALSE.
  Otherwise, test are set to default settings in this list, or set explicitly in the formula of \code{tableby}.

If \code{format.p} is \code{FALSE}, \code{digits.p} denotes the number of significant digits shown. The
  p-values will be in exponential notation if necessary. If \code{format.p} is \code{TRUE},
  \code{digits.p} will determine the number of digits after the decimal point to show. If the p-value
  is less than the resulting number of places, it will be formatted to show so. If \code{format.p} is a character string,
  it will be treated as a \code{\link[glue]{glue}} specification: the p-value is exposed as "p", and "digits.p" as "digits.p".

Options for statistics are described more thoroughly in the vignette and are listed in \link{tableby.stats}
}
\examples{
set.seed(100)
## make 3+ categories for Response
mdat <- data.frame(Response=c(0,0,0,0,0,1,1,1,1,1),
                   Sex=sample(c("Male", "Female"), 10,replace=TRUE),
                   Age=round(rnorm(10,mean=40, sd=5)),
                   HtIn=round(rnorm(10,mean=65,sd=5)))

## allow default summaries in RHS variables, and pass control args to
## main function, to be picked up with ... when calling tableby.control
outResp <- tableby(Response ~ Sex + Age + HtIn, data=mdat, total=FALSE, test=TRUE)
outCtl <- tableby(Response ~ Sex + Age + HtIn, data=mdat,
                  control=tableby.control(total=TRUE, cat.simplify=TRUE,
                  cat.stats=c("Nmiss","countpct"),digits=1))
summary(outResp, text=TRUE)
summary(outCtl, text=TRUE)
}
\seealso{
\code{\link[stats]{anova}}, \code{\link[stats]{chisq.test}}, \code{\link{tableby}}, \code{\link{summary.tableby}},
  \code{\link{tableby.stats}}.
}
\author{
Jason Sinnwell, Beth Atkinson, Ethan Heinzen, Terry Therneau, adapted from SAS Macros written by Paul Novotny and Ryan Lennon
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comparedf.control.R
\name{comparedf.control}
\alias{comparedf.control}
\title{Control settings for \code{comparedf} function}
\usage{
comparedf.control(
  tol.logical = "none",
  tol.num = c("absolute", "percent", "pct"),
  tol.num.val = sqrt(.Machine$double.eps),
  int.as.num = FALSE,
  tol.char = c("none", "trim", "case", "both"),
  tol.factor = c("none", "levels", "labels"),
  factor.as.char = FALSE,
  tol.date = "absolute",
  tol.date.val = 0,
  tol.other = "none",
  tol.vars = "none",
  max.print.vars = NA,
  max.print.obs = NA,
  max.print.diffs.per.var = 10,
  max.print.diffs = 50,
  max.print.attrs = NA,
  ...,
  max.print.diff = 10
)
}
\arguments{
\item{tol.logical, tol.num, tol.char, tol.factor, tol.date, tol.other}{A function or one of the shortcut character strings or a list thereof,
denoting the tolerance function to use for a given data type. See "details", below.}

\item{tol.num.val}{Numeric; maximum value of differences allowed in numerics (fed to the function given in \code{tol.num}).}

\item{int.as.num}{Logical; should integers be coerced to numeric before comparison? Default FALSE.}

\item{factor.as.char}{Logical; should factors be coerced to character before comparison? Default FALSE.}

\item{tol.date.val}{Numeric; maximum value of differences allowed in dates (fed to the function given in \code{tol.date}).}

\item{tol.vars}{Either \code{"none"} (the default), denoting that variable names are to be matched as-is,
a named vector manually specifying variable names to compare (where the names correspond to columns of
\code{x} and the values correspond to columns of \code{y}), or a
character vector denoting equivalence classes for characters in the variable names. See "details", below.}

\item{max.print.vars}{Integer denoting maximum number of variables to report in the "variables not shared" and "variables not compared"
output. \code{NA} will print all differences.}

\item{max.print.obs}{Integer denoting maximum number of not-shared observations to report. \code{NA} will print all differences.}

\item{max.print.diffs.per.var, max.print.diffs}{Integers denoting the maximum number of differences to report for each variable or overall.
\code{NA} will print all differences for each variable or overall.}

\item{max.print.attrs}{Integers denoting the maximum number of non-identical attributes to report.\code{NA} will print all differences.}

\item{...}{Other arguments (not in use at this time).}

\item{max.print.diff}{Deprecated.}
}
\value{
A list containing the necessary parameters for the \code{\link{comparedf}} function.
}
\description{
Control tolerance definitions for the \code{\link{comparedf}} function.
}
\details{
The following character strings are accepted:
\itemize{
  \item{\code{tol.logical = "none"}: compare logicals exactly as they are.}
  \item{\code{tol.num = "absolute"}: compare absolute differences in numerics.}
  \item{\code{tol.num = "percent"}, \code{tol.num = "pct"} compare percent differences in numerics.}
  \item{\code{tol.char = "none"}: compare character strings exactly as they are.}
  \item{\code{tol.char = "trim"}: left-justify and trim all trailing white space.}
  \item{\code{tol.char = "case"}: allow differences in upper/lower case.}
  \item{\code{tol.char = "both"}: combine \code{"trim"} and \code{"case"}.}
  \item{\code{tol.factor = "none"}: match both character labels and numeric levels.}
  \item{\code{tol.factor = "levels"}: match only the numeric levels.}
  \item{\code{tol.factor = "labels"}: match only the labels.}
  \item{\code{tol.date = "absolute"}: compare absolute differences in dates.}
  \item{\code{tol.other = "none"}: expect objects of other classes to be exactly identical.}
}

A list with names mapped to \code{x} can be used to specify tolerances by variable. One unnamed element is supported
as the default.

\code{tol.vars}: If not set to \code{"none"} (the default) or a named vector,
  the \code{tol.vars} argument is a character vector denoting equivalence classes
  for the characters in the variable names. A single character in this vector means to replace that character
  with \code{""}. All other strings in this vector are split by character and replaced by the first character in the string.

E.g., a character vector \code{c("._", "aA", " ")} would denote that the dot and underscore are equivalent (to be translated to a dot),
  that "a" and "A" are equivalent (to be translated to "a"), and that spaces should be removed.

The special character string \code{"case"} in this vector is the same as specifying \code{paste0(letters, LETTERS)}.
}
\examples{
cntl <- comparedf.control(
  tol.num = "pct",     # calculate percent differences
  tol.vars = c("case", # ignore case
               "._",   # set all underscores to dots.
               "e")    # remove all letter e's
)

cntl <- comparedf.control(tol.char = list(
  "none",      # the default
  x1 = "case", # be case-insensitive for the variable "x1"
  x2 = function(x, y) tol.NA(x, y, x != y | y == "NA") # a custom-defined tolerance
))
}
\seealso{
\code{\link{comparedf}}, \code{\link{comparedf.tolerances}}, \code{\link{summary.comparedf}}
}
\author{
Ethan Heinzen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tableby.R
\name{tableby}
\alias{tableby}
\title{Summary Statistics of a Set of Independent Variables by a Categorical Variable}
\usage{
tableby(
  formula,
  data,
  na.action,
  subset = NULL,
  weights = NULL,
  strata,
  control = NULL,
  ...
)
}
\arguments{
\item{formula}{an object of class \code{\link{formula}}; a symbolic description of the variables to be summarized by the group,
or categorical variable, of interest. See "Details" for more information. To only view overall summary
statistics, a one-sided formula can be used.}

\item{data}{an optional data frame, list or environment (or object coercible by \code{\link{as.data.frame}} to a data frame)
containing the variables in the model. If not found in data, the variables are taken from \code{environment(formula)},
typically the environment from which the function is called.}

\item{na.action}{a function which indicates what should happen when the data contain \code{NA}s.
The default is \code{na.tableby(TRUE)} if there is a by-variable, and \code{na.tableby(FALSE)} if there is not.
This schema thus includes observations with \code{NA}s in x variables,
but removes those with \code{NA} in the categorical group variable and strata (if used).}

\item{subset}{an optional vector specifying a subset of observations (rows of data) to be used in the results.
Works as vector of logicals or an index.}

\item{weights}{a vector of weights. Using weights will disable statistical tests.}

\item{strata}{a vector of strata to separate summaries by an additional group.}

\item{control}{control parameters to handle optional settings within \code{tableby}.
Two aspects of \code{tableby} are controlled with these: test options of RHS variables across levels of the categorical
grouping variable, and x variable summaries within the grouping variable. Arguments for \code{tableby.control}
can be passed to \code{tableby} via the \code{...} argument, but if a control object and \code{...} arguments are both supplied,
the latter are used. See \code{\link{tableby.control}} for more details.}

\item{...}{additional arguments to be passed to internal \code{tableby} functions or \code{\link{tableby.control}}.}
}
\value{
An object with class \code{c("tableby", "arsenal_table")}
}
\description{
Summarize one or more variables (x) by a categorical variable (y). Variables
  on the right side of the formula, i.e. independent variables, are summarized by the
  levels of a categorical variable on the left of the formula. Optionally, an appropriate test is performed to test the
  distribution of the independent variables across the levels of the categorical variable.
}
\details{
The group variable (if any) is categorical, which could be an integer, character,
factor, or ordered factor. \code{tableby} makes a simple summary of
the counts within the k-levels of the independent variables on the
right side of the formula. Note that unused levels are dropped.

The \code{data} argument allows data.frames with label attributes for the columns, and those
labels will be used in the summary methods for the \code{tableby} class.

The independent variables are a mixture of types: categorical (discrete),
numeric (continuous), and time to event (survival). These variables
are split by the levels of the group variable (if any), then summarized within
those levels, specific to the variable type. A statistical test is
performed to compare the distribution of the independent variables across the
levels of the grouping variable.

The tests differ by the independent variable type, but can be specified
explicitly in the formula statement or in the control function.
These tests are accepted:
\itemize{
  \item{
    \code{anova}: analysis of variance test; the default test for continuous variables. When
    LHS variable has two levels, equivalent to two-sample t-test.
  }
  \item{
    \code{kwt}: Kruskal-Wallis Rank Test, optional test for continuous
    variables. When LHS variable has two levels, equivalent to Wilcoxon test.
  }
  \item{
    \code{wt}: An explicit Wilcoxon test.
  }
  \item{
    \code{medtest}: A median test.
  }
  \item{
    \code{chisq}: chi-square goodness of fit test for equal counts of a
    categorical variable across categories; the default for categorical
    or factor variables
  }
  \item{
    \code{fe}: Fisher's exact test for categorical variables
  }
  \item{
    \code{trend}: trend test for equal distribution of an ordered variable
    across a categorical variable; the default for ordered factor variables
  }
  \item{
    \code{logrank}: log-rank, the default for time-to-event variables
  }
  \item{
    \code{notest}: no test is performed.
  }
}

To perform a mixture of asymptotic and rank-based tests on two
different continuous variables, an example formula is:
\code{formula = group ~ anova(age) + kwt(height)}. The test settings
in \code{tableby.control} apply to all independent variables of a given type.

The summary statistics reported for each independent variable within the
group variable can be set in \code{\link{tableby.control}}.

Finally, multiple by-variables can be set using \code{list()}. See the examples for more details.
}
\examples{
data(mockstudy)
tab1 <- tableby(arm ~ sex + age, data=mockstudy)
summary(tab1, text=TRUE)

mylabels <- list(sex = "SEX", age ="Age, yrs")
summary(tab1, labelTranslations = mylabels, text=TRUE)

tab3 <- tableby(arm ~ sex + age, data=mockstudy, test=FALSE, total=FALSE,
                numeric.stats=c("median","q1q3"), numeric.test="kwt")
summary(tab3, text=TRUE)

# multiple LHS
summary(tableby(list(arm, sex) ~ age, data = mockstudy, strata = ps), text = TRUE)

tab.test <- tableby(arm ~ kwt(age) + anova(bmi) + kwt(ast), data=mockstudy)
tests(tab.test)

}
\seealso{
\code{\link{arsenal_table}}, \code{\link[stats]{anova}}, \code{\link[stats]{chisq.test}}, \code{\link{tableby.control}},
  \code{\link{summary.tableby}}, \code{\link{tableby.internal}}, \code{\link{formulize}}, \code{\link{selectall}}
}
\author{
Jason Sinnwell, Beth Atkinson, Gregory Dougherty, and Ethan Heinzen, adapted from SAS Macros written by Paul Novotny and Ryan Lennon
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/padjust.R
\name{padjust}
\alias{padjust}
\alias{padjust.default}
\alias{padjust.tableby}
\alias{padjust.summary.tableby}
\title{Adjust P-values for Multiple Comparisons}
\usage{
padjust(p, method, n, ...)

\method{padjust}{default}(p, method, n, ...)

\method{padjust}{tableby}(p, method, n, suffix = " (adjusted for multiple comparisons)", ...)

\method{padjust}{summary.tableby}(p, method, n, suffix = " (adjusted for multiple comparisons)", ...)
}
\arguments{
\item{p}{An object.}

\item{method}{correction method, a \code{\link{character}} string.
    Can be abbreviated.}

\item{n}{number of comparisons, must be at least \code{length(p)};
    only set this (to non-default) when you know what you are doing!}

\item{...}{Other arguments.}

\item{suffix}{A suffix to add to the footnotes indicating that the tests were adjusted.}
}
\description{
Adjust P-values for Multiple Comparisons
}
\seealso{
\code{\link[stats]{p.adjust}}, \code{\link{modpval.tableby}}, \code{\link{tests.tableby}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modelsum.control.R
\name{modelsum.control}
\alias{modelsum.control}
\title{Control settings for \code{modelsum} function}
\usage{
modelsum.control(
  digits = 3L,
  digits.ratio = 3L,
  digits.p = 3L,
  format.p = TRUE,
  show.adjust = TRUE,
  show.intercept = TRUE,
  conf.level = 0.95,
  ordinal.stats = c("OR", "CI.lower.OR", "CI.upper.OR", "p.value", "Nmiss"),
  binomial.stats = c("OR", "CI.lower.OR", "CI.upper.OR", "p.value", "concordance",
    "Nmiss"),
  gaussian.stats = c("estimate", "std.error", "p.value", "adj.r.squared", "Nmiss"),
  poisson.stats = c("RR", "CI.lower.RR", "CI.upper.RR", "p.value", "Nmiss"),
  negbin.stats = c("RR", "CI.lower.RR", "CI.upper.RR", "p.value", "Nmiss"),
  relrisk.stats = c("RR", "CI.lower.RR", "CI.upper.RR", "p.value", "Nmiss"),
  clog.stats = c("OR", "CI.lower.OR", "CI.upper.OR", "p.value", "concordance", "Nmiss"),
  survival.stats = c("HR", "CI.lower.HR", "CI.upper.HR", "p.value", "concordance",
    "Nmiss"),
  stat.labels = list(),
  ...
)
}
\arguments{
\item{digits}{Numeric, denoting the number of digits after the decimal point for beta coefficients and standard errors.}

\item{digits.ratio}{Numeric, denoting the number of digits after the decimal point for ratios, e.g. OR, RR, HR.}

\item{digits.p}{Numeric, denoting the number of digits for p-values. See "Details", below.}

\item{format.p}{Logical, denoting whether to format p-values. See "Details", below.}

\item{show.adjust}{Logical, denoting whether to show adjustment terms.}

\item{show.intercept}{Logical, denoting whether to show intercept terms.}

\item{conf.level}{Numeric, giving the confidence level.}

\item{ordinal.stats, binomial.stats, survival.stats, gaussian.stats, poisson.stats, negbin.stats, clog.stats, relrisk.stats}{Character vectors denoting which stats to show for the various model types.}

\item{stat.labels}{A named list of labels for all the stats used above.}

\item{...}{Other arguments (not in use at this time).}
}
\value{
A list with settings to be used within the \code{modelsum} function.
}
\description{
Control test and summary settings for \code{\link{modelsum}} function.
}
\details{
If \code{format.p} is \code{FALSE}, \code{digits.p} denotes the number of significant digits shown. The
  p-values will be in exponential notation if necessary. If \code{format.p} is \code{TRUE},
  \code{digits.p} will determine the number of digits after the decimal point to show. If the p-value
  is less than the resulting number of places, it will be formatted to show so.
}
\seealso{
\code{\link{modelsum}}, \code{\link{summary.modelsum}}, \code{\link{modelsum.internal}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comparedf.R
\name{comparedf}
\alias{comparedf}
\alias{print.comparedf}
\title{Compare two data.frames and report differences}
\usage{
comparedf(x, y, by = NULL, by.x = by, by.y = by, control = NULL, ...)

\method{print}{comparedf}(x, ...)
}
\arguments{
\item{x, y}{A data.frame to compare}

\item{by, by.x, by.y}{Which variables are IDs to merge the two data.frames?
If set to \code{"row.names"}, merging will occur over the row.names.
If set to \code{NULL} (default), merging will occur row-by-row.}

\item{control}{A list of control parameters from \code{\link{comparedf.control}}.}

\item{...}{Other arguments, passed to \code{\link{comparedf.control}} when appropriate.}
}
\description{
Compare two data.frames and report any differences between them,
 much like SAS's \code{PROC COMPARE} procedure.
}
\examples{

df1 <- data.frame(id = paste0("person", 1:3), a = c("a", "b", "c"),
                  b = c(1, 3, 4), c = c("f", "e", "d"),
                  row.names = paste0("rn", 1:3), stringsAsFactors = FALSE)
df2 <- data.frame(id = paste0("person", 3:1), a = c("c", "b", "a"),
                  b = c(1, 3, 4), d = paste0("rn", 1:3),
                  row.names = paste0("rn", c(1,3,2)), stringsAsFactors = FALSE)
summary(comparedf(df1, df2))
summary(comparedf(df1, df2, by = "id"))
summary(comparedf(df1, df2, by = "row.names"))
}
\seealso{
\code{\link{summary.comparedf}}, \code{\link{comparedf.control}},
  \code{\link{diffs}}, \code{\link{n.diffs}}, \code{\link{n.diff.obs}}
}
\author{
Ethan Heinzen, adapted from code from Andrew Hanson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tableby.internal.R
\name{tableby.internal}
\alias{tableby.internal}
\alias{is.tableby}
\alias{is.summary.tableby}
\alias{modpval.tableby}
\alias{tests}
\alias{tests.tableby}
\alias{na.tableby}
\alias{xtfrm.tableby}
\alias{sort.tableby}
\alias{Ops.tableby}
\alias{head.tableby}
\alias{tail.tableby}
\title{Helper functions for tableby}
\usage{
is.tableby(x)

is.summary.tableby(x)

modpval.tableby(x, pdata, use.pname = FALSE)

tests(x)

\method{tests}{tableby}(x)

na.tableby(lhs = TRUE)

\method{xtfrm}{tableby}(x)

\method{sort}{tableby}(x, ...)

\method{Ops}{tableby}(e1, e2)

\method{head}{tableby}(x, n = 6L, ...)

\method{tail}{tableby}(x, n = 6L, ...)
}
\arguments{
\item{x}{A \code{tableby} object.}

\item{pdata}{A named data.frame where the first column is the by-variable names, the (optional) second is the strata value, the next is
the x variable names, the next is p-values (or some test stat), and the (optional) next column is the method name.}

\item{use.pname}{Logical, denoting whether the column name in \code{pdata} corresponding to the p-values should be used
in the output of the object.}

\item{lhs}{Logical, denoting whether to remove \code{NA}s from the first column of the data.frame (the "left-hand side")}

\item{...}{Other arguments.}

\item{e1, e2}{\code{\link{tableby}} objects, or numbers to compare them to.}

\item{n}{A single integer. See \code{\link[utils]{head}} or \code{\link[utils:head]{tail}} for more details}
}
\value{
\code{na.tableby} returns a subsetted version of \code{object} (with attributes). \code{Ops.tableby} returns
  a logical vector. \code{xtfrm.tableby} returns the p-values (which are ordered by \code{\link{order}} to \code{\link{sort}}).
}
\description{
A set of helper functions for \code{\link{tableby}}.
}
\details{
Logical comparisons are implemented for \code{Ops.tableby}.
}
\seealso{
\code{\link{arsenal_table}}, \code{\link{sort}}, \code{\link[utils]{head}}, \code{\link[utils:head]{tail}},
  \code{\link{tableby}}, \code{\link{summary.tableby}}, \code{\link{tableby.control}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comparedf.internal.R
\name{diffs}
\alias{diffs}
\alias{n.diff.obs}
\alias{n.diff.obs.comparedf}
\alias{n.diff.obs.summary.comparedf}
\alias{n.diffs}
\alias{n.diffs.comparedf}
\alias{n.diffs.summary.comparedf}
\alias{diffs.comparedf}
\alias{diffs.summary.comparedf}
\title{Extract differences}
\usage{
n.diff.obs(object, ...)

\method{n.diff.obs}{comparedf}(object, ...)

\method{n.diff.obs}{summary.comparedf}(object, ...)

n.diffs(object, ...)

\method{n.diffs}{comparedf}(object, ...)

\method{n.diffs}{summary.comparedf}(object, ...)

diffs(object, ...)

\method{diffs}{comparedf}(
  object,
  what = c("differences", "observations"),
  vars = NULL,
  ...,
  by.var = FALSE
)

\method{diffs}{summary.comparedf}(
  object,
  what = c("differences", "observations"),
  vars = NULL,
  ...,
  by.var = FALSE
)
}
\arguments{
\item{object}{An object of class \code{comparedf} or \code{summary.comparedf}.}

\item{...}{Other arguments (not in use at this time).}

\item{what}{Should differences or the not-shared observations be returned?}

\item{vars}{A character vector of variable names to subset the results to.}

\item{by.var}{Logical: should the number of differences by variable be reported, or should
all differences be reported (the default).}
}
\description{
Extract differences (\code{diffs()}), number of differences (\code{n.diffs()}),
or number of not-shared observations (\code{n.diff.obs()}) from a \code{comparedf} object.
}
\seealso{
\code{\link{comparedf}} \code{\link{summary.comparedf}}
}
\author{
Ethan Heinzen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.data.frame.modelsum.R
\name{as.data.frame.modelsum}
\alias{as.data.frame.modelsum}
\title{as.data.frame.modelsum}
\usage{
\method{as.data.frame}{modelsum}(x, ..., labelTranslations = NULL, list.ok = FALSE)
}
\arguments{
\item{x}{A \code{\link{modelsum}} object.}

\item{...}{Arguments to pass to \code{\link{modelsum.control}}.}

\item{labelTranslations}{A named list (or vector) where the name is the label in the
output to be replaced in the pretty rendering by the character string
value for the named element of the list, e.g., \code{list(age = "Age(Years)", meansd = "Mean(SD)")}.}

\item{list.ok}{If the object has multiple by-variables, is it okay to return a list of data.frames instead of a single data.frame?
If \code{FALSE} but there are multiple by-variables, a warning is issued.}
}
\value{
A \code{data.frame}.
}
\description{
Coerce a \code{\link{modelsum}} object to a \code{data.frame}.
}
\seealso{
\code{\link{modelsum}}, \code{\link{summary.modelsum}}
}
\author{
Ethan Heinzen, based on code originally by Greg Dougherty
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tableby.stats.R
\name{tableby.stats}
\alias{tableby.stats}
\alias{arsenal_sum}
\alias{arsenal_min}
\alias{arsenal_max}
\alias{arsenal_mean}
\alias{arsenal_sd}
\alias{arsenal_var}
\alias{meansd}
\alias{meanse}
\alias{meanpmsd}
\alias{meanpmse}
\alias{meanCI}
\alias{medianrange}
\alias{medianmad}
\alias{arsenal_median}
\alias{arsenal_range}
\alias{gmean}
\alias{gsd}
\alias{gmeansd}
\alias{gmeanCI}
\alias{Nsigntest}
\alias{Nevents}
\alias{medSurv}
\alias{medSurvCI}
\alias{medSurvQuant}
\alias{NeventsSurv}
\alias{NriskSurv}
\alias{Nrisk}
\alias{medTime}
\alias{q1q3}
\alias{medianq1q3}
\alias{iqr}
\alias{Nmiss}
\alias{Nmisspct}
\alias{Nmiss2}
\alias{Nmisspct2}
\alias{N}
\alias{Npct}
\alias{Nrowpct}
\alias{count}
\alias{countpct}
\alias{pct}
\alias{countN}
\alias{countrowpct}
\alias{rowpct}
\alias{countcellpct}
\alias{binomCI}
\alias{rowbinomCI}
\title{tableby Summary Statistics Functions}
\usage{
arsenal_sum(x, na.rm = TRUE, ...)

arsenal_min(x, na.rm = TRUE, ...)

arsenal_max(x, na.rm = TRUE, ...)

arsenal_mean(x, na.rm = TRUE, weights = NULL, ...)

arsenal_sd(x, na.rm = TRUE, weights = NULL, ...)

arsenal_var(x, na.rm = TRUE, weights = NULL, ...)

meansd(x, na.rm = TRUE, weights = NULL, ...)

meanse(x, na.rm = TRUE, weights = NULL, ...)

meanpmsd(x, na.rm = TRUE, weights = NULL, ...)

meanpmse(x, na.rm = TRUE, weights = NULL, ...)

meanCI(x, na.rm = TRUE, weights = NULL, conf.level = 0.95, ...)

medianrange(x, na.rm = TRUE, weights = NULL, ...)

medianmad(x, na.rm = TRUE, weights = NULL, ...)

arsenal_median(x, na.rm = TRUE, weights = NULL, ...)

arsenal_range(x, na.rm = TRUE, ...)

gmean(x, na.rm = TRUE, weights = NULL, ...)

gsd(x, na.rm = TRUE, weights = NULL, ...)

gmeansd(x, na.rm = TRUE, weights = NULL, ...)

gmeanCI(x, na.rm = TRUE, weights = NULL, conf.level = 0.95, ...)

Nsigntest(x, na.rm = TRUE, weights = NULL, ...)

Nevents(x, na.rm = TRUE, weights = NULL, ...)

medSurv(x, na.rm = TRUE, weights = NULL, ...)

medSurvCI(
  x,
  na.rm = TRUE,
  weights = NULL,
  robust = FALSE,
  conf.type = "log",
  ...
)

medSurvQuant(x, na.rm = TRUE, weights = NULL, robust = FALSE, ...)

NeventsSurv(x, na.rm = TRUE, weights = NULL, times = 1:5, ...)

NriskSurv(x, na.rm = TRUE, weights = NULL, times = 1:5, ...)

Nrisk(x, na.rm = TRUE, weights = NULL, times = 1:5, ...)

medTime(x, na.rm = TRUE, weights = NULL, ...)

q1q3(x, na.rm = TRUE, weights = NULL, ...)

medianq1q3(x, na.rm = TRUE, weights = NULL, ...)

iqr(x, na.rm = TRUE, weights = NULL, ...)

Nmiss(x, weights = NULL, ...)

Nmisspct(x, weights = NULL, ...)

Nmiss2(x, weights = NULL, ...)

Nmisspct2(x, weights = NULL, ...)

N(x, weights = NULL, ...)

Npct(x, weights = NULL, ...)

Nrowpct(
  x,
  levels = NULL,
  by,
  by.levels = sort(unique(by)),
  weights = NULL,
  ...,
  totallab = "Total"
)

count(x, levels = NULL, na.rm = TRUE, weights = NULL, ...)

countpct(x, levels = NULL, na.rm = TRUE, weights = NULL, ...)

pct(x, levels = NULL, na.rm = TRUE, weights = NULL, ...)

countN(x, levels = NULL, na.rm = TRUE, weights = NULL, ...)

countrowpct(
  x,
  levels = NULL,
  by,
  by.levels = sort(unique(by)),
  na.rm = TRUE,
  weights = NULL,
  ...,
  totallab = "Total"
)

rowpct(
  x,
  levels = NULL,
  by,
  by.levels = sort(unique(by)),
  na.rm = TRUE,
  weights = NULL,
  ...,
  totallab = "Total"
)

countcellpct(
  x,
  levels = NULL,
  by,
  by.levels = sort(unique(by)),
  na.rm = TRUE,
  weights = NULL,
  ...,
  totallab = "Total"
)

binomCI(x, levels = NULL, na.rm = TRUE, weights = NULL, conf.level = 0.95, ...)

rowbinomCI(
  x,
  levels = NULL,
  by,
  by.levels = sort(unique(by)),
  na.rm = TRUE,
  weights = NULL,
  conf.level = 0.95,
  ...,
  totallab = "Total"
)
}
\arguments{
\item{x}{Usually a vector.}

\item{na.rm}{Should NAs be removed?}

\item{...}{Other arguments.}

\item{weights}{A vector of weights.}

\item{conf.level}{Numeric, denoting what confidence level to use for confidence intervals.}

\item{robust, conf.type}{Passed to \code{survival::\link[survival]{survfit}}.}

\item{times}{A vector of times to use for survival summaries.}

\item{levels}{A vector of levels that character \code{x}s should have.}

\item{by}{a vector of the by-values.}

\item{by.levels}{a vector of the levels of \code{by}.}

\item{totallab}{What to call the total "column"}
}
\value{
Usually a vector of the appropriate numbers.
}
\description{
A collection of functions that will report summary statistics. To create a custom function,
  consider using a function with all three arguments and \code{...}. See the \code{\link{tableby}} vignette
  for an example.
}
\details{
Not all these functions are exported, in order to avoid conflicting NAMESPACES.
  Note also that the functions prefixed with \code{"arsenal_"} can be referred to by their short names
  (e.g., \code{"min"} for \code{"arsenal_min"}).
}
\seealso{
\code{\link{includeNA}}, \code{\link{tableby.control}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write2specific.R
\name{write2specific}
\alias{write2specific}
\alias{write2word}
\alias{write2pdf}
\alias{write2html}
\title{write2word, write2html, write2pdf}
\usage{
write2word(object, file, ...)

write2pdf(object, file, ...)

write2html(object, file, ...)
}
\arguments{
\item{object}{An object.}

\item{file}{A single character string denoting the filename for the output document.}

\item{...}{Additional arguments to be passed to \code{FUN}, \code{rmarkdown::render}, etc.
One popular option is to use \code{quiet = TRUE} to suppress the command line output.}
}
\value{
\code{object} is returned invisibly, and \code{file} is written.
}
\description{
Functions to output tables to a single Word, HTML, or PDF document.
}
\details{
To generate the appropriate file type, the \code{write2*} functions use one of \code{rmarkdown::word_document}, \code{rmarkdown::html_document},
   and \code{rmarkdown::pdf_document} to get the job done. \code{"..."} arguments are passed to these functions, too.
}
\examples{
\dontrun{
data(mockstudy)
# tableby example
tab1 <- tableby(arm ~ sex + age, data=mockstudy)
write2html(tab1, "~/trash.html")

# freqlist example
tab.ex <- table(mockstudy[, c("arm", "sex", "mdquality.s")], useNA = "ifany")
noby <- freqlist(tab.ex, na.options = "include")
write2pdf(noby, "~/trash2.pdf")

# A more complicated example
write2word(tab1, "~/trash.doc",
  keep.md = TRUE,
  reference_docx = mystyles.docx, # passed to rmarkdown::word_document
  quiet = TRUE, # passed to rmarkdown::render
  title = "My cool new title") # passed to summary.tableby
}
}
\seealso{
\code{\link{write2}}
}
\author{
Ethan Heinzen, adapted from code from Krista Goergen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modelsum.families.R
\name{modelsum.family}
\alias{modelsum.family}
\alias{survival}
\alias{ordinal}
\alias{negbin}
\alias{clog}
\alias{relrisk}
\title{Family functions for modelsum}
\usage{
survival()

ordinal(method = c("logistic", "probit", "loglog", "cloglog", "cauchit"))

negbin(link = c("log", "identity", "sqrt"))

clog()

relrisk(link = "log")
}
\arguments{
\item{method}{See \code{MASS::\link[MASS]{polr}}.}

\item{link}{See \code{MASS::\link[MASS]{glm.nb}}.}
}
\value{
A list, in particular with element \code{family}.
}
\description{
A set of family functions for \code{\link{modelsum}}.
}
\seealso{
\code{\link[stats]{family}}, \code{\link[survival]{coxph}}, \code{\link[MASS]{polr}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/keep.labels.R
\name{keep.labels}
\alias{keep.labels}
\alias{keep.labels.data.frame}
\alias{keep.labels.default}
\alias{[.keep_labels}
\alias{[<-.keep_labels}
\alias{loosen.labels}
\alias{loosen.labels.data.frame}
\alias{loosen.labels.default}
\title{Keep Labels}
\usage{
keep.labels(x, ...)

\method{keep.labels}{data.frame}(x, ...)

\method{keep.labels}{default}(x, ...)

\method{[}{keep_labels}(x, ...)

\method{[}{keep_labels}(x, i) <- value

loosen.labels(x, ...)

\method{loosen.labels}{data.frame}(x, ...)

\method{loosen.labels}{default}(x, ...)
}
\arguments{
\item{x}{An R object}

\item{...}{Other arguments (not in use at this time).}

\item{i, value}{See \code{\link{[<-}}.}
}
\value{
A copy of \code{x} with a "keep_labels" class appended on or removed. Note that for the \code{data.frame} method,
  only classes on the columns are changed; the \code{data.frame} won't have an extra class appended. This is different from previous
  versions of \code{arsenal}.
}
\description{
Keep the \code{'label'} attribute on an R object when subsetting. \code{loosen.labels} allows the \code{'label'}
  attribute to be lost again.
}
\seealso{
\code{\link{labels}}
}
\author{
Ethan Heinzen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defunct.R
\name{arsenal-defunct}
\alias{arsenal-defunct}
\title{Defunct functions in \code{arsenal}}
\arguments{
\item{x, y}{See \code{\link{comparedf}}.}

\item{...}{Other arguments.}
}
\description{
Details about defunct functions in \code{arsenal}
}
\details{
\code{comparison.control} was renamed to \code{\link{comparedf.control}} in version 3.0.0.

\code{compare.data.frame} was renamed to \code{\link{comparedf}} in version 3.0.0.

\code{length.tableby} was removed in version 2.0.0.

\code{includeNA.character} and \code{includeNA.numeric} were removed in version 2.0.0 and
  replaced with a default method.

\code{rangeTime} was removed in version 1.5.0.
}
\seealso{
\code{\link{arsenal-deprecated}}, \code{\link{comparedf}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/paired.R
\name{paired}
\alias{paired}
\title{Summary Statistics of a Set of Independent Variables Paired Across Two Timepoints}
\usage{
paired(
  formula,
  data,
  id,
  na.action,
  subset = NULL,
  strata,
  control = NULL,
  ...
)
}
\arguments{
\item{formula}{an object of class \code{\link{formula}} of the form \code{time ~ var1 + ...}.
See "Details" for more information.}

\item{data}{an optional data frame, list or environment (or object coercible by \code{\link{as.data.frame}} to a data frame)
containing the variables in the model. If not found in data, the variables are taken from \code{environment(formula)},
typically the environment from which the function is called.}

\item{id}{The vector giving IDs to match up data for the same subject across two timepoints.}

\item{na.action}{a function which indicates what should happen when the data contain \code{NA}s.
The default is \code{na.paired("in.both")}. See \code{\link{na.paired}} for more details}

\item{subset}{an optional vector specifying a subset of observations (rows of data) to be used in the results.
Works as vector of logicals or an index.}

\item{strata}{a vector of strata to separate summaries by an additional group.}

\item{control}{control parameters to handle optional settings within \code{paired}.
Two aspects of \code{paired} are controlled with these: test options of RHS variables and x variable summaries.
Arguments for \code{paired.control} can be passed to \code{paired} via the \code{...} argument,
but if a control object and \code{...} arguments are both supplied,
the latter are used. See \code{\link{paired.control}} for more details.}

\item{...}{additional arguments to be passed to internal \code{paired} functions or \code{\link{paired.control}}.}
}
\value{
An object with class \code{c("paired", "tableby", "arsenal_table")}
}
\description{
Summarize one or more variables (x) by a paired time variable (y). Variables
  on the right side of the formula, i.e. independent variables, are summarized by the
  two time points on the left of the formula. Optionally, an appropriate test is performed to test the
  distribution of the independent variables across the time points.
}
\details{
Do note that this function piggybacks off of \code{\link{tableby}} quite heavily, so there is no
\code{summary.paired} function (for instance).

These tests are accepted:
\itemize{
  \item{
    \code{paired.t}: a paired \code{\link[stats:t.test]{t-test}}.
  }
  \item{
    \code{mcnemar}: \link[stats:mcnemar.test]{McNemar's test}.
  }
  \item{
    \code{signed.rank}: a \link[stats:wilcox.test]{signed rank test}.
  }
  \item{
    \code{signtest}: a sign test.
  }
  \item{
    \code{notest}: no test is performed.
  }
}
}
\seealso{
\code{\link{arsenal_table}}, \code{\link{paired.control}}, \code{\link{tableby}}, \code{\link{formulize}}, \code{\link{selectall}}
}
\author{
Jason Sinnwell, Beth Atkinson, Ryan Lennon, and Ethan Heinzen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.tableby.R
\name{summary.tableby}
\alias{summary.tableby}
\alias{as.data.frame.summary.tableby}
\title{The summary method for a \code{tableby} object}
\usage{
\method{summary}{tableby}(
  object,
  ...,
  labelTranslations = NULL,
  text = FALSE,
  title = NULL,
  pfootnote = FALSE,
  term.name = ""
)

\method{as.data.frame}{summary.tableby}(
  x,
  ...,
  text = x$text,
  pfootnote = x$pfootnote,
  term.name = x$term.name,
  width = NULL,
  min.split = NULL,
  list.ok = FALSE
)
}
\arguments{
\item{object}{An object of class \code{"tableby"}, made by the \code{\link{tableby}} function.}

\item{...}{For \code{summary.tableby}, other arguments passed to \code{\link{as.data.frame.tableby}}.
For \code{print}ing the summary object, these are passed to both \code{as.data.frame.summary.tableby} and
\code{\link[knitr]{kable}}.}

\item{labelTranslations}{A named list (or vector) where the name is the label in the
output to be replaced in the pretty rendering by the character string
value for the named element of the list, e.g., \code{list(age = "Age(Years)", meansd = "Mean(SD)")}.}

\item{text}{An argument denoting how to print the summary to the screen.
Default is \code{FALSE} (show markdown output). \code{TRUE} and \code{NULL} output a text-only version, with
the latter avoiding all formatting.
\code{"html"} uses the HTML tag \code{<strong>} instead of the markdown formatting, and \code{"latex"} uses
the LaTeX command \code{\\textbf}.}

\item{title}{Title/caption for the table, defaulting to \code{NULL} (no title). Passed to \code{\link[knitr]{kable}}.
Can be length > 1 if the more than one table is being printed.}

\item{pfootnote}{Logical, denoting whether to put footnotes describing the tests used to generate the p-values. Alternatively,
"html" to surround the outputted footnotes with \code{<li>}.}

\item{term.name}{A character vector denoting the column name for the "terms" column. It should be the same length
as the number of tables or less (it will get recycled if needed). The special value \code{TRUE} will
use the y-variable's label for each table.}

\item{x}{An object of class \code{"summary.tableby"}.}

\item{width, min.split}{Passed to \code{\link{smart.split}} for formatting of the "term" column.}

\item{list.ok}{If the object has multiple by-variables, is it okay to return a list of data.frames instead of a single data.frame?
If \code{FALSE} but there are multiple by-variables, a warning is issued.}
}
\value{
An object of class \code{summary.tableby}
}
\description{
The summary method for a \code{\link{tableby}} object, which is a pretty rendering of a \code{\link{tableby}}
object into a publication-quality results table in R Markdown, and can render well in text-only.
}
\examples{

set.seed(100)
## make 3+ categories for response
nsubj <- 90
mdat <- data.frame(Response=sample(c(1,2,3),nsubj, replace=TRUE),
                   Sex=sample(c("Male", "Female"), nsubj,replace=TRUE),
                   Age=round(rnorm(nsubj,mean=40, sd=5)),
                   HtIn=round(rnorm(nsubj,mean=65,sd=5)))

## allow default summaries on RHS variables
out <- tableby(Response ~ Sex + Age + HtIn, data=mdat)
summary(out, text=TRUE)
labels(out)
labels(out) <- c(Age="Age (years)", HtIn="Height (inches)")
summary(out, stats.labels=c(meansd="Mean-SD", q1q3 = "Q1-Q3"), text=TRUE)

}
\seealso{
\code{\link{tableby.control}}, \code{\link{tableby}}
}
\author{
Ethan Heinzen, based on code by Gregory Dougherty, Jason Sinnwell, Beth Atkinson,
  adapted from SAS Macros written by Paul Novotny and Ryan Lennon
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comparedf.tolerances.R
\name{comparedf.tolerances}
\alias{comparedf.tolerances}
\alias{tol.NA}
\alias{tol.num.absolute}
\alias{tol.num.percent}
\alias{tol.num.pct}
\alias{tol.factor.none}
\alias{tol.factor.levels}
\alias{tol.factor.labels}
\alias{tol.char.both}
\alias{tol.char.case}
\alias{tol.char.trim}
\alias{tol.char.none}
\alias{tol.date.absolute}
\alias{tol.logical.none}
\alias{tol.other.none}
\title{\code{comparedf} tolerances}
\usage{
tol.NA(x, y, idx)

tol.num.absolute(x, y, tol)

tol.num.percent(x, y, tol)

tol.num.pct(x, y, tol)

tol.factor.none(x, y)

tol.factor.levels(x, y)

tol.factor.labels(x, y)

tol.char.both(x, y)

tol.char.case(x, y)

tol.char.trim(x, y)

tol.char.none(x, y)

tol.date.absolute(x, y, tol)

tol.logical.none(x, y)

tol.other.none(x, y)
}
\arguments{
\item{x, y}{vectors of the appropriate lengths and types.}

\item{idx}{A logical vector of appropriate length.}

\item{tol}{A numeric tolerance}
}
\value{
A logical vector of length equal to that of \code{x} and \code{y}, where \code{TRUE} denotes a
  difference between \code{x} and \code{y}, and \code{FALSE} denotes no difference between \code{x} and \code{y}.
}
\description{
Internal functions defining tolerances for the \code{\link{comparedf.control}} function.
  To create your own tolerance definitions, see the vignette.
}
\details{
\code{tol.NA} takes as differences between two vectors any elements which are NA in one but not the other,
  or which are non-NA in both and \code{TRUE} in \code{idx}. It is useful for handling NAs in custom tolerance functions.
}
\seealso{
\code{\link{comparedf.control}}, \code{\link{comparedf}}
}
\author{
Ethan Heinzen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formulize.R
\name{formulize}
\alias{formulize}
\title{formulize}
\usage{
formulize(
  y = "",
  x,
  ...,
  data = NULL,
  collapse = "+",
  collapse.y = collapse,
  escape = FALSE
)
}
\arguments{
\item{y, x, ...}{Character vectors, names, or calls to be collapsed (by \code{"+"}) and put left-to-right in the formula.
If \code{data} is supplied, these can also be numeric, denoting which column name to use. See examples.}

\item{data}{An R object with non-null column names.}

\item{collapse}{How should terms be collapsed? Default is addition.}

\item{collapse.y}{How should the y-terms be collapsed? Default is addition. Also accepts the special string "list",
which combines them into a multiple-left-hand-side formula, for use in other functions.}

\item{escape}{A logical indicating whether character vectors should be coerced to names (that is, whether names with spaces should
be surrounded with backticks or not)}
}
\description{
A shortcut to generate one-, two-, or many-sided formulas from vectors of variable names.
}
\examples{
## two-sided formula
f1 <- formulize("y", c("x1", "x2", "x3"))

## one-sided formula
f2 <- formulize(x = c("x1", "x2", "x3"))

## multi-sided formula
f3 <- formulize("y", c("x1", "x2", "x3"), c("z1", "z2"), "w1")

## can use numerics for column names
data(mockstudy)
f4 <- formulize(y = 1, x = 2:4, data = mockstudy)

## mix and match
f5 <- formulize(1, c("x1", "x2", "x3"), data = mockstudy)

## get an interaction
f6 <- formulize("y", c("x1*x2", "x3"))

## get only interactions
f7 <- formulize("y", c("x1", "x2", "x3"), collapse = "*")

## no intercept
f8 <- formulize("y", "x1 - 1")
f9 <- formulize("y", c("x1", "x2", "-1"))

## LHS as a list to use in arsenal functions
f10 <- formulize(c("y1", "y2", "y3"), c("x", "z"), collapse.y = "list")

## use in an lm
f11 <- formulize(2, 3:4, data = mockstudy)
summary(lm(f11, data = mockstudy))

## using non-syntactic names or calls (like reformulate example)
f12 <- formulize(as.name("+-"), c("`P/E`", "`\% Growth`"))
f12 <- formulize("+-", c("P/E", "\% Growth"), escape = TRUE)

f <- Surv(ft, case) ~ a + b
f13 <- formulize(f[[2]], f[[3]])

}
\seealso{
\code{\link[stats:delete.response]{reformulate}}
}
\author{
Ethan Heinzen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/paired.internal.R
\name{paired.internal}
\alias{paired.internal}
\alias{na.paired}
\title{Helper functions for paired}
\usage{
na.paired(missings = c("in.both", "fill", "asis"))
}
\arguments{
\item{missings}{A character string denoting which action to take. See "Details", below.}
}
\value{
\code{na.paired} returns a function used to subset data.frames in \code{\link{paired}}.
}
\description{
A set of helper functions for \code{\link{paired}}.
}
\details{
All methods subset out any NA time points or IDs.
  \code{"in.both"} (the default) subsets the data.frame to individuals who appear at both time points.
  \code{"fill"} adds explicit missings for the people missing second time points.
  \code{"asis"} does nothing to add or remove missings.
}
\seealso{
\link{tableby.internal}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mockstudy.R
\docType{data}
\name{mockstudy}
\alias{mockstudy}
\alias{muck_up_mockstudy}
\title{Mock study data for examples}
\format{
A data frame with 1499 observations on the following 15 variables:
  \describe{
    \item{\code{case}}{a numeric identifier-patient ID}
    \item{\code{age}}{age in years}
    \item{\code{arm}}{treatment arm divided into 3 groups, character string }
    \item{\code{sex}}{a factor with levels \code{Male} \code{Female}}
    \item{\code{race}}{self-reported race/ethnicity, character string}
    \item{\code{fu.time}}{survival or censoring time in years}
    \item{\code{fu.stat}}{censoring status; 1=censor, 2=death}
    \item{\code{ps}}{integer, ECOG performance score }
    \item{\code{hgb}}{numeric, hemoglobin count}
    \item{\code{bmi}}{numeric, body mass index, kg/m^2}
    \item{\code{alk.phos}}{numeric, alkaline phosphatase}
    \item{\code{ast}}{numeric, aspartate transaminase }
    \item{\code{mdquality.s}}{integer, LASA QOL 0=Clinically Deficient, 1=Not Clinically Deficient  }
    \item{\code{age.ord}}{an ordered factor split of age, with levels
      \code{10-19} < \code{20-29} < \code{30-39} < \code{40-49} <
        \code{50-59} < \code{60-69} < \code{70-79} < \code{80-89}}
  }

An object of class \code{data.frame} with 1499 rows and 14 columns.
}
\usage{
mockstudy

muck_up_mockstudy()
}
\description{
Mock clinical study data for examples to test data manipulation and statistical functions.
  The function \code{muck_up_mockstudy()} is used in examples for \code{\link{comparedf}}.
}
\examples{
data(mockstudy)
str(mockstudy)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/yaml.R
\name{yaml}
\alias{yaml}
\alias{print.yaml}
\alias{c.yaml}
\alias{is.yaml}
\title{Include a YAML header in \code{write2}}
\usage{
yaml(...)

\method{print}{yaml}(x, ...)

\method{c}{yaml}(..., recursive = FALSE)

is.yaml(x)
}
\arguments{
\item{...}{For \code{yaml()}, arguments to be bundled into a list and passed to \code{\link[yaml]{as.yaml}}.
For \code{print.yaml()}, extra arguments. For \code{c.yaml()}, "yaml" objects to be concatenated.}

\item{x}{An object of class \code{"yaml"}.}

\item{recursive}{Not in use at this time.}
}
\value{
A text string of class \code{"yaml"}.
}
\description{
Include a YAML header in \code{write2}
}
\examples{
x <- yaml(title = "My cool title", author = "Ethan P Heinzen")
x
y <- yaml("header-includes" = list("\\\\usepackage[labelformat=empty]{caption}"))
y
c(x, y)
}
\seealso{
\code{\link[yaml]{as.yaml}}, \code{\link{write2}}
}
\author{
Ethan Heinzen, adapted from an idea by Brendan Broderick
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.comparedf.R
\name{summary.comparedf}
\alias{summary.comparedf}
\alias{print.summary.comparedf}
\title{The summary method for a \code{comparedf} object}
\usage{
\method{summary}{comparedf}(object, ..., show.attrs = FALSE)

\method{print}{summary.comparedf}(x, ..., format = "pandoc")
}
\arguments{
\item{object}{An object of class \code{"comparedf"}, as made by the \code{\link{comparedf}} S3 method.}

\item{...}{Other arguments passed to \code{\link{comparedf.control}}. In \code{print}, these are passed to \code{\link[knitr]{kable}}.}

\item{show.attrs}{Logical, denoting whether to show the actual attributes which are different. For (e.g.) factors with lots
of levels, this can make the tables quite wide, so this feature is \code{FALSE} by default.}

\item{x}{An object returned by the \code{summary.comparedf} function.}

\item{format}{Passed to \code{\link[knitr]{kable}}: the format for the table. The default here is "pandoc".
To use the default in \code{kable}, pass \code{NULL}.}
}
\value{
An object of class \code{"summary.comparedf"} is returned.
}
\description{
Print a more detailed output of the \code{\link{comparedf}} object.
}
\seealso{
\code{\link{comparedf}}, \code{\link{comparedf.control}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/freqlist.internal.R
\name{freqlist.internal}
\alias{freqlist.internal}
\alias{is.freqlist}
\alias{is.summary.freqlist}
\alias{head.summary.freqlist}
\alias{tail.summary.freqlist}
\alias{sort.freqlist}
\title{Helper functions for freqlist}
\usage{
is.freqlist(x)

is.summary.freqlist(x)

\method{head}{summary.freqlist}(x, n = 6L, ...)

\method{tail}{summary.freqlist}(x, n = 6L, ...)

\method{sort}{freqlist}(x, decreasing = FALSE, ...)
}
\arguments{
\item{x}{A \code{freqlist} object.}

\item{n}{A single integer. See \code{\link[utils]{head}} or \code{\link[utils:head]{tail}} for more details}

\item{...}{Other arguments.}

\item{decreasing}{Should the sort be increasing or decreasing?}
}
\description{
A set of helper functions for \code{\link{freqlist}}.
}
\details{
Note that \code{sort()} has to recalculate cumulative statistics. Note also that the reordering of rows
  will also affect which labels are duplicates; you may also want to consider using
  \code{dupLabels=TRUE} in \code{\link{freq.control}()}.
}
\seealso{
\code{\link{merge.freqlist}}, \code{\link{arsenal_table}}, \code{\link{sort}},
  \code{\link{freqlist}}, \code{\link{summary.freqlist}}, \code{\link{freq.control}},
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arsenal_table.R
\name{arsenal_table}
\alias{arsenal_table}
\alias{has_strata}
\alias{[.arsenal_table}
\alias{[.tableby}
\alias{labels.arsenal_table}
\alias{labels<-.arsenal_table}
\alias{print.arsenal_table}
\alias{merge.arsenal_table}
\alias{merge.freqlist}
\alias{print.summary.arsenal_table}
\title{\code{arsenal} tables with common structure}
\usage{
has_strata(x)

\method{[}{arsenal_table}(x, i, j, ...)

\method{[}{tableby}(x, i, j, ...)

\method{labels}{arsenal_table}(object, ...)

\method{labels}{arsenal_table}(x) <- value

\method{print}{arsenal_table}(x, ...)

\method{merge}{arsenal_table}(x, y, all = FALSE, all.x = all, all.y = all, ...)

\method{merge}{freqlist}(x, y, all = TRUE, ...)

\method{print}{summary.arsenal_table}(
  x,
  ...,
  format = if (!is.null(x$text) && x$text \%in\% c("html", "latex")) x$text else
    "markdown",
  escape = x$text \%nin\% c("html", "latex"),
  width = NULL,
  min.split = NULL
)
}
\arguments{
\item{x, y, object}{An object of class \code{"arsenal_table"}}

\item{i, j}{A vector to index \code{x} with: either names of variables, a numeric vector, or a logical vector of appropriate length.
\code{i} indexes the x-variables, and \code{j} indexes the by-variables.}

\item{...}{Other arguments (only used in \code{print.summary.arsenal_table})}

\item{value}{A list of new labels.}

\item{all, all.x, all.y}{Logicals, denoting which terms to keep if not all are in common.}

\item{format}{Passed to \code{\link[knitr]{kable}}: the format for the table. The default here is "markdown".
To use the default in \code{kable}, pass \code{NULL}. If \code{x$text} specifies LaTeX or HTML formatting,
that format is used in the table.}

\item{escape}{Passed to \code{\link[knitr]{kable}}: should special characters be escaped when printed?}

\item{width, min.split}{Passed to \code{\link{smart.split}} for formatting of the "term" column.}
}
\description{
\code{arsenal} tables with common structure
}
\seealso{
\code{\link{merge}}, \code{\link{labels}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/selectall.R
\name{selectall}
\alias{selectall}
\alias{as.selectall}
\alias{as.matrix.selectall}
\alias{[.selectall}
\alias{is.na.selectall}
\alias{is.selectall}
\title{Make a column for "select all" input}
\usage{
selectall(...)

as.selectall(x)

\method{as.matrix}{selectall}(x, ...)

\method{[}{selectall}(x, i, j, drop = FALSE)

\method{is.na}{selectall}(x)

is.selectall(x)
}
\arguments{
\item{...}{Named arguments of the same length. These should be logical, numeric (0/1) or a factor with two levels.}

\item{x}{An object of class "selectall"}

\item{i, j, drop}{Arguments to `[.matrix`}
}
\description{
Make a column for "select all" input
}
\examples{
d <- data.frame(grp = rep(c("A", "B"), each = 5))
d$s <- selectall(
  `Option 1` = c(rep(1, 4), rep(0, 6)),
  `Option 2` = c(0, 1, 0, 0, 0, 1, 1, 1, 0, 0),
  `Option 3` = 1,
  `Option 4` = 0
)
summary(tableby(grp ~ s, data = d), text = TRUE)
}
\seealso{
\code{\link{tableby}}, \code{\link{paired}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/freq.control.R
\name{freq.control}
\alias{freq.control}
\title{Control settings for \code{freqlist} function}
\usage{
freq.control(
  sparse = FALSE,
  single = FALSE,
  dupLabels = FALSE,
  digits.count = 0L,
  digits.pct = 2L,
  ...,
  digits = NULL
)
}
\arguments{
\item{sparse}{a logical value indicating whether to keep rows with counts of zero.
The default is \code{FALSE} (drop zero-count rows).}

\item{single}{logical, indicating whether to collapse results created using a strata variable into a single table for printing}

\item{dupLabels}{logical: should labels which are the same as the row above be printed? The default (\code{FALSE}) more
closely approximates \code{PROC FREQ} output from SAS, where a label carried down from the row above is left blank.}

\item{digits.count}{Number of decimal places for count values.}

\item{digits.pct}{Number of decimal places for percents.}

\item{...}{additional arguments.}

\item{digits}{A deprecated argument}
}
\value{
A list with settings to be used within the \code{freqlist} function.
}
\description{
Control test and summary settings for the \code{\link{freqlist}} function.
}
\seealso{
\code{\link{freqlist}}, \code{\link{summary.freqlist}}, \code{\link{freqlist.internal}}
}
\author{
Ethan Heinzen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mdy.Date.R
\name{mdy.Date}
\alias{mdy.Date}
\alias{Date.mdy}
\alias{is.Date}
\title{Convert numeric dates to Date object, and vice versa}
\usage{
mdy.Date(month, day, year, yearcut = 120)

Date.mdy(date)

is.Date(x)
}
\arguments{
\item{month}{integer, month (1-12).}

\item{day}{integer, day of the month (1-31, depending on the month).}

\item{year}{integer, either 2- or 4-digit year. If two-digit number, will add 1900 onto it, depending on range.}

\item{yearcut}{cutoff for method to know if to convert to 4-digit year.}

\item{date}{A date value.}

\item{x}{An object.}
}
\value{
\code{mdy.Date} returns a Date object, and Date.mdy returns a list with integer values for month, day, and year.
  \code{is.Date} returns a single logical value.
}
\description{
Convert numeric dates for month, day, and year to Date object, and vice versa.
}
\details{
Test if an object is a date.

More work may need to be done with yearcut and 2-digit years. Best to give a full 4-digit year.
}
\examples{
mdy.Date(9, 2, 2013)

tmp <- mdy.Date(9, 2, 2013)
Date.mdy(tmp)

is.Date(tmp)
}
\seealso{
\code{\link{Date}}, \code{\link{DateTimeClasses}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tableby.internal.R
\name{tbfmt}
\alias{tbfmt}
\alias{tableby.stats.internal}
\alias{as.tbstat}
\alias{as.tbstat_multirow}
\title{Internal \code{tableby} functions}
\usage{
tbfmt(x, digits = NULL, digits.count = NULL, digits.pct = NULL, ...)

as.tbstat(
  x,
  oldClass = NULL,
  fmt = "{y}",
  which.count = 0L,
  which.pct = 0L,
  ...
)

as.tbstat_multirow(x)
}
\arguments{
\item{x}{Usually a vector.}

\item{digits, digits.pct, digits.count}{Digits specifications}

\item{...}{arguments to pass to \code{as.tbstat}.}

\item{oldClass}{class(es) to add to the resulting object.}

\item{fmt}{A \code{\link[glue]{glue}} string, where the object is exposed as the variable \code{x},
and a default-formatted version (using \code{tbfmt}) exposed as the variable \code{y}. \code{digits}, \code{digits.count},
and \code{digits.pct} are also exposed.}

\item{which.count}{Which statistics are counts? The default is 0, indicating that none are.}

\item{which.pct}{Which statistics are percents? The default is 0, indicating that none are.}
}
\description{
A collection of functions that may help users create custom functions that are formatted correctly.
}
\details{
The vignette has an example on how to use these.

  \code{as.tbstat} defines a tableby statistic with its appropriate formatting.

  \code{tbfmt} applies some default formatting.

  \code{as.tbstat_multirow} marks an object (usually a list) for multiple-row printing.
}
